@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,14 +17,19 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ALTER_COLUMN_H
+#define DRIZZLED_ALTER_COLUMN_H
 
-#include <drizzled/memory/sql_alloc.h>
+#include "drizzled/memory/sql_alloc.h"
 
-namespace drizzled {
-
-class AlterColumn : public memory::SqlAlloc 
+namespace drizzled
 {
+
+namespace memory { class Root; }
+
+class Item;
+
+class AlterColumn :public memory::SqlAlloc {
 public:
   const char *name;
   Item *def;
@@ -32,7 +37,17 @@ public:
     name(par_name),
     def(literal)
   {}
+  
+  /**
+    Used to make a clone of this object for ALTER/CREATE TABLE
+    @sa comment for Key_part_spec::clone
+  */
+  AlterColumn *clone(memory::Root *mem_root) const
+  {
+    return new (mem_root) AlterColumn(*this);
+  }
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ALTER_COLUMN_H */
