@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,9 +25,8 @@
  * @TODO Convert HA_XXX defines into enums and/or bitmaps
  */
 
-#include <drizzled/definitions.h>
-
-#pragma once
+#ifndef DRIZZLED_BASE_H
+#define DRIZZLED_BASE_H
 
 namespace drizzled
 {
@@ -86,6 +85,14 @@ enum ha_key_alg {
   HA_KEY_ALG_HASH=	3		/* HASH keys (HEAP tables) */
 };
 
+	/* Index and table build methods */
+
+enum ha_build_method {
+  HA_BUILD_DEFAULT,
+  HA_BUILD_ONLINE,
+  HA_BUILD_OFFLINE
+};
+
 	/* The following is parameter to ha_extra() */
 
 enum ha_extra_function {
@@ -185,6 +192,7 @@ enum ha_base_keytype {
   HA_KEYTYPE_ULONG_INT=9,
   HA_KEYTYPE_LONGLONG=10,
   HA_KEYTYPE_ULONGLONG=11,
+  HA_KEYTYPE_UINT24=13,
   /* Varchar (0-255 bytes) with length packed with 1 byte */
   HA_KEYTYPE_VARTEXT1=15,               /* Key is sorted as letters */
   HA_KEYTYPE_VARBINARY1=16,             /* Key is sorted as unsigned chars */
@@ -299,7 +307,84 @@ enum ha_base_keytype {
 */
 #define HA_ERR_FIRST            120     /* Copy of first error nr.*/
 
-/* Other constants */
+#define HA_ERR_KEY_NOT_FOUND	120	/* Didn't find key on read or update */
+#define HA_ERR_FOUND_DUPP_KEY	121	/* Dupplicate key on write */
+#define HA_ERR_INTERNAL_ERROR   122     /* Internal error */
+#define HA_ERR_RECORD_CHANGED	123	/* Uppdate with is recoverable */
+#define HA_ERR_WRONG_INDEX	124	/* Wrong index given to function */
+#define HA_ERR_CRASHED		126	/* Indexfile is crashed */
+#define HA_ERR_WRONG_IN_RECORD	127	/* Record-file is crashed */
+#define HA_ERR_OUT_OF_MEM	128	/* Record-file is crashed */
+#define HA_ERR_NOT_A_TABLE      130     /* not a MYI file - no signature */
+#define HA_ERR_WRONG_COMMAND	131	/* Command not supported */
+#define HA_ERR_OLD_FILE		132	/* old databasfile */
+#define HA_ERR_NO_ACTIVE_RECORD 133	/* No record read in update() */
+#define HA_ERR_RECORD_DELETED	134	/* A record is not there */
+#define HA_ERR_RECORD_FILE_FULL 135	/* No more room in file */
+#define HA_ERR_INDEX_FILE_FULL	136	/* No more room in file */
+#define HA_ERR_END_OF_FILE	137	/* end in next/prev/first/last */
+#define HA_ERR_UNSUPPORTED	138	/* unsupported extension used */
+#define HA_ERR_TO_BIG_ROW	139	/* Too big row */
+#define HA_WRONG_CREATE_OPTION	140	/* Wrong create option */
+#define HA_ERR_FOUND_DUPP_UNIQUE 141	/* Dupplicate unique on write */
+#define HA_ERR_UNKNOWN_CHARSET	 142	/* Can't open charset */
+#define HA_ERR_WRONG_MRG_TABLE_DEF 143  /* conflicting tables in MERGE */
+#define HA_ERR_CRASHED_ON_REPAIR 144	/* Last (automatic?) repair failed */
+#define HA_ERR_CRASHED_ON_USAGE  145	/* Table must be repaired */
+#define HA_ERR_LOCK_WAIT_TIMEOUT 146
+#define HA_ERR_LOCK_TABLE_FULL   147
+#define HA_ERR_READ_ONLY_TRANSACTION 148 /* Updates not allowed */
+#define HA_ERR_LOCK_DEADLOCK	 149
+#define HA_ERR_CANNOT_ADD_FOREIGN 150    /* Cannot add a foreign key constr. */
+#define HA_ERR_NO_REFERENCED_ROW 151     /* Cannot add a child row */
+#define HA_ERR_ROW_IS_REFERENCED 152     /* Cannot delete a parent row */
+#define HA_ERR_NO_SAVEPOINT	 153     /* No savepoint with that name */
+#define HA_ERR_NON_UNIQUE_BLOCK_SIZE 154 /* Non unique key block size */
+#define HA_ERR_NO_SUCH_TABLE     155  /* The table does not exist in engine */
+#define HA_ERR_TABLE_EXIST       156  /* The table existed in storage engine */
+#define HA_ERR_NO_CONNECTION     157  /* Could not connect to storage engine */
+/* NULLs are not supported in spatial index */
+#define HA_ERR_NULL_IN_SPATIAL   158
+#define HA_ERR_TABLE_DEF_CHANGED 159  /* The table changed in storage engine */
+/* There's no partition in table for given value */
+#define HA_ERR_NO_PARTITION_FOUND 160
+#define HA_ERR_RBR_LOGGING_FAILED 161  /* Row-based binlogging of row failed */
+#define HA_ERR_DROP_INDEX_FK      162  /* Index needed in foreign key constr */
+/*
+  Upholding foreign key constraints would lead to a duplicate key error
+  in some other table.
+*/
+#define HA_ERR_FOREIGN_DUPLICATE_KEY 163
+/* The table changed in storage engine */
+#define HA_ERR_TABLE_NEEDS_UPGRADE 164
+#define HA_ERR_TABLE_READONLY      165   /* The table is not writable */
+
+#define HA_ERR_AUTOINC_READ_FAILED 166   /* Failed to get next autoinc value */
+#define HA_ERR_AUTOINC_ERANGE    167     /* Failed to set row autoinc value */
+#define HA_ERR_GENERIC           168     /* Generic error */
+/* row not actually updated: new values same as the old values */
+#define HA_ERR_RECORD_IS_THE_SAME 169
+/* It is not possible to log this statement */
+#define HA_ERR_LOGGING_IMPOSSIBLE 170    /* It is not possible to log this
+                                            statement */
+#define HA_ERR_TABLESPACE_EXIST   171
+#define HA_ERR_CORRUPT_EVENT      172    /* The event was corrupt, leading to
+                                            illegal data being read */
+#define HA_ERR_NEW_FILE	          173	 /* New file format */
+#define HA_ERR_ROWS_EVENT_APPLY   174    /* The event could not be processed
+                                            no other hanlder error happened */
+#define HA_ERR_INITIALIZATION     175    /* Error during initialization */
+#define HA_ERR_FILE_TOO_SHORT	  176	 /* File too short */
+#define HA_ERR_WRONG_CRC	  177	 /* Wrong CRC on page */
+#define HA_ERR_LOCK_OR_ACTIVE_TRANSACTION 178
+#define HA_ERR_NO_SUCH_TABLESPACE 179
+#define HA_ERR_TABLESPACE_NOT_EMPTY 180
+#define HA_ERR_LAST               180    /* Copy of last error nr */
+
+/* Number of different errors */
+#define HA_ERR_ERRORS            (HA_ERR_LAST - HA_ERR_FIRST + 1)
+
+	/* Other constants */
 
 typedef unsigned long key_part_map;
 #define HA_WHOLE_KEY  (~(key_part_map)0)
@@ -389,27 +474,26 @@ enum data_file_type {
 */
 #define NULL_RANGE	64
 
-class key_range
+typedef struct st_key_range
 {
-public:
   const unsigned char *key;
   uint32_t length;
   enum ha_rkey_function flag;
   key_part_map keypart_map;
-};
+} key_range;
 
-class KEY_MULTI_RANGE
+typedef struct st_key_multi_range
 {
-public:
   key_range start_key;
   key_range end_key;
   char  *ptr;                 /* Free to use by caller (ptr to row etc) */
   uint32_t  range_flag;           /* key range flags see above */
-};
+} KEY_MULTI_RANGE;
 
 
 /* For number of records */
 typedef uint64_t	ha_rows;
+#define rows2double(A)	uint64_t2double(A)
 
 #define HA_POS_ERROR	(~ (::drizzled::ha_rows) 0)
 #define HA_OFFSET_ERROR	(~ (::drizzled::internal::my_off_t) 0)
@@ -420,11 +504,8 @@ typedef uint64_t	ha_rows;
 #define MAX_FILE_SIZE	INT64_MAX
 #endif
 
-inline static uint32_t ha_varchar_packlength(uint32_t field_length)
-{
-  return (field_length < 256 ? 1 :2);
-}
-
+#define HA_VARCHAR_PACKLENGTH(field_length) ((field_length) < 256 ? 1 :2)
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_BASE_H */
