@@ -11,12 +11,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/internal/m_string.h>
-#include <drizzled/charset.h>
+#include "drizzled/internal/m_string.h"
+#include "drizzled/charset_info.h"
 #include <cerrno>
 #include <cstdio>
 #include <cstdlib>
@@ -27,13 +27,14 @@
 
 using namespace std;
 
-namespace drizzled {
+namespace drizzled
+{
 
 /*
   Returns the number of bytes required for strnxfrm().
 */
 
-size_t my_strnxfrmlen_simple(const charset_info_st * const cs, size_t len)
+size_t my_strnxfrmlen_simple(const CHARSET_INFO * const cs, size_t len)
 {
   return len * (cs->strxfrm_multiply ? cs->strxfrm_multiply : 1);
 }
@@ -46,7 +47,7 @@ size_t my_strnxfrmlen_simple(const charset_info_st * const cs, size_t len)
    end buffer must be checked.
 */
 
-size_t my_snprintf_8bit(const charset_info_st * const,
+size_t my_snprintf_8bit(const CHARSET_INFO * const,
                         char* to, size_t n,
 		     const char* fmt, ...)
 {
@@ -59,16 +60,16 @@ size_t my_snprintf_8bit(const charset_info_st * const,
 }
 
 
-long my_strntol_8bit(const charset_info_st * const cs,
+long my_strntol_8bit(const CHARSET_INFO * const cs,
 		     const char *nptr, size_t l, int base,
 		     char **endptr, int *err)
 {
   int negative;
-  uint32_t cutoff;
-  uint32_t cutlim;
-  uint32_t i;
-  const char *s;
-  unsigned char c;
+  register uint32_t cutoff;
+  register uint32_t cutlim;
+  register uint32_t i;
+  register const char *s;
+  register unsigned char c;
   const char *save, *e;
   int overflow;
 
@@ -81,7 +82,7 @@ long my_strntol_8bit(const charset_info_st * const cs,
   s = nptr;
   e = nptr+l;
 
-  for ( ; s<e && cs->isspace(*s) ; s++) {}
+  for ( ; s<e && my_isspace(cs, *s) ; s++) {}
 
   if (s == e)
   {
@@ -182,16 +183,16 @@ noconv:
 }
 
 
-ulong my_strntoul_8bit(const charset_info_st * const cs,
+ulong my_strntoul_8bit(const CHARSET_INFO * const cs,
 		       const char *nptr, size_t l, int base,
 		       char **endptr, int *err)
 {
   int negative;
-  uint32_t cutoff;
-  uint32_t cutlim;
-  uint32_t i;
-  const char *s;
-  unsigned char c;
+  register uint32_t cutoff;
+  register uint32_t cutlim;
+  register uint32_t i;
+  register const char *s;
+  register unsigned char c;
   const char *save, *e;
   int overflow;
 
@@ -204,7 +205,7 @@ ulong my_strntoul_8bit(const charset_info_st * const cs,
   s = nptr;
   e = nptr+l;
 
-  for( ; s<e && cs->isspace(*s); s++) {}
+  for( ; s<e && my_isspace(cs, *s); s++) {}
 
   if (s==e)
   {
@@ -283,7 +284,7 @@ ulong my_strntoul_8bit(const charset_info_st * const cs,
   if (overflow)
   {
     err[0]= ERANGE;
-    return UINT32_MAX;
+    return (~(uint32_t) 0);
   }
 
   return (negative ? -((long) i) : (long) i);
@@ -296,15 +297,15 @@ noconv:
 }
 
 
-int64_t my_strntoll_8bit(const charset_info_st * const cs,
+int64_t my_strntoll_8bit(const CHARSET_INFO * const cs,
 			  const char *nptr, size_t l, int base,
 			  char **endptr,int *err)
 {
   int negative;
-  uint64_t cutoff;
-  uint32_t cutlim;
-  uint64_t i;
-  const char *s, *e;
+  register uint64_t cutoff;
+  register uint32_t cutlim;
+  register uint64_t i;
+  register const char *s, *e;
   const char *save;
   int overflow;
 
@@ -317,7 +318,7 @@ int64_t my_strntoll_8bit(const charset_info_st * const cs,
   s = nptr;
   e = nptr+l;
 
-  for(; s<e && cs->isspace(*s); s++) {}
+  for(; s<e && my_isspace(cs,*s); s++) {}
 
   if (s == e)
   {
@@ -369,7 +370,7 @@ int64_t my_strntoll_8bit(const charset_info_st * const cs,
   i = 0;
   for ( ; s != e; s++)
   {
-    unsigned char c= *s;
+    register unsigned char c= *s;
     if (c>='0' && c<='9')
       c -= '0';
     else if (c>='A' && c<='Z')
@@ -419,15 +420,15 @@ noconv:
 }
 
 
-uint64_t my_strntoull_8bit(const charset_info_st * const cs,
+uint64_t my_strntoull_8bit(const CHARSET_INFO * const cs,
 			   const char *nptr, size_t l, int base,
 			   char **endptr, int *err)
 {
   int negative;
-  uint64_t cutoff;
-  uint32_t cutlim;
-  uint64_t i;
-  const char *s, *e;
+  register uint64_t cutoff;
+  register uint32_t cutlim;
+  register uint64_t i;
+  register const char *s, *e;
   const char *save;
   int overflow;
 
@@ -440,7 +441,7 @@ uint64_t my_strntoull_8bit(const charset_info_st * const cs,
   s = nptr;
   e = nptr+l;
 
-  for(; s<e && cs->isspace(*s); s++) {}
+  for(; s<e && my_isspace(cs,*s); s++) {}
 
   if (s == e)
   {
@@ -492,7 +493,7 @@ uint64_t my_strntoull_8bit(const charset_info_st * const cs,
   i = 0;
   for ( ; s != e; s++)
   {
-    unsigned char c= *s;
+    register unsigned char c= *s;
 
     if (c>='0' && c<='9')
       c -= '0';
@@ -558,7 +559,7 @@ noconv:
 */
 
 
-double my_strntod_8bit(const charset_info_st * const,
+double my_strntod_8bit(const CHARSET_INFO * const,
 		       char *str, size_t length,
 		       char **end, int *err)
 {
@@ -575,11 +576,11 @@ double my_strntod_8bit(const charset_info_st * const,
   Assume len >= 1
 */
 
-size_t my_long10_to_str_8bit(const charset_info_st * const,
+size_t my_long10_to_str_8bit(const CHARSET_INFO * const,
                              char *dst, size_t len, int radix, long int val)
 {
   char buffer[66];
-  char *p, *e;
+  register char *p, *e;
   long int new_val;
   uint32_t sign=0;
   unsigned long int uval = (unsigned long int) val;
@@ -616,12 +617,12 @@ size_t my_long10_to_str_8bit(const charset_info_st * const,
 }
 
 
-size_t my_int64_t10_to_str_8bit(const charset_info_st * const,
+size_t my_int64_t10_to_str_8bit(const CHARSET_INFO * const,
                                 char *dst, size_t len, int radix,
                                 int64_t val)
 {
   char buffer[65];
-  char *p, *e;
+  register char *p, *e;
   long long_val;
   uint32_t sign= 0;
   uint64_t uval = (uint64_t)val;
@@ -678,26 +679,16 @@ cnv:
 **	 1 if matched with wildcard
 */
 
-inline static int likeconv(const charset_info_st *cs, const char c) 
-{
 #ifdef LIKE_CMP_TOUPPER
-  return (unsigned char) cs->toupper(c);
+#define likeconv(s,A) (unsigned char) my_toupper(s,A)
 #else
-  return cs->sort_order[(unsigned char)c];
-#endif    
-}
+#define likeconv(s,A) (unsigned char) (s)->sort_order[(unsigned char) (A)]
+#endif
+
+#define INC_PTR(cs,A,B) (A)++
 
 
-inline static const char* inc_ptr(const charset_info_st *cs, const char *str, const char *str_end)
-{
-  // (Strange this macro have been used. If str_end would actually
-  // have been used it would have made sense. /Gustaf)
-  (void)cs;
-  (void)str_end;
-  return str++; 
-}
-
-int my_wildcmp_8bit(const charset_info_st * const cs,
+int my_wildcmp_8bit(const CHARSET_INFO * const cs,
 		    const char *str,const char *str_end,
 		    const char *wildstr,const char *wildend,
 		    int escape, int w_one, int w_many)
@@ -712,7 +703,7 @@ int my_wildcmp_8bit(const charset_info_st * const cs,
 	wildstr++;
 
       if (str == str_end || likeconv(cs,*wildstr++) != likeconv(cs,*str++))
-	return 1;				/* No match */
+	return(1);				/* No match */
       if (wildstr == wildend)
 	return(str != str_end);		/* Match if both are at end */
       result=1;					/* Found an anchor char     */
@@ -723,7 +714,7 @@ int my_wildcmp_8bit(const charset_info_st * const cs,
       {
 	if (str == str_end)			/* Skip one char if possible */
 	  return(result);
-	inc_ptr(cs,str,str_end);
+	INC_PTR(cs,str,str_end);
       } while (++wildstr < wildend && *wildstr == w_one);
       if (wildstr == wildend)
 	break;
@@ -742,20 +733,20 @@ int my_wildcmp_8bit(const charset_info_st * const cs,
 	{
 	  if (str == str_end)
 	    return(-1);
-	  inc_ptr(cs,str,str_end);
+	  INC_PTR(cs,str,str_end);
 	  continue;
 	}
 	break;					/* Not a wild character */
       }
       if (wildstr == wildend)
-	return 0;				/* Ok if w_many is last */
+	return(0);				/* Ok if w_many is last */
       if (str == str_end)
 	return(-1);
 
       if ((cmp= *wildstr) == escape && wildstr+1 != wildend)
 	cmp= *++wildstr;
 
-      inc_ptr(cs,wildstr,wildend);	/* This is compared trough cmp */
+      INC_PTR(cs,wildstr,wildend);	/* This is compared trough cmp */
       cmp=likeconv(cs,cmp);
       do
       {
@@ -794,7 +785,7 @@ int my_wildcmp_8bit(const charset_info_st * const cs,
 ** optimized !
 */
 
-bool my_like_range_simple(const charset_info_st * const cs,
+bool my_like_range_simple(const CHARSET_INFO * const cs,
                              const char *ptr, size_t ptr_length,
                              char escape, char w_one, char w_many,
                              size_t res_length,
@@ -844,7 +835,7 @@ bool my_like_range_simple(const charset_info_st * const cs,
 }
 
 
-size_t my_scan_8bit(const charset_info_st * const cs, const char *str, const char *end, int sq)
+size_t my_scan_8bit(const CHARSET_INFO * const cs, const char *str, const char *end, int sq)
 {
   const char *str0= str;
   switch (sq)
@@ -860,7 +851,7 @@ size_t my_scan_8bit(const charset_info_st * const cs, const char *str, const cha
   case MY_SEQ_SPACES:
     for ( ; str < end ; str++)
     {
-      if (!cs->isspace(*str))
+      if (!my_isspace(cs,*str))
         break;
     }
     return (size_t) (str - str0);
@@ -870,38 +861,45 @@ size_t my_scan_8bit(const charset_info_st * const cs, const char *str, const cha
 }
 
 
-void my_fill_8bit(const charset_info_st * const, char *s, size_t l, int fill)
+void my_fill_8bit(const CHARSET_INFO * const,
+		  char *s, size_t l, int fill)
 {
   memset(s, fill, l);
 }
 
 
-size_t my_numchars_8bit(const charset_info_st * const, const char *b, const char *e)
+size_t my_numchars_8bit(const CHARSET_INFO * const,
+		        const char *b, const char *e)
 {
   return (size_t) (e - b);
 }
 
 
-size_t my_numcells_8bit(const charset_info_st * const, const char *b, const char *e)
+size_t my_numcells_8bit(const CHARSET_INFO * const,
+                        const char *b, const char *e)
 {
   return (size_t) (e - b);
 }
 
 
-size_t my_charpos_8bit(const charset_info_st * const, const char *, const char *, size_t pos)
+size_t my_charpos_8bit(const CHARSET_INFO * const,
+                       const char *, const char *, size_t pos)
 {
   return pos;
 }
 
 
-size_t my_well_formed_len_8bit(const charset_info_st&, str_ref str, size_t nchars, int *error)
+size_t my_well_formed_len_8bit(const CHARSET_INFO * const,
+                               const char *start, const char *end,
+                               size_t nchars, int *error)
 {
+  size_t nbytes= (size_t) (end-start);
   *error= 0;
-  return min(str.size(), nchars);
+  return min(nbytes, nchars);
 }
 
 
-size_t my_lengthsp_8bit(const charset_info_st * const,
+size_t my_lengthsp_8bit(const CHARSET_INFO * const,
                         const char *ptr, size_t length)
 {
   const char *end;
@@ -910,12 +908,12 @@ size_t my_lengthsp_8bit(const charset_info_st * const,
 }
 
 
-uint32_t my_instr_simple(const charset_info_st * const cs,
+uint32_t my_instr_simple(const CHARSET_INFO * const cs,
                      const char *b, size_t b_length,
                      const char *s, size_t s_length,
                      my_match_t *match, uint32_t nmatch)
 {
-  const unsigned char *str, *search, *end, *search_end;
+  register const unsigned char *str, *search, *end, *search_end;
 
   if (s_length <= b_length)
   {
@@ -940,7 +938,7 @@ skip:
     {
       if (cs->sort_order[*str++] == cs->sort_order[*search])
       {
-	const unsigned char *i,*j;
+	register const unsigned char *i,*j;
 
 	i= str;
 	j= search+1;
@@ -978,10 +976,7 @@ typedef struct
 
 #define PLANE_SIZE	0x100
 #define PLANE_NUM	0x100
-inline static int plane_number(uint16_t x)
-{
-  return ((x >> 8) % PLANE_NUM);
-}
+#define PLANE_NUMBER(x)	(((x)>>8) % PLANE_NUM)
 
 static int pcmp(const void * f, const void * s)
 {
@@ -994,7 +989,7 @@ static int pcmp(const void * f, const void * s)
   return res;
 }
 
-static bool create_fromuni(charset_info_st *cs, cs_alloc_func alloc)
+static bool create_fromuni(CHARSET_INFO *cs, cs_alloc_func alloc)
 {
   uni_idx	idx[PLANE_NUM];
   int		i,n;
@@ -1015,7 +1010,7 @@ static bool create_fromuni(charset_info_st *cs, cs_alloc_func alloc)
   for (i=0; i< 0x100; i++)
   {
     uint16_t wc=cs->tab_to_uni[i];
-    int pl= plane_number(wc);
+    int pl= PLANE_NUMBER(wc);
 
     if (wc || !i)
     {
@@ -1073,7 +1068,7 @@ static bool create_fromuni(charset_info_st *cs, cs_alloc_func alloc)
   return false;
 }
 
-bool my_cset_init_8bit(charset_info_st *cs, cs_alloc_func alloc)
+bool my_cset_init_8bit(CHARSET_INFO *cs, cs_alloc_func alloc)
 {
   cs->caseup_multiply= 1;
   cs->casedn_multiply= 1;
@@ -1081,7 +1076,7 @@ bool my_cset_init_8bit(charset_info_st *cs, cs_alloc_func alloc)
   return create_fromuni(cs, alloc);
 }
 
-static void set_max_sort_char(charset_info_st *cs)
+static void set_max_sort_char(CHARSET_INFO *cs)
 {
   unsigned char max_char;
   uint32_t  i;
@@ -1100,21 +1095,21 @@ static void set_max_sort_char(charset_info_st *cs)
   }
 }
 
-bool my_coll_init_simple(charset_info_st *cs, cs_alloc_func)
+bool my_coll_init_simple(CHARSET_INFO *cs, cs_alloc_func)
 {
   set_max_sort_char(cs);
   return false;
 }
 
 
-int64_t my_strtoll10_8bit(const charset_info_st * const,
+int64_t my_strtoll10_8bit(const CHARSET_INFO * const,
                           const char *nptr, char **endptr, int *error)
 {
   return internal::my_strtoll10(nptr, endptr, error);
 }
 
 
-int my_mb_ctype_8bit(const charset_info_st * const cs, int *ctype,
+int my_mb_ctype_8bit(const CHARSET_INFO * const cs, int *ctype,
                    const unsigned char *s, const unsigned char *e)
 {
   if (s >= e)
@@ -1216,7 +1211,7 @@ static uint64_t d10[DIGITS_IN_ULONGLONG]=
 */
 
 uint64_t
-my_strntoull10rnd_8bit(const charset_info_st * const,
+my_strntoull10rnd_8bit(const CHARSET_INFO * const,
                        const char *str, size_t length, int unsigned_flag,
                        char **endptr, int *error)
 {
@@ -1494,16 +1489,83 @@ ret_too_big:
 
 
 
-bool my_propagate_simple()
+bool my_propagate_simple(const CHARSET_INFO * const, const unsigned char *,
+                         size_t)
 {
   return 1;
 }
 
-bool my_propagate_complex()
+
+bool my_propagate_complex(const CHARSET_INFO * const, const unsigned char *,
+                          size_t)
 {
   return 0;
 }
 
+
+
+/*
+  Normalize strxfrm flags
+
+  SYNOPSIS:
+    my_strxfrm_flag_normalize()
+    flags    - non-normalized flags
+    nlevels  - number of levels
+
+  NOTES:
+    If levels are omitted, then 1-maximum is assumed.
+    If any level number is greater than the maximum,
+    it is treated as the maximum.
+
+  RETURN
+    normalized flags
+*/
+
+uint32_t my_strxfrm_flag_normalize(uint32_t flags, uint32_t maximum)
+{
+  assert(maximum >= 1 && maximum <= MY_STRXFRM_NLEVELS);
+
+  /* If levels are omitted, then 1-maximum is assumed*/
+  if (!(flags & MY_STRXFRM_LEVEL_ALL))
+  {
+    static uint32_t def_level_flags[]= {0, 0x01, 0x03, 0x07, 0x0F, 0x1F, 0x3F };
+    uint32_t flag_pad= flags & MY_STRXFRM_PAD_WITH_SPACE;
+    flags= def_level_flags[maximum] | flag_pad;
+  }
+  else
+  {
+    uint32_t i;
+    uint32_t flag_lev= flags & MY_STRXFRM_LEVEL_ALL;
+    uint32_t flag_dsc= (flags >> MY_STRXFRM_DESC_SHIFT) & MY_STRXFRM_LEVEL_ALL;
+    uint32_t flag_rev= (flags >> MY_STRXFRM_REVERSE_SHIFT) & MY_STRXFRM_LEVEL_ALL;
+    uint32_t flag_pad= flags & MY_STRXFRM_PAD_WITH_SPACE;
+
+    /*
+      If any level number is greater than the maximum,
+      it is treated as the maximum.
+    */
+    for (maximum--, flags= 0, i= 0; i < MY_STRXFRM_NLEVELS; i++)
+    {
+      uint32_t src_bit= 1 << i;
+      uint32_t dst_bit= 1 << min(i, maximum);
+      if (flag_lev & src_bit)
+      {
+        flags|= dst_bit;
+        flags|= (flag_dsc & dst_bit) << MY_STRXFRM_DESC_SHIFT;
+        flags|= (flag_rev & dst_bit) << MY_STRXFRM_REVERSE_SHIFT;
+      }
+      else
+      {
+        /* Check that there are no DESC or REVERSE flag for skipped level */
+        assert(!(flag_dsc & src_bit) && !(flag_rev & src_bit));
+      }
+    }
+    flags|= flag_pad;
+  }
+
+  return flags;
+}
+
 /*
   Apply DESC and REVERSE collation rules.
 
@@ -1561,7 +1623,7 @@ void my_strxfrm_desc_and_reverse(unsigned char *str, unsigned char *strend,
 
 
 size_t
-my_strxfrm_pad_desc_and_reverse(const charset_info_st * const cs,
+my_strxfrm_pad_desc_and_reverse(const CHARSET_INFO * const cs,
                                 unsigned char *str, unsigned char *frmend, unsigned char *strend,
                                 uint32_t nweights, uint32_t flags, uint32_t level)
 {
