@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,34 +17,26 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 #include <drizzled/current_session.h>
 
-namespace drizzled {
+#include <pthread.h>
 
-static boost::thread_specific_ptr<Session> THR_Session;
-static boost::thread_specific_ptr<memory::Root> THR_Mem_root;
-
-Session* _current_session()
+namespace drizzled
 {
-  return THR_Session.get();
+
+extern pthread_key_t THR_Session;
+extern pthread_key_t THR_Mem_root;
+
+Session *_current_session(void)
+{
+  return static_cast<Session *>(pthread_getspecific(THR_Session));
 }
 
-memory::Root* current_mem_root()
-{
-  return THR_Mem_root.get();
-}
 
-void setCurrentSession(Session* v)
+memory::Root *current_mem_root(void)
 {
-  THR_Session.release();
-  THR_Session.reset(v);
-}
-
-void setCurrentMemRoot(memory::Root* v)
-{
-  THR_Mem_root.release();
-  THR_Mem_root.reset(v);
+  return *(static_cast<memory::Root **>(pthread_getspecific(THR_Mem_root)));
 }
 
 } /* namespace drizzled */
