@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,20 +18,20 @@
  */
 
 
-#pragma once
+#ifndef DRIZZLED_CURRENT_SESSION_H
+#define DRIZZLED_CURRENT_SESSION_H
 
-#include <pthread.h>
-#include <boost/thread/tss.hpp>
-#include <drizzled/common_fwd.h>
-#include <drizzled/visibility.h>
+namespace drizzled
+{
 
-namespace drizzled {
+class Session;
 
-DRIZZLED_API Session *_current_session();
+namespace memory { class Root; }
+
+Session *_current_session(void);
 #define current_session ::drizzled::_current_session()
-memory::Root *current_mem_root();
-
-void setCurrentSession(Session*);
-void setCurrentMemRoot(memory::Root*);
+memory::Root *current_mem_root(void);
 
 } /* namespace drizzled */
+
+#endif /* DRIZZLED_CURRENT_SESSION_H */
