@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,39 +17,88 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_CURSOR_H
+#define DRIZZLED_CURSOR_H
 
-#include <drizzled/atomics.h>
-#include <drizzled/definitions.h>
+#include <drizzled/xid.h>
 #include <drizzled/discrete_interval.h>
-#include <drizzled/error_t.h>
-#include <drizzled/ha_statistics.h>
-#include <drizzled/handler_structs.h>
-#include <drizzled/identifier.h>
-#include <drizzled/key_map.h>
-#include <drizzled/message/table.h>
-#include <drizzled/sql_list.h>
+#include <drizzled/table_identifier.h>
+
+/* Definitions for parameters to do with Cursor-routines */
+
 #include <drizzled/thr_lock.h>
+#include <drizzled/sql_string.h>
+#include <drizzled/sql_list.h>
+#include <drizzled/plugin/storage_engine.h>
+#include <drizzled/handler_structs.h>
+#include <drizzled/ha_statistics.h>
+#include <drizzled/atomics.h>
+
+#include <drizzled/message/table.pb.h>
+
+/* Bits to show what an alter table will do */
+#include <drizzled/sql_bitmap.h>
+
+#include <drizzled/cursor.h>
 
 #include <bitset>
 #include <algorithm>
 
-#include <drizzled/visibility.h>
-
-namespace drizzled {
+namespace drizzled
+{
 
 #define HA_MAX_ALTER_FLAGS 40
 
+
 typedef std::bitset<HA_MAX_ALTER_FLAGS> HA_ALTER_FLAGS;
 
+extern uint64_t refresh_version;  /* Increments on each reload */
+
+
+typedef bool (*qc_engine_callback)(Session *session, char *table_key,
+                                      uint32_t key_length,
+                                      uint64_t *engine_data);
+
+
+/* The Cursor for a table type.  Will be included in the Table structure */
+
+class Table;
+class TableList;
+class TableShare;
+class Select_Lex_Unit;
+struct st_foreign_key_info;
+typedef struct st_foreign_key_info FOREIGN_KEY_INFO;
+struct order_st;
+
+class Item;
+struct st_table_log_memory_entry;
+
+class LEX;
+class Select_Lex;
+class AlterInfo;
+class select_result;
+class CreateField;
+class sys_var_str;
+class Item_ident;
+typedef struct st_sort_field SORT_FIELD;
+
 typedef List<Item> List_item;
 extern KEY_CREATE_INFO default_key_create_info;
 
+/* Forward declaration for condition pushdown to storage engine */
+typedef class Item COND;
+
+typedef struct system_status_var system_status_var;
+
+class COST_VECT;
+
+uint32_t calculate_key_len(Table *, uint, const unsigned char *, key_part_map);
 /*
   bitmap with first N+1 bits set
   (keypart_map for a key prefix of [0..N] keyparts)
 */
-inline key_part_map make_keypart_map(int a)
+template<class T>
+inline key_part_map make_keypart_map(T a)
 {
   return (((key_part_map)2 << a) - 1);
 }
@@ -58,7 +107,8 @@ inline key_part_map make_keypart_map(int a)
   bitmap with first N bits set
   (keypart_map for a key prefix of [0..N-1] keyparts)
 */
-inline key_part_map make_prev_keypart_map(int a)
+template<class T>
+inline key_part_map make_prev_keypart_map(T a)
 {
   return (((key_part_map)1 << a) - 1);
 }
@@ -99,6 +149,10 @@ inline key_part_map make_prev_keypart_map(int a)
   present, its length is one byte <not-sure> which must be set to 0xFF
   at all times. </not-sure>
 
+  If the table has columns of type BIT, then certain bits from those columns
+  may be stored in null_bytes as well. Grep around for Field_bit for
+  details.
+
   For blob columns (see Field_blob), the record buffer stores length of the
   data, following by memory pointer to the blob data. The pointer is owned
   by the storage engine and is valid until the next operation.
@@ -106,35 +160,28 @@ inline key_part_map make_prev_keypart_map(int a)
   If a blob column has NULL value, then its length and blob data pointer
   must be set to 0.
 */
-class DRIZZLED_API Cursor
+
+class Cursor :public memory::SqlAlloc
 {
-  friend class SEAPITesterCursor;
-  Table &table;               /* The current open table */
-  plugin::StorageEngine &engine;      /* storage engine of this Cursor */
-
 protected:
-  ha_rows estimation_rows_to_insert;
+  TableShare *table_share;   /* The table definition */
+  Table *table;               /* The current open table */
 
+  ha_rows estimation_rows_to_insert;
 public:
+  plugin::StorageEngine *engine;      /* storage engine of this Cursor */
   inline plugin::StorageEngine *getEngine() const	/* table_type for handler */
   {
-    return &engine;
+    return engine;
   }
   unsigned char *ref;				/* Pointer to current row */
   unsigned char *dup_ref;			/* Pointer to duplicate row */
 
-  TableShare *getShare();
-
-  Table *getTable() const
-  {
-    return &table;
-  }
-
   ha_statistics stats;
   /** MultiRangeRead-related members: */
   range_seq_t mrr_iter;    /* Interator to traverse the range sequence */
   RANGE_SEQ_IF mrr_funcs;  /* Range sequence traversal functions */
-
+  HANDLER_BUFFER *multi_range_buffer; /* MRR buffer info */
   uint32_t ranges_in_seq; /* Total number of ranges in the traversed sequence */
   /* true <=> source MRR ranges and the output are ordered */
   bool mrr_is_output_sorted;
@@ -143,13 +190,18 @@ public:
   bool mrr_have_range;
 
   bool eq_range;
+  /*
+    true <=> the engine guarantees that returned records are within the range
+    being scanned.
+  */
+  bool in_range_check_pushed_down;
 
   /** Current range (the one we're now returning rows from) */
   KEY_MULTI_RANGE mrr_cur_range;
 
   /** The following are for read_range() */
   key_range save_end_range, *end_range;
-  KeyPartInfo *range_key_part;
+  KEY_PART_INFO *range_key_part;
   int key_compare_result_on_equal;
 
   uint32_t errkey;				/* Last dup key */
@@ -159,6 +211,7 @@ public:
   uint32_t ref_length;
   enum {NONE=0, INDEX, RND} inited;
   bool locked;
+  bool implicit_emptied;                /* Can be !=0 only if HEAP */
 
   /**
     next_insert_id is the next value which should be inserted into the
@@ -170,19 +223,11 @@ public:
     get_auto_increment().
   */
   uint64_t next_insert_id;
-  uint64_t getNextInsertId() const
+  uint64_t getNextInsertId()
   {
     return next_insert_id;
   }
 
-  /**
-    Used by SHOW TABLE STATUS to get the current auto_inc from the engine
-  */
-  uint64_t getAutoIncrement() const
-  {
-    return stats.auto_increment_value;
-  }
-
   /**
     insert id for the current row (*autogenerated*; if not
     autogenerated, it's 0).
@@ -196,17 +241,17 @@ public:
   */
   Discrete_interval auto_inc_interval_for_cur_row;
 
-  Cursor(plugin::StorageEngine &engine_arg, Table &share_arg);
+  Cursor(plugin::StorageEngine &engine_arg, TableShare &share_arg);
   virtual ~Cursor(void);
   virtual Cursor *clone(memory::Root *mem_root);
 
   /* ha_ methods: pubilc wrappers for private virtual API */
 
-  int ha_open(const identifier::Table &identifier, int mode, int test_if_locked);
-  int startIndexScan(uint32_t idx, bool sorted) __attribute__ ((warn_unused_result));
-  int endIndexScan();
-  int startTableScan(bool scan) __attribute__ ((warn_unused_result));
-  int endTableScan();
+  int ha_open(Table *table, const char *name, int mode, int test_if_locked);
+  int ha_index_init(uint32_t idx, bool sorted);
+  int ha_index_end();
+  int ha_rnd_init(bool scan);
+  int ha_rnd_end();
   int ha_reset();
 
   /* this is necessary in many places, e.g. in HANDLER command */
@@ -215,45 +260,50 @@ public:
   /**
     These functions represent the public interface to *users* of the
     Cursor class, hence they are *not* virtual. For the inheritance
-    interface, see the (private) functions doInsertRecord(), doUpdateRecord(),
-    and doDeleteRecord() below.
+    interface, see the (private) functions write_row(), update_row(),
+    and delete_row() below.
   */
   int ha_external_lock(Session *session, int lock_type);
-  int insertRecord(unsigned char * buf) __attribute__ ((warn_unused_result));
-  int updateRecord(const unsigned char * old_data, unsigned char * new_data) __attribute__ ((warn_unused_result));
-  int deleteRecord(const unsigned char * buf) __attribute__ ((warn_unused_result));
+  int ha_write_row(unsigned char * buf);
+  int ha_update_row(const unsigned char * old_data, unsigned char * new_data);
+  int ha_delete_row(const unsigned char * buf);
   void ha_release_auto_increment();
-  int ha_check(Session*);
+
+  /** to be actually called to get 'check()' functionality*/
+  int ha_check(Session *session, HA_CHECK_OPT *check_opt);
+
   void ha_start_bulk_insert(ha_rows rows);
   int ha_end_bulk_insert();
   int ha_delete_all_rows();
   int ha_reset_auto_increment(uint64_t value);
-  int ha_analyze(Session*);
+  int ha_analyze(Session* session, HA_CHECK_OPT* check_opt);
 
   int ha_disable_indexes(uint32_t mode);
   int ha_enable_indexes(uint32_t mode);
   int ha_discard_or_import_tablespace(bool discard);
-  void closeMarkForDelete();
+  void closeMarkForDelete(const char *name);
 
   void adjust_next_insert_id_after_explicit_value(uint64_t nr);
   int update_auto_increment();
+  virtual void change_table_ptr(Table *table_arg, TableShare *share);
 
   /* Estimates calculation */
   virtual double scan_time(void)
-  { return static_cast<double>(stats.data_file_length) / IO_SIZE + 2; }
+  { return uint64_t2double(stats.data_file_length) / IO_SIZE + 2; }
   virtual double read_time(uint32_t, uint32_t ranges, ha_rows rows)
-  { return ranges + rows; }
+  { return rows2double(ranges+rows); }
 
   virtual double index_only_read_time(uint32_t keynr, double records);
 
   virtual ha_rows multi_range_read_info_const(uint32_t keyno, RANGE_SEQ_IF *seq,
                                               void *seq_init_param,
                                               uint32_t n_ranges, uint32_t *bufsz,
-                                              uint32_t *flags, optimizer::CostVector *cost);
+                                              uint32_t *flags, COST_VECT *cost);
   virtual int multi_range_read_info(uint32_t keyno, uint32_t n_ranges, uint32_t keys,
-                                    uint32_t *bufsz, uint32_t *flags, optimizer::CostVector *cost);
+                                    uint32_t *bufsz, uint32_t *flags, COST_VECT *cost);
   virtual int multi_range_read_init(RANGE_SEQ_IF *seq, void *seq_init_param,
-                                    uint32_t n_ranges, uint32_t mode);
+                                    uint32_t n_ranges, uint32_t mode,
+                                    HANDLER_BUFFER *buf);
   virtual int multi_range_read_next(char **range_info);
 
 
@@ -288,6 +338,12 @@ public:
   virtual ha_rows estimate_rows_upper_bound()
   { return stats.records+EXTRA_RECORDS; }
 
+  /**
+    Get the row type from the storage engine.  If this method returns
+    ROW_TYPE_NOT_USED, the information in HA_CREATE_INFO should be used.
+  */
+  virtual enum row_type get_row_type() const { return ROW_TYPE_NOT_USED; }
+
   virtual const char *index_type(uint32_t)
   { assert(0); return "";}
 
@@ -301,11 +357,11 @@ public:
      row if available. If the key value is null, begin at the first key of the
      index.
   */
-  virtual int index_read_map(unsigned char * buf, const unsigned char *key,
+  virtual int index_read_map(unsigned char * buf, const unsigned char * key,
                              key_part_map keypart_map,
                              enum ha_rkey_function find_flag)
   {
-    uint32_t key_len= calculate_key_len(active_index, keypart_map);
+    uint32_t key_len= calculate_key_len(table, active_index, key, keypart_map);
     return  index_read(buf, key, key_len, find_flag);
   }
   /**
@@ -318,7 +374,7 @@ public:
                                  const unsigned char * key,
                                  key_part_map keypart_map,
                                  enum ha_rkey_function find_flag);
-  virtual int index_next(unsigned char *) __attribute__ ((warn_unused_result))
+  virtual int index_next(unsigned char *)
    { return  HA_ERR_WRONG_COMMAND; }
   virtual int index_prev(unsigned char *)
    { return  HA_ERR_WRONG_COMMAND; }
@@ -327,11 +383,6 @@ public:
   virtual int index_last(unsigned char *)
    { return  HA_ERR_WRONG_COMMAND; }
   virtual int index_next_same(unsigned char *, const unsigned char *, uint32_t);
-
-private:
-  uint32_t calculate_key_len(uint32_t key_position, key_part_map keypart_map_arg);
-public:
-
   /**
      @brief
      The following functions works like index_read, but it find the last
@@ -340,7 +391,7 @@ public:
   virtual int index_read_last_map(unsigned char * buf, const unsigned char * key,
                                   key_part_map keypart_map)
   {
-    uint32_t key_len= calculate_key_len(active_index, keypart_map);
+    uint32_t key_len= calculate_key_len(table, active_index, key, keypart_map);
     return index_read_last(buf, key, key_len);
   }
   virtual int read_range_first(const key_range *start_key,
@@ -348,9 +399,23 @@ public:
                                bool eq_range, bool sorted);
   virtual int read_range_next();
   int compare_key(key_range *range);
+  int compare_key2(key_range *range);
   virtual int rnd_next(unsigned char *)=0;
   virtual int rnd_pos(unsigned char *, unsigned char *)=0;
+  /**
+    One has to use this method when to find
+    random position by record as the plain
+    position() call doesn't work for some
+    handlers for random position.
+  */
+  virtual int rnd_pos_by_record(unsigned char *record);
   virtual int read_first_row(unsigned char *buf, uint32_t primary_key);
+  /**
+    The following function is only needed for tables that may be temporary
+    tables during joins.
+  */
+  virtual int restart_rnd_next(unsigned char *, unsigned char *)
+    { return HA_ERR_WRONG_COMMAND; }
   virtual int rnd_same(unsigned char *, uint32_t)
     { return HA_ERR_WRONG_COMMAND; }
   virtual ha_rows records_in_range(uint32_t, key_range *, key_range *)
@@ -388,8 +453,7 @@ public:
   virtual void get_auto_increment(uint64_t offset, uint64_t increment,
                                   uint64_t nb_desired_values,
                                   uint64_t *first_value,
-                                  uint64_t *nb_reserved_values)= 0;
-
+                                  uint64_t *nb_reserved_values);
   void set_next_insert_id(uint64_t id)
   {
     next_insert_id= id;
@@ -410,6 +474,8 @@ public:
       insert_id_for_cur_row;
   }
 
+  virtual void update_create_info(HA_CREATE_INFO *) {}
+  int check_old_types(void);
   /* end of the list of admin commands */
 
   virtual int indexes_are_disabled(void) {return 0;}
@@ -432,10 +498,20 @@ public:
    */
   virtual bool can_switch_engines(void) { return true; }
   /** used in REPLACE; is > 0 if table is referred by a FOREIGN KEY */
-  virtual int get_foreign_key_list(Session *, List<ForeignKeyInfo> *)
+  virtual int get_foreign_key_list(Session *, List<FOREIGN_KEY_INFO> *)
   { return 0; }
   virtual uint32_t referenced_by_foreign_key() { return 0;}
   virtual void free_foreign_key_create_info(char *) {}
+  /** The following can be called without an open Cursor */
+
+  virtual int add_index(Table *, KEY *, uint32_t)
+  { return (HA_ERR_WRONG_COMMAND); }
+  virtual int prepare_drop_index(Table *, uint32_t *, uint32_t)
+  { return (HA_ERR_WRONG_COMMAND); }
+  virtual int final_drop_index(Table *)
+  { return (HA_ERR_WRONG_COMMAND); }
+
+  virtual uint32_t checksum(void) const { return 0; }
 
   /**
     Is not invoked for non-transactional temporary tables.
@@ -444,7 +520,7 @@ public:
     or partitioned.
 
     @note that one can NOT rely on table->in_use in store_lock().  It may
-    refer to a different thread if called from abortLockForThread().
+    refer to a different thread if called from mysql_lock_abort_for_thread().
 
     @note If the table is MERGE, store_lock() can return less locks
     than lock_count() claimed. This can happen when the MERGE children
@@ -478,8 +554,9 @@ public:
 
 protected:
   /* Service methods for use by storage engines. */
-  void ha_statistic_increment(uint64_t system_status_var::*offset) const;
+  void ha_statistic_increment(ulong system_status_var::*offset) const;
   void **ha_data(Session *) const;
+  Session *ha_session(void) const;
 
 private:
   /* Private helpers */
@@ -491,31 +568,30 @@ private:
     the corresponding 'ha_*' method above.
   */
 
-  virtual int open(const char *, int , uint32_t ) { assert(0); return -1; }
-  virtual int doOpen(const identifier::Table &identifier, int mode, uint32_t test_if_locked);
-  virtual int doStartIndexScan(uint32_t idx, bool)
+  virtual int open(const char *name, int mode, uint32_t test_if_locked)=0;
+  virtual int index_init(uint32_t idx, bool)
   { active_index= idx; return 0; }
-  virtual int doEndIndexScan() { active_index= MAX_KEY; return 0; }
+  virtual int index_end() { active_index= MAX_KEY; return 0; }
   /**
-    doStartTableScan() can be called two times without doEndTableScan() in between
+    rnd_init() can be called two times without rnd_end() in between
     (it only makes sense if scan=1).
     then the second call should prepare for the new table scan (e.g
     if rnd_init allocates the cursor, second call should position it
     to the start of the table, no need to deallocate and allocate it again
   */
-  virtual int doStartTableScan(bool scan) __attribute__ ((warn_unused_result)) = 0;
-  virtual int doEndTableScan() { return 0; }
-  virtual int doInsertRecord(unsigned char *)
+  virtual int rnd_init(bool scan)= 0;
+  virtual int rnd_end() { return 0; }
+  virtual int write_row(unsigned char *)
   {
     return HA_ERR_WRONG_COMMAND;
   }
 
-  virtual int doUpdateRecord(const unsigned char *, unsigned char *)
+  virtual int update_row(const unsigned char *, unsigned char *)
   {
     return HA_ERR_WRONG_COMMAND;
   }
 
-  virtual int doDeleteRecord(const unsigned char *)
+  virtual int delete_row(const unsigned char *)
   {
     return HA_ERR_WRONG_COMMAND;
   }
@@ -552,7 +628,7 @@ private:
   {
     return 0;
   }
-  virtual void release_auto_increment(void) { return; }
+  virtual void release_auto_increment(void) { return; };
   /** admin commands - called from mysql_admin_table */
   virtual int check(Session *)
   { return HA_ADMIN_NOT_IMPLEMENTED; }
@@ -600,20 +676,21 @@ private:
     no new engine should ever use it). Right
     now HEAP does rely on it, so we cannot remove it.
   */
-  virtual void drop_table();
+  virtual void drop_table(const char *name);
 };
 
 extern const char *ha_row_type[];
 
 /* basic stuff */
-void ha_init_errors(void);
+int ha_init_errors(void);
+int ha_end(void);
 
-SortField *make_unireg_sortorder(Order *order, uint32_t *length,
-                                 SortField *sortorder);
+SORT_FIELD * make_unireg_sortorder(order_st *order, uint32_t *length,
+                                   SORT_FIELD *sortorder);
 int setup_order(Session *session, Item **ref_pointer_array, TableList *tables,
-                List<Item> &fields, List <Item> &all_fields, Order *order);
+                List<Item> &fields, List <Item> &all_fields, order_st *order);
 int setup_group(Session *session, Item **ref_pointer_array, TableList *tables,
-                List<Item> &fields, List<Item> &all_fields, Order *order,
+                List<Item> &fields, List<Item> &all_fields, order_st *order,
                 bool *hidden_group_fields);
 bool fix_inner_refs(Session *session, List<Item> &all_fields, Select_Lex *select,
                     Item **ref_pointer_array);
@@ -622,76 +699,83 @@ bool handle_select(Session *session, LEX *lex, select_result *result,
                    uint64_t setup_tables_done_option);
 void free_underlaid_joins(Session *session, Select_Lex *select);
 
-bool handle_derived(LEX *lex, bool (*processor)(Session *session,
+bool mysql_handle_derived(LEX *lex, bool (*processor)(Session *session,
                                                       LEX *lex,
                                                       TableList *table));
-bool derived_prepare(Session *session, LEX *lex, TableList *t);
-bool derived_filling(Session *session, LEX *lex, TableList *t);
+bool mysql_derived_prepare(Session *session, LEX *lex, TableList *t);
+bool mysql_derived_filling(Session *session, LEX *lex, TableList *t);
 int prepare_create_field(CreateField *sql_field,
                          uint32_t *blob_columns,
                          int *timestamps, int *timestamps_with_niladic);
 
-bool create_table(Session *session,
-                  const identifier::Table &identifier,
-                  HA_CREATE_INFO *create_info,
-                  message::Table &table_proto,
-                  AlterInfo *alter_info,
-                  bool tmp_table, uint32_t select_field_count,
-                  bool is_if_not_exists);
+bool mysql_create_table(Session *session,
+                        TableIdentifier &identifier,
+                        HA_CREATE_INFO *create_info,
+                        message::Table &table_proto,
+                        AlterInfo *alter_info,
+                        bool tmp_table, uint32_t select_field_count,
+                        bool is_if_not_exists);
 
-bool create_table_no_lock(Session *session,
-                          const identifier::Table &identifier,
-                          HA_CREATE_INFO *create_info,
-                          message::Table &table_proto,
-                          AlterInfo *alter_info,
-                          bool tmp_table, uint32_t select_field_count,
-                          bool is_if_not_exists);
+bool mysql_create_table_no_lock(Session *session,
+                                TableIdentifier &identifier,
+                                HA_CREATE_INFO *create_info,
+                                message::Table &table_proto,
+                                AlterInfo *alter_info,
+                                bool tmp_table,
+                                uint32_t select_field_count,
+                                bool is_if_not_exists);
 
-bool create_like_table(Session* session,
-                       const identifier::Table& destination_identifier,
-                       const identifier::Table& source_identifier,
-                       message::Table &create_table_proto,
-                       bool is_if_not_exists,
-                       bool is_engine_set);
+bool mysql_create_like_table(Session* session,
+                             TableIdentifier &destination_identifier,
+                             TableList* table, TableList* src_table,
+                             message::Table &create_table_proto,
+                             bool is_if_not_exists,
+                             bool is_engine_set);
 
-bool rename_table(Session &session,
-                        plugin::StorageEngine *base,
-                        const identifier::Table &old_identifier,
-                        const identifier::Table &new_identifier);
+bool mysql_rename_table(plugin::StorageEngine *base, const char *old_db,
+                        const char * old_name, const char *new_db,
+                        const char * new_name, uint32_t flags);
 
-bool prepare_update(Session *session, TableList *table_list,
-                          Item **conds, uint32_t order_num, Order *order);
-int update_query(Session *session,TableList *tables,List<Item> &fields,
+bool mysql_prepare_update(Session *session, TableList *table_list,
+                          Item **conds, uint32_t order_num, order_st *order);
+int mysql_update(Session *session,TableList *tables,List<Item> &fields,
                  List<Item> &values,COND *conds,
-                 uint32_t order_num, Order *order, ha_rows limit,
+                 uint32_t order_num, order_st *order, ha_rows limit,
                  enum enum_duplicates handle_duplicates, bool ignore);
-bool prepare_insert(Session *session, TableList *table_list, Table *table,
+bool mysql_prepare_insert(Session *session, TableList *table_list, Table *table,
                           List<Item> &fields, List_item *values,
                           List<Item> &update_fields,
                           List<Item> &update_values, enum_duplicates duplic,
                           COND **where, bool select_insert,
                           bool check_fields, bool abort_on_warning);
-bool insert_query(Session *session,TableList *table,List<Item> &fields,
+bool mysql_insert(Session *session,TableList *table,List<Item> &fields,
                   List<List_item> &values, List<Item> &update_fields,
                   List<Item> &update_values, enum_duplicates flag,
                   bool ignore);
 int check_that_all_fields_are_given_values(Session *session, Table *entry,
                                            TableList *table_list);
-int prepare_delete(Session *session, TableList *table_list, Item **conds);
-bool delete_query(Session *session, TableList *table_list, COND *conds,
+int mysql_prepare_delete(Session *session, TableList *table_list, Item **conds);
+bool mysql_delete(Session *session, TableList *table_list, COND *conds,
                   SQL_LIST *order, ha_rows rows, uint64_t options,
                   bool reset_auto_increment);
-bool truncate(Session& session, TableList *table_list);
+bool mysql_truncate(Session& session, TableList *table_list);
 TableShare *get_table_share(Session *session, TableList *table_list, char *key,
                              uint32_t key_length, uint32_t db_flags, int *error);
 TableShare *get_cached_table_share(const char *db, const char *table_name);
 bool reopen_name_locked_table(Session* session, TableList* table_list, bool link_in);
+Table *table_cache_insert_placeholder(Session *session, const char *key,
+                                      uint32_t key_length);
+bool lock_table_name_if_not_cached(Session *session, const char *db,
+                                   const char *table_name, Table **table);
+bool reopen_table(Table *table);
 bool reopen_tables(Session *session,bool get_locks,bool in_refresh);
+void close_data_files_and_morph_locks(Session *session, const char *db,
+                                      const char *table_name);
 void close_handle_and_leave_table_as_lock(Table *table);
 bool wait_for_tables(Session *session);
 bool table_is_used(Table *table, bool wait_for_name_lock);
-Table *drop_locked_tables(Session *session, const drizzled::identifier::Table &identifier);
-void abort_locked_tables(Session *session, const drizzled::identifier::Table &identifier);
+Table *drop_locked_tables(Session *session,const char *db, const char *table_name);
+void abort_locked_tables(Session *session,const char *db, const char *table_name);
 extern Field *not_found_field;
 extern Field *view_ref_found;
 
@@ -714,3 +798,4 @@ find_field_in_table(Session *session, Table *table, const char *name, uint32_t l
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_CURSOR_H */
