@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,17 +17,16 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_DATA_HOME_H
+#define DRIZZLED_DATA_HOME_H
 
-#include <boost/filesystem.hpp>
-#include <drizzled/visibility.h>
+namespace drizzled
+{
 
-namespace drizzled {
-
-DRIZZLED_API const boost::filesystem::path& getFullDataHome();
-DRIZZLED_API const boost::filesystem::path& getDataHome();
-DRIZZLED_API boost::filesystem::path& getMutableDataHome();
-DRIZZLED_API void setFullDataHome(const boost::filesystem::path&);
+extern uint32_t drizzle_data_home_len;
+extern char *drizzle_data_home,
+             drizzle_real_data_home[], drizzle_unpacked_real_data_home[];
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_DATA_HOME_H */
