@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008-2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2008-2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,11 +17,11 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ENUM_NESTED_LOOP_STATE_H
+#define DRIZZLED_ENUM_NESTED_LOOP_STATE_H
 
-#include <drizzled/common_fwd.h>
-
-namespace drizzled {
+namespace drizzled
+{
 
 /** The states in which a nested loop join can be in */
 enum enum_nested_loop_state
@@ -34,9 +34,13 @@ enum enum_nested_loop_state
   NESTED_LOOP_CURSOR_LIMIT= 4
 };
 
-typedef enum_nested_loop_state (*Next_select_func)(Join *, JoinTable *, bool);
+class JOIN;
+class JoinTable;
+
+typedef enum_nested_loop_state (*Next_select_func)(JOIN *, JoinTable *, bool);
 typedef int (*Read_record_func)(JoinTable *tab);
-Next_select_func setup_end_select_func(Join *join);
+Next_select_func setup_end_select_func(JOIN *join);
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ENUM_NESTED_LOOP_STATE_H */
