@@ -1,7 +1,6 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2010 Brian Aker
  *  Copyright (C) 2008 MySQL
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,50 +22,38 @@
  * @file This file implements the Field class and API
  */
 
-#include <config.h>
-#include <cstdio>
+#include "config.h"
 #include <errno.h>
 #include <float.h>
-#include <drizzled/sql_select.h>
-#include <drizzled/error.h>
-#include <drizzled/field/str.h>
-#include <drizzled/field/num.h>
-#include <drizzled/field/blob.h>
-#include <drizzled/field/boolean.h>
-#include <drizzled/field/enum.h>
-#include <drizzled/field/null.h>
-#include <drizzled/field/date.h>
-#include <drizzled/field/decimal.h>
-#include <drizzled/field/real.h>
-#include <drizzled/field/double.h>
-#include <drizzled/field/int32.h>
-#include <drizzled/field/int64.h>
-#include <drizzled/field/num.h>
-#include <drizzled/field/time.h>
-#include <drizzled/field/epoch.h>
-#include <drizzled/field/datetime.h>
-#include <drizzled/field/microtime.h>
-#include <drizzled/field/varstring.h>
-#include <drizzled/field/uuid.h>
-#include <drizzled/field/ipv6.h>
-#include <drizzled/time_functions.h>
-#include <drizzled/internal/m_string.h>
-#include <drizzled/table.h>
-#include <drizzled/util/test.h>
-#include <drizzled/session.h>
-#include <drizzled/current_session.h>
-#include <drizzled/display.h>
-#include <drizzled/typelib.h>
-#include <drizzled/create_field.h>
+#include "drizzled/sql_select.h"
+#include "drizzled/error.h"
+#include "drizzled/field/str.h"
+#include "drizzled/field/num.h"
+#include "drizzled/field/blob.h"
+#include "drizzled/field/enum.h"
+#include "drizzled/field/null.h"
+#include "drizzled/field/date.h"
+#include "drizzled/field/decimal.h"
+#include "drizzled/field/real.h"
+#include "drizzled/field/double.h"
+#include "drizzled/field/long.h"
+#include "drizzled/field/int64_t.h"
+#include "drizzled/field/num.h"
+#include "drizzled/field/timestamp.h"
+#include "drizzled/field/datetime.h"
+#include "drizzled/field/varstring.h"
+#include "drizzled/time_functions.h"
+#include "drizzled/internal/m_string.h"
 
-namespace drizzled {
+namespace drizzled
+{
 
 /*****************************************************************************
   Instansiate templates and static variables
 *****************************************************************************/
 
 static enum_field_types
-field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
+field_types_merge_rules [DRIZZLE_TYPE_MAX+1][DRIZZLE_TYPE_MAX+1]=
 {
   /* DRIZZLE_TYPE_LONG -> */
   {
@@ -92,16 +79,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_DOUBLE -> */
   {
@@ -127,16 +104,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_NULL -> */
   {
@@ -162,16 +129,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_ENUM,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_BOOLEAN,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_UUID,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_MICROTIME,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_TIMESTAMP -> */
   {
@@ -197,16 +154,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_LONGLONG -> */
   {
@@ -231,16 +178,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_DATETIME -> */
   {
@@ -266,16 +203,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_DATE -> */
   {
@@ -301,16 +228,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_VARCHAR -> */
   {
@@ -336,16 +253,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_DECIMAL -> */
   {
@@ -371,16 +278,6 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
   /* DRIZZLE_TYPE_ENUM -> */
   {
@@ -406,17 +303,7 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_VARCHAR,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
-   },
+  },
   /* DRIZZLE_TYPE_BLOB -> */
   {
     //DRIZZLE_TYPE_LONG
@@ -441,195 +328,10 @@ field_types_merge_rules [enum_field_types_size][enum_field_types_size]=
     DRIZZLE_TYPE_BLOB,
     //DRIZZLE_TYPE_BLOB
     DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
-  },
-  /* DRIZZLE_TYPE_TIME -> */
-  {
-    //DRIZZLE_TYPE_LONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DOUBLE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_NULL
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_TIMESTAMP
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_LONGLONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATETIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DECIMAL
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR,
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_BLOB
-    DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_UUID,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
-  },
-  /* DRIZZLE_TYPE_BOOLEAN -> */
-  {
-    //DRIZZLE_TYPE_LONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DOUBLE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_NULL
-    DRIZZLE_TYPE_BOOLEAN,
-    //DRIZZLE_TYPE_TIMESTAMP
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_LONGLONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATETIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DECIMAL
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR,
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_BLOB
-    DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_BOOLEAN,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
-  },
-  /* DRIZZLE_TYPE_UUID -> */
-  {
-    //DRIZZLE_TYPE_LONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DOUBLE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_NULL
-    DRIZZLE_TYPE_UUID,
-    //DRIZZLE_TYPE_TIMESTAMP
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_LONGLONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATETIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DECIMAL
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR,
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_BLOB
-    DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_UUID,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
-  },
-  /* DRIZZLE_TYPE_MICROTIME -> */
-  {
-    //DRIZZLE_TYPE_LONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DOUBLE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_NULL
-    DRIZZLE_TYPE_MICROTIME,
-    //DRIZZLE_TYPE_TIMESTAMP
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_LONGLONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATETIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DECIMAL
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR,
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_BLOB
-    DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_UUID,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_MICROTIME,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
-  },
-  /* DRIZZLE_TYPE_IPV6 -> */
-  {
-    //DRIZZLE_TYPE_LONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DOUBLE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_NULL
-    DRIZZLE_TYPE_IPV6,
-    //DRIZZLE_TYPE_TIMESTAMP
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_LONGLONG
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATETIME
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DATE
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_DECIMAL
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_VARCHAR,
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_BLOB
-    DRIZZLE_TYPE_BLOB,
-    //DRIZZLE_TYPE_TIME
-    DRIZZLE_TYPE_TIME,
-    //DRIZZLE_TYPE_BOOLEAN
-    DRIZZLE_TYPE_VARCHAR,
-    //DRIZZLE_TYPE_UUID
-    DRIZZLE_TYPE_UUID,
-    //DRIZZLE_TYPE_MICROTIME
-    DRIZZLE_TYPE_MICROTIME,
-    //DRIZZLE_TYPE_IPV6
-    DRIZZLE_TYPE_VARCHAR,
   },
 };
 
-static Item_result field_types_result_type [enum_field_types_size]=
+static Item_result field_types_result_type [DRIZZLE_TYPE_MAX+1]=
 {
   //DRIZZLE_TYPE_LONG
   INT_RESULT,
@@ -653,19 +355,9 @@ static Item_result field_types_result_type [enum_field_types_size]=
   STRING_RESULT,
   //DRIZZLE_TYPE_BLOB
   STRING_RESULT,
-  //DRIZZLE_TYPE_TIME
-  STRING_RESULT,
-  //DRIZZLE_TYPE_BOOLEAN
-  STRING_RESULT,
-  //DRIZZLE_TYPE_UUID
-  STRING_RESULT,
-  //DRIZZLE_TYPE_MICROTIME
-  STRING_RESULT,
-  //DRIZZLE_TYPE_IPV6
-  STRING_RESULT,
 };
 
-bool test_if_important_data(const charset_info_st * const cs, 
+bool test_if_important_data(const CHARSET_INFO * const cs, 
                             const char *str,
                             const char *strend)
 {
@@ -681,20 +373,20 @@ void *Field::operator new(size_t size)
 
 void *Field::operator new(size_t size, memory::Root *mem_root)
 {
-  return mem_root->alloc(size);
+  return alloc_root(mem_root, static_cast<uint32_t>(size));
 }
 
 enum_field_types Field::field_type_merge(enum_field_types a,
                                          enum_field_types b)
 {
-  assert(a < enum_field_types_size);
-  assert(b < enum_field_types_size);
+  assert(a <= DRIZZLE_TYPE_MAX);
+  assert(b <= DRIZZLE_TYPE_MAX);
   return field_types_merge_rules[a][b];
 }
 
 Item_result Field::result_merge_type(enum_field_types field_type)
 {
-  assert(field_type < enum_field_types_size);
+  assert(field_type <= DRIZZLE_TYPE_MAX);
   return field_types_result_type[field_type];
 }
 
@@ -745,7 +437,7 @@ void Field::reset_fields()
 
 void Field::set_default()
 {
-  ptrdiff_t l_offset= (ptrdiff_t) (table->getDefaultValues() - table->getInsertRecord());
+  ptrdiff_t l_offset= (ptrdiff_t) (table->getDefaultValues() - table->record[0]);
   memcpy(ptr, ptr + l_offset, pack_length());
   if (null_ptr)
     *null_ptr= ((*null_ptr & (unsigned char) ~null_bit) | (null_ptr[l_offset] & null_bit));
@@ -814,24 +506,30 @@ uint32_t Field::decimals() const
   return 0;
 }
 
-bool Field::is_null(ptrdiff_t row_offset) const
+bool Field::is_null(ptrdiff_t row_offset)
 {
-  return null_ptr ? (null_ptr[row_offset] & null_bit ? true : false) : table->null_row;
+  return null_ptr ?
+    (null_ptr[row_offset] & null_bit ? true : false) :
+    table->null_row;
 }
 
-bool Field::is_real_null(ptrdiff_t row_offset) const
+bool Field::is_real_null(ptrdiff_t row_offset)
 {
-  return null_ptr && (null_ptr[row_offset] & null_bit);
+  return null_ptr ? (null_ptr[row_offset] & null_bit ? true : false) : false;
 }
 
-bool Field::is_null_in_record(const unsigned char *record) const
+bool Field::is_null_in_record(const unsigned char *record)
 {
-  return null_ptr && test(record[(uint32_t) (null_ptr -table->getInsertRecord())] & null_bit);
+  if (! null_ptr)
+    return false;
+  return test(record[(uint32_t) (null_ptr -table->record[0])] & null_bit);
 }
 
-bool Field::is_null_in_record_with_offset(ptrdiff_t with_offset) const
+bool Field::is_null_in_record_with_offset(ptrdiff_t with_offset)
 {
-  return null_ptr && test(null_ptr[with_offset] & null_bit);
+  if (! null_ptr)
+    return false;
+  return test(null_ptr[with_offset] & null_bit);
 }
 
 void Field::set_null(ptrdiff_t row_offset)
@@ -846,20 +544,19 @@ void Field::set_notnull(ptrdiff_t row_offset)
     null_ptr[row_offset]&= (unsigned char) ~null_bit;
 }
 
-bool Field::maybe_null(void) const
+bool Field::maybe_null(void)
 {
   return null_ptr != 0 || table->maybe_null;
 }
 
-bool Field::real_maybe_null(void) const
+bool Field::real_maybe_null(void)
 {
   return null_ptr != 0;
 }
 
 bool Field::type_can_have_key_part(enum enum_field_types type)
 {
-  switch (type) 
-  {
+  switch (type) {
   case DRIZZLE_TYPE_VARCHAR:
   case DRIZZLE_TYPE_BLOB:
     return true;
@@ -886,6 +583,7 @@ int Field::warn_if_overflow(int op_result)
 void Field::init(Table *table_arg)
 {
   orig_table= table= table_arg;
+  table_name= &table_arg->alias;
 }
 
 /// This is used as a table name when the table structure is not set up
@@ -894,11 +592,13 @@ Field::Field(unsigned char *ptr_arg,
              unsigned char *null_ptr_arg,
              unsigned char null_bit_arg,
              utype unireg_check_arg, 
-             const char *field_name_arg) :
+             const char *field_name_arg)
+  :
     ptr(ptr_arg),
     null_ptr(null_ptr_arg),
     table(NULL),
     orig_table(NULL),
+    table_name(NULL),
     field_name(field_name_arg),
     key_start(0),
     part_of_key(0),
@@ -906,14 +606,16 @@ Field::Field(unsigned char *ptr_arg,
     part_of_sortkey(0),
     unireg_check(unireg_check_arg),
     field_length(length_arg),
-    flags(null_ptr ? 0: NOT_NULL_FLAG),
-    field_index(0),
     null_bit(null_bit_arg),
     is_created_from_null_item(false)
 {
+  flags= null_ptr ? 0: NOT_NULL_FLAG;
+  comment.str= (char*) "";
+  comment.length= 0;
+  field_index= 0;
 }
 
-void Field::hash(uint32_t *nr, uint32_t *nr2) const
+void Field::hash(uint32_t *nr, uint32_t *nr2)
 {
   if (is_null())
   {
@@ -922,7 +624,7 @@ void Field::hash(uint32_t *nr, uint32_t *nr2) const
   else
   {
     uint32_t len= pack_length();
-    const charset_info_st * const cs= charset();
+    const CHARSET_INFO * const cs= charset();
     cs->coll->hash_sort(cs, ptr, len, nr, nr2);
   }
 }
@@ -939,15 +641,15 @@ void Field::copy_from_tmp(int row_offset)
   }
 }
 
-int Field::store_and_check(enum_check_fields check_level,
-                           const char *to, 
-                           uint32_t length,
-                           const charset_info_st * const cs)
-
+int Field::store(const char *to, 
+                 uint32_t length,
+                 const CHARSET_INFO * const cs,
+                 enum_check_fields check_level)
 {
+  int res;
   enum_check_fields old_check_level= table->in_use->count_cuted_fields;
   table->in_use->count_cuted_fields= check_level;
-  int res= store(to, length, cs);
+  res= store(to, length, cs);
   table->in_use->count_cuted_fields= old_check_level;
   return res;
 }
@@ -962,7 +664,8 @@ unsigned char *Field::pack(unsigned char *to, const unsigned char *from, uint32_
 
 unsigned char *Field::pack(unsigned char *to, const unsigned char *from)
 {
-  return this->pack(to, from, UINT32_MAX, table->getShare()->db_low_byte_first);
+  unsigned char *result= this->pack(to, from, UINT32_MAX, table->s->db_low_byte_first);
+  return(result);
 }
 
 const unsigned char *Field::unpack(unsigned char* to,
@@ -999,29 +702,30 @@ const unsigned char *Field::unpack(unsigned char* to,
 
 const unsigned char *Field::unpack(unsigned char* to, const unsigned char *from)
 {
-  return unpack(to, from, 0, table->getShare()->db_low_byte_first);
+  const unsigned char *result= unpack(to, from, 0U, table->s->db_low_byte_first);
+  return(result);
 }
 
-type::Decimal *Field::val_decimal(type::Decimal *) const
+my_decimal *Field::val_decimal(my_decimal *)
 {
   /* This never have to be called */
-  assert(false);
+  assert(0);
   return 0;
 }
 
 
 void Field::make_field(SendField *field)
 {
-  if (orig_table && orig_table->getShare()->getSchemaName() && *orig_table->getShare()->getSchemaName())
+  if (orig_table && orig_table->s->getSchemaName() && *orig_table->s->getSchemaName())
   {
-    field->db_name= orig_table->getShare()->getSchemaName();
-    field->org_table_name= orig_table->getShare()->getTableName();
+    field->db_name= orig_table->s->getSchemaName();
+    field->org_table_name= orig_table->s->table_name.str;
   }
   else
     field->org_table_name= field->db_name= "";
   if (orig_table)
   {
-    field->table_name= orig_table->getAlias();
+    field->table_name= orig_table->alias;
     field->org_col_name= field_name;
   }
   else
@@ -1037,12 +741,12 @@ void Field::make_field(SendField *field)
   field->decimals= 0;
 }
 
-int64_t Field::convert_decimal2int64_t(const type::Decimal *val, bool, int *err)
+int64_t Field::convert_decimal2int64_t(const my_decimal *val, bool, int *err)
 {
   int64_t i;
-  if (warn_if_overflow(val->val_int32(E_DEC_ERROR &
+  if (warn_if_overflow(my_decimal2int(E_DEC_ERROR &
                                       ~E_DEC_OVERFLOW & ~E_DEC_TRUNCATED,
-                                      false, &i)))
+                                      val, false, &i)))
   {
     i= (val->sign() ? INT64_MIN : INT64_MAX);
     *err= 1;
@@ -1050,7 +754,7 @@ int64_t Field::convert_decimal2int64_t(const type::Decimal *val, bool, int *err)
   return i;
 }
 
-uint32_t Field::fill_cache_field(CacheField *copy)
+uint32_t Field::fill_cache_field(CACHE_FIELD *copy)
 {
   uint32_t store_length;
   copy->str=ptr;
@@ -1060,7 +764,7 @@ uint32_t Field::fill_cache_field(CacheField *copy)
   {
     copy->blob_field=(Field_blob*) this;
     copy->strip=0;
-    copy->length-= table->getShare()->sizeBlobPtr();
+    copy->length-= table->s->blob_ptr_size;
     return copy->length;
   }
   else
@@ -1071,31 +775,32 @@ uint32_t Field::fill_cache_field(CacheField *copy)
   return copy->length+ store_length;
 }
 
-bool Field::get_date(type::Time &ltime, uint32_t fuzzydate) const
+bool Field::get_date(DRIZZLE_TIME *ltime,uint32_t fuzzydate)
 {
-  char buff[type::Time::MAX_STRING_LENGTH];
-  String tmp(buff,sizeof(buff),&my_charset_bin);
-
-  assert(getTable() and getTable()->getSession());
-
-  String* res= val_str_internal(&tmp);
-  return not res or str_to_datetime_with_warn(*getTable()->getSession(), *res, ltime, fuzzydate) <= type::DRIZZLE_TIMESTAMP_ERROR;
+  char buff[40];
+  String tmp(buff,sizeof(buff),&my_charset_bin),*res;
+  if (!(res=val_str(&tmp)) ||
+      str_to_datetime_with_warn(res->ptr(), res->length(),
+                                ltime, fuzzydate) <= DRIZZLE_TIMESTAMP_ERROR)
+    return 1;
+  return 0;
 }
 
-bool Field::get_time(type::Time &ltime) const
+bool Field::get_time(DRIZZLE_TIME *ltime)
 {
-  char buff[type::Time::MAX_STRING_LENGTH];
-  String tmp(buff,sizeof(buff),&my_charset_bin);
-
-  String* res= val_str_internal(&tmp);
-  return not res or str_to_time_with_warn(*getTable()->getSession(), *res, ltime);
+  char buff[40];
+  String tmp(buff,sizeof(buff),&my_charset_bin),*res;
+  if (!(res=val_str(&tmp)) ||
+      str_to_time_with_warn(res->ptr(), res->length(), ltime))
+    return 1;
+  return 0;
 }
 
-int Field::store_time(type::Time &ltime, type::timestamp_t)
+int Field::store_time(DRIZZLE_TIME *ltime, enum enum_drizzle_timestamp_type)
 {
-  String tmp;
-  ltime.convert(tmp);
-  return store(tmp.ptr(), tmp.length(), &my_charset_bin);
+  char buff[MAX_DATE_STRING_REP_LENGTH];
+  uint32_t length= (uint32_t) my_TIME_to_str(ltime, buff);
+  return store(buff, length, &my_charset_bin);
 }
 
 bool Field::optimize_range(uint32_t idx, uint32_t)
@@ -1105,7 +810,10 @@ bool Field::optimize_range(uint32_t idx, uint32_t)
 
 Field *Field::new_field(memory::Root *root, Table *new_table, bool)
 {
-  Field* tmp= (Field*) root->memdup(this,size_of());
+  Field *tmp;
+  if (!(tmp= (Field*) memdup_root(root,(char*) this,size_of())))
+    return 0;
+
   if (tmp->table->maybe_null)
     tmp->flags&= ~NOT_NULL_FLAG;
   tmp->table= new_table;
@@ -1123,99 +831,242 @@ Field *Field::new_key_field(memory::Root *root, Table *new_table,
                             unsigned char *new_null_ptr,
                             uint32_t new_null_bit)
 {
-  Field *tmp= new_field(root, new_table, table == new_table);
-  tmp->ptr= new_ptr;
-  tmp->null_ptr= new_null_ptr;
-  tmp->null_bit= new_null_bit;
+  Field *tmp;
+  if ((tmp= new_field(root, new_table, table == new_table)))
+  {
+    tmp->ptr= new_ptr;
+    tmp->null_ptr= new_null_ptr;
+    tmp->null_bit= new_null_bit;
+  }
   return tmp;
 }
 
 Field *Field::clone(memory::Root *root, Table *new_table)
 {
-  Field *tmp= (Field*) root->memdup(this,size_of());
-  tmp->init(new_table);
-  tmp->move_field_offset((ptrdiff_t) (new_table->getInsertRecord() - new_table->getDefaultValues()));
+  Field *tmp;
+  if ((tmp= (Field*) memdup_root(root,(char*) this,size_of())))
+  {
+    tmp->init(new_table);
+    tmp->move_field_offset((ptrdiff_t) (new_table->record[0] -
+                                           new_table->s->default_values));
+  }
   return tmp;
 }
 
+
 uint32_t Field::is_equal(CreateField *new_field_ptr)
 {
-  return new_field_ptr->sql_type == real_type();
+  return (new_field_ptr->sql_type == real_type());
 }
 
 bool Field::eq_def(Field *field)
 {
-  return real_type() == field->real_type() && charset() == field->charset() && pack_length() == field->pack_length();
+  if (real_type() != field->real_type() || charset() != field->charset() ||
+      pack_length() != field->pack_length())
+    return 0;
+  return 1;
 }
 
 bool Field_enum::eq_def(Field *field)
 {
   if (!Field::eq_def(field))
     return 0;
-
   TYPELIB *from_lib=((Field_enum*) field)->typelib;
 
   if (typelib->count < from_lib->count)
     return 0;
-
   for (uint32_t i=0 ; i < from_lib->count ; i++)
-  {
     if (my_strnncoll(field_charset,
-                     (const unsigned char*)typelib->type_names[i], strlen(typelib->type_names[i]),
-                     (const unsigned char*)from_lib->type_names[i], strlen(from_lib->type_names[i])))
+                     (const unsigned char*)typelib->type_names[i],
+                     strlen(typelib->type_names[i]),
+                     (const unsigned char*)from_lib->type_names[i],
+                     strlen(from_lib->type_names[i])))
       return 0;
-  }
-
   return 1;
 }
 
 uint32_t calc_pack_length(enum_field_types type,uint32_t length)
 {
-  switch (type) 
-  {
+  switch (type) {
   case DRIZZLE_TYPE_VARCHAR: return (length + (length < 256 ? 1: 2));
-  case DRIZZLE_TYPE_UUID: return field::Uuid::max_string_length();
-  case DRIZZLE_TYPE_MICROTIME: return field::Microtime::max_string_length();
-  case DRIZZLE_TYPE_TIMESTAMP: return field::Epoch::max_string_length();
-  case DRIZZLE_TYPE_BOOLEAN: return field::Boolean::max_string_length();
-  case DRIZZLE_TYPE_IPV6: return field::IPv6::max_string_length();
-  case DRIZZLE_TYPE_DATE:
-  case DRIZZLE_TYPE_ENUM:
+  case DRIZZLE_TYPE_DATE: return 3;
+  case DRIZZLE_TYPE_TIMESTAMP:
   case DRIZZLE_TYPE_LONG: return 4;
   case DRIZZLE_TYPE_DOUBLE: return sizeof(double);
-  case DRIZZLE_TYPE_TIME:
   case DRIZZLE_TYPE_DATETIME:
   case DRIZZLE_TYPE_LONGLONG: return 8;	/* Don't crash if no int64_t */
   case DRIZZLE_TYPE_NULL: return 0;
   case DRIZZLE_TYPE_BLOB: return 4 + portable_sizeof_char_ptr;
+  case DRIZZLE_TYPE_ENUM:
   case DRIZZLE_TYPE_DECIMAL:
-                          break;
+    abort();
+  default:
+    return 0;
   }
-
-  assert(0);
-  abort();
 }
 
 uint32_t pack_length_to_packflag(uint32_t type)
 {
-  switch (type) 
-  {
-  case 1: return 1 << FIELDFLAG_PACK_SHIFT;
-  case 2: assert(0);
-  case 3: assert(0);
-  case 4: return f_settype(DRIZZLE_TYPE_LONG);
-  case 8: return f_settype(DRIZZLE_TYPE_LONGLONG);
+  switch (type) {
+    case 1: return 1 << FIELDFLAG_PACK_SHIFT;
+    case 2: assert(1);
+    case 3: assert(1);
+    case 4: return f_settype((uint32_t) DRIZZLE_TYPE_LONG);
+    case 8: return f_settype((uint32_t) DRIZZLE_TYPE_LONGLONG);
   }
-  assert(false);
   return 0;					// This shouldn't happen
 }
 
+Field *make_field(TableShare *share,
+                  memory::Root *root,
+                  unsigned char *ptr,
+                  uint32_t field_length,
+                  bool is_nullable,
+                  unsigned char *null_pos,
+                  unsigned char null_bit,
+                  uint8_t decimals,
+                  enum_field_types field_type,
+                  const CHARSET_INFO * field_charset,
+                  Field::utype unireg_check,
+                  TYPELIB *interval,
+                  const char *field_name)
+{
+  if(! root)
+    root= current_mem_root();
+
+  if (! is_nullable)
+  {
+    null_pos=0;
+    null_bit=0;
+  }
+  else
+  {
+    null_bit= ((unsigned char) 1) << null_bit;
+  }
+
+  switch (field_type) 
+  {
+  case DRIZZLE_TYPE_DATE:
+  case DRIZZLE_TYPE_DATETIME:
+  case DRIZZLE_TYPE_TIMESTAMP:
+    field_charset= &my_charset_bin;
+  default: break;
+  }
+
+  if (field_type == DRIZZLE_TYPE_VARCHAR ||
+      field_type == DRIZZLE_TYPE_BLOB ||
+      field_type == DRIZZLE_TYPE_ENUM)
+  {
+    if (field_type == DRIZZLE_TYPE_VARCHAR)
+      return new (root) Field_varstring(ptr,field_length,
+                                  HA_VARCHAR_PACKLENGTH(field_length),
+                                  null_pos,null_bit,
+                                  field_name,
+                                  share,
+                                  field_charset);
+
+    if (field_type == DRIZZLE_TYPE_BLOB)
+    {
+      return new (root) Field_blob(ptr,
+                                   null_pos,
+                                   null_bit,
+                                   field_name,
+                                   share,
+                                   calc_pack_length(DRIZZLE_TYPE_LONG, 0),
+                                   field_charset);
+    }
+
+    if (interval)
+    {
+      return new (root) Field_enum(ptr,
+                                   field_length,
+                                   null_pos,
+                                   null_bit,
+                                   field_name,
+                                   get_enum_pack_length(interval->count),
+                                   interval,
+                                   field_charset);
+    }
+  }
+
+  switch (field_type)
+  {
+  case DRIZZLE_TYPE_DECIMAL:
+    return new (root) Field_decimal(ptr,
+                                    field_length,
+                                    null_pos,
+                                    null_bit,
+                                    unireg_check,
+                                    field_name,
+                                    decimals,
+                                    false,
+                                    false /* is_unsigned */);
+  case DRIZZLE_TYPE_DOUBLE:
+    return new (root) Field_double(ptr,
+                                   field_length,
+                                   null_pos,
+                                   null_bit,
+                                   unireg_check,
+                                   field_name,
+                                   decimals,
+                                   false,
+                                   false /* is_unsigned */);
+  case DRIZZLE_TYPE_LONG:
+    return new (root) Field_long(ptr,
+                                 field_length,
+                                 null_pos,
+                                 null_bit,
+                                 unireg_check,
+                                 field_name,
+                                 false,
+                                 false /* is_unsigned */);
+  case DRIZZLE_TYPE_LONGLONG:
+    return new (root) Field_int64_t(ptr,
+                                    field_length,
+                                    null_pos,
+                                    null_bit,
+                                    unireg_check,
+                                    field_name,
+                                    false,
+                                    false /* is_unsigned */);
+  case DRIZZLE_TYPE_TIMESTAMP:
+    return new (root) Field_timestamp(ptr,
+                                      field_length,
+                                      null_pos,
+                                      null_bit,
+                                      unireg_check,
+                                      field_name,
+                                      share,
+                                      field_charset);
+  case DRIZZLE_TYPE_DATE:
+    return new (root) Field_date(ptr,
+                                 null_pos,
+                                 null_bit,
+                                 field_name,
+                                 field_charset);
+  case DRIZZLE_TYPE_DATETIME:
+    return new (root) Field_datetime(ptr,
+                                     null_pos,
+                                     null_bit,
+                                     field_name,
+                                     field_charset);
+  case DRIZZLE_TYPE_NULL:
+    return new (root) Field_null(ptr,
+                                 field_length,
+                                 field_name,
+                                 field_charset);
+  default: // Impossible (Wrong version)
+    break;
+  }
+  return 0;
+}
+
 /*****************************************************************************
  Warning handling
 *****************************************************************************/
 
 bool Field::set_warning(DRIZZLE_ERROR::enum_warning_level level,
-                        drizzled::error_t code,
+                        uint32_t code,
                         int cuted_increment)
 {
   /*
@@ -1235,55 +1086,55 @@ bool Field::set_warning(DRIZZLE_ERROR::enum_warning_level level,
 
 
 void Field::set_datetime_warning(DRIZZLE_ERROR::enum_warning_level level,
-                                 drizzled::error_t code,
+                                 unsigned int code,
                                  const char *str, 
                                  uint32_t str_length,
-                                 type::timestamp_t ts_type, 
+                                 enum enum_drizzle_timestamp_type ts_type, 
                                  int cuted_increment)
 {
-  Session *session= (getTable() and getTable()->getSession()) ? getTable()->getSession() : current_session;
-
-  if ((session->abortOnWarning() and
+  Session *session= table ? table->in_use : current_session;
+  if ((session->really_abort_on_warning() &&
        level >= DRIZZLE_ERROR::WARN_LEVEL_WARN) ||
       set_warning(level, code, cuted_increment))
-    make_truncated_value_warning(*session, level, str_ref(str, str_length), ts_type, field_name);
+    make_truncated_value_warning(session, level, str, str_length, ts_type,
+                                 field_name);
 }
 
 void Field::set_datetime_warning(DRIZZLE_ERROR::enum_warning_level level, 
-                                 drizzled::error_t code,
+                                 uint32_t code,
                                  int64_t nr, 
-                                 type::timestamp_t ts_type,
+                                 enum enum_drizzle_timestamp_type ts_type,
                                  int cuted_increment)
 {
-  Session *session= (getTable() and getTable()->getSession()) ? getTable()->getSession() : current_session;
-
-  if (session->abortOnWarning() or
+  Session *session= table ? table->in_use : current_session;
+  if (session->really_abort_on_warning() ||
       set_warning(level, code, cuted_increment))
   {
-    char str_nr[DECIMAL_LONGLONG_DIGITS];
+    char str_nr[22];
     char *str_end= internal::int64_t10_to_str(nr, str_nr, -10);
-    make_truncated_value_warning(*session, level, str_ref(str_nr, (uint32_t) (str_end - str_nr)), ts_type, field_name);
+    make_truncated_value_warning(session, level, str_nr, (uint32_t) (str_end - str_nr),
+                                 ts_type, field_name);
   }
 }
 
 void Field::set_datetime_warning(DRIZZLE_ERROR::enum_warning_level level,
-                                 const drizzled::error_t code,
+                                 const uint32_t code,
                                  double nr, 
-                                 type::timestamp_t ts_type)
+                                 enum enum_drizzle_timestamp_type ts_type)
 {
-  Session *session= (getTable() and getTable()->getSession()) ? getTable()->getSession() : current_session;
-
-  if (session->abortOnWarning() or
+  Session *session= table ? table->in_use : current_session;
+  if (session->really_abort_on_warning() ||
       set_warning(level, code, 1))
   {
     /* DBL_DIG is enough to print '-[digits].E+###' */
     char str_nr[DBL_DIG + 8];
-    uint32_t str_len= snprintf(str_nr, sizeof(str_nr), "%g", nr);
-    make_truncated_value_warning(*session, level, str_ref(str_nr, str_len), ts_type, field_name);
+    uint32_t str_len= sprintf(str_nr, "%g", nr);
+    make_truncated_value_warning(session, level, str_nr, str_len, ts_type,
+                                 field_name);
   }
 }
 
-bool Field::isReadSet() const 
+bool Field::isReadSet() 
 { 
   return table->isReadSet(field_index); 
 }
@@ -1309,74 +1160,4 @@ void Field::setWriteSet(bool arg)
     table->clearWriteSet(field_index);
 }
 
-void Field::pack_num(uint64_t arg, unsigned char *destination)
-{
-  if (not destination)
-    destination= ptr;
-
-  int64_tstore(destination, arg);
-}
-
-void Field::pack_num(uint32_t arg, unsigned char *destination)
-{
-  if (not destination)
-    destination= ptr;
-
-  longstore(destination, arg);
-}
-
-uint64_t Field::unpack_num(uint64_t &destination, const unsigned char *arg) const
-{
-  if (not arg)
-    arg= ptr;
-
-  int64_tget(destination, arg);
-
-  return destination;
-}
-
-uint32_t Field::unpack_num(uint32_t &destination, const unsigned char *arg) const
-{
-  if (not arg)
-    arg= ptr;
-
-  longget(destination, arg);
-
-  return destination;
-}
-
-std::ostream& operator<<(std::ostream& output, const Field &field)
-{
-  output << "Field:(";
-  output <<  field.field_name;
-  output << ", ";
-  output << drizzled::display::type(field.real_type());
-  output << ", { ";
-
-  if (field.flags & NOT_NULL_FLAG)
-    output << " NOT_NULL";
-
-  if (field.flags & PRI_KEY_FLAG)
-    output << ", PRIMARY KEY";
-
-  if (field.flags & UNIQUE_KEY_FLAG)
-    output << ", UNIQUE KEY";
-
-  if (field.flags & MULTIPLE_KEY_FLAG)
-    output << ", MULTIPLE KEY";
-
-  if (field.flags & BLOB_FLAG)
-    output << ", BLOB";
-
-  if (field.flags & UNSIGNED_FLAG)
-    output << ", UNSIGNED";
-
-  if (field.flags & BINARY_FLAG)
-    output << ", BINARY";
-  output << "}, ";
-  output << ")";
-
-  return output;  // for multiple << operators.
-}
-
 } /* namespace drizzled */
