@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,36 +22,52 @@
   variables must declare the size_of() member function.
 */
 
+#ifndef DRIZZLED_FIELD_H
+#define DRIZZLED_FIELD_H
 
-
-#pragma once
-
-#include <drizzled/common_fwd.h>
-#include <drizzled/sql_error.h>
-#include <drizzled/type/decimal.h>
-#include <drizzled/key_map.h>
-#include <drizzled/sql_list.h>
-#include <drizzled/structs.h>
-#include <drizzled/charset.h>
-#include <drizzled/item_result.h>
+#include "drizzled/sql_error.h"
+#include "drizzled/my_decimal.h"
+#include "drizzled/key_map.h"
+#include "drizzled/sql_bitmap.h"
+#include "drizzled/sql_list.h"
+#include "drizzled/structs.h"
+#include "drizzled/charset_info.h"
+#include "drizzled/item_result.h"
 
 #include <string>
 #include <vector>
 
-#include <drizzled/visibility.h>
-
-namespace drizzled {
+namespace drizzled
+{
 
 #define DATETIME_DEC                     6
 #define DOUBLE_TO_STRING_CONVERSION_BUFFER_SIZE FLOATING_POINT_BUFFER
 
-#define ASSERT_COLUMN_MARKED_FOR_READ assert(getTable() && (not getTable()->read_set || isReadSet()))
-#define ASSERT_COLUMN_MARKED_FOR_WRITE assert(getTable() && (not getTable()->write_set || isWriteSet()))
+#ifdef DEBUG
+#define ASSERT_COLUMN_MARKED_FOR_READ assert(!table || (table->read_set == NULL || isReadSet()))
+#define ASSERT_COLUMN_MARKED_FOR_WRITE assert(!table || (table->write_set == NULL || isWriteSet()))
+#else
+#define ASSERT_COLUMN_MARKED_FOR_READ
+#define ASSERT_COLUMN_MARKED_FOR_WRITE
+#endif
+
+typedef struct st_typelib TYPELIB;
 
 const uint32_t max_field_size= (uint32_t) 4294967295U;
 
+class SendField;
+class CreateField;
+class TableShare;
+class Field;
+struct st_cache_field;
+
 int field_conv(Field *to,Field *from);
 
+inline uint32_t get_enum_pack_length(int elements)
+{
+  return elements < 256 ? 1 : 2;
+}
+
 /**
  * Class representing a Field in a Table
  *
@@ -65,8 +81,11 @@ int field_conv(Field *to,Field *from);
  * The store_xxx() methods take various input and convert
  * the input into the raw bytes stored in the ptr member variable.
  */
-class DRIZZLED_API Field : boost::noncopyable
+class Field
 {
+  /* Prevent use of these */
+  Field(const Field&);
+  void operator=(Field &);
 public:
   unsigned char *ptr; /**< Position to field in record. Stores raw field value */
   unsigned char *null_ptr; /**< Byte where null_bit is */
@@ -77,30 +96,11 @@ public:
    * @note You can use table->in_use as replacement for current_session member
    * only inside of val_*() and store() members (e.g. you can't use it in cons)
    */
-private:
   Table *table;
-
-public:
-  Table *getTable()
-  {
-    assert(table);
-    return table;
-  }
-
-  Table *getTable() const
-  {
-    assert(table);
-    return table;
-  }
-
-  void setTable(Table *table_arg)
-  {
-    table= table_arg;
-  }
-
   Table *orig_table; /**< Pointer to the original Table. @TODO What is "the original table"? */
+  const char **table_name; /**< Pointer to the name of the table. @TODO This is redundant with Table::table_name. */
   const char *field_name; /**< Name of the field */
-  str_ref comment; /**< A comment about the field */
+  LEX_STRING comment; /**< A comment about the field */
 
   /** The field is part of the following keys */
   key_map	key_start;
@@ -128,27 +128,7 @@ public:
   utype	unireg_check;
   uint32_t field_length; /**< Length of this field in bytes */
   uint32_t flags;
-
-  bool isUnsigned() const
-  {
-    return flags & UNSIGNED_FLAG;
-  }
-
-private:
   uint16_t field_index; /**< Index of this Field in Table::fields array */
-
-public:
-
-  uint16_t position() const
-  {
-    return field_index;
-  }
-
-  void setPosition(uint32_t arg)
-  {
-    field_index= arg;
-  }
-
   unsigned char null_bit; /**< Bit used to test null bit */
   /**
      If true, this field was created in create_tmp_field_from_item from a NULL
@@ -164,8 +144,6 @@ public:
   static void *operator new(size_t size, memory::Root *mem_root);
   static void operator delete(void *, size_t)
   { }
-  static void operator delete(void *, memory::Root *)
-  { }
 
   Field(unsigned char *ptr_arg,
         uint32_t length_arg,
@@ -174,38 +152,31 @@ public:
         utype unireg_check_arg,
         const char *field_name_arg);
   virtual ~Field() {}
-
-  bool hasDefault() const
-  {
-    return not (flags & NO_DEFAULT_VALUE_FLAG);
-  }
-
   /* Store functions returns 1 on overflow and -1 on fatal error */
   virtual int store(const char *to,
                     uint32_t length,
-                    const charset_info_st * const cs)=0;
+                    const CHARSET_INFO * const cs)=0;
   virtual int store(double nr)=0;
   virtual int store(int64_t nr, bool unsigned_val)=0;
-  virtual int store_decimal(const type::Decimal *d)=0;
-  int store_and_check(enum_check_fields check_level,
-                      const char *to,
-                      uint32_t length,
-                      const charset_info_st * const cs);
+  virtual int store_decimal(const my_decimal *d)=0;
+  int store(const char *to,
+            uint32_t length,
+            const CHARSET_INFO * const cs,
+            enum_check_fields check_level);
   /**
     This is called when storing a date in a string.
 
     @note
       Needs to be changed if/when we want to support different time formats.
   */
-  virtual int store_time(type::Time &ltime, type::timestamp_t t_type);
-  virtual double val_real() const=0;
-  virtual int64_t val_int() const =0;
-  virtual type::Decimal *val_decimal(type::Decimal *) const;
-  String *val_str_internal(String *str) const
+  virtual int store_time(DRIZZLE_TIME *ltime, enum enum_drizzle_timestamp_type t_type);
+  virtual double val_real(void)=0;
+  virtual int64_t val_int(void)=0;
+  virtual my_decimal *val_decimal(my_decimal *);
+  inline String *val_str(String *str)
   {
     return val_str(str, str);
   }
-
   /*
      val_str(buf1, buf2) gets two buffers and should use them as follows:
      if it needs a temp buffer to convert result to string - use buf1
@@ -218,8 +189,7 @@ public:
      an unnecessary free (and later, may be an alloc).
      This trickery is used to decrease a number of malloc calls.
   */
-  virtual String *val_str(String*, String *) const =0;
-
+  virtual String *val_str(String*, String *)=0;
   /*
    str_needs_quotes() returns true if the value returned by val_str() needs
    to be quoted when used in constructing an SQL query.
@@ -228,7 +198,6 @@ public:
   virtual Item_result result_type () const=0;
   virtual Item_result cmp_type () const { return result_type(); }
   virtual Item_result cast_to_int_type () const { return result_type(); }
-
   /**
      Check whether a field type can be partially indexed by a key.
 
@@ -275,11 +244,6 @@ public:
    */
   virtual bool eq_def(Field *field);
 
-  virtual bool is_timestamp() const
-  {
-    return false;
-  }
-
   /**
    * Returns size (in bytes) used to store field data in memory
    * (i.e. it returns the maximum size of the field in a row of the table,
@@ -323,9 +287,10 @@ public:
   virtual uint32_t key_length() const;
   virtual enum_field_types type() const =0;
   virtual enum_field_types real_type() const;
-  virtual int cmp_max(const unsigned char *a, const unsigned char *b, uint32_t max_len);
+  inline  int cmp(const unsigned char *str) { return cmp(ptr,str); }
+  virtual int cmp_max(const unsigned char *a, const unsigned char *b,
+                      uint32_t max_len);
   virtual int cmp(const unsigned char *,const unsigned char *)=0;
-  int cmp_internal(const unsigned char *str) { return cmp(ptr,str); }
   virtual int cmp_binary(const unsigned char *a,const unsigned char *b,
                          uint32_t max_length=UINT32_MAX);
   virtual int cmp_offset(uint32_t row_offset);
@@ -334,17 +299,24 @@ public:
   virtual int key_cmp(const unsigned char *str, uint32_t length);
   virtual uint32_t decimals() const;
 
+  /*
+    Caller beware: sql_type can change str.Ptr, so check
+    ptr() to see if it changed if you are using your own buffer
+    in str and restore it with set() if needed
+  */
+  virtual void sql_type(String &str) const =0;
+
   // For new field
   virtual uint32_t size_of() const =0;
 
-  bool is_null(ptrdiff_t row_offset= 0) const;
-  bool is_real_null(ptrdiff_t row_offset= 0) const;
-  bool is_null_in_record(const unsigned char *record) const;
-  bool is_null_in_record_with_offset(ptrdiff_t offset) const;
+  bool is_null(ptrdiff_t row_offset= 0);
+  bool is_real_null(ptrdiff_t row_offset= 0);
+  bool is_null_in_record(const unsigned char *record);
+  bool is_null_in_record_with_offset(ptrdiff_t offset);
   void set_null(ptrdiff_t row_offset= 0);
   void set_notnull(ptrdiff_t row_offset= 0);
-  bool maybe_null(void) const;
-  bool real_maybe_null(void) const;
+  bool maybe_null(void);
+  bool real_maybe_null(void);
 
   virtual void make_field(SendField *);
   virtual void sort_string(unsigned char *buff,uint32_t length)=0;
@@ -361,35 +333,37 @@ public:
     return false;
   }
   virtual void free() {}
-  virtual Field *new_field(memory::Root*, Table*, bool keep_type);
+  virtual Field *new_field(memory::Root *root,
+                           Table *new_table,
+                           bool keep_type);
   virtual Field *new_key_field(memory::Root *root, Table *new_table,
                                unsigned char *new_ptr,
                                unsigned char *new_null_ptr,
                                uint32_t new_null_bit);
   /** This is used to generate a field in Table from TableShare */
-  Field* clone(memory::Root*, Table*);
-  void move_field(unsigned char *ptr_arg,unsigned char *null_ptr_arg,unsigned char null_bit_arg)
+  Field *clone(memory::Root *mem_root, Table *new_table);
+  inline void move_field(unsigned char *ptr_arg,unsigned char *null_ptr_arg,unsigned char null_bit_arg)
   {
     ptr= ptr_arg;
     null_ptr= null_ptr_arg;
     null_bit= null_bit_arg;
   }
-  void move_field(unsigned char *ptr_arg) { ptr=ptr_arg; }
+  inline void move_field(unsigned char *ptr_arg) { ptr=ptr_arg; }
   virtual void move_field_offset(ptrdiff_t ptr_diff)
   {
     ptr= ADD_TO_PTR(ptr,ptr_diff, unsigned char*);
     if (null_ptr)
       null_ptr= ADD_TO_PTR(null_ptr,ptr_diff,unsigned char*);
   }
-  virtual void get_image(unsigned char *buff, uint32_t length, const charset_info_st * const)
+  virtual void get_image(unsigned char *buff, uint32_t length, const CHARSET_INFO * const)
   {
     memcpy(buff,ptr,length);
   }
-  virtual void get_image(std::basic_string<unsigned char> &buff, uint32_t length, const charset_info_st * const)
+  virtual void get_image(std::basic_string<unsigned char> &buff, uint32_t length, const CHARSET_INFO * const)
   {
     buff.append(ptr,length);
   }
-  virtual void set_image(const unsigned char *buff,uint32_t length, const charset_info_st * const)
+  virtual void set_image(const unsigned char *buff,uint32_t length, const CHARSET_INFO * const)
   {
     memcpy(ptr,buff,length);
   }
@@ -432,7 +406,7 @@ public:
   {
     set_image(buff,length, &my_charset_bin);
   }
-  int64_t val_int_offset(uint32_t row_offset)
+  inline int64_t val_int_offset(uint32_t row_offset)
   {
     ptr+=row_offset;
     int64_t tmp=val_int();
@@ -440,20 +414,20 @@ public:
     return tmp;
   }
 
-  int64_t val_int_internal(const unsigned char *new_ptr)
+  inline int64_t val_int(const unsigned char *new_ptr)
   {
     unsigned char *old_ptr= ptr;
+    int64_t return_value;
     ptr= const_cast<unsigned char*>(new_ptr);
-    int64_t return_value= val_int();
+    return_value= val_int();
     ptr= old_ptr;
     return return_value;
   }
-
-  String *val_str_internal(String *str, const unsigned char *new_ptr)
+  inline String *val_str(String *str, const unsigned char *new_ptr)
   {
     unsigned char *old_ptr= ptr;
     ptr= const_cast<unsigned char*>(new_ptr);
-    val_str_internal(str);
+    val_str(str);
     ptr= old_ptr;
     return str;
   }
@@ -562,18 +536,18 @@ public:
     return max_length;
   }
 
-  uint32_t offset(const unsigned char *record)
+  inline uint32_t offset(unsigned char *record)
   {
     return (uint32_t) (ptr - record);
   }
   void copy_from_tmp(int offset);
-  uint32_t fill_cache_field(CacheField *copy);
-  virtual bool get_date(type::Time &ltime,uint32_t fuzzydate) const;
-  virtual bool get_time(type::Time &ltime) const;
-  virtual const charset_info_st *charset(void) const { return &my_charset_bin; }
-  virtual const charset_info_st *sort_charset(void) const { return charset(); }
+  uint32_t fill_cache_field(struct st_cache_field *copy);
+  virtual bool get_date(DRIZZLE_TIME *ltime,uint32_t fuzzydate);
+  virtual bool get_time(DRIZZLE_TIME *ltime);
+  virtual const CHARSET_INFO *charset(void) const { return &my_charset_bin; }
+  virtual const CHARSET_INFO *sort_charset(void) const { return charset(); }
   virtual bool has_charset(void) const { return false; }
-  virtual void set_charset(const charset_info_st * const)
+  virtual void set_charset(const CHARSET_INFO * const)
   {}
   virtual enum Derivation derivation(void) const
   {
@@ -601,7 +575,7 @@ public:
       0 otherwise
   */
   bool set_warning(DRIZZLE_ERROR::enum_warning_level,
-                   drizzled::error_t code,
+                   unsigned int code,
                    int cuted_increment);
   /**
     Produce warning or note about datetime string data saved into field.
@@ -619,10 +593,10 @@ public:
       thread.
   */
   void set_datetime_warning(DRIZZLE_ERROR::enum_warning_level,
-                            drizzled::error_t code,
+                            uint32_t code,
                             const char *str,
                             uint32_t str_len,
-                            type::timestamp_t ts_type,
+                            enum enum_drizzle_timestamp_type ts_type,
                             int cuted_increment);
   /**
     Produce warning or note about integer datetime value saved into field.
@@ -639,9 +613,9 @@ public:
       thread.
   */
   void set_datetime_warning(DRIZZLE_ERROR::enum_warning_level,
-                            drizzled::error_t code,
+                            uint32_t code,
                             int64_t nr,
-                            type::timestamp_t ts_type,
+                            enum enum_drizzle_timestamp_type ts_type,
                             int cuted_increment);
   /**
     Produce warning or note about double datetime data saved into field.
@@ -657,10 +631,10 @@ public:
       thread.
   */
   void set_datetime_warning(DRIZZLE_ERROR::enum_warning_level,
-                            const drizzled::error_t code,
+                            const uint32_t code,
                             double nr,
-                            type::timestamp_t ts_type);
-  bool check_overflow(int op_result)
+                            enum enum_drizzle_timestamp_type ts_type);
+  inline bool check_overflow(int op_result)
   {
     return (op_result == E_DEC_OVERFLOW);
   }
@@ -694,23 +668,23 @@ public:
     @return
       value converted from val
   */
-  int64_t convert_decimal2int64_t(const type::Decimal *val,
+  int64_t convert_decimal2int64_t(const my_decimal *val,
                                   bool unsigned_flag,
                                   int *err);
   /* The max. number of characters */
-  uint32_t char_length() const
+  inline uint32_t char_length() const
   {
     return field_length / charset()->mbmaxlen;
   }
 
-  enum column_format_type column_format() const
+  inline enum column_format_type column_format() const
   {
     return (enum column_format_type)
       ((flags >> COLUMN_FORMAT_FLAGS) & COLUMN_FORMAT_MASK);
   }
 
   /* Hash value */
-  virtual void hash(uint32_t *nr, uint32_t *nr2) const;
+  virtual void hash(uint32_t *nr, uint32_t *nr2);
   friend bool reopen_table(Session *,Table *,bool);
 
   friend class CopyField;
@@ -726,53 +700,19 @@ public:
   friend class Item_sum_max;
   friend class Item_func_group_concat;
 
-  bool isReadSet() const;
+  bool isReadSet();
   bool isWriteSet();
   void setReadSet(bool arg= true);
   void setWriteSet(bool arg= true);
-
-protected:
-
-  void pack_num(uint64_t arg, unsigned char *destination= NULL);
-  void pack_num(uint32_t arg, unsigned char *destination= NULL);
-  uint64_t unpack_num(uint64_t &destination, const unsigned char *arg= NULL) const;
-  uint32_t unpack_num(uint32_t &destination, const unsigned char *arg= NULL) const;
 };
 
-namespace field {
+} /* namespace drizzled */
 
-inline bool isDateTime(const enum_field_types &arg)
+/** @TODO Why is this in the middle of the file???*/
+#include "drizzled/create_field.h"
+
+namespace drizzled
 {
-  switch (arg)
-  {
-  case DRIZZLE_TYPE_DATE:
-  case DRIZZLE_TYPE_DATETIME:
-  case DRIZZLE_TYPE_MICROTIME:
-  case DRIZZLE_TYPE_TIME:
-  case DRIZZLE_TYPE_TIMESTAMP:
-    return true;
-
-  case DRIZZLE_TYPE_BLOB:
-  case DRIZZLE_TYPE_BOOLEAN:
-  case DRIZZLE_TYPE_DECIMAL:
-  case DRIZZLE_TYPE_DOUBLE:
-  case DRIZZLE_TYPE_ENUM:
-  case DRIZZLE_TYPE_LONG:
-  case DRIZZLE_TYPE_LONGLONG:
-  case DRIZZLE_TYPE_NULL:
-  case DRIZZLE_TYPE_UUID:
-  case DRIZZLE_TYPE_IPV6:
-  case DRIZZLE_TYPE_VARCHAR:
-    return false;
-  }
-
-  assert(0);
-  abort();
-}
-
-} // namespace field
-
-std::ostream& operator<<(std::ostream& output, const Field &field);
 
 /**
  * A class for sending field information to a client.
@@ -795,8 +735,56 @@ public:
   uint32_t flags;
   uint32_t decimals;
   enum_field_types type;
+  SendField() {}
 };
 
+/**
+ * A class for quick copying data to fields
+ */
+class CopyField :public memory::SqlAlloc
+{
+  /**
+    Convenience definition of a copy function returned by
+    get_copy_func.
+  */
+  typedef void Copy_func(CopyField*);
+  Copy_func *get_copy_func(Field *to, Field *from);
+public:
+  unsigned char *from_ptr;
+  unsigned char *to_ptr;
+  unsigned char *from_null_ptr;
+  unsigned char *to_null_ptr;
+  bool *null_row;
+  uint32_t from_bit;
+  uint32_t to_bit;
+  uint32_t from_length;
+  uint32_t to_length;
+  Field *from_field;
+  Field *to_field;
+  String tmp;					// For items
+
+  CopyField() {}
+  ~CopyField() {}
+  void set(Field *to,Field *from,bool save);	// Field to field
+  void set(unsigned char *to,Field *from);		// Field to string
+  void (*do_copy)(CopyField *);
+  void (*do_copy2)(CopyField *);		// Used to handle null values
+};
+
+Field *make_field(TableShare *share,
+                  memory::Root *root,
+                  unsigned char *ptr,
+                  uint32_t field_length,
+                  bool is_nullable,
+                  unsigned char *null_pos,
+                  unsigned char null_bit,
+                  uint8_t decimals,
+                  enum_field_types field_type,
+                  const CHARSET_INFO * cs,
+                  Field::utype unireg_check,
+                  TYPELIB *interval,
+                  const char *field_name);
+
 uint32_t pack_length_to_packflag(uint32_t type);
 uint32_t calc_pack_length(enum_field_types type,uint32_t length);
 int set_field_to_null(Field *field);
@@ -815,9 +803,10 @@ int set_field_to_null_with_conversions(Field *field, bool no_conversions);
  * @retval
  *  true  - If string has some important data
  */
-bool test_if_important_data(const charset_info_st * const cs,
+bool test_if_important_data(const CHARSET_INFO * const cs,
                             const char *str,
                             const char *strend);
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FIELD_H */
