@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 
 /**
@@ -24,33 +24,29 @@
     gives much more speed.
 */
 
-#include <config.h>
-
+#include "config.h"
 #include <drizzled/error.h>
 #include <drizzled/table.h>
 #include <drizzled/session.h>
-#include <drizzled/current_session.h>
 
-#include <drizzled/copy_field.h>
-#include <drizzled/field/blob.h>
-#include <drizzled/field/date.h>
-#include <drizzled/field/datetime.h>
-#include <drizzled/field/decimal.h>
-#include <drizzled/field/double.h>
-#include <drizzled/field/enum.h>
-#include <drizzled/field/epoch.h>
-#include <drizzled/field/int32.h>
-#include <drizzled/field/int64.h>
-#include <drizzled/field/null.h>
-#include <drizzled/field/num.h>
-#include <drizzled/field/num.h>
-#include <drizzled/field/real.h>
 #include <drizzled/field/str.h>
+#include <drizzled/field/num.h>
+#include <drizzled/field/blob.h>
+#include <drizzled/field/enum.h>
+#include <drizzled/field/null.h>
+#include <drizzled/field/date.h>
+#include <drizzled/field/decimal.h>
+#include <drizzled/field/real.h>
+#include <drizzled/field/double.h>
+#include <drizzled/field/long.h>
+#include <drizzled/field/int64_t.h>
+#include <drizzled/field/num.h>
+#include <drizzled/field/timestamp.h>
+#include <drizzled/field/datetime.h>
 #include <drizzled/field/varstring.h>
-#include <drizzled/util/test.h>
-#include <drizzled/system_variables.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
 static void do_field_eq(CopyField *copy)
 {
@@ -147,12 +143,12 @@ set_field_to_null(Field *field)
     return 0;
   }
   field->reset();
-  if (field->getTable()->in_use->count_cuted_fields == CHECK_FIELD_WARN)
+  if (field->table->in_use->count_cuted_fields == CHECK_FIELD_WARN)
   {
     field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN, ER_WARN_DATA_TRUNCATED, 1);
     return 0;
   }
-  if (!field->getTable()->in_use->no_errors)
+  if (!field->table->in_use->no_errors)
     my_error(ER_BAD_NULL_ERROR, MYF(0), field->field_name);
   return -1;
 }
@@ -184,7 +180,6 @@ set_field_to_null_with_conversions(Field *field, bool no_conversions)
     field->reset();
     return 0;
   }
-
   if (no_conversions)
     return -1;
 
@@ -193,28 +188,24 @@ set_field_to_null_with_conversions(Field *field, bool no_conversions)
     when set to NULL (TIMESTAMP fields which allow setting to NULL
     are handled by first check).
   */
-  if (field->is_timestamp())
+  if (field->type() == DRIZZLE_TYPE_TIMESTAMP)
   {
-    ((field::Epoch::pointer) field)->set_time();
+    ((Field_timestamp*) field)->set_time();
     return 0;					// Ok to set time to NULL
   }
-
   field->reset();
-  if (field == field->getTable()->next_number_field)
+  if (field == field->table->next_number_field)
   {
-    field->getTable()->auto_increment_field_not_null= false;
+    field->table->auto_increment_field_not_null= false;
     return 0;				  // field is set in fill_record()
   }
-
-  if (field->getTable()->in_use->count_cuted_fields == CHECK_FIELD_WARN)
+  if (field->table->in_use->count_cuted_fields == CHECK_FIELD_WARN)
   {
     field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN, ER_BAD_NULL_ERROR, 1);
     return 0;
   }
-
-  if (!field->getTable()->in_use->no_errors)
+  if (!field->table->in_use->no_errors)
     my_error(ER_BAD_NULL_ERROR, MYF(0), field->field_name);
-
   return -1;
 }
 
@@ -257,20 +248,14 @@ static void do_outer_field_null(CopyField *copy)
 
 static void do_copy_not_null(CopyField *copy)
 {
-  if (copy->to_field->hasDefault() and *copy->from_null_ptr & copy->from_bit)
-  {
-    copy->to_field->set_default();
-  }
-  else if (*copy->from_null_ptr & copy->from_bit)
+  if (*copy->from_null_ptr & copy->from_bit)
   {
     copy->to_field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN,
                                 ER_WARN_DATA_TRUNCATED, 1);
     copy->to_field->reset();
   }
   else
-  {
     (copy->do_copy2)(copy);
-  }
 }
 
 
@@ -287,12 +272,10 @@ static void do_copy_timestamp(CopyField *copy)
   if (*copy->from_null_ptr & copy->from_bit)
   {
     /* Same as in set_field_to_null_with_conversions() */
-    ((field::Epoch::pointer) copy->to_field)->set_time();
+    ((Field_timestamp*) copy->to_field)->set_time();
   }
   else
-  {
     (copy->do_copy2)(copy);
-  }
 }
 
 
@@ -301,13 +284,11 @@ static void do_copy_next_number(CopyField *copy)
   if (*copy->from_null_ptr & copy->from_bit)
   {
     /* Same as in set_field_to_null_with_conversions() */
-    copy->to_field->getTable()->auto_increment_field_not_null= false;
+    copy->to_field->table->auto_increment_field_not_null= false;
     copy->to_field->reset();
   }
   else
-  {
     (copy->do_copy2)(copy);
-  }
 }
 
 
@@ -320,7 +301,7 @@ static void do_copy_blob(CopyField *copy)
 
 static void do_conv_blob(CopyField *copy)
 {
-  copy->from_field->val_str_internal(&copy->tmp);
+  copy->from_field->val_str(&copy->tmp);
   ((Field_blob *) copy->to_field)->store(copy->tmp.ptr(),
                                          copy->tmp.length(),
                                          copy->tmp.charset());
@@ -332,7 +313,7 @@ static void do_save_blob(CopyField *copy)
 {
   char buff[MAX_FIELD_WIDTH];
   String res(buff, sizeof(buff), copy->tmp.charset());
-  copy->from_field->val_str_internal(&res);
+  copy->from_field->val_str(&res);
   copy->tmp.copy(res);
   ((Field_blob *) copy->to_field)->store(copy->tmp.ptr(),
                                          copy->tmp.length(),
@@ -344,8 +325,9 @@ static void do_field_string(CopyField *copy)
 {
   char buff[MAX_FIELD_WIDTH];
   copy->tmp.set_quick(buff,sizeof(buff),copy->tmp.charset());
-  copy->from_field->val_str_internal(&copy->tmp);
-  copy->to_field->store(copy->tmp.data(),copy->tmp.length(), copy->tmp.charset());
+  copy->from_field->val_str(&copy->tmp);
+  copy->to_field->store(copy->tmp.c_ptr_quick(),copy->tmp.length(),
+                        copy->tmp.charset());
 }
 
 
@@ -374,7 +356,7 @@ static void do_field_real(CopyField *copy)
 
 static void do_field_decimal(CopyField *copy)
 {
-  type::Decimal value;
+  my_decimal value;
   copy->to_field->store_decimal(copy->from_field->val_decimal(&value));
 }
 
@@ -386,8 +368,8 @@ static void do_field_decimal(CopyField *copy)
 
 static void do_cut_string(CopyField *copy)
 {
-  const charset_info_st * const cs= copy->from_field->charset();
-  memcpy(copy->to_ptr, copy->from_ptr, copy->to_length);
+  const CHARSET_INFO * const cs= copy->from_field->charset();
+  memcpy(copy->to_ptr,copy->from_ptr,copy->to_length);
 
   /* Check if we loosed any important characters */
   if (cs->cset->scan(cs,
@@ -409,9 +391,13 @@ static void do_cut_string(CopyField *copy)
 static void do_cut_string_complex(CopyField *copy)
 {						// Shorter string field
   int well_formed_error;
-  const charset_info_st * const cs= copy->from_field->charset();
+  const CHARSET_INFO * const cs= copy->from_field->charset();
   const unsigned char *from_end= copy->from_ptr + copy->from_length;
-  uint32_t copy_length= cs->cset->well_formed_len(*cs, str_ref(copy->from_ptr, from_end), copy->to_length / cs->mbmaxlen, &well_formed_error);
+  uint32_t copy_length= cs->cset->well_formed_len(cs,
+                                              (char*) copy->from_ptr,
+                                              (char*) from_end,
+                                              copy->to_length / cs->mbmaxlen,
+                                              &well_formed_error);
   if (copy->to_length < copy_length)
     copy_length= copy->to_length;
   memcpy(copy->to_ptr, copy->from_ptr, copy_length);
@@ -436,7 +422,7 @@ static void do_cut_string_complex(CopyField *copy)
 
 static void do_expand_binary(CopyField *copy)
 {
-  const charset_info_st * const cs= copy->from_field->charset();
+  const CHARSET_INFO * const cs= copy->from_field->charset();
   memcpy(copy->to_ptr, copy->from_ptr, copy->from_length);
   cs->cset->fill(cs, (char*) copy->to_ptr+copy->from_length,
                  copy->to_length-copy->from_length, '\0');
@@ -446,7 +432,7 @@ static void do_expand_binary(CopyField *copy)
 
 static void do_expand_string(CopyField *copy)
 {
-  const charset_info_st * const cs= copy->from_field->charset();
+  const CHARSET_INFO * const cs= copy->from_field->charset();
   memcpy(copy->to_ptr,copy->from_ptr,copy->from_length);
   cs->cset->fill(cs, (char*) copy->to_ptr+copy->from_length,
                  copy->to_length-copy->from_length, ' ');
@@ -459,11 +445,9 @@ static void do_varstring1(CopyField *copy)
   if (length > copy->to_length- 1)
   {
     length= copy->to_length - 1;
-    if (copy->from_field->getTable()->in_use->count_cuted_fields)
-    {
+    if (copy->from_field->table->in_use->count_cuted_fields)
       copy->to_field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN,
                                   ER_WARN_DATA_TRUNCATED, 1);
-    }
   }
   *(unsigned char*) copy->to_ptr= (unsigned char) length;
   memcpy(copy->to_ptr+1, copy->from_ptr + 1, length);
@@ -473,17 +457,18 @@ static void do_varstring1(CopyField *copy)
 static void do_varstring1_mb(CopyField *copy)
 {
   int well_formed_error;
-  const charset_info_st * const cs= copy->from_field->charset();
+  const CHARSET_INFO * const cs= copy->from_field->charset();
   uint32_t from_length= (uint32_t) *(unsigned char*) copy->from_ptr;
   const unsigned char *from_ptr= copy->from_ptr + 1;
   uint32_t to_char_length= (copy->to_length - 1) / cs->mbmaxlen;
-  uint32_t length= cs->cset->well_formed_len(*cs, str_ref(from_ptr, from_length), to_char_length, &well_formed_error);
+  uint32_t length= cs->cset->well_formed_len(cs, (char*) from_ptr,
+                                         (char*) from_ptr + from_length,
+                                         to_char_length, &well_formed_error);
   if (length < from_length)
   {
     if (current_session->count_cuted_fields)
-    {
-      copy->to_field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN, ER_WARN_DATA_TRUNCATED, 1);
-    }
+      copy->to_field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN,
+                                  ER_WARN_DATA_TRUNCATED, 1);
   }
   *copy->to_ptr= (unsigned char) length;
   memcpy(copy->to_ptr + 1, from_ptr, length);
@@ -496,11 +481,9 @@ static void do_varstring2(CopyField *copy)
   if (length > copy->to_length- HA_KEY_BLOB_LENGTH)
   {
     length=copy->to_length-HA_KEY_BLOB_LENGTH;
-    if (copy->from_field->getTable()->in_use->count_cuted_fields)
-    {
+    if (copy->from_field->table->in_use->count_cuted_fields)
       copy->to_field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN,
                                   ER_WARN_DATA_TRUNCATED, 1);
-    }
   }
   int2store(copy->to_ptr,length);
   memcpy(copy->to_ptr+HA_KEY_BLOB_LENGTH, copy->from_ptr + HA_KEY_BLOB_LENGTH,
@@ -511,18 +494,18 @@ static void do_varstring2(CopyField *copy)
 static void do_varstring2_mb(CopyField *copy)
 {
   int well_formed_error;
-  const charset_info_st * const cs= copy->from_field->charset();
+  const CHARSET_INFO * const cs= copy->from_field->charset();
   uint32_t char_length= (copy->to_length - HA_KEY_BLOB_LENGTH) / cs->mbmaxlen;
   uint32_t from_length= uint2korr(copy->from_ptr);
   const unsigned char *from_beg= copy->from_ptr + HA_KEY_BLOB_LENGTH;
-  uint32_t length= cs->cset->well_formed_len(*cs, str_ref(from_beg, from_length), char_length, &well_formed_error);
+  uint32_t length= cs->cset->well_formed_len(cs, (char*) from_beg,
+                                             (char*) from_beg + from_length,
+                                             char_length, &well_formed_error);
   if (length < from_length)
   {
     if (current_session->count_cuted_fields)
-    {
       copy->to_field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN,
                                   ER_WARN_DATA_TRUNCATED, 1);
-    }
   }
   int2store(copy->to_ptr, length);
   memcpy(copy->to_ptr+HA_KEY_BLOB_LENGTH, from_beg, length);
@@ -553,9 +536,9 @@ void CopyField::set(unsigned char *to,Field *from)
     to_ptr[0]= 1;                             // Null as default value
     to_null_ptr= (unsigned char*) to_ptr++;
     to_bit= 1;
-    if (from->getTable()->maybe_null)
+    if (from->table->maybe_null)
     {
-      null_row= &from->getTable()->null_row;
+      null_row= &from->table->null_row;
       do_copy= do_outer_field_to_null_str;
     }
     else
@@ -611,29 +594,21 @@ void CopyField::set(Field *to,Field *from,bool save)
       to_null_ptr= to->null_ptr;
       to_bit= to->null_bit;
       if (from_null_ptr)
-      {
         do_copy= do_copy_null;
-      }
       else
       {
-        null_row= &from->getTable()->null_row;
+        null_row= &from->table->null_row;
         do_copy= do_outer_field_null;
       }
     }
     else
     {
-      if (to_field->is_timestamp())
-      {
+      if (to_field->type() == DRIZZLE_TYPE_TIMESTAMP)
         do_copy= do_copy_timestamp;               // Automatic timestamp
-      }
-      else if (to_field == to_field->getTable()->next_number_field)
-      {
+      else if (to_field == to_field->table->next_number_field)
         do_copy= do_copy_next_number;
-      }
       else
-      {
         do_copy= do_copy_not_null;
-      }
     }
   }
   else if (to_field->real_maybe_null())
@@ -657,8 +632,8 @@ void CopyField::set(Field *to,Field *from,bool save)
 CopyField::Copy_func *
 CopyField::get_copy_func(Field *to,Field *from)
 {
-  bool compatible_db_low_byte_first= (to->getTable()->getShare()->db_low_byte_first ==
-                                     from->getTable()->getShare()->db_low_byte_first);
+  bool compatible_db_low_byte_first= (to->table->s->db_low_byte_first ==
+                                     from->table->s->db_low_byte_first);
   if (to->flags & BLOB_FLAG)
   {
     if (!(from->flags & BLOB_FLAG) || from->charset() != to->charset())
@@ -666,8 +641,8 @@ CopyField::get_copy_func(Field *to,Field *from)
     if (from_length != to_length || !compatible_db_low_byte_first)
     {
       // Correct pointer to point at char pointer
-      to_ptr+= to_length - to->getTable()->getShare()->sizeBlobPtr();
-      from_ptr+= from_length- from->getTable()->getShare()->sizeBlobPtr();
+      to_ptr+= to_length - to->table->s->blob_ptr_size;
+      from_ptr+= from_length- from->table->s->blob_ptr_size;
       return do_copy_blob;
     }
   }
@@ -685,7 +660,7 @@ CopyField::get_copy_func(Field *to,Field *from)
       */
       if (to->real_type() != from->real_type() ||
           !compatible_db_low_byte_first ||
-          (((to->getTable()->in_use->variables.sql_mode & (MODE_NO_ZERO_DATE | MODE_INVALID_DATES)) && to->type() == DRIZZLE_TYPE_DATE) || to->type() == DRIZZLE_TYPE_DATETIME))
+          (((to->table->in_use->variables.sql_mode & (MODE_NO_ZERO_DATE | MODE_INVALID_DATES)) && to->type() == DRIZZLE_TYPE_DATE) || to->type() == DRIZZLE_TYPE_DATETIME))
       {
         if (from->real_type() == DRIZZLE_TYPE_ENUM)
         {
@@ -693,11 +668,11 @@ CopyField::get_copy_func(Field *to,Field *from)
           {
             return do_field_int;  // Convert SET to number
           }
-
+          
           return do_field_string;
         }
       }
-
+      
       if (to->real_type() == DRIZZLE_TYPE_ENUM)
       {
         if (!to->eq_def(from))
@@ -713,30 +688,19 @@ CopyField::get_copy_func(Field *to,Field *from)
         return do_field_string;
       else if (to->real_type() == DRIZZLE_TYPE_VARCHAR)
       {
-        /* Field_blob is not part of the Field_varstring hierarchy,
-          and casting to varstring for calling pack_length_no_ptr()
-          is always incorrect. Previously the below comparison has
-          always evaluated to false as pack_length_no_ptr() for BLOB
-          will return 4 and varstring can only be <= 2.
-          If your brain slightly bleeds as to why this worked for
-          so many years, you are in no way alone.
-        */
-        if (from->flags & BLOB_FLAG)
-          return do_field_string;
-
-        if ((static_cast<Field_varstring*>(to))->pack_length_no_ptr() !=
-            (static_cast<Field_varstring*>(from))->pack_length_no_ptr())
+        if (((Field_varstring*) to)->length_bytes !=
+            ((Field_varstring*) from)->length_bytes)
         {
           return do_field_string;
         }
-
+        
         if (to_length != from_length)
         {
-          return (((Field_varstring*) to)->pack_length_no_ptr() == 1 ?
+          return (((Field_varstring*) to)->length_bytes == 1 ?
                   (from->charset()->mbmaxlen == 1 ? do_varstring1 :
-                   do_varstring1_mb) :
+                                                    do_varstring1_mb) :
                   (from->charset()->mbmaxlen == 1 ? do_varstring2 :
-                   do_varstring2_mb));
+                                                    do_varstring2_mb));
         }
       }
       else if (to_length < from_length)
@@ -795,7 +759,7 @@ CopyField::get_copy_func(Field *to,Field *from)
 int field_conv(Field *to,Field *from)
 {
   if (to->real_type() == from->real_type() &&
-      !(to->type() == DRIZZLE_TYPE_BLOB && to->getTable()->copy_blobs))
+      !(to->type() == DRIZZLE_TYPE_BLOB && to->table->copy_blobs))
   {
     /* Please god, will someone rewrite this to be readable :( */
     if (to->pack_length() == from->pack_length() &&
@@ -803,9 +767,9 @@ int field_conv(Field *to,Field *from)
         to->real_type() != DRIZZLE_TYPE_ENUM &&
         (to->real_type() != DRIZZLE_TYPE_DECIMAL || (to->field_length == from->field_length && (((Field_num*)to)->dec == ((Field_num*)from)->dec))) &&
         from->charset() == to->charset() &&
-	to->getTable()->getShare()->db_low_byte_first == from->getTable()->getShare()->db_low_byte_first &&
-        (!(to->getTable()->in_use->variables.sql_mode & (MODE_NO_ZERO_DATE | MODE_INVALID_DATES)) || (to->type() != DRIZZLE_TYPE_DATE && to->type() != DRIZZLE_TYPE_DATETIME)) &&
-        (from->real_type() != DRIZZLE_TYPE_VARCHAR || ((Field_varstring*)from)->pack_length_no_ptr() == ((Field_varstring*)to)->pack_length_no_ptr()))
+	to->table->s->db_low_byte_first == from->table->s->db_low_byte_first &&
+        (!(to->table->in_use->variables.sql_mode & (MODE_NO_ZERO_DATE | MODE_INVALID_DATES)) || (to->type() != DRIZZLE_TYPE_DATE && to->type() != DRIZZLE_TYPE_DATETIME)) &&
+        (from->real_type() != DRIZZLE_TYPE_VARCHAR || ((Field_varstring*)from)->length_bytes == ((Field_varstring*)to)->length_bytes))
     {						// Identical fields
       /* This may happen if one does 'UPDATE ... SET x=x' */
       if (to->ptr != from->ptr)
@@ -816,12 +780,12 @@ int field_conv(Field *to,Field *from)
   if (to->type() == DRIZZLE_TYPE_BLOB)
   {						// Be sure the value is stored
     Field_blob *blob=(Field_blob*) to;
-    from->val_str_internal(&blob->value);
+    from->val_str(&blob->value);
     /*
       Copy value if copy_blobs is set, or source is not a string and
       we have a pointer to its internal string conversion buffer.
     */
-    if (to->getTable()->copy_blobs ||
+    if (to->table->copy_blobs ||
         (!blob->value.is_alloced() &&
          from->real_type() != DRIZZLE_TYPE_VARCHAR))
       blob->value.copy();
@@ -840,28 +804,24 @@ int field_conv(Field *to,Field *from)
   {
     char buff[MAX_FIELD_WIDTH];
     String result(buff,sizeof(buff),from->charset());
-    from->val_str_internal(&result);
+    from->val_str(&result);
     /*
       We use c_ptr_quick() here to make it easier if to is a float/double
       as the conversion routines will do a copy of the result doesn't
       end with \0. Can be replaced with .ptr() when we have our own
       string->double conversion.
     */
-    return to->store(result.c_str(),result.length(),from->charset());
+    return to->store(result.c_ptr_quick(),result.length(),from->charset());
   }
   else if (from->result_type() == REAL_RESULT)
-  {
     return to->store(from->val_real());
-  }
   else if (from->result_type() == DECIMAL_RESULT)
   {
-    type::Decimal buff;
+    my_decimal buff;
     return to->store_decimal(from->val_decimal(&buff));
   }
   else
-  {
     return to->store(from->val_int(), test(from->flags & UNSIGNED_FLAG));
-  }
 }
 
 } /* namespace drizzled */
