@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_LOCATE_H
+#define DRIZZLED_FUNCTION_LOCATE_H
 
 #include <drizzled/function/func.h>
 #include <drizzled/function/math/int.h>
@@ -35,8 +36,9 @@ public:
   const char *func_name() const { return "locate"; }
   int64_t val_int();
   void fix_length_and_dec();
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_LOCATE_H */
