@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_MATH_DEC_H
+#define DRIZZLED_FUNCTION_MATH_DEC_H
 
 #include <drizzled/function/func.h>
 #include <drizzled/function/math/real.h>
@@ -29,7 +30,6 @@ namespace drizzled
 class Item_dec_func :public Item_real_func
 {
  public:
-  Item_dec_func(): Item_real_func() {}
   Item_dec_func(Item *a) :Item_real_func(a) {}
   Item_dec_func(Item *a,Item *b) :Item_real_func(a,b) {}
   void fix_length_and_dec()
@@ -42,3 +42,4 @@ class Item_dec_func :public Item_real_func
 } /* namespace drizzled */
 
 
+#endif /* DRIZZLED_FUNCTION_MATH_DEC_H */
