@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_MATH_DIVIDE_H
+#define DRIZZLED_FUNCTION_MATH_DIVIDE_H
 
 #include <drizzled/function/func.h>
 #include <drizzled/function/num_op.h>
@@ -27,13 +28,12 @@ namespace drizzled
 
 class Item_func_div :public Item_num_op
 {
-  Session *session;
 public:
   uint32_t prec_increment;
-  Item_func_div(Session *arg, Item *a,Item *b) :Item_num_op(a,b), session(arg) {}
+  Item_func_div(Item *a,Item *b) :Item_num_op(a,b) {}
   int64_t int_op() { assert(0); return 0; }
   double real_op();
-  type::Decimal *decimal_op(type::Decimal *);
+  my_decimal *decimal_op(my_decimal *);
   const char *func_name() const { return "/"; }
   void fix_length_and_dec();
   void result_precision();
@@ -41,3 +41,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_MATH_DIVIDE_H */
