@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_MATH_INT_VAL_H
+#define DRIZZLED_FUNCTION_MATH_INT_VAL_H
 
 #include <drizzled/function/func.h>
 #include <drizzled/function/num1.h>
@@ -29,10 +30,10 @@ class Item_func_int_val :public Item_func_num1
 {
 public:
   Item_func_int_val(Item *a) :Item_func_num1(a) {}
-  Item_func_int_val() :Item_func_num1() {}
   void fix_num_length_and_dec();
   void find_num_type();
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_MATH_INT_VAL_H */
