@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_NUM1_H
+#define DRIZZLED_FUNCTION_NUM1_H
 
 #include <drizzled/function/numhybrid.h>
 
@@ -28,7 +29,6 @@ namespace drizzled
 class Item_func_num1: public Item_func_numhybrid
 {
 public:
-  Item_func_num1(): Item_func_numhybrid() {}
   Item_func_num1(Item *a) :Item_func_numhybrid(a) {}
   Item_func_num1(Item *a, Item *b) :Item_func_numhybrid(a, b) {}
 
@@ -40,3 +40,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_NUM1_H */
