@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <drizzled/function/row_count.h>
 #include <drizzled/session.h>
@@ -28,7 +28,9 @@ namespace drizzled
 int64_t Item_func_row_count::val_int()
 {
   assert(fixed == 1);
-  return getSession().rowCount();
+  Session *session= current_session;
+
+  return session->row_count_func;
 }
 
 
