@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_STR_INSERT_H
+#define DRIZZLED_FUNCTION_STR_INSERT_H
 
 #include <drizzled/function/str/strfunc.h>
 
@@ -26,17 +27,10 @@ namespace drizzled
 
 class Item_func_insert :public Item_str_func
 {
-  Session &session;
   String tmp_value;
 public:
-  Item_func_insert(Session &session_arg,
-                   Item *org,
-                   Item *start,
-                   Item *length,
-                   Item *new_str) :
-    Item_str_func(org,start,length,new_str),
-    session(session_arg)
-  {}
+  Item_func_insert(Item *org,Item *start,Item *length,Item *new_str)
+    :Item_str_func(org,start,length,new_str) {}
   String *val_str(String *);
   void fix_length_and_dec();
   const char *func_name() const { return "insert"; }
@@ -44,3 +38,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_STR_INSERT_H */
