@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,20 +17,17 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_STR_LOAD_FILE_H
+#define DRIZZLED_FUNCTION_STR_LOAD_FILE_H
 
 namespace drizzled
 {
 
 class Item_load_file :public Item_str_func
 {
-  Session &session;
   String tmp_value;
 public:
-  Item_load_file(Session &session_arg, Item *a) :
-    Item_str_func(a),
-    session(session_arg)
-  {}
+  Item_load_file(Item *a) :Item_str_func(a) {}
   String *val_str(String *);
   const char *func_name() const { return "load_file"; }
   void fix_length_and_dec()
@@ -43,3 +40,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_STR_LOAD_FILE_H */
