@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_TIME_TIMESTAMP_DIFF_H
+#define DRIZZLED_FUNCTION_TIME_TIMESTAMP_DIFF_H
 
 #include <drizzled/function/math/int.h>
 
@@ -37,9 +38,10 @@ public:
     decimals=0;
     maybe_null=1;
   }
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_TIME_TIMESTAMP_DIFF_H */
 
