@@ -11,30 +11,32 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* Sort of string pointers in string-order with radix or qsort */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/internal/my_sys.h>
-#include <drizzled/internal/m_string.h>
+#include "drizzled/internal/my_sys.h"
+#include "drizzled/internal/m_string.h"
 
-namespace drizzled {
-namespace internal {
+namespace drizzled
+{
+namespace internal
+{
 
 void my_string_ptr_sort(unsigned char *base, uint32_t items, size_t size)
 {
 #if INT_MAX > 65536L
-  if (size <= 20 && items >= 1000 && items < 100000)
+  unsigned char **ptr= NULL;
+
+  if (size <= 20 && items >= 1000 && items < 100000 &&
+      (ptr= (unsigned char**) malloc(items*sizeof(char*))))
   {
-    unsigned char** ptr= (unsigned char**) malloc(items*sizeof(char*));
     radixsort_for_str_ptr((unsigned char**) base,items,size,ptr);
     free((unsigned char*) ptr);
   }
   else
-#else
-  assert(false);
 #endif
   {
     if (size && items)
