@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,20 +17,44 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-
-
-#pragma once
+#ifndef DRIZZLED_ITEM_H
+#define DRIZZLED_ITEM_H
 
 #include <drizzled/dtcollation.h>
-#include <drizzled/charset.h>
-#include <drizzled/item_result.h>
-#include <drizzled/memory/sql_alloc.h>
+#include <drizzled/my_time.h>
+#include <drizzled/my_decimal.h>
+#include <drizzled/sql_bitmap.h>
 #include <drizzled/sql_list.h>
-#include <drizzled/sql_string.h>
+#include "drizzled/memory/sql_alloc.h"
+#include <drizzled/table.h>
+#include "drizzled/item_result.h"
 
-#include <drizzled/visibility.h>
+namespace drizzled
+{
 
-namespace drizzled {
+class TableList;
+class Item_field;
+class Name_resolution_context;
+class Select_Lex;
+class Item_equal;
+class user_var_entry;
+class Item_sum;
+class Item_in_subselect;
+class SendField;
+class Field;
+
+namespace plugin
+{
+class Client;
+}
+
+/**
+  Dummy error processor used by default by Name_resolution_context.
+
+  @note
+    do nothing
+*/
+void dummy_error_processor(Session *session, void *data);
 
 /*
   Analyzer function
@@ -53,8 +77,12 @@ typedef bool (Item::*Item_processor) (unsigned char *arg);
  * statement "tree" or Lex.  Each item represents something in the
  * execution plan.
  */
-class DRIZZLED_API Item : public memory::SqlAlloc, boost::noncopyable
+class Item: public memory::SqlAlloc
 {
+  /* Prevent use of these */
+  Item(const Item &);
+  void operator=(Item &);
+
   /* Cache of the result of is_expensive(). */
   int8_t is_expensive_cache;
   virtual bool is_expensive_processor(unsigned char *arg);
@@ -84,7 +112,6 @@ public:
     ROW_ITEM, CACHE_ITEM,
     TYPE_HOLDER,
     PARAM_ITEM,
-    BOOLEAN_ITEM,
     DECIMAL_ITEM
   };
   enum traverse_order
@@ -107,11 +134,12 @@ public:
   String str_value;
 
   /** Name from select */
-  const char *name;
-
+  char *name;
   /** Length of name */
   uint32_t name_length;
 
+  /** Original item name (if it was renamed) */
+  char *orig_name;
   Item *next;
   uint32_t max_length;
 
@@ -121,20 +149,8 @@ public:
   bool maybe_null; /**< True if item may be null */
   bool null_value; /**< True if item is null */
   bool unsigned_flag;
-
-  bool is_unsigned() const
-  {
-    return unsigned_flag;
-  }
-
-  virtual bool negative() const
-  {
-    return false;
-  }
-
   bool with_sum_func;
   bool is_autogenerated_name; /**< indicates whether name of this Item was autogenerated or set by user */
-
   /**
    * If this item is a subselect or some of its arguments is or contains a
    * subselect. Computed by fix_fields.
@@ -142,7 +158,6 @@ public:
   bool with_subselect;
   DTCollation collation;
   Item_result cmp_context; /**< Comparison context */
-
   /**
    * Constructor
    *
@@ -151,7 +166,6 @@ public:
    * Alloc & destruct is done as start of select using memory::sql_alloc
    */
   Item();
-
   /**
    * Constructor used by Item_field, Item_ref & aggregate (sum) functions.
    *
@@ -163,23 +177,26 @@ public:
    * optimisation changes in prepared statements
    */
   Item(Session *session, Item *item);
-
   virtual ~Item()
   {
+#ifdef EXTRA_DEBUG
+    name=0;
+#endif
   }
 
-  void set_name(str_ref arg)
+  void set_name(const std::string &arg)
   {
-    set_name(arg.data(), arg.size());
+    set_name(arg.c_str(), arg.length(), system_charset_info);
   }
 
-  void set_name(const char* arg)
-  {
-    set_name(str_ref(arg));
-  }
-
-  void set_name(const char *str, uint32_t length, const charset_info_st* cs= system_charset_info);
-  void init_make_field(SendField *tmp_field, enum_field_types type);
+  void set_name(const char *str, uint32_t length, const CHARSET_INFO * const cs= system_charset_info);
+  /**
+   * Renames item (used for views, cleanup() return original name).
+   *
+   * @param new_name	new name of item;
+   */
+  void rename(char *new_name);
+  void init_make_field(SendField *tmp_field,enum enum_field_types type);
   virtual void cleanup();
   virtual void make_field(SendField *field);
   /**
@@ -221,7 +238,7 @@ public:
   /**
    * This is only called from items that is not of type item_field.
    */
-  virtual void send(plugin::Client *client, String *str);
+  virtual bool send(plugin::Client *client, String *str);
   /**
     Compares this Item to another Item, returning true if Item's
     are functionally equal.
@@ -346,7 +363,6 @@ public:
    *   If value is not null null_value flag will be reset to false.
    */
   virtual String *val_str(String *str)=0;
-
   /**
    * Return decimal representation of item with fixed point.
    *
@@ -360,13 +376,12 @@ public:
    *
    * @retval
    *
-   * Return pointer on type::Decimal (it can be other then passed via argument)
+   * Return pointer on my_decimal (it can be other then passed via argument)
    * if value is not NULL (null_value flag will be reset to false).
    * In case of NULL value it return 0 pointer and set null_value flag
    * to true.
    */
-  virtual type::Decimal *val_decimal(type::Decimal *decimal_buffer)= 0;
-
+  virtual my_decimal *val_decimal(my_decimal *decimal_buffer)= 0;
   /**
    * Return boolean value of item.
    *
@@ -376,22 +391,20 @@ public:
    * true value is true (not equal to 0)
    */
   virtual bool val_bool();
-
   /* Helper functions, see item_sum.cc */
   String *val_string_from_real(String *str);
   String *val_string_from_int(String *str);
   String *val_string_from_decimal(String *str);
-  type::Decimal *val_decimal_from_real(type::Decimal *decimal_value);
-  type::Decimal *val_decimal_from_int(type::Decimal *decimal_value);
-  type::Decimal *val_decimal_from_string(type::Decimal *decimal_value);
-  type::Decimal *val_decimal_from_date(type::Decimal *decimal_value);
-  type::Decimal *val_decimal_from_time(type::Decimal *decimal_value);
+  my_decimal *val_decimal_from_real(my_decimal *decimal_value);
+  my_decimal *val_decimal_from_int(my_decimal *decimal_value);
+  my_decimal *val_decimal_from_string(my_decimal *decimal_value);
+  my_decimal *val_decimal_from_date(my_decimal *decimal_value);
+  my_decimal *val_decimal_from_time(my_decimal *decimal_value);
   int64_t val_int_from_decimal();
   double val_real_from_decimal();
 
-  bool save_time_in_field(Field *field);
-  bool save_date_in_field(Field *field);
-
+  int save_time_in_field(Field *field);
+  int save_date_in_field(Field *field);
   /**
    * Stores a string value in field directly
    *
@@ -437,7 +450,7 @@ public:
   {
     return val_str(tmp);
   }
-  virtual type::Decimal *val_decimal_result(type::Decimal *val)
+  virtual my_decimal *val_decimal_result(my_decimal *val)
   {
     return val_decimal(val);
   }
@@ -514,11 +527,11 @@ public:
 
     For more information about view definition query, INFORMATION_SCHEMA
     query and why they should be generated from the Item-tree, @see
-    register_view().
+    mysql_register_view().
   */
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
 
-  void print_item_w_name(String *);
+  void print_item_w_name(String *, enum_query_type query_type);
   virtual void update_used_tables() {}
   virtual void split_sum_func(Session *session, 
                               Item **ref_pointer_array,
@@ -548,17 +561,17 @@ public:
                       bool skip_registered);
 
   /**
-    Get the value of the function as a type::Time structure.
+    Get the value of the function as a DRIZZLE_TIME structure.
     As a extra convenience the time structure is reset on error!
   */
-  virtual bool get_date(type::Time &ltime, uint32_t fuzzydate);
+  virtual bool get_date(DRIZZLE_TIME *ltime,uint32_t fuzzydate);
   /**
     Get time of first argument.
 
     As a extra convenience the time structure is reset on error!
   */
-  virtual bool get_time(type::Time &ltime);
-  virtual bool get_date_result(type::Time &ltime,uint32_t fuzzydate);
+  virtual bool get_time(DRIZZLE_TIME *ltime);
+  virtual bool get_date_result(DRIZZLE_TIME *ltime,uint32_t fuzzydate);
 
   /**
     The method allows to determine nullness of a complex expression
@@ -605,8 +618,8 @@ public:
   virtual const Item *real_item(void) const;
   virtual Item *get_tmp_table_item(Session *session);
 
-  static const charset_info_st *default_charset();
-  virtual const charset_info_st *compare_collation();
+  static const CHARSET_INFO *default_charset();
+  virtual const CHARSET_INFO *compare_collation();
 
   virtual bool walk(Item_processor processor,
                     bool walk_subquery,
@@ -656,6 +669,7 @@ public:
                              traverse_order order);
 
   virtual bool remove_dependence_processor(unsigned char * arg);
+  virtual bool remove_fixed(unsigned char * arg);
   virtual bool collect_item_field_processor(unsigned char * arg);
   virtual bool find_item_in_field_list_processor(unsigned char *arg);
   virtual bool change_context_processor(unsigned char *context);
@@ -694,7 +708,7 @@ public:
 
   virtual Item *neg_transformer(Session *session);
   virtual Item *update_value_transformer(unsigned char *select_arg);
-  virtual Item *safe_charset_converter(const charset_info_st * const tocs);
+  virtual Item *safe_charset_converter(const CHARSET_INFO * const tocs);
   void delete_self();
 
   /**
@@ -718,11 +732,6 @@ public:
    * cost Item::execution_cost(),
    * where 'cost' is either 'double' or some structure of various cost
    * parameters.
-   *
-   *NOTE
-   *   This function is now used to prevent evaluation of materialized IN
-   *   subquery predicates before it is allowed. grep for
-   *   DontEvaluateMaterializedSubqueryTooEarly to see the uses.
    */
   virtual bool is_expensive();
 
@@ -747,34 +756,9 @@ public:
    * @retval
    *  false otherwise
    */
-  bool eq_by_collation(Item *item, bool binary_cmp, const charset_info_st * const cs);
-
-  inline uint32_t char_to_byte_length_safe(uint32_t char_length_arg, uint32_t mbmaxlen_arg)
-  { 
-    uint64_t tmp= ((uint64_t) char_length_arg) * mbmaxlen_arg;
-    return (tmp > UINT32_MAX) ? (uint32_t) UINT32_MAX : (uint32_t) tmp;
-  } 
-
-  uint32_t max_char_length() const;
-
-  void fix_char_length(uint32_t max_char_length_arg);
-
-protected:
-  Session &getSession()
-  {
-    return _session;
-  }
-
-private:
-  Session &_session;
+  bool eq_by_collation(Item *item, bool binary_cmp, const CHARSET_INFO * const cs);
 };
 
-namespace display {
-const std::string &type(Item::Type type);
-} /* namespace display */
-
-std::ostream& operator<<(std::ostream& output, const Item &item);
-
 } /* namespace drizzled */
 
 /** @TODO Why is this in the middle? */
@@ -919,3 +903,4 @@ Field *create_tmp_field(Session *session,
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ITEM_H */
