@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,25 +17,29 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ITEM_CACHE_DECIMAL_H
+#define DRIZZLED_ITEM_CACHE_DECIMAL_H
 
-#include <drizzled/type/decimal.h>
 #include <drizzled/item/cache.h>
 
-namespace drizzled {
-
-class Item_cache_decimal : public Item_cache
+namespace drizzled
 {
+
+class Item_cache_decimal: public Item_cache
+{
+protected:
+  my_decimal decimal_value;
 public:
-  void store(Item*);
+  Item_cache_decimal(): Item_cache() {}
+
+  void store(Item *item);
   double val_real();
   int64_t val_int();
-  String* val_str(String*);
-  type::Decimal *val_decimal(type::Decimal*);
+  String* val_str(String *str);
+  my_decimal *val_decimal(my_decimal *);
   enum Item_result result_type() const { return DECIMAL_RESULT; }
-protected:
-  type::Decimal decimal_value;
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ITEM_CACHE_DECIMAL_H */
