@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,11 +17,12 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <cmath>
+#include "config.h"
+#include <math.h>
 #include <drizzled/item/cache_real.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
 void Item_cache_real::store(Item *item)
 {
@@ -29,12 +30,14 @@ void Item_cache_real::store(Item *item)
   null_value= item->null_value;
 }
 
+
 int64_t Item_cache_real::val_int()
 {
   assert(fixed == 1);
   return (int64_t) rint(value);
 }
 
+
 String* Item_cache_real::val_str(String *str)
 {
   assert(fixed == 1);
@@ -42,10 +45,11 @@ String* Item_cache_real::val_str(String *str)
   return str;
 }
 
-type::Decimal *Item_cache_real::val_decimal(type::Decimal *decimal_val)
+
+my_decimal *Item_cache_real::val_decimal(my_decimal *decimal_val)
 {
   assert(fixed == 1);
-  double2_class_decimal(E_DEC_FATAL_ERROR, value, decimal_val);
+  double2my_decimal(E_DEC_FATAL_ERROR, value, decimal_val);
   return decimal_val;
 }
 
