@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <drizzled/session.h>
 #include <drizzled/table.h>
@@ -29,12 +29,9 @@
 #include <drizzled/item/field.h>
 #include <drizzled/item/outer_ref.h>
 #include <drizzled/plugin/client.h>
-#include <drizzled/item/subselect.h>
-#include <drizzled/sql_lex.h>
 
-#include <boost/dynamic_bitset.hpp>
-
-namespace drizzled {
+namespace drizzled
+{
 
 /**
   Store the pointer to this item field into a list if not already there.
@@ -57,14 +54,15 @@ namespace drizzled {
 bool Item_field::collect_item_field_processor(unsigned char *arg)
 {
   List<Item_field> *item_list= (List<Item_field>*) arg;
-  List<Item_field>::iterator item_list_it(item_list->begin());
-  while (Item_field* curr_item= item_list_it++)
+  List_iterator<Item_field> item_list_it(*item_list);
+  Item_field *curr_item;
+  while ((curr_item= item_list_it++))
   {
     if (curr_item->eq(this, 1))
-      return false; /* Already in the set. */
+      return(false); /* Already in the set. */
   }
   item_list->push_back(this);
-  return false;
+  return(false);
 }
 
 
@@ -86,10 +84,11 @@ bool Item_field::collect_item_field_processor(unsigned char *arg)
 
 bool Item_field::find_item_in_field_list_processor(unsigned char *arg)
 {
-  KeyPartInfo *first_non_group_part= *((KeyPartInfo **) arg);
-  KeyPartInfo *last_part= *(((KeyPartInfo **) arg) + 1);
+  KEY_PART_INFO *first_non_group_part= *((KEY_PART_INFO **) arg);
+  KEY_PART_INFO *last_part= *(((KEY_PART_INFO **) arg) + 1);
+  KEY_PART_INFO *cur_part;
 
-  for (KeyPartInfo* cur_part= first_non_group_part; cur_part != last_part; cur_part++)
+  for (cur_part= first_non_group_part; cur_part != last_part; cur_part++)
   {
     if (field->eq(cur_part->field))
       return true;
@@ -109,15 +108,15 @@ bool Item_field::find_item_in_field_list_processor(unsigned char *arg)
 bool Item_field::register_field_in_read_map(unsigned char *arg)
 {
   Table *table= (Table *) arg;
-  if (field->getTable() == table || !table)
-    field->getTable()->setReadSet(field->position());
+  if (field->table == table || !table)
+    field->table->setReadSet(field->field_index);
 
   return 0;
 }
 
 
 Item_field::Item_field(Field *f)
-  :Item_ident(0, NULL, f->getTable()->getAlias(), f->field_name),
+  :Item_ident(0, NULL, *f->table_name, f->field_name),
    item_equal(0), no_const_subst(0),
    have_privileges(0), any_privileges(0)
 {
@@ -141,8 +140,8 @@ Item_field::Item_field(Session *,
                        Name_resolution_context *context_arg,
                        Field *f) :
   Item_ident(context_arg,
-             f->getTable()->getShare()->getSchemaName(),
-             f->getTable()->getAlias(),
+             f->table->s->getSchemaName(),
+             *f->table_name,
              f->field_name),
    item_equal(0),
    no_const_subst(0),
@@ -164,7 +163,7 @@ Item_field::Item_field(Name_resolution_context *context_arg,
    have_privileges(0),
    any_privileges(0)
 {
-  Select_Lex *select= getSession().lex().current_select;
+  Select_Lex *select= current_session->lex->current_select;
   collation.set(DERIVATION_IMPLICIT);
 
   if (select && select->parsing_place != IN_HAVING)
@@ -193,13 +192,16 @@ void Item_field::set_field(Field *field_par)
   maybe_null=field->maybe_null();
   decimals= field->decimals();
   max_length= field_par->max_display_length();
-  table_name= field_par->getTable()->getAlias();
+  table_name= *field_par->table_name;
   field_name= field_par->field_name;
-  db_name= field_par->getTable()->getShare()->getSchemaName();
-  alias_name_used= field_par->getTable()->alias_name_used;
+  db_name= field_par->table->s->getSchemaName();
+  alias_name_used= field_par->table->alias_name_used;
   unsigned_flag=test(field_par->flags & UNSIGNED_FLAG);
   collation.set(field_par->charset(), field_par->derivation());
   fixed= 1;
+
+  if (field->table->s->tmp_table == SYSTEM_TMP_TABLE)
+    any_privileges= 0;
 }
 
 
@@ -213,7 +215,7 @@ void Item_field::reset_field(Field *f)
 {
   set_field(f);
   /* 'name' is pointing at field->field_name of old field */
-  name= f->field_name;
+  name= (char*) f->field_name;
 }
 
 /* ARGSUSED */
@@ -245,7 +247,7 @@ int64_t Item_field::val_int()
 }
 
 
-type::Decimal *Item_field::val_decimal(type::Decimal *decimal_value)
+my_decimal *Item_field::val_decimal(my_decimal *decimal_value)
 {
   if ((null_value= field->is_null()))
     return 0;
@@ -261,32 +263,32 @@ String *Item_field::str_result(String *str)
   return result_field->val_str(str,&str_value);
 }
 
-bool Item_field::get_date(type::Time &ltime,uint32_t fuzzydate)
+bool Item_field::get_date(DRIZZLE_TIME *ltime,uint32_t fuzzydate)
 {
   if ((null_value=field->is_null()) || field->get_date(ltime,fuzzydate))
   {
-    ltime.reset();
+    memset(ltime, 0, sizeof(*ltime));
     return 1;
   }
   return 0;
 }
 
-bool Item_field::get_date_result(type::Time &ltime,uint32_t fuzzydate)
+bool Item_field::get_date_result(DRIZZLE_TIME *ltime,uint32_t fuzzydate)
 {
   if ((null_value=result_field->is_null()) ||
       result_field->get_date(ltime,fuzzydate))
   {
-    ltime.reset();
+    memset(ltime, 0, sizeof(*ltime));
     return 1;
   }
   return 0;
 }
 
-bool Item_field::get_time(type::Time &ltime)
+bool Item_field::get_time(DRIZZLE_TIME *ltime)
 {
   if ((null_value=field->is_null()) || field->get_time(ltime))
   {
-    ltime.reset();
+    memset(ltime, 0, sizeof(*ltime));
     return 1;
   }
   return 0;
@@ -307,7 +309,7 @@ int64_t Item_field::val_int_result()
 }
 
 
-type::Decimal *Item_field::val_decimal_result(type::Decimal *decimal_value)
+my_decimal *Item_field::val_decimal_result(my_decimal *decimal_value)
 {
   if ((null_value= result_field->is_null()))
     return 0;
@@ -318,35 +320,26 @@ type::Decimal *Item_field::val_decimal_result(type::Decimal *decimal_value)
 bool Item_field::val_bool_result()
 {
   if ((null_value= result_field->is_null()))
-  {
     return false;
-  }
-
-  switch (result_field->result_type()) 
-  {
+  switch (result_field->result_type()) {
   case INT_RESULT:
     return result_field->val_int() != 0;
-
   case DECIMAL_RESULT:
-    {
-      type::Decimal decimal_value;
-      type::Decimal *val= result_field->val_decimal(&decimal_value);
-      if (val)
-        return not val->isZero();
-      return 0;
-    }
-
+  {
+    my_decimal decimal_value;
+    my_decimal *val= result_field->val_decimal(&decimal_value);
+    if (val)
+      return !my_decimal_is_zero(val);
+    return 0;
+  }
   case REAL_RESULT:
   case STRING_RESULT:
     return result_field->val_real() != 0.0;
-
   case ROW_RESULT:
-    assert(false);
-    return 0;
+  default:
+    assert(0);
+    return 0;                                   // Shut up compiler
   }
-
-  assert(false);
-  return 0;
 }
 
 
@@ -369,22 +362,22 @@ bool Item_field::eq(const Item *item, bool) const
     (In cases where we would choose wrong we would have to generate a
     ER_NON_UNIQ_ERROR).
   */
-  return (not system_charset_info->strcasecmp(item_field->name, field_name) &&
-          (not item_field->table_name || not table_name ||
-           (not table_alias_charset->strcasecmp(item_field->table_name, table_name) &&
-            (not item_field->db_name || not db_name ||
-             (item_field->db_name && not system_charset_info->strcasecmp(item_field->db_name, db_name))))));
+  return (!my_strcasecmp(system_charset_info, item_field->name,
+			 field_name) &&
+	  (!item_field->table_name || !table_name ||
+	   (!my_strcasecmp(table_alias_charset, item_field->table_name,
+			   table_name) &&
+	    (!item_field->db_name || !db_name ||
+	     (item_field->db_name && !strcmp(item_field->db_name,
+					     db_name))))));
 }
 
 
 table_map Item_field::used_tables() const
 {
-  if (field->getTable()->const_table)
-  {
+  if (field->table->const_table)
     return 0;					// const item
-  }
-
-  return depended_from ? OUTER_REF_TABLE_BIT : field->getTable()->map;
+  return (depended_from ? OUTER_REF_TABLE_BIT : field->table->map);
 }
 
 enum Item_result Item_field::result_type () const
@@ -427,7 +420,8 @@ bool Item_field::is_null()
 Item *Item_field::get_tmp_table_item(Session *session)
 {
   Item_field *new_item= new Item_field(session, this);
-  new_item->field= new_item->result_field;
+  if (new_item)
+    new_item->field= new_item->result_field;
   return new_item;
 }
 
@@ -495,7 +489,7 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
   */
   Name_resolution_context *last_checked_context= context;
   Item **ref= (Item **) not_found_item;
-  Select_Lex *current_sel= (Select_Lex *) session->lex().current_select;
+  Select_Lex *current_sel= (Select_Lex *) session->lex->current_select;
   Name_resolution_context *outer_context= 0;
   Select_Lex *select= 0;
   /* Currently derived tables cannot be correlated */
@@ -539,13 +533,14 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
       {
         if (*from_field != view_ref_found)
         {
-          prev_subselect_item->used_tables_cache|= (*from_field)->getTable()->map;
-          prev_subselect_item->const_item_cache= false;
+          prev_subselect_item->used_tables_cache|= (*from_field)->table->map;
+          prev_subselect_item->const_item_cache= 0;
           set_field(*from_field);
           if (!last_checked_context->select_lex->having_fix_field &&
               select->group_list.elements &&
               (place == SELECT_LIST || place == IN_HAVING))
           {
+            Item_outer_ref *rf;
             /*
               If an outer field is resolved in a grouping select then it
               is replaced for an Item_outer_ref object. Otherwise an
@@ -555,21 +550,23 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
               after the original field has been fixed and this is done in the
               fix_inner_refs() function.
             */
-            Item_outer_ref* rf= new Item_outer_ref(context, this);
-            *reference= rf;
+            ;
+            if (!(rf= new Item_outer_ref(context, this)))
+              return -1;
+            session->change_item_tree(reference, rf);
             select->inner_refs_list.push_back(rf);
-            rf->in_sum_func= session->lex().in_sum_func;
+            rf->in_sum_func= session->lex->in_sum_func;
           }
           /*
             A reference is resolved to a nest level that's outer or the same as
             the nest level of the enclosing set function : adjust the value of
             max_arg_level for the function if it's needed.
           */
-          if (session->lex().in_sum_func &&
-              session->lex().in_sum_func->nest_level >= select->nest_level)
+          if (session->lex->in_sum_func &&
+              session->lex->in_sum_func->nest_level >= select->nest_level)
           {
             Item::Type ref_type= (*reference)->type();
-            set_if_bigger(session->lex().in_sum_func->max_arg_level,
+            set_if_bigger(session->lex->in_sum_func->max_arg_level,
                           select->nest_level);
             set_field(*from_field);
             fixed= 1;
@@ -584,10 +581,15 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
         else
         {
           Item::Type ref_type= (*reference)->type();
-          prev_subselect_item->used_tables_cache|= (*reference)->used_tables();
-          prev_subselect_item->const_item_cache&= (*reference)->const_item();
+          prev_subselect_item->used_tables_cache|=
+            (*reference)->used_tables();
+          prev_subselect_item->const_item_cache&=
+            (*reference)->const_item();
           mark_as_dependent(session, last_checked_context->select_lex,
-            context->select_lex, this, ((ref_type == REF_ITEM || ref_type == FIELD_ITEM) ? (Item_ident*) (*reference) : 0));
+                            context->select_lex, this,
+                            ((ref_type == REF_ITEM || ref_type == FIELD_ITEM) ?
+                             (Item_ident*) (*reference) :
+                             0));
           /*
             A reference to a view field had been found and we
             substituted it instead of this Item (find_field_in_tables
@@ -620,7 +622,7 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
       case it does not matter which used tables bits we set)
     */
     prev_subselect_item->used_tables_cache|= OUTER_REF_TABLE_BIT;
-    prev_subselect_item->const_item_cache= false;
+    prev_subselect_item->const_item_cache= 0;
   }
 
   assert(ref != 0);
@@ -631,7 +633,7 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
     if (upward_lookup)
     {
       // We can't say exactly what absent table or field
-      my_error(ER_BAD_FIELD_ERROR, MYF(0), full_name(), session->where());
+      my_error(ER_BAD_FIELD_ERROR, MYF(0), full_name(), session->where);
     }
     else
     {
@@ -645,6 +647,9 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
   }
   else if (ref != not_found_item)
   {
+    Item *save;
+    Item_ref *rf;
+
     /* Should have been checked in resolve_ref_in_select_and_group(). */
     assert(*ref && (*ref)->fixed);
     /*
@@ -653,13 +658,16 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
       constructor, so no initialization is performed, and call
       fix_fields() below.
     */
-    Item* save= *ref;
+    save= *ref;
     *ref= NULL;                             // Don't call set_properties()
-    Item_ref* rf= (place == IN_HAVING ?
-         new Item_ref(context, ref, table_name, field_name, alias_name_used) :
+    rf= (place == IN_HAVING ?
+         new Item_ref(context, ref, (char*) table_name,
+                      (char*) field_name, alias_name_used) :
          (!select->group_list.elements ?
-         new Item_direct_ref(context, ref, table_name, field_name, alias_name_used) :
-         new Item_outer_ref(context, ref, table_name, field_name, alias_name_used)));
+         new Item_direct_ref(context, ref, (char*) table_name,
+                             (char*) field_name, alias_name_used) :
+         new Item_outer_ref(context, ref, (char*) table_name,
+                            (char*) field_name, alias_name_used)));
     *ref= save;
     if (!rf)
       return -1;
@@ -667,9 +675,9 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
     if (place != IN_HAVING && select->group_list.elements)
     {
       outer_context->select_lex->inner_refs_list.push_back((Item_outer_ref*)rf);
-      ((Item_outer_ref*)rf)->in_sum_func= session->lex().in_sum_func;
+      ((Item_outer_ref*)rf)->in_sum_func= session->lex->in_sum_func;
     }
-    *reference= rf;
+    session->change_item_tree(reference, rf);
     /*
       rf is Item_ref => never substitute other items (in this case)
       during fix_fields() => we can use rf after fix_fields()
@@ -678,16 +686,25 @@ Item_field::fix_outer_field(Session *session, Field **from_field, Item **referen
     if (rf->fix_fields(session, reference) || rf->check_cols(1))
       return -1;
 
-    mark_as_dependent(session, last_checked_context->select_lex, context->select_lex, this, rf);
+    mark_as_dependent(session, last_checked_context->select_lex,
+                      context->select_lex, this,
+                      rf);
     return 0;
   }
   else
   {
-    mark_as_dependent(session, last_checked_context->select_lex, context->select_lex, this, (Item_ident*)*reference);
+    mark_as_dependent(session, last_checked_context->select_lex,
+                      context->select_lex,
+                      this, (Item_ident*)*reference);
     if (last_checked_context->select_lex->having_fix_field)
     {
-      Item_ref* rf= new Item_ref(context, (cached_table->getSchemaName()[0] ? cached_table->getSchemaName() : 0), cached_table->alias, field_name);
-      *reference= rf;
+      Item_ref *rf;
+      rf= new Item_ref(context,
+                       (cached_table->db[0] ? cached_table->db : 0),
+                       (char*) cached_table->alias, (char*) field_name);
+      if (!rf)
+        return -1;
+      session->change_item_tree(reference, rf);
       /*
         rf is Item_ref => never substitute other items (in this case)
         during fix_fields() => we can use rf after fix_fields()
@@ -764,19 +781,18 @@ bool Item_field::fix_fields(Session *session, Item **reference)
                                           context->first_name_resolution_table,
                                           context->last_name_resolution_table,
                                           reference,
-                                          session->lex().use_only_table_context ?
+                                          session->lex->use_only_table_context ?
                                             REPORT_ALL_ERRORS :
                                             IGNORE_EXCEPT_NON_UNIQUE, true)) ==
         not_found_field)
     {
       int ret;
       /* Look up in current select's item_list to find aliased fields */
-      if (session->lex().current_select->is_item_list_lookup)
+      if (session->lex->current_select->is_item_list_lookup)
       {
         uint32_t counter;
         enum_resolution_type resolution;
-        Item** res= find_item_in_list(session,
-                                      this, session->lex().current_select->item_list,
+        Item** res= find_item_in_list(this, session->lex->current_select->item_list,
                                       &counter, REPORT_EXCEPT_NOT_FOUND,
                                       &resolution);
         if (!res)
@@ -800,8 +816,8 @@ bool Item_field::fix_fields(Session *session, Item **reference)
             {
               /* The column to which we link isn't valid. */
               my_error(ER_BAD_FIELD_ERROR, MYF(0), (*res)->name,
-                       session->where());
-              return 1;
+                       current_session->where);
+              return(1);
             }
 
             set_field(new_field);
@@ -815,7 +831,9 @@ bool Item_field::fix_fields(Session *session, Item **reference)
               Item_field created by the parser with the new Item_ref.
             */
             Item_ref *rf= new Item_ref(context, db_name,table_name,field_name);
-            *reference= rf;
+            if (!rf)
+              return 1;
+            session->change_item_tree(reference, rf);
             /*
               Because Item_ref never substitutes itself with other items
               in Item_ref::fix_fields(), we can safely use the original
@@ -826,13 +844,13 @@ bool Item_field::fix_fields(Session *session, Item **reference)
         }
       }
       if ((ret= fix_outer_field(session, &from_field, reference)) < 0)
-        return true;
+        goto error;
       outer_fixed= true;
       if (!ret)
         goto mark_non_agg_field;
     }
     else if (!from_field)
-      return true;
+      goto error;
 
     if (!outer_fixed && cached_table && cached_table->select_lex &&
         context->select_lex &&
@@ -840,7 +858,7 @@ bool Item_field::fix_fields(Session *session, Item **reference)
     {
       int ret;
       if ((ret= fix_outer_field(session, &from_field, reference)) < 0)
-        return true;
+        goto error;
       outer_fixed= 1;
       if (!ret)
         goto mark_non_agg_field;
@@ -862,18 +880,18 @@ bool Item_field::fix_fields(Session *session, Item **reference)
       return false;
 
     set_field(from_field);
-    if (session->lex().in_sum_func &&
-        session->lex().in_sum_func->nest_level ==
-        session->lex().current_select->nest_level)
+    if (session->lex->in_sum_func &&
+        session->lex->in_sum_func->nest_level ==
+        session->lex->current_select->nest_level)
     {
-      set_if_bigger(session->lex().in_sum_func->max_arg_level,
-                    session->lex().current_select->nest_level);
+      set_if_bigger(session->lex->in_sum_func->max_arg_level,
+                    session->lex->current_select->nest_level);
     }
   }
   else if (session->mark_used_columns != MARK_COLUMNS_NONE)
   {
-    Table *table= field->getTable();
-    boost::dynamic_bitset<> *current_bitmap, *other_bitmap;
+    Table *table= field->table;
+    MyBitmap *current_bitmap, *other_bitmap;
     if (session->mark_used_columns == MARK_COLUMNS_READ)
     {
       current_bitmap= table->read_set;
@@ -884,10 +902,9 @@ bool Item_field::fix_fields(Session *session, Item **reference)
       current_bitmap= table->write_set;
       other_bitmap=   table->read_set;
     }
-    //if (! current_bitmap->testAndSet(field->position()))
-    if (! current_bitmap->test(field->position()))
+    if (! current_bitmap->testAndSet(field->field_index))
     {
-      if (! other_bitmap->test(field->position()))
+      if (! other_bitmap->isBitSet(field->field_index))
       {
         /* First usage of column */
         table->used_fields++;                     // Used to optimize loops
@@ -898,9 +915,13 @@ bool Item_field::fix_fields(Session *session, Item **reference)
   fixed= 1;
 mark_non_agg_field:
   return false;
+
+error:
+  context->process_error(session);
+  return true;
 }
 
-Item *Item_field::safe_charset_converter(const charset_info_st * const tocs)
+Item *Item_field::safe_charset_converter(const CHARSET_INFO * const tocs)
 {
   no_const_subst= 1;
   return Item::safe_charset_converter(tocs);
@@ -917,6 +938,7 @@ void Item_field::cleanup()
    */
   field= result_field= 0;
   null_value= false;
+  return;
 }
 
 
@@ -946,10 +968,11 @@ bool Item_field::result_as_int64_t()
 
 Item_equal *Item_field::find_item_equal(COND_EQUAL *cond_equal)
 {
+  Item_equal *item= 0;
   while (cond_equal)
   {
-    List<Item_equal>::iterator li(cond_equal->current_level.begin());
-    while (Item_equal* item= li++)
+    List_iterator_fast<Item_equal> li(cond_equal->current_level);
+    while ((item= li++))
     {
       if (item->contains(field))
         return item;
@@ -1163,13 +1186,13 @@ int Item_field::save_in_field(Field *to, bool no_conversions)
     res= field_conv(to,result_field);
     null_value=0;
   }
-  return res;
+  return(res);
 }
 
 
-void Item_field::send(plugin::Client *client, String *)
+bool Item_field::send(plugin::Client *client, String *)
 {
-  client->store(result_field);
+  return client->store(result_field);
 }
 
 
@@ -1179,8 +1202,10 @@ void Item_field::update_null_value()
     need to set no_errors to prevent warnings about type conversion
     popping up.
   */
-  Session *session= field->getTable()->in_use;
-  int no_errors= session->no_errors;
+  Session *session= field->table->in_use;
+  int no_errors;
+
+  no_errors= session->no_errors;
   session->no_errors= 1;
   Item::update_null_value();
   session->no_errors= no_errors;
@@ -1214,38 +1239,36 @@ Item *Item_field::update_value_transformer(unsigned char *select_arg)
   Select_Lex *select= (Select_Lex*)select_arg;
   assert(fixed);
 
-  if (field->getTable() != select->context.table_list->table)
+  if (field->table != select->context.table_list->table)
   {
     List<Item> *all_fields= &select->join->all_fields;
     Item **ref_pointer_array= select->ref_pointer_array;
-    int el= all_fields->size();
+    int el= all_fields->elements;
+    Item_ref *ref;
+
     ref_pointer_array[el]= (Item*)this;
     all_fields->push_front((Item*)this);
-    Item_ref* ref= new Item_ref(&select->context, ref_pointer_array + el, table_name, field_name);
+    ref= new Item_ref(&select->context, ref_pointer_array + el,
+                      table_name, field_name);
     return ref;
   }
   return this;
 }
 
 
-void Item_field::print(String *str)
+void Item_field::print(String *str, enum_query_type query_type)
 {
-  if (field && field->getTable()->const_table)
+  if (field && field->table->const_table)
   {
     char buff[MAX_FIELD_WIDTH];
     String tmp(buff,sizeof(buff),str->charset());
-    field->val_str_internal(&tmp);
-    if (field->is_null())  {
-      str->append(STRING_WITH_LEN("NULL"));
-    }
-    else {
-      str->append('\'');
-      str->append(tmp);
-      str->append('\'');
-    }
+    field->val_str(&tmp);
+    str->append('\'');
+    str->append(tmp);
+    str->append('\'');
     return;
   }
-  Item_ident::print(str);
+  Item_ident::print(str, query_type);
 }
 
 
