@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,18 +17,17 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/session.h>
-#include <drizzled/error.h>
-#include <drizzled/show.h>
-#include <drizzled/item/ref.h>
-#include <drizzled/plugin/client.h>
-#include <drizzled/item/sum.h>
-#include <drizzled/item/subselect.h>
-#include <drizzled/sql_lex.h>
+#include "drizzled/session.h"
+#include "drizzled/error.h"
+#include "drizzled/show.h"
+#include "drizzled/item/ref.h"
+#include "drizzled/plugin/client.h"
+#include "drizzled/item/sum.h"
 
-namespace drizzled {
+namespace drizzled
+{
 
 Item_ref::Item_ref(Name_resolution_context *context_arg,
                    Item **item, const char *table_name_arg,
@@ -114,12 +113,13 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
 {
   enum_parsing_place place= NO_MATTER;
   assert(fixed == 0);
-  Select_Lex *current_sel= session->lex().current_select;
+  Select_Lex *current_sel= session->lex->current_select;
 
   if (!ref || ref == not_found_item)
   {
-    if (!(ref= resolve_ref_in_select_and_group(session, this, context->select_lex)))
-      return true;             /* Some error occurred (e.g. ambiguous names). */
+    if (!(ref= resolve_ref_in_select_and_group(session, this,
+                                               context->select_lex)))
+      goto error;             /* Some error occurred (e.g. ambiguous names). */
 
     if (ref == not_found_item) /* This reference was not resolved. */
     {
@@ -131,8 +131,9 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
       if (!outer_context)
       {
         /* The current reference cannot be resolved in this query. */
-        my_error(ER_BAD_FIELD_ERROR,MYF(0), full_name(), session->where());
-        return true;
+        my_error(ER_BAD_FIELD_ERROR,MYF(0),
+                 this->full_name(), current_session->where);
+        goto error;
       }
 
       /*
@@ -157,7 +158,7 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
         if (outer_context->resolve_in_select_list)
         {
           if (!(ref= resolve_ref_in_select_and_group(session, this, select)))
-            return true; /* Some error occurred (e.g. ambiguous names). */
+            goto error; /* Some error occurred (e.g. ambiguous names). */
           if (ref != not_found_item)
           {
             assert(*ref && (*ref)->fixed);
@@ -183,8 +184,6 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
           condition, so that we can give a better error message -
           ER_WRONG_FIELD_WITH_GROUP, instead of the less informative
           ER_BAD_FIELD_ERROR which we produce now.
-
-          @todo determine if this is valid.
         */
         if ((place != IN_HAVING ||
              (!select->with_sum_func &&
@@ -203,7 +202,7 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
                                            reference,
                                            IGNORE_EXCEPT_NON_UNIQUE, true);
           if (! from_field)
-            return true;
+            goto error;
           if (from_field == view_ref_found)
           {
             Item::Type refer_type= (*reference)->type();
@@ -245,8 +244,8 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
               } while (outer_context && outer_context->select_lex &&
                        cached_table->select_lex != outer_context->select_lex);
             }
-            prev_subselect_item->used_tables_cache|= from_field->getTable()->map;
-            prev_subselect_item->const_item_cache= false;
+            prev_subselect_item->used_tables_cache|= from_field->table->map;
+            prev_subselect_item->const_item_cache= 0;
             break;
           }
         }
@@ -254,7 +253,7 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
 
         /* Reference is not found => depend on outer (or just error). */
         prev_subselect_item->used_tables_cache|= OUTER_REF_TABLE_BIT;
-        prev_subselect_item->const_item_cache= false;
+        prev_subselect_item->const_item_cache= 0;
 
         outer_context= outer_context->outer_context;
       } while (outer_context);
@@ -263,27 +262,29 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
       if (from_field != not_found_field)
       {
         Item_field* fld;
-        fld= new Item_field(from_field);
-        *reference= fld;
+        if (!(fld= new Item_field(from_field)))
+          goto error;
+        session->change_item_tree(reference, fld);
         mark_as_dependent(session, last_checked_context->select_lex,
-                          session->lex().current_select, this, fld);
+                          session->lex->current_select, this, fld);
         /*
           A reference is resolved to a nest level that's outer or the same as
           the nest level of the enclosing set function : adjust the value of
           max_arg_level for the function if it's needed.
         */
-        if (session->lex().in_sum_func &&
-            session->lex().in_sum_func->nest_level >=
+        if (session->lex->in_sum_func &&
+            session->lex->in_sum_func->nest_level >=
             last_checked_context->select_lex->nest_level)
-          set_if_bigger(session->lex().in_sum_func->max_arg_level,
+          set_if_bigger(session->lex->in_sum_func->max_arg_level,
                         last_checked_context->select_lex->nest_level);
         return false;
       }
       if (ref == 0)
       {
         /* The item was not a table field and not a reference */
-        my_error(ER_BAD_FIELD_ERROR, MYF(0), full_name(), session->where());
-        return true;
+        my_error(ER_BAD_FIELD_ERROR, MYF(0),
+                 this->full_name(), current_session->where);
+        goto error;
       }
       /* Should be checked in resolve_ref_in_select_and_group(). */
       assert(*ref && (*ref)->fixed);
@@ -294,10 +295,10 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
         the nest level of the enclosing set function : adjust the value of
         max_arg_level for the function if it's needed.
       */
-      if (session->lex().in_sum_func &&
-          session->lex().in_sum_func->nest_level >=
+      if (session->lex->in_sum_func &&
+          session->lex->in_sum_func->nest_level >=
           last_checked_context->select_lex->nest_level)
-        set_if_bigger(session->lex().in_sum_func->max_arg_level,
+        set_if_bigger(session->lex->in_sum_func->max_arg_level,
                       last_checked_context->select_lex->nest_level);
     }
   }
@@ -316,15 +317,22 @@ bool Item_ref::fix_fields(Session *session, Item **reference)
           current_sel->having_fix_field)) ||
        !(*ref)->fixed))
   {
-    my_error(ER_ILLEGAL_REFERENCE, MYF(0), name, ((*ref)->with_sum_func ? "reference to group function" : "forward reference in item list"));
-    return true;
+    my_error(ER_ILLEGAL_REFERENCE, MYF(0),
+             name, ((*ref)->with_sum_func?
+                    "reference to group function":
+                    "forward reference in item list"));
+    goto error;
   }
 
   set_properties();
 
   if ((*ref)->check_cols(1))
-    return true;
+    goto error;
   return false;
+
+error:
+  context->process_error(session);
+  return true;
 }
 
 
@@ -358,30 +366,27 @@ void Item_ref::cleanup()
 }
 
 
-void Item_ref::print(String *str)
+void Item_ref::print(String *str, enum_query_type query_type)
 {
   if (ref)
   {
     if ((*ref)->type() != Item::CACHE_ITEM &&
         !table_name && name && alias_name_used)
     {
-      str->append_identifier(str_ref(name));
+      str->append_identifier(name, (uint32_t) strlen(name));
     }
     else
-      (*ref)->print(str);
+      (*ref)->print(str, query_type);
   }
   else
-    Item_ident::print(str);
+    Item_ident::print(str, query_type);
 }
 
 
-void Item_ref::send(plugin::Client *client, String *tmp)
+bool Item_ref::send(plugin::Client *client, String *tmp)
 {
   if (result_field)
-  {
-    client->store(result_field);
-    return;
-  }
+    return client->store(result_field);
   return (*ref)->send(client, tmp);
 }
 
@@ -423,7 +428,7 @@ String *Item_ref::str_result(String* str)
 }
 
 
-type::Decimal *Item_ref::val_decimal_result(type::Decimal *decimal_value)
+my_decimal *Item_ref::val_decimal_result(my_decimal *decimal_value)
 {
   if (result_field)
   {
@@ -444,25 +449,22 @@ bool Item_ref::val_bool_result()
     switch (result_field->result_type()) {
     case INT_RESULT:
       return result_field->val_int() != 0;
-
     case DECIMAL_RESULT:
-      {
-        type::Decimal decimal_value;
-        type::Decimal *val= result_field->val_decimal(&decimal_value);
-        if (val)
-          return not val->isZero();
-        return 0;
-      }
-
+    {
+      my_decimal decimal_value;
+      my_decimal *val= result_field->val_decimal(&decimal_value);
+      if (val)
+        return !my_decimal_is_zero(val);
+      return 0;
+    }
     case REAL_RESULT:
     case STRING_RESULT:
       return result_field->val_real() != 0.0;
-
     case ROW_RESULT:
+    default:
       assert(0);
     }
   }
-
   return val_bool();
 }
 
@@ -510,15 +512,15 @@ bool Item_ref::is_null()
 }
 
 
-bool Item_ref::get_date(type::Time &ltime,uint32_t fuzzydate)
+bool Item_ref::get_date(DRIZZLE_TIME *ltime,uint32_t fuzzydate)
 {
   return (null_value=(*ref)->get_date_result(ltime,fuzzydate));
 }
 
 
-type::Decimal *Item_ref::val_decimal(type::Decimal *decimal_value)
+my_decimal *Item_ref::val_decimal(my_decimal *decimal_value)
 {
-  type::Decimal *val= (*ref)->val_decimal_result(decimal_value);
+  my_decimal *val= (*ref)->val_decimal_result(decimal_value);
   null_value= (*ref)->null_value;
   return val;
 }
