@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,12 +17,14 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ITEM_RETURN_DATE_TIME_H
+#define DRIZZLED_ITEM_RETURN_DATE_TIME_H
 
-namespace drizzled {
+namespace drizzled
+{
 
 /* for show tables */
-class Item_return_date_time : public Item_string
+class Item_return_date_time :public Item_string
 {
   enum_field_types date_time_field_type;
 public:
@@ -35,3 +37,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ITEM_RETURN_DATE_TIME_H */
