@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,17 +17,36 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ITEM_SUBSELECT_H
+#define DRIZZLED_ITEM_SUBSELECT_H
 
-#include <drizzled/comp_creator.h>
-#include <drizzled/item/ref.h>
-#include <drizzled/item/field.h>
-#include <drizzled/item/bin_string.h>
-#include <drizzled/util/test.h>
+/* subselect Item */
 
-namespace drizzled {
 
-class Item_subselect : public Item_result_field
+#include "drizzled/comp_creator.h"
+#include "drizzled/item/ref.h"
+#include "drizzled/item/field.h"
+#include "drizzled/item/bin_string.h"
+
+namespace drizzled
+{
+
+class Select_Lex;
+class Select_Lex_Unit;
+class JOIN;
+class select_result_interceptor;
+class subselect_engine;
+class subselect_hash_sj_engine;
+class Item_bool_func2;
+class Cached_item;
+class Item_in_optimizer;
+class Item_func_not_all;
+class Tmp_Table_Param;
+
+
+/* base class for subselects */
+
+class Item_subselect :public Item_result_field
 {
   bool value_assigned; /* value already assigned to subselect */
 public:
@@ -84,7 +103,7 @@ public:
   {
     null_value= 1;
   }
-  virtual trans_res select_transformer(Join *join);
+  virtual trans_res select_transformer(JOIN *join);
   bool assigned() { return value_assigned; }
   void assigned(bool a) { value_assigned= a; }
   enum Type type() const;
@@ -103,7 +122,7 @@ public:
   inline bool get_const_item_cache() { return const_item_cache; }
   Item *get_tmp_table_item(Session *session);
   void update_used_tables();
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   virtual bool have_guarded_conds() { return false; }
   bool change_engine(subselect_engine *eng)
   {
@@ -145,6 +164,7 @@ public:
 
 /* single value subselect */
 
+class Item_cache;
 class Item_singlerow_subselect :public Item_subselect
 {
 protected:
@@ -157,12 +177,12 @@ public:
   subs_type substype() { return SINGLEROW_SUBS; }
 
   void reset();
-  trans_res select_transformer(Join *join);
+  trans_res select_transformer(JOIN *join);
   void store(uint32_t i, Item* item);
   double val_real();
   int64_t val_int ();
   String *val_str (String *);
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   bool val_bool();
   enum Item_result result_type() const;
   enum_field_types field_type() const;
@@ -193,6 +213,7 @@ public:
 };
 
 /* used in static ALL/ANY optimization */
+class select_max_min_finder_subselect;
 class Item_maxmin_subselect :public Item_singlerow_subselect
 {
 protected:
@@ -201,7 +222,7 @@ protected:
 public:
   Item_maxmin_subselect(Session *session, Item_subselect *parent,
 			Select_Lex *select_lex, bool max);
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   void cleanup();
   bool any_value() { return was_values; }
   void register_value() { was_values= true; }
@@ -229,10 +250,10 @@ public:
   int64_t val_int();
   double val_real();
   String *val_str(String*);
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   bool val_bool();
   void fix_length_and_dec();
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
 
   friend class select_exists_subselect;
   friend class subselect_uniquesubquery_engine;
@@ -335,24 +356,24 @@ public:
     null_value= 0;
     was_null= 0;
   }
-  trans_res select_transformer(Join *join);
-  trans_res select_in_like_transformer(Join *join, const Comp_creator *func);
-  trans_res single_value_transformer(Join *join, const Comp_creator *func);
-  trans_res row_value_transformer(Join * join);
-  trans_res single_value_in_to_exists_transformer(Join * join,
+  trans_res select_transformer(JOIN *join);
+  trans_res select_in_like_transformer(JOIN *join, const Comp_creator *func);
+  trans_res single_value_transformer(JOIN *join, const Comp_creator *func);
+  trans_res row_value_transformer(JOIN * join);
+  trans_res single_value_in_to_exists_transformer(JOIN * join,
                                                   const Comp_creator *func);
-  trans_res row_value_in_to_exists_transformer(Join * join);
+  trans_res row_value_in_to_exists_transformer(JOIN * join);
   virtual bool exec();
   int64_t val_int();
   double val_real();
   String *val_str(String*);
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   void update_null_value () { (void) val_bool(); }
   bool val_bool();
   void top_level_item() { abort_on_null=1; }
   inline bool is_top_level_item() { return abort_on_null; }
   bool test_limit(Select_Lex_Unit *unit);
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   bool fix_fields(Session *session, Item **ref);
   bool setup_engine();
   bool init_left_expr_cache();
@@ -379,8 +400,8 @@ public:
 
   // only ALL subquery has upper not
   subs_type substype() { return all?ALL_SUBS:ANY_SUBS; }
-  trans_res select_transformer(Join *join);
-  virtual void print(String *str);
+  trans_res select_transformer(JOIN *join);
+  virtual void print(String *str, enum_query_type query_type);
 };
 
 
@@ -408,7 +429,7 @@ public:
     res_field_type= DRIZZLE_TYPE_VARCHAR;
     maybe_null= 0;
   }
-  virtual ~subselect_engine() {} // to satisfy compiler
+  virtual ~subselect_engine() {}; // to satisfy compiler
   virtual void cleanup()= 0;
 
   /*
@@ -441,15 +462,14 @@ public:
   */
   virtual int exec()= 0;
   virtual uint32_t cols()= 0; /* return number of columns in select */
-  virtual bool uncacheable()= 0; /* query is uncacheable */
-  virtual bool uncacheable(uint32_t bit_pos)= 0; /* query is uncacheable */
+  virtual uint8_t uncacheable()= 0; /* query is uncacheable */
   enum Item_result type() { return res_type; }
   enum_field_types field_type() { return res_field_type; }
   virtual void exclude()= 0;
-  virtual bool may_be_null() { return maybe_null; }
+  virtual bool may_be_null() { return maybe_null; };
   virtual table_map upper_select_const_tables()= 0;
   static table_map calc_const_tables(TableList *);
-  virtual void print(String *str)= 0;
+  virtual void print(String *str, enum_query_type query_type)= 0;
   virtual bool change_result(Item_subselect *si,
                              select_result_interceptor *result)= 0;
   virtual bool no_tables()= 0;
@@ -469,7 +489,7 @@ class subselect_single_select_engine: public subselect_engine
   bool optimized; /* simple subselect is optimized */
   bool executed; /* simple subselect is executed */
   Select_Lex *select_lex; /* corresponding select_lex */
-  Join * join; /* corresponding JOIN structure */
+  JOIN * join; /* corresponding JOIN structure */
 public:
   subselect_single_select_engine(Select_Lex *select,
 				 select_result_interceptor *result,
@@ -479,18 +499,16 @@ public:
   void fix_length_and_dec(Item_cache** row);
   int exec();
   uint32_t cols();
-  bool uncacheable();
-  bool uncacheable(uint32_t bit_pos);
+  uint8_t uncacheable();
   void exclude();
   table_map upper_select_const_tables();
-  virtual void print (String *str);
+  virtual void print (String *str, enum_query_type query_type);
   bool change_result(Item_subselect *si, select_result_interceptor *result);
   bool no_tables();
   bool may_be_null();
   bool is_executed() const { return executed; }
   bool no_rows();
   virtual enum_engine_type engine_type() { return SINGLE_SELECT_ENGINE; }
-  void save_join_if_explain();
 
   friend class subselect_hash_sj_engine;
   friend class Item_in_subselect;
@@ -509,11 +527,10 @@ public:
   void fix_length_and_dec(Item_cache** row);
   int exec();
   uint32_t cols();
-  bool uncacheable();
-  bool uncacheable(uint32_t bit_pos);
+  uint8_t uncacheable();
   void exclude();
   table_map upper_select_const_tables();
-  virtual void print (String *str);
+  virtual void print (String *str, enum_query_type query_type);
   bool change_result(Item_subselect *si, select_result_interceptor *result);
   bool no_tables();
   bool is_executed() const;
@@ -522,6 +539,9 @@ public:
 };
 
 
+class JoinTable;
+
+
 /*
   A subquery execution engine that evaluates the subquery by doing one index
   lookup in a unique index.
@@ -552,7 +572,7 @@ protected:
   bool null_keypart; /* TRUE <=> constructed search tuple has a NULL */
 public:
 
-  // constructor can assign Session because it will be called after Join::prepare
+  // constructor can assign Session because it will be called after JOIN::prepare
   subselect_uniquesubquery_engine(Session *session_arg, JoinTable *tab_arg,
 				  Item_subselect *subs, Item *where)
     :subselect_engine(subs, 0), tab(tab_arg), cond(where)
@@ -564,11 +584,10 @@ public:
   void fix_length_and_dec(Item_cache** row);
   int exec();
   uint32_t cols() { return 1; }
-  bool uncacheable() { return true; }
-  bool uncacheable(uint32_t) { return true; }
+  uint8_t uncacheable() { return UNCACHEABLE_DEPENDENT; }
   void exclude();
   table_map upper_select_const_tables() { return 0; }
-  virtual void print (String *str);
+  virtual void print (String *str, enum_query_type query_type);
   bool change_result(Item_subselect *si, select_result_interceptor *result);
   bool no_tables();
   int scan_table();
@@ -614,7 +633,7 @@ class subselect_indexsubquery_engine: public subselect_uniquesubquery_engine
   Item *having;
 public:
 
-  // constructor can assign Session because it will be called after Join::prepare
+  // constructor can assign Session because it will be called after JOIN::prepare
   subselect_indexsubquery_engine(Session *session_arg, JoinTable *tab_arg,
 				 Item_subselect *subs, Item *where,
                                  Item *having_arg, bool chk_null)
@@ -623,7 +642,7 @@ public:
      having(having_arg)
   {}
   int exec();
-  virtual void print (String *str);
+  virtual void print (String *str, enum_query_type query_type);
   virtual enum_engine_type engine_type() { return INDEXSUBQUERY_ENGINE; }
 };
 
@@ -662,7 +681,7 @@ protected:
     QEP to execute the subquery and materialize its result into a
     temporary table. Created during the first call to exec().
   */
-  Join *materialize_join;
+  JOIN *materialize_join;
   /* Temp table context of the outer select's JOIN. */
   Tmp_Table_Param *tmp_param;
 
@@ -676,11 +695,11 @@ public:
   ~subselect_hash_sj_engine();
 
   bool init_permanent(List<Item> *tmp_columns);
-  void init_runtime();
+  bool init_runtime();
   void cleanup();
   int prepare() { return 0; }
   int exec();
-  virtual void print (String *str);
+  virtual void print (String *str, enum_query_type query_type);
   uint32_t cols()
   {
     return materialize_engine->cols();
@@ -690,3 +709,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ITEM_SUBSELECT_H */
