@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,22 +17,29 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ITEM_SUM_H
+#define DRIZZLED_ITEM_SUM_H
 
 /* classes for sum functions */
 
-#include <drizzled/tree.h>
+
+#include "drizzled/my_tree.h"
 #include <drizzled/hybrid_type.h>
 #include <drizzled/item.h>
 #include <drizzled/item/field.h>
 #include <drizzled/item/bin_string.h>
-#include <drizzled/charset.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
-int group_concat_key_cmp_with_distinct(void* arg, const void* key1, const void* key2);
+int group_concat_key_cmp_with_distinct(void* arg, const void* key1,
+                                       const void* key2);
 
-int group_concat_key_cmp_with_order(void* arg, const void* key1, const void* key2);
+int group_concat_key_cmp_with_order(void* arg, const void* key1,
+                                    const void* key2);
+
+class Select_Lex;
+struct order_st;
 
 /*
   Class Item_sum is the base class used for special expressions that SQL calls
@@ -359,9 +366,8 @@ public:
     forced_const= true;
   }
   virtual bool const_item() const { return forced_const; }
-  virtual bool const_during_execution() const { return false; }
   void make_field(SendField *field);
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   void fix_num_length_and_dec();
 
   /*
@@ -410,7 +416,7 @@ public:
   bool fix_fields(Session *, Item **);
   int64_t val_int();
   String *val_str(String*str);
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   void reset_field();
 };
 
@@ -423,7 +429,7 @@ public:
   Item_sum_int(Session *session, Item_sum_int *item) :Item_sum_num(session, item) {}
   double val_real() { assert(fixed == 1); return (double) val_int(); }
   String *val_str(String*str);
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   enum Item_result result_type () const { return INT_RESULT; }
   void fix_length_and_dec()
   { decimals=0; max_length=21; maybe_null=null_value=0; }
@@ -435,7 +441,7 @@ class Item_sum_sum :public Item_sum_num
 protected:
   Item_result hybrid_type;
   double sum;
-  type::Decimal dec_buffs[2];
+  my_decimal dec_buffs[2];
   uint32_t curr_dec_buff;
   void fix_length_and_dec();
 
@@ -448,7 +454,7 @@ public:
   double val_real();
   int64_t val_int();
   String *val_str(String*str);
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   enum Item_result result_type () const { return hybrid_type; }
   void reset_field();
   void update_field();
@@ -461,6 +467,8 @@ public:
 
 /* Common class for SUM(DISTINCT), AVG(DISTINCT) */
 
+class Unique;
+
 class Item_sum_distinct :public Item_sum_num
 {
 protected:
@@ -483,7 +491,7 @@ public:
   void cleanup();
   bool add();
   double val_real();
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   int64_t val_int();
   String *val_str(String *str);
 
@@ -568,6 +576,8 @@ class Item_sum_count :public Item_sum_int
 };
 
 
+class Tmp_Table_Param;
+
 class Item_sum_count_distinct :public Item_sum_int
 {
   Table *table;
@@ -634,6 +644,8 @@ public:
 
 /* Item to get the value of a stored sum function */
 
+class Item_sum_avg;
+
 class Item_avg_field :public Item_result_field
 {
 public:
@@ -645,7 +657,7 @@ public:
   enum Type type() const { return FIELD_AVG_ITEM; }
   double val_real();
   int64_t val_int();
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   bool is_null() { update_null_value(); return null_value; }
   String *val_str(String*);
   enum_field_types field_type() const
@@ -677,7 +689,7 @@ public:
   double val_real();
   // In SPs we might force the "wrong" type with select into a declare variable
   int64_t val_int();
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   String *val_str(String *str);
   void reset_field();
   void update_field();
@@ -694,6 +706,8 @@ public:
   }
 };
 
+class Item_sum_variance;
+
 class Item_variance_field :public Item_result_field
 {
 public:
@@ -710,7 +724,7 @@ public:
   int64_t val_int();
   String *val_str(String *str)
   { return val_string_from_real(str); }
-  type::Decimal *val_decimal(type::Decimal *dec_buf)
+  my_decimal *val_decimal(my_decimal *dec_buf)
   { return val_decimal_from_real(dec_buf); }
   bool is_null() { update_null_value(); return null_value; }
   enum_field_types field_type() const
@@ -767,7 +781,7 @@ public:
   bool add();
   double val_real();
   int64_t val_int();
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   void reset_field();
   void update_field();
   Item *result_item(Field *)
@@ -785,13 +799,15 @@ public:
   }
 };
 
+class Item_sum_std;
+
 class Item_std_field :public Item_variance_field
 {
 public:
   Item_std_field(Item_sum_std *item);
   enum Type type() const { return FIELD_STD_ITEM; }
   double val_real();
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   enum Item_result result_type () const { return REAL_RESULT; }
   enum_field_types field_type() const { return DRIZZLE_TYPE_DOUBLE;}
 };
@@ -826,7 +842,7 @@ protected:
   String value,tmp_value;
   double sum;
   int64_t sum_int;
-  type::Decimal sum_dec;
+  my_decimal sum_dec;
   Item_result hybrid_type;
   enum_field_types hybrid_field_type;
   int cmp_sign;
@@ -843,7 +859,7 @@ protected:
   void clear();
   double val_real();
   int64_t val_int();
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   void reset_field();
   String *val_str(String *);
   bool keep_field_type(void) const { return 1; }
@@ -944,14 +960,18 @@ class Item_sum_xor :public Item_sum_bit
   Item *copy_or_same(Session* session);
 };
 
+
+
+class DRIZZLE_ERROR;
+
 class Item_func_group_concat : public Item_sum
 {
   Tmp_Table_Param *tmp_table_param;
   DRIZZLE_ERROR *warning;
   String result;
   String *separator;
-  Tree tree_base;
-  Tree *tree;
+  TREE tree_base;
+  TREE *tree;
 
   /**
      If DISTINCT is used with this GROUP_CONCAT, this member is used to filter
@@ -962,7 +982,7 @@ class Item_func_group_concat : public Item_sum
    */
   Unique *unique_filter;
   Table *table;
-  Order **order;
+  order_st **order;
   Name_resolution_context *context;
   /** The number of ORDER BY items. */
   uint32_t arg_count_order;
@@ -1015,17 +1035,18 @@ public:
   void make_unique();
   double val_real();
   int64_t val_int();
-  type::Decimal *val_decimal(type::Decimal *decimal_value)
+  my_decimal *val_decimal(my_decimal *decimal_value)
   {
     return val_decimal_from_string(decimal_value);
   }
   String* val_str(String* str);
   Item *copy_or_same(Session* session);
   void no_rows_in_result() {}
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   virtual bool change_context_processor(unsigned char *cntx)
     { context= (Name_resolution_context *)cntx; return false; }
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ITEM_SUM_H */
