@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,48 +20,29 @@
 
 /* This struct includes all reserved words and functions */
 
-#pragma once
+#ifndef DRIZZLED_LEX_SYMBOL_H
+#define DRIZZLED_LEX_SYMBOL_H
 
-namespace drizzled {
-
-struct SYM_GROUP 
+namespace drizzled
 {
-  const char* name;
-  const char* needed_define;
-};
 
-struct SYMBOL
+typedef struct st_sym_group {
+  const char *name;
+  const char *needed_define;
+} SYM_GROUP;
+
+typedef struct st_symbol {
+  const char *name;
+  uint	tok;
+} SYMBOL;
+
+typedef struct st_lex_symbol
 {
-  const char* name;
-  uint tok;
-};
+  const SYMBOL *symbol;
+  char   *str;
+  uint32_t   length;
+} LEX_SYMBOL;
 
-struct LEX_SYMBOL
-{
-  const char* begin() const
-  {
-    return data();
-  }
-
-  const char* end() const
-  {
-    return data() + size();
-  }
-
-  const char* data() const
-  {
-    return str;
-  }
-
-  size_t size() const
-  {
-    return length;
-  }
-
-  const SYMBOL* symbol;
-  char* str;
-  uint32_t length;
-};
 
 extern SYM_GROUP sym_group_common;
 extern SYM_GROUP sym_group_geom;
@@ -69,3 +50,4 @@ extern SYM_GROUP sym_group_rtree;
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_LEX_SYMBOL_H */
