@@ -1,8 +1,8 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
- *  Copyright (C) 2010 Jay Pipes
+ *  Copyright (C) 2009 Sun Microsystems
+ *  Copyright (c) 2010 Jay Pipes
  *
  *  Authors:
  *
@@ -29,20 +29,15 @@
  * Statement messages to other formats, including SQL strings.
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <boost/lexical_cast.hpp>
-
-#include <drizzled/charset.h>
-#include <drizzled/message.h>
-#include <drizzled/message/statement_transform.h>
-#include <drizzled/message/transaction.pb.h>
-#include <drizzled/message/access.h>
+#include "drizzled/message/statement_transform.h"
+#include "drizzled/message/transaction.pb.h"
+#include "drizzled/message/table.pb.h"
 
 #include <string>
 #include <vector>
 #include <sstream>
-#include <cstdio>
 
 using namespace std;
 
@@ -52,71 +47,6 @@ namespace drizzled
 namespace message
 {
 
-static void escapeEmbeddedQuotes(string &s, const char quote='\'')
-{
-  string::iterator it;
-
-  for (it= s.begin(); it != s.end(); ++it)
-  {
-    if (*it == quote)
-    {
-      it= s.insert(it, quote);
-      ++it;  // advance back to the quote
-    }
-  }
-}
-
-/* Incredibly similar to append_unescaped() in table.cc, but for std::string */
-static void append_escaped_string(std::string *res, const std::string &input, const char quote='\'')
-{
-  const char *pos= input.c_str();
-  const char *end= input.c_str()+input.length();
-  res->push_back(quote);
-
-  for (; pos != end ; pos++)
-  {
-    uint32_t mblen;
-    if (use_mb(default_charset_info) &&
-        (mblen= my_ismbchar(default_charset_info, pos, end)))
-    {
-      res->append(pos, mblen);
-      pos+= mblen - 1;
-      if (pos >= end)
-        break;
-      continue;
-    }
-
-    switch (*pos) {
-    case 0:				/* Must be escaped for 'mysql' */
-      res->push_back('\\');
-      res->push_back('0');
-      break;
-    case '\n':				/* Must be escaped for logs */
-      res->push_back('\\');
-      res->push_back('n');
-      break;
-    case '\r':
-      res->push_back('\\');		/* This gives better readability */
-      res->push_back('r');
-      break;
-    case '\\':
-      res->push_back('\\');		/* Because of the sql syntax */
-      res->push_back('\\');
-      break;
-    default:
-      if (*pos == quote) /* SQL syntax for quoting a quote */
-      {
-        res->push_back(quote);
-        res->push_back(quote);
-      }
-      else
-        res->push_back(*pos);
-      break;
-    }
-  }
-  res->push_back(quote);
-}
-
 enum TransformSqlError
 transformStatementToSql(const Statement &source,
                         vector<string> &sql_strings,
@@ -127,15 +57,6 @@ transformStatementToSql(const Statement &source,
 
   switch (source.type())
   {
-  case Statement::ROLLBACK_STATEMENT:
-    {
-      break;
-    }
-  case Statement::ROLLBACK:
-    {
-      sql_strings.push_back("ROLLBACK");
-      break;
-    }
   case Statement::INSERT:
     {
       if (! source.has_insert_header())
@@ -152,11 +73,12 @@ transformStatementToSql(const Statement &source,
       const InsertHeader &insert_header= source.insert_header();
       const InsertData &insert_data= source.insert_data();
       size_t num_keys= insert_data.record_size();
+      size_t x;
 
       if (num_keys > 1 && ! already_in_transaction)
         sql_strings.push_back("START TRANSACTION");
 
-      for (size_t x= 0; x < num_keys; ++x)
+      for (x= 0; x < num_keys; ++x)
       {
         string destination;
 
@@ -317,16 +239,6 @@ transformStatementToSql(const Statement &source,
       sql_strings.push_back(destination);
     }
     break;
-  case Statement::ALTER_SCHEMA:
-    {
-      assert(source.has_alter_schema_statement());
-      string destination;
-      error= transformAlterSchemaStatementToSql(source.alter_schema_statement(),
-                                                destination,
-                                                sql_variant);
-      sql_strings.push_back(destination);
-    }
-    break;
   case Statement::SET_VARIABLE:
     {
       assert(source.has_set_variable_statement());
@@ -338,16 +250,6 @@ transformStatementToSql(const Statement &source,
     }
     break;
   case Statement::RAW_SQL:
-    {
-      if (source.has_raw_sql_schema())
-      {
-        string destination("USE ");
-        destination.append(source.raw_sql_schema());
-        sql_strings.push_back(destination);
-      }
-      sql_strings.push_back(source.sql());
-    }
-    break;
   default:
     sql_strings.push_back(source.sql());
     break;
@@ -402,6 +304,10 @@ transformInsertRecordToSql(const InsertHeader &header,
                                                            destination,
                                                            sql_variant);
 
+  char quoted_identifier= '`';
+  if (sql_variant == ANSI)
+    quoted_identifier= '"';
+
   destination.append(") VALUES (");
 
   /* Add insert values */
@@ -416,41 +322,25 @@ transformInsertRecordToSql(const InsertHeader &header,
 
     const FieldMetadata &field_metadata= header.field_metadata(x);
 
-    if (record.is_null(x))
-    {
-      should_quote_field_value= false;
-    }
-    else 
-    {
-      should_quote_field_value= shouldQuoteFieldValue(field_metadata.type());
-    }
+    should_quote_field_value= shouldQuoteFieldValue(field_metadata.type());
 
     if (should_quote_field_value)
       destination.push_back('\'');
 
-    if (record.is_null(x))
+    if (field_metadata.type() == Table::Field::BLOB)
     {
-      destination.append("NULL");
+      /* 
+        * We do this here because BLOB data is returned
+        * in a string correctly, but calling append()
+        * without a length will result in only the string
+        * up to a \0 being output here.
+        */
+      string raw_data(record.insert_value(x));
+      destination.append(raw_data.c_str(), raw_data.size());
     }
     else
     {
-      if (field_metadata.type() == Table::Field::BLOB)
-      {
-        /*
-         * We do this here because BLOB data is returned
-         * in a string correctly, but calling append()
-         * without a length will result in only the string
-         * up to a \0 being output here.
-         */
-        string raw_data(record.insert_value(x));
-        destination.append(raw_data.c_str(), raw_data.size());
-      }
-      else
-      {
-        string tmp(record.insert_value(x));
-        escapeEmbeddedQuotes(tmp);
-        destination.append(tmp);
-      }
+      destination.append(record.insert_value(x));
     }
 
     if (should_quote_field_value)
@@ -471,6 +361,10 @@ transformInsertStatementToSql(const InsertHeader &header,
                                                            destination,
                                                            sql_variant);
 
+  char quoted_identifier= '`';
+  if (sql_variant == ANSI)
+    quoted_identifier= '"';
+
   destination.append(") VALUES (", 10);
 
   /* Add insert values */
@@ -509,9 +403,7 @@ transformInsertStatementToSql(const InsertHeader &header,
       }
       else
       {
-        string tmp(data.record(x).insert_value(y));
-        escapeEmbeddedQuotes(tmp);
-        destination.append(tmp);
+        destination.append(data.record(x).insert_value(y));
       }
 
       if (should_quote_field_value)
@@ -575,41 +467,25 @@ transformUpdateRecordToSql(const UpdateHeader &header,
     destination.push_back(quoted_identifier);
     destination.push_back('=');
 
-    if (record.is_null(x))
-    {
-      should_quote_field_value= false;
-    }
-    else 
-    {
-      should_quote_field_value= shouldQuoteFieldValue(field_metadata.type());
-    }    
+    should_quote_field_value= shouldQuoteFieldValue(field_metadata.type());
 
     if (should_quote_field_value)
       destination.push_back('\'');
 
-    if (record.is_null(x))
+    if (field_metadata.type() == Table::Field::BLOB)
     {
-      destination.append("NULL");
+      /* 
+       * We do this here because BLOB data is returned
+       * in a string correctly, but calling append()
+       * without a length will result in only the string
+       * up to a \0 being output here.
+       */
+      string raw_data(record.after_value(x));
+      destination.append(raw_data.c_str(), raw_data.size());
     }
-    else 
+    else
     {
-      if (field_metadata.type() == Table::Field::BLOB)
-      {
-        /*
-         * We do this here because BLOB data is returned
-         * in a string correctly, but calling append()
-         * without a length will result in only the string
-         * up to a \0 being output here.
-         */
-        string raw_data(record.after_value(x));
-        destination.append(raw_data.c_str(), raw_data.size());
-      }
-      else 
-      {
-        string tmp(record.after_value(x));
-        escapeEmbeddedQuotes(tmp);
-        destination.append(tmp);
-      }
+      destination.append(record.after_value(x));
     }
 
     if (should_quote_field_value)
@@ -731,9 +607,7 @@ transformDeleteRecordToSql(const DeleteHeader &header,
     }
     else
     {
-      string tmp(record.key_value(x));
-      escapeEmbeddedQuotes(tmp);
-      destination.append(tmp);
+      destination.append(record.key_value(x));
     }
 
     if (should_quote_field_value)
@@ -802,9 +676,7 @@ transformDeleteStatementToSql(const DeleteHeader &header,
       }
       else
       {
-        string tmp(data.record(x).key_value(y));
-        escapeEmbeddedQuotes(tmp);
-        destination.append(tmp);
+        destination.append(data.record(x).key_value(y));
       }
 
       if (should_quote_field_value)
@@ -816,37 +688,6 @@ transformDeleteStatementToSql(const DeleteHeader &header,
   return error;
 }
 
-enum TransformSqlError
-transformAlterSchemaStatementToSql(const AlterSchemaStatement &statement,
-                                   string &destination,
-                                   enum TransformSqlVariant sql_variant)
-{
-  const Schema &before= statement.before();
-  const Schema &after= statement.after();
-
-  /* Make sure we are given the before and after for the same object */
-  if (before.uuid() != after.uuid())
-    return UUID_MISMATCH;
-
-  char quoted_identifier= '`';
-  if (sql_variant == ANSI)
-    quoted_identifier= '"';
-
-  destination.append("ALTER SCHEMA ");
-  destination.push_back(quoted_identifier);
-  destination.append(before.name());
-  destination.push_back(quoted_identifier);
-
-  /*
-   * Diff our schemas. Currently, only collation can change so a
-   * diff of the two structures is not really necessary.
-   */
-  destination.append(" COLLATE = ");
-  destination.append(after.collation());
-
-  return NONE;
-}
-
 enum TransformSqlError
 transformDropSchemaStatementToSql(const DropSchemaStatement &statement,
                                   string &destination,
@@ -875,30 +716,17 @@ transformCreateSchemaStatementToSql(const CreateSchemaStatement &statement,
 
   const Schema &schema= statement.schema();
 
-  destination.append("CREATE SCHEMA ");
+  destination.append("CREATE SCHEMA ", 14);
   destination.push_back(quoted_identifier);
   destination.append(schema.name());
   destination.push_back(quoted_identifier);
 
   if (schema.has_collation())
   {
-    destination.append(" COLLATE ");
+    destination.append(" COLLATE ", 9);
     destination.append(schema.collation());
   }
 
-  if (not message::is_replicated(schema))
-  {
-    destination.append(" REPLICATE = FALSE");
-  }
-
-  if (message::has_definer(schema))
-  {
-    destination.append(" DEFINER ");
-    destination.push_back('\'');
-    destination.append(message::definer(schema));
-    destination.push_back('\'');
-  }
-
   return NONE;
 }
 
@@ -913,13 +741,13 @@ transformDropTableStatementToSql(const DropTableStatement &statement,
 
   const TableMetadata &table_metadata= statement.table_metadata();
 
-  destination.append("DROP TABLE ");
+  destination.append("DROP TABLE ", 11);
 
   /* Add the IF EXISTS clause if necessary */
   if (statement.has_if_exists_clause() &&
       statement.if_exists_clause() == true)
   {
-    destination.append("IF EXISTS ");
+    destination.append("IF EXISTS ", 10);
   }
 
   destination.push_back(quoted_identifier);
@@ -944,7 +772,7 @@ transformTruncateTableStatementToSql(const TruncateTableStatement &statement,
 
   const TableMetadata &table_metadata= statement.table_metadata();
 
-  destination.append("TRUNCATE TABLE ");
+  destination.append("TRUNCATE TABLE ", 15);
   destination.push_back(quoted_identifier);
   destination.append(table_metadata.schema_name());
   destination.push_back(quoted_identifier);
@@ -965,7 +793,7 @@ transformSetVariableStatementToSql(const SetVariableStatement &statement,
   const FieldMetadata &variable_metadata= statement.variable_metadata();
   bool should_quote_field_value= shouldQuoteFieldValue(variable_metadata.type());
 
-  destination.append("SET GLOBAL "); /* Only global variables are replicated */
+  destination.append("SET GLOBAL ", 11); /* Only global variables are replicated */
   destination.append(variable_metadata.name());
   destination.push_back('=');
 
@@ -991,25 +819,22 @@ transformCreateTableStatementToSql(const CreateTableStatement &statement,
 enum TransformSqlError
 transformTableDefinitionToSql(const Table &table,
                               string &destination,
-                              enum TransformSqlVariant sql_variant, bool with_schema)
+                              enum TransformSqlVariant sql_variant)
 {
   char quoted_identifier= '`';
   if (sql_variant == ANSI)
     quoted_identifier= '"';
 
-  destination.append("CREATE ");
+  destination.append("CREATE ", 7);
 
   if (table.type() == Table::TEMPORARY)
-    destination.append("TEMPORARY ");
+    destination.append("TEMPORARY ", 10);
   
-  destination.append("TABLE ");
-  if (with_schema)
-  {
-    append_escaped_string(&destination, table.schema(), quoted_identifier);
-    destination.push_back('.');
-  }
-  append_escaped_string(&destination, table.name(), quoted_identifier);
-  destination.append(" (\n");
+  destination.append("TABLE ", 6);
+  destination.push_back(quoted_identifier);
+  destination.append(table.name());
+  destination.push_back(quoted_identifier);
+  destination.append(" (\n", 3);
 
   enum TransformSqlError result= NONE;
   size_t num_fields= table.field_size();
@@ -1018,12 +843,10 @@ transformTableDefinitionToSql(const Table &table,
     const Table::Field &field= table.field(x);
 
     if (x != 0)
-      destination.append(",\n");
-
-    destination.append("  ");
+      destination.append(",\n", 2);
 
     result= transformFieldDefinitionToSql(field, destination, sql_variant);
-
+    
     if (result != NONE)
       return result;
   }
@@ -1031,80 +854,44 @@ transformTableDefinitionToSql(const Table &table,
   size_t num_indexes= table.indexes_size();
   
   if (num_indexes > 0)
-    destination.append(",\n");
+    destination.append(",\n", 2);
 
   for (size_t x= 0; x < num_indexes; ++x)
   {
     const message::Table::Index &index= table.indexes(x);
 
     if (x != 0)
-      destination.append(",\n");
+      destination.append(",\n", 2);
 
     result= transformIndexDefinitionToSql(index, table, destination, sql_variant);
     
     if (result != NONE)
       return result;
   }
-
-  size_t num_foreign_keys= table.fk_constraint_size();
-
-  if (num_foreign_keys > 0)
-    destination.append(",\n");
-
-  for (size_t x= 0; x < num_foreign_keys; ++x)
-  {
-    const message::Table::ForeignKeyConstraint &fkey= table.fk_constraint(x);
-
-    if (x != 0)
-      destination.append(",\n");
-
-    result= transformForeignKeyConstraintDefinitionToSql(fkey, table, destination, sql_variant);
-
-    if (result != NONE)
-      return result;
-  }
-
-  destination.append("\n)");
+  destination.append("\n)", 2);
 
   /* Add ENGINE = " clause */
   if (table.has_engine())
   {
-    destination.append(" ENGINE=");
-    destination.append(table.engine().name());
+    const Table::StorageEngine &engine= table.engine();
+    destination.append("\nENGINE = ", 10);
+    destination.append(engine.name());
 
-    size_t num_engine_options= table.engine().options_size();
-    if (num_engine_options > 0)
-      destination.append(" ", 1);
+    size_t num_engine_options= engine.option_size();
     for (size_t x= 0; x < num_engine_options; ++x)
     {
-      const Engine::Option &option= table.engine().options(x);
-      destination.append(option.name());
-      destination.append("='");
-      destination.append(option.state());
-      destination.append("'");
-      if (x != num_engine_options-1)
-      {
-        destination.append(", ");
-      }
+      const Table::StorageEngine::EngineOption &option= engine.option(x);
+      destination.push_back('\n');
+      destination.append(option.option_name());
+      destination.append(" = ", 3);
+      destination.append(option.option_value());
+      destination.push_back('\n');
     }
   }
 
   if (table.has_options())
     (void) transformTableOptionsToSql(table.options(), destination, sql_variant);
 
-  if (not message::is_replicated(table))
-  {
-    destination.append(" REPLICATE = FALSE");
-  }
-
-  if (message::has_definer(table))
-  {
-    destination.append(" DEFINER ");
-    destination.push_back('\'');
-    destination.append(message::definer(table));
-    destination.push_back('\'');
-  }
-
   return NONE;
 }
 
@@ -1116,62 +903,111 @@ transformTableOptionsToSql(const Table::TableOptions &options,
   if (sql_variant == ANSI)
     return NONE; /* ANSI does not support table options... */
 
+  stringstream ss;
+
   if (options.has_comment())
   {
-    destination.append(" COMMENT=");
-    append_escaped_string(&destination, options.comment());
+    destination.append("\nCOMMENT = '", 12);
+    destination.append(options.comment());
+    destination.push_back('\'');
   }
 
   if (options.has_collation())
   {
-    destination.append(" COLLATE = ");
+    destination.append("\nCOLLATE = ", 11);
     destination.append(options.collation());
   }
 
+  if (options.has_auto_increment())
+  {
+    ss << options.auto_increment();
+    destination.append("\nAUTOINCREMENT_OFFSET = ", 24);
+    destination.append(ss.str());
+    ss.clear();
+  }
+  
+  if (options.has_row_type())
+  {
+    ss << options.row_type();
+    destination.append("\nROW_TYPE = ", 12);
+    destination.append(ss.str());
+    ss.clear();
+  }
+
   if (options.has_data_file_name())
   {
-    destination.append("\nDATA_FILE_NAME = '");
+    destination.append("\nDATA_FILE_NAME = '", 19);
     destination.append(options.data_file_name());
     destination.push_back('\'');
   }
 
   if (options.has_index_file_name())
   {
-    destination.append("\nINDEX_FILE_NAME = '");
+    destination.append("\nINDEX_FILE_NAME = '", 20);
     destination.append(options.index_file_name());
     destination.push_back('\'');
   }
 
   if (options.has_max_rows())
   {
-    destination.append("\nMAX_ROWS = ");
-    destination.append(boost::lexical_cast<string>(options.max_rows()));
+    ss << options.max_rows();
+    destination.append("\nMAX_ROWS = ", 12);
+    destination.append(ss.str());
+    ss.clear();
   }
 
   if (options.has_min_rows())
   {
-    destination.append("\nMIN_ROWS = ");
-    destination.append(boost::lexical_cast<string>(options.min_rows()));
+    ss << options.min_rows();
+    destination.append("\nMIN_ROWS = ", 12);
+    destination.append(ss.str());
+    ss.clear();
   }
 
-  if (options.has_user_set_auto_increment_value()
-      && options.has_auto_increment_value())
+  if (options.has_auto_increment_value())
   {
-    destination.append(" AUTO_INCREMENT=");
-    destination.append(boost::lexical_cast<string>(options.auto_increment_value()));
+    ss << options.auto_increment_value();
+    destination.append("\nAUTO_INCREMENT = ", 18);
+    destination.append(ss.str());
+    ss.clear();
   }
 
   if (options.has_avg_row_length())
   {
-    destination.append("\nAVG_ROW_LENGTH = ");
-    destination.append(boost::lexical_cast<string>(options.avg_row_length()));
+    ss << options.avg_row_length();
+    destination.append("\nAVG_ROW_LENGTH = ", 18);
+    destination.append(ss.str());
+    ss.clear();
   }
 
-  if (options.has_checksum() && options.checksum())
-    destination.append("\nCHECKSUM = TRUE");
+  if (options.has_key_block_size())
+  {
+    ss << options.key_block_size();
+    destination.append("\nKEY_BLOCK_SIZE = ", 18);
+    destination.append(ss.str());
+    ss.clear();
+  }
 
-  if (options.has_page_checksum() && options.page_checksum())
-    destination.append("\nPAGE_CHECKSUM = TRUE");
+  if (options.has_block_size())
+  {
+    ss << options.block_size();
+    destination.append("\nBLOCK_SIZE = ", 14);
+    destination.append(ss.str());
+    ss.clear();
+  }
+
+  if (options.has_pack_keys() &&
+      options.pack_keys())
+    destination.append("\nPACK_KEYS = TRUE", 17);
+  if (options.has_pack_record() &&
+      options.pack_record())
+    destination.append("\nPACK_RECORD = TRUE", 19);
+  if (options.has_checksum() &&
+      options.checksum())
+    destination.append("\nCHECKSUM = TRUE", 16);
+  if (options.has_page_checksum() &&
+      options.page_checksum())
+    destination.append("\nPAGE_CHECKSUM = TRUE", 21);
 
   return NONE;
 }
@@ -1186,24 +1022,17 @@ transformIndexDefinitionToSql(const Table::Index &index,
   if (sql_variant == ANSI)
     quoted_identifier= '"';
 
-  destination.append("  ", 2);
-
   if (index.is_primary())
-    destination.append("PRIMARY ");
+    destination.append("PRIMARY ", 8);
   else if (index.is_unique())
-    destination.append("UNIQUE ");
+    destination.append("UNIQUE ", 7);
 
   destination.append("KEY ", 4);
-  if (! (index.is_primary() && index.name().compare("PRIMARY")==0))
-  {
-    destination.push_back(quoted_identifier);
-    destination.append(index.name());
-    destination.push_back(quoted_identifier);
-    destination.append(" (", 2);
-  }
-  else
-    destination.append("(", 1);
-
+  destination.push_back(quoted_identifier);
+  destination.append(index.name());
+  destination.push_back(quoted_identifier);
+  destination.append(" (", 2);
+  
   size_t num_parts= index.index_part_size();
   for (size_t x= 0; x < num_parts; ++x)
   {
@@ -1227,10 +1056,19 @@ transformIndexDefinitionToSql(const Table::Index &index,
     {
       if (part.has_compare_length())
       {
-        if (part.compare_length() != field.string_options().length())
+        size_t compare_length_in_chars= part.compare_length();
+        
+        /* hack: compare_length() is bytes, not chars, but
+         * only for VARCHAR. Ass. */
+        if (field.type() == Table::Field::VARCHAR)
+          compare_length_in_chars/= 4;
+
+        if (compare_length_in_chars != field.string_options().length())
         {
+          stringstream ss;
           destination.push_back('(');
-          destination.append(boost::lexical_cast<string>(part.compare_length()));
+          ss << compare_length_in_chars;
+          destination.append(ss.str());
           destination.push_back(')');
         }
       }
@@ -1238,115 +1076,6 @@ transformIndexDefinitionToSql(const Table::Index &index,
   }
   destination.push_back(')');
 
-  switch (index.type())
-  {
-  case Table::Index::UNKNOWN_INDEX:
-    break;
-  case Table::Index::BTREE:
-    destination.append(" USING BTREE");
-    break;
-  case Table::Index::RTREE:
-    destination.append(" USING RTREE");
-    break;
-  case Table::Index::HASH:
-    destination.append(" USING HASH");
-    break;
-  case Table::Index::FULLTEXT:
-    destination.append(" USING FULLTEXT");
-    break;
-  }
-
-  if (index.has_comment())
-  {
-    destination.append(" COMMENT ");
-    append_escaped_string(&destination, index.comment());
-  }
-
-  return NONE;
-}
-
-static void transformForeignKeyOptionToSql(Table::ForeignKeyConstraint::ForeignKeyOption opt, string &destination)
-{
-  switch (opt)
-  {
-  case Table::ForeignKeyConstraint::OPTION_RESTRICT:
-    destination.append("RESTRICT");
-    break;
-  case Table::ForeignKeyConstraint::OPTION_CASCADE:
-    destination.append("CASCADE");
-    break;
-  case Table::ForeignKeyConstraint::OPTION_SET_NULL:
-    destination.append("SET NULL");
-    break;
-  case Table::ForeignKeyConstraint::OPTION_UNDEF:
-  case Table::ForeignKeyConstraint::OPTION_NO_ACTION:
-    destination.append("NO ACTION");
-    break;
-  case Table::ForeignKeyConstraint::OPTION_SET_DEFAULT:
-    destination.append("SET DEFAULT");
-    break;
-  }
-}
-
-enum TransformSqlError
-transformForeignKeyConstraintDefinitionToSql(const Table::ForeignKeyConstraint &fkey,
-                                             const Table &,
-                                             string &destination,
-                                             enum TransformSqlVariant sql_variant)
-{
-  char quoted_identifier= '`';
-  if (sql_variant == ANSI)
-    quoted_identifier= '"';
-
-  destination.append("  ");
-
-  if (fkey.has_name())
-  {
-    destination.append("CONSTRAINT ");
-    append_escaped_string(&destination, fkey.name(), quoted_identifier);
-    destination.append(" ", 1);
-  }
-
-  destination.append("FOREIGN KEY (");
-
-  for (ssize_t x= 0; x < fkey.column_names_size(); ++x)
-  {
-    if (x != 0)
-      destination.append(", ");
-
-    append_escaped_string(&destination, fkey.column_names(x),
-                          quoted_identifier);
-  }
-
-  destination.append(") REFERENCES ");
-
-  append_escaped_string(&destination, fkey.references_table_name(),
-                        quoted_identifier);
-  destination.append(" (");
-
-  for (ssize_t x= 0; x < fkey.references_columns_size(); ++x)
-  {
-    if (x != 0)
-      destination.append(", ");
-
-    append_escaped_string(&destination, fkey.references_columns(x),
-                          quoted_identifier);
-  }
-
-  destination.push_back(')');
-
-  if (fkey.has_update_option() and fkey.update_option() != Table::ForeignKeyConstraint::OPTION_UNDEF)
-  {
-    destination.append(" ON UPDATE ");
-    transformForeignKeyOptionToSql(fkey.update_option(), destination);
-  }
-
-  if (fkey.has_delete_option() and fkey.delete_option() != Table::ForeignKeyConstraint::OPTION_UNDEF)
-  {
-    destination.append(" ON DELETE ");
-    transformForeignKeyOptionToSql(fkey.delete_option(), destination);
-  }
-
   return NONE;
 }
 
@@ -1356,61 +1085,38 @@ transformFieldDefinitionToSql(const Table::Field &field,
                               enum TransformSqlVariant sql_variant)
 {
   char quoted_identifier= '`';
-  char quoted_default;
-
   if (sql_variant == ANSI)
     quoted_identifier= '"';
 
-  if (sql_variant == DRIZZLE)
-    quoted_default= '\'';
-  else
-    quoted_default= quoted_identifier;
-
-  append_escaped_string(&destination, field.name(), quoted_identifier);
+  destination.push_back(quoted_identifier);
+  destination.append(field.name());
+  destination.push_back(quoted_identifier);
 
   Table::Field::FieldType field_type= field.type();
 
   switch (field_type)
   {
     case Table::Field::DOUBLE:
-    destination.append(" DOUBLE");
-    if (field.has_numeric_options()
-        && field.numeric_options().has_precision())
-    {
-      stringstream ss;
-      ss << "(" << field.numeric_options().precision() << ",";
-      ss << field.numeric_options().scale() << ")";
-      destination.append(ss.str());
-    }
+    destination.append(" DOUBLE", 7);
     break;
   case Table::Field::VARCHAR:
     {
-      if (field.string_options().has_collation()
-          && field.string_options().collation().compare("binary") == 0)
-        destination.append(" VARBINARY(");
-      else
-        destination.append(" VARCHAR(");
-
-      destination.append(boost::lexical_cast<string>(field.string_options().length()));
-      destination.append(")");
+      destination.append(" VARCHAR(", 9);
+      stringstream ss;
+      ss << field.string_options().length() << ")";
+      destination.append(ss.str());
     }
     break;
   case Table::Field::BLOB:
-    {
-      if (field.string_options().has_collation()
-          && field.string_options().collation().compare("binary") == 0)
-        destination.append(" BLOB");
-      else
-        destination.append(" TEXT");
-    }
+    destination.append(" BLOB", 5);
     break;
   case Table::Field::ENUM:
     {
-      size_t num_field_values= field.enumeration_values().field_value_size();
-      destination.append(" ENUM(");
+      size_t num_field_values= field.set_options().field_value_size();
+      destination.append(" ENUM(", 6);
       for (size_t x= 0; x < num_field_values; ++x)
       {
-        const string &type= field.enumeration_values().field_value(x);
+        const string &type= field.set_options().field_value(x);
 
         if (x != 0)
           destination.push_back(',');
@@ -1422,32 +1128,15 @@ transformFieldDefinitionToSql(const Table::Field &field,
       destination.push_back(')');
       break;
     }
-  case Table::Field::UUID:
-    destination.append(" UUID");
-    break;
-  case Table::Field::IPV6:
-    destination.append(" IPV6");
-    break;
-  case Table::Field::BOOLEAN:
-    destination.append(" BOOLEAN");
-    break;
   case Table::Field::INTEGER:
-    destination.append(" INT");
+    destination.append(" INT", 4);
     break;
   case Table::Field::BIGINT:
-    if (field.has_constraints() and
-        field.constraints().is_unsigned())
-    {
-      destination.append(" BIGINT UNSIGNED");
-    }
-    else
-    {
-      destination.append(" BIGINT");
-    }
+    destination.append(" BIGINT", 7);
     break;
   case Table::Field::DECIMAL:
     {
-      destination.append(" DECIMAL(");
+      destination.append(" DECIMAL(", 9);
       stringstream ss;
       ss << field.numeric_options().precision() << ",";
       ss << field.numeric_options().scale() << ")";
@@ -1455,52 +1144,34 @@ transformFieldDefinitionToSql(const Table::Field &field,
     }
     break;
   case Table::Field::DATE:
-    destination.append(" DATE");
+    destination.append(" DATE", 5);
     break;
-
-  case Table::Field::EPOCH:
-    if (field.time_options().microseconds())
-    {
-      destination.append(" TIMESTAMP(6)");
-    }
-    else
-    {
-      destination.append(" TIMESTAMP");
-    }
+  case Table::Field::TIMESTAMP:
+    destination.append(" TIMESTAMP",  10);
     break;
-
   case Table::Field::DATETIME:
-    destination.append(" DATETIME");
-    break;
-  case Table::Field::TIME:
-    destination.append(" TIME");
+    destination.append(" DATETIME",  9);
     break;
   }
 
-  if (field.type() == Table::Field::BLOB ||
-      field.type() == Table::Field::VARCHAR)
+  if (field.type() == Table::Field::INTEGER || 
+      field.type() == Table::Field::BIGINT)
   {
-    if (field.string_options().has_collation()
-        && field.string_options().collation().compare("binary"))
+    if (field.has_constraints() &&
+        field.constraints().has_is_unsigned() &&
+        field.constraints().is_unsigned())
     {
-      destination.append(" COLLATE ");
-      destination.append(field.string_options().collation());
+      destination.append(" UNSIGNED", 9);
     }
   }
 
-  if (field.has_constraints() and field.constraints().is_unique())
-  {
-    destination.append(" UNIQUE");
-  }
 
-  if (field.has_constraints() && field.constraints().is_notnull())
+  if (! (field.has_constraints() &&
+         field.constraints().is_nullable()))
   {
-    destination.append(" NOT NULL");
-  }
-  else if (field.type() == Table::Field::EPOCH)
-  {
-    destination.append(" NULL");
+    destination.append(" NOT", 4);
   }
+  destination.append(" NULL", 5);
 
   if (field.type() == Table::Field::INTEGER || 
       field.type() == Table::Field::BIGINT)
@@ -1509,55 +1180,49 @@ transformFieldDefinitionToSql(const Table::Field &field,
     if (field.has_numeric_options() &&
         field.numeric_options().is_autoincrement())
     {
-      destination.append(" AUTO_INCREMENT");
+      destination.append(" AUTO_INCREMENT", 15);
+    }
+  }
+
+  if (field.type() == Table::Field::BLOB ||
+      field.type() == Table::Field::VARCHAR)
+  {
+    if (field.string_options().has_collation())
+    {
+      destination.append(" COLLATE ", 9);
+      destination.append(field.string_options().collation());
     }
   }
 
   if (field.options().has_default_value())
   {
-    destination.append(" DEFAULT ");
-    append_escaped_string(&destination, field.options().default_value());
-  }
-  else if (field.options().has_default_expression())
-  {
-    destination.append(" DEFAULT ");
-    destination.append(field.options().default_expression());
-  }
-  else if (field.options().has_default_bin_value())
-  {
-    const string &v= field.options().default_bin_value();
-    if (v.length() == 0)
-    {
-      destination.append(" DEFAULT ''");
-    }
-    else
-    {
-      destination.append(" DEFAULT 0x");
-      for (size_t x= 0; x < v.length(); x++)
-      {
-        char hex[3];
-        snprintf(hex, sizeof(hex), "%.2X", *(v.c_str() + x));
-        destination.append(hex, 2);
-      }
-    }
-  }
-  else if (field.options().has_default_null()
-           && field.options().default_null()
-           && field.type() != Table::Field::BLOB)
-  {
-    destination.append(" DEFAULT NULL");
+    destination.append(" DEFAULT ", 9);
+    destination.push_back(quoted_identifier);
+    destination.append(field.options().default_value());
+    destination.push_back(quoted_identifier);
   }
 
-  if (field.has_options() && field.options().has_update_expression())
+  if (field.options().has_default_bin_value())
   {
-    destination.append(" ON UPDATE ");
-    destination.append(field.options().update_expression());
+    const string &v= field.options().default_bin_value();
+    destination.append(" DEFAULT 0x", 11);
+    for (size_t x= 0; x < v.length(); x++)
+    {
+      printf("%.2x", *(v.c_str() + x));
+    }
   }
 
+  if (field.type() == Table::Field::TIMESTAMP)
+    if (field.timestamp_options().has_auto_updates() &&
+        field.timestamp_options().auto_updates())
+      destination.append(" ON UPDATE CURRENT_TIMESTAMP", 28);
+
   if (field.has_comment())
   {
-    destination.append(" COMMENT ");
-    append_escaped_string(&destination, field.comment(), quoted_default);
+    destination.append(" COMMENT ", 9);
+    destination.push_back(quoted_identifier);
+    destination.append(field.comment());
+    destination.push_back(quoted_identifier);
   }
   return NONE;
 }
@@ -1570,6 +1235,7 @@ bool shouldQuoteFieldValue(Table::Field::FieldType in_type)
   case Table::Field::DECIMAL:
   case Table::Field::INTEGER:
   case Table::Field::BIGINT:
+  case Table::Field::ENUM:
     return false;
   default:
     return true;
@@ -1586,15 +1252,12 @@ Table::Field::FieldType internalFieldTypeToFieldProtoType(enum enum_field_types
   case DRIZZLE_TYPE_NULL:
     assert(false); /* Not a user definable type */
     return Table::Field::INTEGER; /* unreachable */
-  case DRIZZLE_TYPE_MICROTIME:
   case DRIZZLE_TYPE_TIMESTAMP:
-    return Table::Field::EPOCH;
+    return Table::Field::TIMESTAMP;
   case DRIZZLE_TYPE_LONGLONG:
     return Table::Field::BIGINT;
   case DRIZZLE_TYPE_DATETIME:
     return Table::Field::DATETIME;
-  case DRIZZLE_TYPE_TIME:
-    return Table::Field::TIME;
   case DRIZZLE_TYPE_DATE:
     return Table::Field::DATE;
   case DRIZZLE_TYPE_VARCHAR:
@@ -1605,55 +1268,11 @@ Table::Field::FieldType internalFieldTypeToFieldProtoType(enum enum_field_types
     return Table::Field::ENUM;
   case DRIZZLE_TYPE_BLOB:
     return Table::Field::BLOB;
-  case DRIZZLE_TYPE_UUID:
-    return Table::Field::UUID;
-  case DRIZZLE_TYPE_BOOLEAN:
-    return Table::Field::BOOLEAN;
-  case DRIZZLE_TYPE_IPV6:
-    return Table::Field::IPV6;
   }
 
   assert(false);
   return Table::Field::INTEGER; /* unreachable */
 }
 
-bool transactionContainsBulkSegment(const Transaction &transaction)
-{
-  size_t num_statements= transaction.statement_size();
-  if (num_statements == 0)
-    return false;
-
-  /*
-   * Only INSERT, UPDATE, and DELETE statements can possibly
-   * have bulk segments.  So, we loop through the statements
-   * checking for segment_id > 1 in those specific submessages.
-   */
-  size_t x;
-  for (x= 0; x < num_statements; ++x)
-  {
-    const Statement &statement= transaction.statement(x);
-    Statement::Type type= statement.type();
-
-    switch (type)
-    {
-      case Statement::INSERT:
-        if (statement.insert_data().segment_id() > 1)
-          return true;
-        break;
-      case Statement::UPDATE:
-        if (statement.update_data().segment_id() > 1)
-          return true;
-        break;
-      case Statement::DELETE:
-        if (statement.delete_data().segment_id() > 1)
-          return true;
-        break;
-      default:
-        break;
-    }
-  }
-  return false;
-}
-
 } /* namespace message */
 } /* namespace drizzled */
