@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008-2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2008-2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,16 +18,12 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-
+#include "config.h"
 #include <drizzled/sql_select.h>
 #include <drizzled/nested_join.h>
 #include <drizzled/item/cmpfunc.h>
-#include <drizzled/table.h>
-#include <drizzled/optimizer/key_field.h>
-#include <drizzled/optimizer/key_use.h>
-#include <drizzled/sql_lex.h>
-#include <drizzled/item/subselect.h>
+#include "drizzled/optimizer/key_field.h"
+#include "drizzled/optimizer/key_use.h"
 
 #include <vector>
 
@@ -40,7 +36,7 @@ void optimizer::add_key_part(DYNAMIC_ARRAY *keyuse_array,
                              optimizer::KeyField *key_field)
 {
   Field *field= key_field->getField();
-  Table *form= field->getTable();
+  Table *form= field->table;
 
   if (key_field->isEqualityCondition() &&
       ! (key_field->getOptimizeFlags() & KEY_OPTIMIZE_EXISTS))
@@ -55,7 +51,7 @@ void optimizer::add_key_part(DYNAMIC_ARRAY *keyuse_array,
       {
         if (field->eq(form->key_info[key].key_part[part].field))
         {
-          optimizer::KeyUse keyuse(field->getTable(),
+          optimizer::KeyUse keyuse(field->table,
                                    key_field->getValue(),
                                    key_field->getValue()->used_tables(),
                                    key,
@@ -65,37 +61,37 @@ void optimizer::add_key_part(DYNAMIC_ARRAY *keyuse_array,
                                    0,
                                    key_field->rejectNullValues(),
                                    key_field->getConditionalGuard());
-          keyuse_array->push_back(&keyuse);
+          insert_dynamic(keyuse_array, (unsigned char*) &keyuse);
         }
       }
     }
   }
 }
 
-void optimizer::add_key_fields_for_nj(Join *join,
+void optimizer::add_key_fields_for_nj(JOIN *join,
                                       TableList *nested_join_table,
                                       optimizer::KeyField **end,
                                       uint32_t *and_level,
                                       vector<optimizer::SargableParam> &sargables)
 {
-  List<TableList>::iterator li(nested_join_table->getNestedJoin()->join_list.begin());
-  List<TableList>::iterator li2(nested_join_table->getNestedJoin()->join_list.begin());
+  List_iterator<TableList> li(nested_join_table->nested_join->join_list);
+  List_iterator<TableList> li2(nested_join_table->nested_join->join_list);
   bool have_another= false;
   table_map tables= 0;
   TableList *table;
-  assert(nested_join_table->getNestedJoin());
+  assert(nested_join_table->nested_join);
 
   while ((table= li++) || (have_another && (li=li2, have_another=false,
                                             (table= li++))))
   {
-    if (table->getNestedJoin())
+    if (table->nested_join)
     {
       if (! table->on_expr)
       {
         /* It's a semi-join nest. Walk into it as if it wasn't a nest */
         have_another= true;
         li2= li;
-        li= List<TableList>::iterator(table->getNestedJoin()->join_list.begin());
+        li= List_iterator<TableList>(table->nested_join->join_list);
       }
       else
         add_key_fields_for_nj(join, table, end, and_level, sargables);
@@ -248,7 +244,7 @@ void optimizer::add_key_field(optimizer::KeyField **key_fields,
   {
     // Don't remove column IS NULL on a LEFT JOIN table
     if (! eq_func || (*value)->type() != Item::NULL_ITEM ||
-        ! field->getTable()->maybe_null || field->null_ptr)
+        ! field->table->maybe_null || field->null_ptr)
       return;					// Not a key. Skip it
     exists_optimize= KEY_OPTIMIZE_EXISTS;
     assert(num_values == 1);
@@ -260,23 +256,23 @@ void optimizer::add_key_field(optimizer::KeyField **key_fields,
     for (uint32_t i= 0; i < num_values; i++)
     {
       used_tables|= (value[i])->used_tables();
-      if (! ((value[i])->used_tables() & (field->getTable()->map | RAND_TABLE_BIT)))
+      if (! ((value[i])->used_tables() & (field->table->map | RAND_TABLE_BIT)))
         optimizable= 1;
     }
     if (! optimizable)
       return;
-    if (! (usable_tables & field->getTable()->map))
+    if (! (usable_tables & field->table->map))
     {
       if (! eq_func || (*value)->type() != Item::NULL_ITEM ||
-          ! field->getTable()->maybe_null || field->null_ptr)
+          ! field->table->maybe_null || field->null_ptr)
         return;					// Can't use left join optimize
       exists_optimize= KEY_OPTIMIZE_EXISTS;
     }
     else
     {
-      JoinTable *stat= field->getTable()->reginfo.join_tab;
+      JoinTable *stat= field->table->reginfo.join_tab;
       key_map possible_keys= field->key_start;
-      possible_keys&= field->getTable()->keys_in_use_for_query;
+      possible_keys&= field->table->keys_in_use_for_query;
       stat[0].keys|= possible_keys;             // Add possible keys
 
       /*
@@ -403,7 +399,7 @@ void optimizer::add_key_equal_fields(optimizer::KeyField **key_fields,
       Add to the set of possible key values every substitution of
       the field for an equal field included into item_equal
     */
-    Item_equal_iterator it(item_equal->begin());
+    Item_equal_iterator it(*item_equal);
     Item_field *item;
     while ((item= it++))
     {
@@ -417,7 +413,7 @@ void optimizer::add_key_equal_fields(optimizer::KeyField **key_fields,
   }
 }
 
-void optimizer::add_key_fields(Join *join,
+void optimizer::add_key_fields(JOIN *join,
                                optimizer::KeyField **key_fields,
                                uint32_t *and_level,
                                COND *cond,
@@ -426,7 +422,7 @@ void optimizer::add_key_fields(Join *join,
 {
   if (cond->type() == Item_func::COND_ITEM)
   {
-    List<Item>::iterator li(((Item_cond*) cond)->argument_list()->begin());
+    List_iterator_fast<Item> li(*((Item_cond*) cond)->argument_list());
     optimizer::KeyField *org_key_fields= *key_fields;
 
     if (((Item_cond*) cond)->functype() == Item_func::COND_AND_FUNC)
@@ -591,7 +587,7 @@ void optimizer::add_key_fields(Join *join,
   case Item_func::OPTIMIZE_EQUAL:
     Item_equal *item_equal= (Item_equal *) cond;
     Item *const_item= item_equal->get_const();
-    Item_equal_iterator it(item_equal->begin());
+    Item_equal_iterator it(*item_equal);
     Item_field *item;
     if (const_item)
     {
@@ -614,7 +610,7 @@ void optimizer::add_key_fields(Join *join,
         field1=field2 as a condition allowing an index access of the table
         with field1 by the keys value of field2.
       */
-      Item_equal_iterator fi(item_equal->begin());
+      Item_equal_iterator fi(*item_equal);
       while ((item= fi++))
       {
         Field *field= item->field;
@@ -627,7 +623,7 @@ void optimizer::add_key_fields(Join *join,
                           sargables);
           }
         }
-        it= item_equal->begin();
+        it.rewind();
       }
     }
     break;
