@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008-2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2008-2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,37 +17,35 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <drizzled/session.h>
-#include <drizzled/sql_select.h>
-#include <drizzled/join.h>
-#include <drizzled/optimizer/range.h>
-#include <drizzled/optimizer/quick_group_min_max_select.h>
-#include <drizzled/optimizer/quick_range.h>
-#include <drizzled/optimizer/quick_range_select.h>
-#include <drizzled/optimizer/sel_arg.h>
-#include <drizzled/internal/m_string.h>
-#include <drizzled/util/functors.h>
-#include <drizzled/key.h>
-#include <drizzled/table.h>
-#include <drizzled/system_variables.h>
+#include "config.h"
+#include "drizzled/session.h"
+#include "drizzled/sql_select.h"
+#include "drizzled/join.h"
+#include "drizzled/optimizer/range.h"
+#include "drizzled/optimizer/quick_group_min_max_select.h"
+#include "drizzled/optimizer/quick_range.h"
+#include "drizzled/optimizer/quick_range_select.h"
+#include "drizzled/optimizer/sel_arg.h"
+#include "drizzled/internal/m_string.h"
+#include "drizzled/util/functors.h"
 
 #include <vector>
 
 using namespace std;
 
-namespace drizzled {
-namespace optimizer {
+namespace drizzled
+{
 
-QuickGroupMinMaxSelect::QuickGroupMinMaxSelect(Table *table,
-                       Join *join_arg,
+optimizer::QuickGroupMinMaxSelect::
+QuickGroupMinMaxSelect(Table *table,
+                       JOIN *join_arg,
                        bool have_min_arg,
                        bool have_max_arg,
-                       KeyPartInfo *min_max_arg_part_arg,
+                       KEY_PART_INFO *min_max_arg_part_arg,
                        uint32_t group_prefix_len_arg,
                        uint32_t group_key_parts_arg,
                        uint32_t used_key_parts_arg,
-                       KeyInfo *index_info_arg,
+                       KEY *index_info_arg,
                        uint32_t use_index,
                        double read_cost_arg,
                        ha_rows records_arg,
@@ -72,7 +70,7 @@ QuickGroupMinMaxSelect::QuickGroupMinMaxSelect(Table *table,
   cursor= head->cursor;
   index= use_index;
   record= head->record[0];
-  tmp_record= head->getUpdateRecord();
+  tmp_record= head->record[1];
   read_time= read_cost_arg;
   records= records_arg;
   used_key_parts= used_key_parts_arg;
@@ -88,7 +86,7 @@ QuickGroupMinMaxSelect::QuickGroupMinMaxSelect(Table *table,
   assert(! parent_alloc);
   if (! parent_alloc)
   {
-    alloc.init(join->session->variables.range_alloc_block_size);
+    memory::init_sql_alloc(&alloc, join->session->variables.range_alloc_block_size, 0);
     join->session->mem_root= &alloc;
   }
   else
@@ -96,17 +94,20 @@ QuickGroupMinMaxSelect::QuickGroupMinMaxSelect(Table *table,
 }
 
 
-int QuickGroupMinMaxSelect::init()
+int optimizer::QuickGroupMinMaxSelect::init()
 {
   if (group_prefix) /* Already initialized. */
     return 0;
 
-  last_prefix= alloc.alloc(group_prefix_len);
+  if (! (last_prefix= (unsigned char*) alloc_root(&alloc, group_prefix_len)))
+      return 1;
   /*
     We may use group_prefix to store keys with all select fields, so allocate
     enough space for it.
   */
-  group_prefix= alloc.alloc(real_prefix_len + min_max_arg_len);
+  if (! (group_prefix= (unsigned char*) alloc_root(&alloc,
+                                                   real_prefix_len + min_max_arg_len)))
+    return 1;
 
   if (key_infix_len > 0)
   {
@@ -114,17 +115,33 @@ int QuickGroupMinMaxSelect::init()
       The memory location pointed to by key_infix will be deleted soon, so
       allocate a new buffer and copy the key_infix into it.
     */
-    unsigned char *tmp_key_infix= alloc.alloc(key_infix_len);
+    unsigned char *tmp_key_infix= (unsigned char*) alloc_root(&alloc, key_infix_len);
+    if (! tmp_key_infix)
+      return 1;
     memcpy(tmp_key_infix, this->key_infix, key_infix_len);
     this->key_infix= tmp_key_infix;
   }
 
   if (min_max_arg_part)
   {
-    min_functions= have_min ? new List<Item_sum> : NULL;
-    max_functions= have_max ? new List<Item_sum> : NULL;
+    if (have_min)
+    {
+      if (! (min_functions= new List<Item_sum>))
+        return 1;
+    }
+    else
+      min_functions= NULL;
+    if (have_max)
+    {
+      if (! (max_functions= new List<Item_sum>))
+        return 1;
+    }
+    else
+      max_functions= NULL;
+
+    Item_sum *min_max_item= NULL;
     Item_sum **func_ptr= join->sum_funcs;
-    while (Item_sum* min_max_item= *(func_ptr++))
+    while ((min_max_item= *(func_ptr++)))
     {
       if (have_min && (min_max_item->sum_func() == Item_sum::MIN_FUNC))
         min_functions->push_back(min_max_item);
@@ -133,18 +150,27 @@ int QuickGroupMinMaxSelect::init()
     }
 
     if (have_min)
-      min_functions_it= new List<Item_sum>::iterator(min_functions->begin());
+    {
+      if (! (min_functions_it= new List_iterator<Item_sum>(*min_functions)))
+        return 1;
+    }
+
     if (have_max)
-      max_functions_it= new List<Item_sum>::iterator(max_functions->begin());
+    {
+      if (! (max_functions_it= new List_iterator<Item_sum>(*max_functions)))
+        return 1;
+    }
   }
+
   return 0;
 }
 
-QuickGroupMinMaxSelect::~QuickGroupMinMaxSelect()
+
+optimizer::QuickGroupMinMaxSelect::~QuickGroupMinMaxSelect()
 {
   if (cursor->inited != Cursor::NONE)
   {
-    cursor->endIndexScan();
+    cursor->ha_index_end();
   }
   if (min_max_arg_part)
   {
@@ -153,16 +179,16 @@ QuickGroupMinMaxSelect::~QuickGroupMinMaxSelect()
              DeletePtr());
   }
   min_max_ranges.clear();
-  alloc.free_root(MYF(0));
+  free_root(&alloc,MYF(0));
   delete min_functions_it;
   delete max_functions_it;
   delete quick_prefix_select;
 }
 
 
-bool QuickGroupMinMaxSelect::add_range(SEL_ARG *sel_range)
+bool optimizer::QuickGroupMinMaxSelect::add_range(optimizer::SEL_ARG *sel_range)
 {
-  QuickRange *range= NULL;
+  optimizer::QuickRange *range= NULL;
   uint32_t range_flag= sel_range->min_flag | sel_range->max_flag;
 
   /* Skip (-inf,+inf) ranges, e.g. (x < 5 or x > 4). */
@@ -179,7 +205,7 @@ bool QuickGroupMinMaxSelect::add_range(SEL_ARG *sel_range)
                     min_max_arg_len) == 0)
       range_flag|= EQ_RANGE;  /* equality condition */
   }
-  range= new QuickRange(sel_range->min_value,
+  range= new optimizer::QuickRange(sel_range->min_value,
                                    min_max_arg_len,
                                    make_keypart_map(sel_range->part),
                                    sel_range->max_value,
@@ -193,24 +219,31 @@ bool QuickGroupMinMaxSelect::add_range(SEL_ARG *sel_range)
 }
 
 
-void QuickGroupMinMaxSelect::adjust_prefix_ranges()
+void optimizer::QuickGroupMinMaxSelect::adjust_prefix_ranges()
 {
   if (quick_prefix_select &&
       group_prefix_len < quick_prefix_select->max_used_key_length)
   {
-    DYNAMIC_ARRAY& arr= quick_prefix_select->ranges;
-    for (size_t inx= 0; inx < arr.size(); inx++)
-      reinterpret_cast<QuickRange**>(arr.buffer)[inx]->flag &= ~(NEAR_MIN | NEAR_MAX);
+    DYNAMIC_ARRAY *arr= NULL;
+    uint32_t inx;
+
+    for (inx= 0, arr= &quick_prefix_select->ranges; inx < arr->elements; inx++)
+    {
+      optimizer::QuickRange *range= NULL;
+
+      get_dynamic(arr, (unsigned char*)&range, inx);
+      range->flag &= ~(NEAR_MIN | NEAR_MAX);
+    }
   }
 }
 
 
-void QuickGroupMinMaxSelect::update_key_stat()
+void optimizer::QuickGroupMinMaxSelect::update_key_stat()
 {
   max_used_key_length= real_prefix_len;
   if (! min_max_ranges.empty())
   {
-    QuickRange *cur_range= NULL;
+    optimizer::QuickRange *cur_range= NULL;
     if (have_min)
     { /* Check if the right-most range has a lower boundary. */
       cur_range= min_max_ranges.back();
@@ -249,12 +282,12 @@ void QuickGroupMinMaxSelect::update_key_stat()
 }
 
 
-int QuickGroupMinMaxSelect::reset(void)
+int optimizer::QuickGroupMinMaxSelect::reset(void)
 {
   int result;
 
   cursor->extra(HA_EXTRA_KEYREAD); /* We need only the key attributes */
-  if ((result= cursor->startIndexScan(index,1)))
+  if ((result= cursor->ha_index_init(index,1)))
     return result;
   if (quick_prefix_select && quick_prefix_select->reset())
     return 0;
@@ -268,7 +301,7 @@ int QuickGroupMinMaxSelect::reset(void)
 }
 
 
-int QuickGroupMinMaxSelect::get_next()
+int optimizer::QuickGroupMinMaxSelect::get_next()
 {
   int min_res= 0;
   int max_res= 0;
@@ -348,7 +381,7 @@ int QuickGroupMinMaxSelect::get_next()
 }
 
 
-int QuickGroupMinMaxSelect::next_min()
+int optimizer::QuickGroupMinMaxSelect::next_min()
 {
   int result= 0;
 
@@ -413,16 +446,23 @@ int QuickGroupMinMaxSelect::next_min()
 }
 
 
-int QuickGroupMinMaxSelect::next_max()
+int optimizer::QuickGroupMinMaxSelect::next_max()
 {
+  int result= 0;
+
   /* Get the last key in the (possibly extended) group. */
-  return min_max_ranges.empty()
-    ? cursor->index_read_map(record, group_prefix, make_prev_keypart_map(real_key_parts), HA_READ_PREFIX_LAST)
-    : next_max_in_range();
+  if (! min_max_ranges.empty())
+    result= next_max_in_range();
+  else
+    result= cursor->index_read_map(record,
+                                   group_prefix,
+                                   make_prev_keypart_map(real_key_parts),
+                                   HA_READ_PREFIX_LAST);
+  return result;
 }
 
 
-int QuickGroupMinMaxSelect::next_prefix()
+int optimizer::QuickGroupMinMaxSelect::next_prefix()
 {
   int result= 0;
 
@@ -468,11 +508,11 @@ int QuickGroupMinMaxSelect::next_prefix()
 }
 
 
-int QuickGroupMinMaxSelect::next_min_in_range()
+int optimizer::QuickGroupMinMaxSelect::next_min_in_range()
 {
   ha_rkey_function find_flag;
   key_part_map keypart_map;
-  QuickRange *cur_range= NULL;
+  optimizer::QuickRange *cur_range= NULL;
   bool found_null= false;
   int result= HA_ERR_KEY_NOT_FOUND;
   basic_string<unsigned char> max_key;
@@ -481,7 +521,9 @@ int QuickGroupMinMaxSelect::next_min_in_range()
 
   assert(! min_max_ranges.empty());
 
-  for (vector<QuickRange *>::iterator it= min_max_ranges.begin(); it != min_max_ranges.end(); ++it)
+  for (vector<optimizer::QuickRange *>::iterator it= min_max_ranges.begin();
+       it != min_max_ranges.end();
+       ++it)
   { /* Search from the left-most range to the right. */
     cur_range= *it;
 
@@ -538,7 +580,7 @@ int QuickGroupMinMaxSelect::next_min_in_range()
         Remember this key, and continue looking for a non-NULL key that
         satisfies some other condition.
       */
-      memcpy(tmp_record, record, head->getShare()->rec_buff_length);
+      memcpy(tmp_record, record, head->s->rec_buff_length);
       found_null= true;
       continue;
     }
@@ -579,25 +621,25 @@ int QuickGroupMinMaxSelect::next_min_in_range()
   */
   if (found_null && result)
   {
-    memcpy(record, tmp_record, head->getShare()->rec_buff_length);
+    memcpy(record, tmp_record, head->s->rec_buff_length);
     result= 0;
   }
   return result;
 }
 
 
-int QuickGroupMinMaxSelect::next_max_in_range()
+int optimizer::QuickGroupMinMaxSelect::next_max_in_range()
 {
   ha_rkey_function find_flag;
   key_part_map keypart_map;
-  QuickRange *cur_range= NULL;
+  optimizer::QuickRange *cur_range= NULL;
   int result= 0;
   basic_string<unsigned char> min_key;
   min_key.reserve(real_prefix_len + min_max_arg_len);
 
   assert(! min_max_ranges.empty());
 
-  for (vector<QuickRange *>::reverse_iterator rit= min_max_ranges.rbegin();
+  for (vector<optimizer::QuickRange *>::reverse_iterator rit= min_max_ranges.rbegin();
        rit != min_max_ranges.rend();
        ++rit)
   { /* Search from the right-most range to the left. */
@@ -676,24 +718,28 @@ int QuickGroupMinMaxSelect::next_max_in_range()
 }
 
 
-void QuickGroupMinMaxSelect::update_min_result()
+void optimizer::QuickGroupMinMaxSelect::update_min_result()
 {
-  *min_functions_it= min_functions->begin();
-  for (Item_sum *min_func; (min_func= (*min_functions_it)++); )
+  Item_sum *min_func= NULL;
+
+  min_functions_it->rewind();
+  while ((min_func= (*min_functions_it)++))
     min_func->reset();
 }
 
 
-void QuickGroupMinMaxSelect::update_max_result()
+void optimizer::QuickGroupMinMaxSelect::update_max_result()
 {
-  *max_functions_it= max_functions->begin();
-  for (Item_sum *max_func; (max_func= (*max_functions_it)++); )
+  Item_sum *max_func= NULL;
+
+  max_functions_it->rewind();
+  while ((max_func= (*max_functions_it)++))
     max_func->reset();
 }
 
 
-void QuickGroupMinMaxSelect::add_keys_and_lengths(string *key_names,
-                                                             string *used_lengths)
+void optimizer::QuickGroupMinMaxSelect::add_keys_and_lengths(String *key_names,
+                                                             String *used_lengths)
 {
   char buf[64];
   key_names->append(index_info->name);
@@ -701,5 +747,4 @@ void QuickGroupMinMaxSelect::add_keys_and_lengths(string *key_names,
   used_lengths->append(buf, length);
 }
 
-}
 } /* namespace drizzled */
