@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008-2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2008-2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,35 +17,34 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-
-#include <drizzled/session.h>
-#include <drizzled/optimizer/quick_range.h>
-#include <drizzled/optimizer/quick_range_select.h>
-#include <drizzled/internal/m_string.h>
-#include <drizzled/current_session.h>
-#include <drizzled/key.h>
-#include <drizzled/table.h>
-#include <drizzled/util/test.h>
-#include <drizzled/system_variables.h>
-
+#include "config.h"
+#include "drizzled/session.h"
+#include "drizzled/optimizer/quick_range.h"
+#include "drizzled/optimizer/quick_range_select.h"
+#include "drizzled/sql_bitmap.h"
+#include "drizzled/internal/m_string.h"
 #include <fcntl.h>
+#include "drizzled/memory/multi_malloc.h"
 
 using namespace std;
 
-namespace drizzled {
+namespace drizzled
+{
 
 
 optimizer::QuickRangeSelect::QuickRangeSelect(Session *session,
                                               Table *table,
                                               uint32_t key_nr,
                                               bool no_alloc,
-                                              memory::Root *parent_alloc)
+                                              memory::Root *parent_alloc,
+                                              bool *create_error)
   :
+    mrr_flags(0),
+    alloc(),
     cursor(NULL),
     ranges(),
     in_ror_merged_scan(false),
-    column_bitmap(NULL),
+    column_bitmap(),
     save_read_set(NULL),
     save_write_set(NULL),
     free_file(false),
@@ -53,24 +52,26 @@ optimizer::QuickRangeSelect::QuickRangeSelect(Session *session,
     last_range(NULL),
     qr_traversal_ctx(),
     mrr_buf_size(0),
+    mrr_buf_desc(NULL),
     key_parts(NULL),
-    dont_free(false),
-    mrr_flags(0),
-    alloc()
+    dont_free(false)
 {
+  my_bitmap_map *bitmap= NULL;
+
   sorted= 0;
   index= key_nr;
   head= table;
   key_part_info= head->key_info[index].key_part;
-  ranges.init(sizeof(optimizer::QuickRange*), 16, 16);
+  my_init_dynamic_array(&ranges, sizeof(optimizer::QuickRange*), 16, 16);
 
   /* 'session' is not accessible in QuickRangeSelect::reset(). */
   mrr_buf_size= session->variables.read_rnd_buff_size;
+  mrr_buf_desc= NULL;
 
   if (! no_alloc && ! parent_alloc)
   {
     // Allocates everything through the internal memroot
-    alloc.init(session->variables.range_alloc_block_size);
+    memory::init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
     session->mem_root= &alloc;
   }
   else
@@ -82,7 +83,22 @@ optimizer::QuickRangeSelect::QuickRangeSelect(Session *session,
   record= head->record[0];
   save_read_set= head->read_set;
   save_write_set= head->write_set;
-  column_bitmap= new boost::dynamic_bitset<>(table->getShare()->sizeFields());
+
+  /* Allocate a bitmap for used columns. Using memory::sql_alloc instead of malloc
+     simply as a "fix" to the MySQL 6.0 code that also free()s it at the
+     same time we destroy the mem_root.
+   */
+
+  bitmap= reinterpret_cast<my_bitmap_map*>(memory::sql_alloc(head->s->column_bitmap_size));
+  if (! bitmap)
+  {
+    column_bitmap.setBitmap(NULL);
+    *create_error= 1;
+  }
+  else
+  {
+    column_bitmap.init(bitmap, head->s->fields);
+  }
 }
 
 
@@ -90,7 +106,7 @@ int optimizer::QuickRangeSelect::init()
 {
   if (cursor->inited != Cursor::NONE)
     cursor->ha_index_or_rnd_end();
-  return (cursor->startIndexScan(index, 1));
+  return (cursor->ha_index_init(index, 1));
 }
 
 
@@ -121,19 +137,22 @@ optimizer::QuickRangeSelect::~QuickRangeSelect()
         delete cursor;
       }
     }
-    ranges.free(); /* ranges are allocated in alloc */
-    delete column_bitmap;
-    alloc.free_root(MYF(0));
+    delete_dynamic(&ranges); /* ranges are allocated in alloc */
+    free_root(&alloc,MYF(0));
+  }
+  head->column_bitmaps_set(save_read_set, save_write_set);
+  assert(mrr_buf_desc == NULL);
+  if (mrr_buf_desc)
+  {
+    free(mrr_buf_desc);
   }
-  head->column_bitmaps_set(*save_read_set, *save_write_set);
 }
 
 
 int optimizer::QuickRangeSelect::init_ror_merged_scan(bool reuse_handler)
 {
-  Cursor* org_file;
-  Cursor* save_file= cursor;
-  Session* session;
+  Cursor *save_file= cursor, *org_file;
+  Session *session;
 
   in_ror_merged_scan= 1;
   if (reuse_handler)
@@ -142,7 +161,7 @@ int optimizer::QuickRangeSelect::init_ror_merged_scan(bool reuse_handler)
     {
       return 0;
     }
-    head->column_bitmaps_set(*column_bitmap, *column_bitmap);
+    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
     goto end;
   }
 
@@ -154,7 +173,7 @@ int optimizer::QuickRangeSelect::init_ror_merged_scan(bool reuse_handler)
   }
 
   session= head->in_use;
-  if (not (cursor= head->cursor->clone(session->mem_root)))
+  if (! (cursor= head->cursor->clone(session->mem_root)))
   {
     /*
       Manually set the error flag. Note: there seems to be quite a few
@@ -168,7 +187,7 @@ int optimizer::QuickRangeSelect::init_ror_merged_scan(bool reuse_handler)
     goto failure;
   }
 
-  head->column_bitmaps_set(*column_bitmap, *column_bitmap);
+  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
 
   if (cursor->ha_external_lock(session, F_RDLCK))
     goto failure;
@@ -192,20 +211,20 @@ end:
   org_file= head->cursor;
   head->cursor= cursor;
   /* We don't have to set 'head->keyread' here as the 'cursor' is unique */
-  if (not head->no_keyread)
+  if (! head->no_keyread)
   {
     head->key_read= 1;
     head->mark_columns_used_by_index(index);
   }
   head->prepare_for_position();
   head->cursor= org_file;
-  *column_bitmap|= *head->read_set;
-  head->column_bitmaps_set(*column_bitmap, *column_bitmap);
+  column_bitmap= *head->read_set;
+  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
 
   return 0;
 
 failure:
-  head->column_bitmaps_set(*save_read_set, *save_write_set);
+  head->column_bitmaps_set(save_read_set, save_write_set);
   delete cursor;
   cursor= save_file;
   return 0;
@@ -220,12 +239,12 @@ void optimizer::QuickRangeSelect::save_last_pos()
 
 bool optimizer::QuickRangeSelect::unique_key_range() const
 {
-  if (ranges.size() == 1)
+  if (ranges.elements == 1)
   {
     optimizer::QuickRange *tmp= *((optimizer::QuickRange**)ranges.buffer);
     if ((tmp->flag & (EQ_RANGE | NULL_RANGE)) == EQ_RANGE)
     {
-      KeyInfo *key=head->key_info+index;
+      KEY *key=head->key_info+index;
       return ((key->flags & (HA_NOSAME)) == HA_NOSAME &&
 	      key->key_length == tmp->min_length);
     }
@@ -236,21 +255,49 @@ bool optimizer::QuickRangeSelect::unique_key_range() const
 
 int optimizer::QuickRangeSelect::reset()
 {
+  uint32_t buf_size= 0;
+  unsigned char *mrange_buff= NULL;
   int error= 0;
+  HANDLER_BUFFER empty_buf;
   last_range= NULL;
   cur_range= (optimizer::QuickRange**) ranges.buffer;
 
-  if (cursor->inited == Cursor::NONE && (error= cursor->startIndexScan(index, 1)))
+  if (cursor->inited == Cursor::NONE && (error= cursor->ha_index_init(index, 1)))
   {
     return error;
   }
 
-  /*
-    (in the past) Allocate buffer if we need one but haven't allocated it yet 
-    There is a later assert in th code that hoped to catch random free() that might
-    have done this.
-  */
-  assert(not (mrr_buf_size));
+  /* Allocate buffer if we need one but haven't allocated it yet */
+  if (mrr_buf_size && ! mrr_buf_desc)
+  {
+    buf_size= mrr_buf_size;
+    while (buf_size && ! memory::multi_malloc(false,
+                                              &mrr_buf_desc,
+                                              sizeof(*mrr_buf_desc),
+                                              &mrange_buff,
+                                              buf_size,
+                                              NULL))
+    {
+      /* Try to shrink the buffers until both are 0. */
+      buf_size/= 2;
+    }
+    if (! mrr_buf_desc)
+    {
+      return HA_ERR_OUT_OF_MEM;
+    }
+
+    /* Initialize the Cursor buffer. */
+    mrr_buf_desc->buffer= mrange_buff;
+    mrr_buf_desc->buffer_end= mrange_buff + buf_size;
+    mrr_buf_desc->end_of_used_area= mrange_buff;
+  }
+
+  if (! mrr_buf_desc)
+  {
+    empty_buf.buffer= NULL;
+    empty_buf.buffer_end= NULL;
+    empty_buf.end_of_used_area= NULL;
+  }
 
   if (sorted)
   {
@@ -262,8 +309,9 @@ int optimizer::QuickRangeSelect::reset()
   };
   error= cursor->multi_range_read_init(&seq_funcs,
                                        (void*) this,
-                                       ranges.size(),
-                                       mrr_flags);
+                                       ranges.elements,
+                                       mrr_flags,
+                                       mrr_buf_desc ? mrr_buf_desc : &empty_buf);
   return error;
 }
 
@@ -277,7 +325,7 @@ int optimizer::QuickRangeSelect::get_next()
       We don't need to signal the bitmap change as the bitmap is always the
       same for this head->cursor
     */
-    head->column_bitmaps_set(*column_bitmap, *column_bitmap);
+    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
   }
 
   int result= cursor->multi_range_read_next(&dummy);
@@ -285,7 +333,7 @@ int optimizer::QuickRangeSelect::get_next()
   if (in_ror_merged_scan)
   {
     /* Restore bitmaps set on entry */
-    head->column_bitmaps_set(*save_read_set, *save_write_set);
+    head->column_bitmaps_set(save_read_set, save_write_set);
   }
   return result;
 }
@@ -311,7 +359,7 @@ int optimizer::QuickRangeSelect::get_next_prefix(uint32_t prefix_length,
         return result;
     }
 
-    uint32_t count= ranges.size() - (cur_range - (optimizer::QuickRange**) ranges.buffer);
+    uint32_t count= ranges.elements - (cur_range - (optimizer::QuickRange**) ranges.buffer);
     if (count == 0)
     {
       /* Ranges have already been used up before. None is left for read. */
@@ -354,12 +402,12 @@ bool optimizer::QuickRangeSelect::row_in_ranges()
 {
   optimizer::QuickRange *res= NULL;
   uint32_t min= 0;
-  uint32_t max= ranges.size() - 1;
+  uint32_t max= ranges.elements - 1;
   uint32_t mid= (max + min) / 2;
 
   while (min != max)
   {
-    if (cmp_next(reinterpret_cast<optimizer::QuickRange**>(ranges.buffer)[mid]))
+    if (cmp_next(*(optimizer::QuickRange**)dynamic_array_ptr(&ranges, mid)))
     {
       /* current row value > mid->max */
       min= mid + 1;
@@ -368,8 +416,8 @@ bool optimizer::QuickRangeSelect::row_in_ranges()
       max= mid;
     mid= (min + max) / 2;
   }
-  res= reinterpret_cast<optimizer::QuickRange**>(ranges.buffer)[mid];
-  return not cmp_next(res) && not cmp_prev(res);
+  res= *(optimizer::QuickRange**)dynamic_array_ptr(&ranges, mid);
+  return (! cmp_next(res) && ! cmp_prev(res));
 }
 
 
@@ -423,19 +471,19 @@ int optimizer::QuickRangeSelect::cmp_prev(optimizer::QuickRange *range_arg)
 }
 
 
-void optimizer::QuickRangeSelect::add_info_string(string *str)
+void optimizer::QuickRangeSelect::add_info_string(String *str)
 {
-  KeyInfo *key_info= head->key_info + index;
+  KEY *key_info= head->key_info + index;
   str->append(key_info->name);
 }
 
 
-void optimizer::QuickRangeSelect::add_keys_and_lengths(string *key_names,
-                                                       string *used_lengths)
+void optimizer::QuickRangeSelect::add_keys_and_lengths(String *key_names,
+                                                       String *used_lengths)
 {
   char buf[64];
   uint32_t length;
-  KeyInfo *key_info= head->key_info + index;
+  KEY *key_info= head->key_info + index;
   key_names->append(key_info->name);
   length= internal::int64_t2str(max_used_key_length, buf, 10) - buf;
   used_lengths->append(buf, length);
@@ -451,13 +499,12 @@ void optimizer::QuickRangeSelect::add_keys_and_lengths(string *key_names,
   which handle the ranges and implement the get_next() function.  But
   for now, this seems to work right at least.
  */
-optimizer::QuickSelectDescending::QuickSelectDescending(optimizer::QuickRangeSelect *q, uint32_t used_key_parts_arg, bool *)
+optimizer::QuickSelectDescending::QuickSelectDescending(optimizer::QuickRangeSelect *q, uint32_t, bool *)
   :
-  optimizer::QuickRangeSelect(*q),
-  used_key_parts(used_key_parts_arg)
+    optimizer::QuickRangeSelect(*q)
 {
   optimizer::QuickRange **pr= (optimizer::QuickRange**) ranges.buffer;
-  optimizer::QuickRange **end_range= pr + ranges.size();
+  optimizer::QuickRange **end_range= pr + ranges.elements;
   for (; pr != end_range; pr++)
   {
     rev_ranges.push_back(*pr);
@@ -465,11 +512,15 @@ optimizer::QuickSelectDescending::QuickSelectDescending(optimizer::QuickRangeSel
   rev_it= rev_ranges.begin();
 
   /* Remove EQ_RANGE flag for keys that are not using the full key */
-  BOOST_FOREACH(QuickRange* it, rev_ranges)
+  for (vector<optimizer::QuickRange *>::iterator it= rev_ranges.begin();
+       it != rev_ranges.end();
+       ++it)
   {
-    if ((it->flag & EQ_RANGE) && head->key_info[index].key_length != it->max_length)
+    optimizer::QuickRange *r= *it;
+    if ((r->flag & EQ_RANGE) &&
+        head->key_info[index].key_length != r->max_length)
     {
-      it->flag&= ~EQ_RANGE;
+      r->flag&= ~EQ_RANGE;
     }
   }
   q->dont_free= 1; // Don't free shared mem
@@ -494,8 +545,7 @@ int optimizer::QuickSelectDescending::get_next()
     int result;
     if (last_range)
     {						// Already read through key
-      result= ((last_range->flag & EQ_RANGE &&
-               used_key_parts <= head->key_info[index].key_parts) ?
+      result= ((last_range->flag & EQ_RANGE) ?
 		           cursor->index_next_same(record, last_range->min_key,
 					                             last_range->min_length) :
 		           cursor->index_prev(record));
@@ -526,8 +576,7 @@ int optimizer::QuickSelectDescending::get_next()
       continue;
     }
 
-    if (last_range->flag & EQ_RANGE
-        && used_key_parts <= head->key_info[index].key_parts)
+    if (last_range->flag & EQ_RANGE)
     {
       result = cursor->index_read_map(record,
                                       last_range->max_key,
@@ -537,8 +586,6 @@ int optimizer::QuickSelectDescending::get_next()
     else
     {
       assert(last_range->flag & NEAR_MAX ||
-             (last_range->flag & EQ_RANGE &&
-              used_key_parts > head->key_info[index].key_parts) ||
              range_reads_after_key(last_range));
       result= cursor->index_read_map(record,
                                      last_range->max_key,
