@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008-2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2008-2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -51,16 +51,11 @@
   (assuming a index for column d of table t2 is defined)
 */
 
-#include <config.h>
-
-#include <drizzled/sql_select.h>
-#include <drizzled/item/sum.h>
-#include <drizzled/item/cmpfunc.h>
-#include <drizzled/optimizer/sum.h>
-#include <drizzled/plugin/storage_engine.h>
-#include <drizzled/table_list.h>
-#include <drizzled/key.h>
-#include <drizzled/error.h>
+#include "config.h"
+#include "drizzled/sql_select.h"
+#include "drizzled/item/sum.h"
+#include "drizzled/item/cmpfunc.h"
+#include "drizzled/optimizer/sum.h"
 
 namespace drizzled
 {
@@ -115,7 +110,7 @@ static uint64_t get_exact_record_count(TableList *tables)
 
 int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
 {
-  List<Item>::iterator it(all_fields.begin());
+  List_iterator_fast<Item> it(all_fields);
   int const_result= 1;
   bool recalc_const_item= false;
   uint64_t count= 1;
@@ -140,7 +135,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
   for (TableList *tl= tables; tl; tl= tl->next_leaf)
   {
     TableList *embedded= NULL;
-    for (embedded= tl; embedded; embedded= embedded->getEmbedding())
+    for (embedded= tl; embedded; embedded= embedded->embedding)
     {
       if (embedded->on_expr)
         break;
@@ -244,7 +239,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
 
               ref.key_buff= key_buff;
               Item_field *item_field= (Item_field*) (expr->real_item());
-              Table *table= item_field->field->getTable();
+              Table *table= item_field->field->table;
 
               /*
                  Look for a partial key that can be used for optimization.
@@ -266,17 +261,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
                 const_result= 0;
                 break;
               }
-              error= table->cursor->startIndexScan(static_cast<uint32_t>(ref.key), 1);
-              if (error)
-              {
-                if (table->key_read)
-                {
-                  table->key_read= 0;
-                  table->cursor->extra(HA_EXTRA_NO_KEYREAD);
-                }
-                table->print_error(error, MYF(0));
-                return error;
-              }
+              error= table->cursor->ha_index_init(static_cast<uint32_t>(ref.key), 1);
 
               if (! ref.key_length)
               {
@@ -366,7 +351,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
                 table->key_read= 0;
                 table->cursor->extra(HA_EXTRA_NO_KEYREAD);
               }
-              table->cursor->endIndexScan();
+              table->cursor->ha_index_end();
               if (error)
               {
                 if (error == HA_ERR_KEY_NOT_FOUND || error == HA_ERR_END_OF_FILE)
@@ -422,7 +407,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
 
               ref.key_buff= key_buff;
               Item_field *item_field= (Item_field*) (expr->real_item());
-              Table *table= item_field->field->getTable();
+              Table *table= item_field->field->table;
 
               /*
                  Look for a partial key that can be used for optimization.
@@ -444,7 +429,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
                 const_result= 0;
                 break;
               }
-              error= table->cursor->startIndexScan(static_cast<uint32_t>(ref.key), 1);
+              error= table->cursor->ha_index_init(static_cast<uint32_t>(ref.key), 1);
 
               if (! ref.key_length)
               {
@@ -474,7 +459,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
                 table->key_read= 0;
                 table->cursor->extra(HA_EXTRA_NO_KEYREAD);
               }
-              table->cursor->endIndexScan();
+              table->cursor->ha_index_end();
               if (error)
               {
                 if (error == HA_ERR_KEY_NOT_FOUND || error == HA_ERR_END_OF_FILE)
@@ -556,7 +541,7 @@ bool optimizer::simple_pred(Item_func *func_item, Item **args, bool &inv_order)
     /* MULT_EQUAL_FUNC */
     {
       Item_equal *item_equal= (Item_equal *) func_item;
-      Item_equal_iterator it(item_equal->begin());
+      Item_equal_iterator it(*item_equal);
       args[0]= it++;
       if (it++)
       {
@@ -662,8 +647,8 @@ bool optimizer::simple_pred(Item_func *func_item, Item **args, bool &inv_order)
 */
 static bool matching_cond(bool max_fl,
                           table_reference_st *ref,
-                          KeyInfo *keyinfo,
-                          KeyPartInfo *field_part,
+                          KEY *keyinfo,
+                          KEY_PART_INFO *field_part,
                           COND *cond,
                           key_part_map *key_part_used,
                           uint32_t *range_fl,
@@ -677,7 +662,7 @@ static bool matching_cond(bool max_fl,
 
   field->setWriteSet();
 
-  if (! (cond->used_tables() & field->getTable()->map))
+  if (! (cond->used_tables() & field->table->map))
   {
     /* Condition doesn't restrict the used table */
     return 1;
@@ -690,7 +675,7 @@ static bool matching_cond(bool max_fl,
     }
 
     /* AND */
-    List<Item>::iterator li(((Item_cond*) cond)->argument_list()->begin());
+    List_iterator_fast<Item> li(*((Item_cond*) cond)->argument_list());
     Item *item;
     while ((item= li++))
     {
@@ -763,7 +748,7 @@ static bool matching_cond(bool max_fl,
 
   /* Check if field is part of the tested partial key */
   unsigned char *key_ptr= ref->key_buff;
-  KeyPartInfo *part= NULL;
+  KEY_PART_INFO *part= NULL;
   for (part= keyinfo->key_part; ; key_ptr+= part++->store_length)
 
   {
@@ -922,16 +907,16 @@ static bool find_key_for_maxmin(bool max_fl,
     return 0; // Not key field
   }
 
-  Table *table= field->getTable();
+  Table *table= field->table;
   uint32_t idx= 0;
 
-  KeyInfo *keyinfo,*keyinfo_end= NULL;
-  for (keyinfo= table->key_info, keyinfo_end= keyinfo+table->getShare()->sizeKeys();
+  KEY *keyinfo,*keyinfo_end= NULL;
+  for (keyinfo= table->key_info, keyinfo_end= keyinfo+table->s->keys;
        keyinfo != keyinfo_end;
        keyinfo++,idx++)
   {
-    KeyPartInfo *part= NULL;
-    KeyPartInfo *part_end= NULL;
+    KEY_PART_INFO *part= NULL;
+    KEY_PART_INFO *part_end= NULL;
     key_part_map key_part_to_use= 0;
     /*
       Perform a check if index is not disabled by ALTER Table
@@ -953,7 +938,7 @@ static bool find_key_for_maxmin(bool max_fl,
       }
 
       /* Check whether the index component is partial */
-      Field *part_field= table->getField(part->fieldnr-1);
+      Field *part_field= table->field[part->fieldnr-1];
       part_field->setWriteSet();
 
       if ((part_field->flags & BLOB_FLAG) ||
@@ -1043,7 +1028,7 @@ static int reckey_in_range(bool max_fl,
                            uint32_t range_fl,
                            uint32_t prefix_len)
 {
-  if (key_cmp_if_same(field->getTable(), ref->key_buff, ref->key, prefix_len))
+  if (key_cmp_if_same(field->table, ref->key_buff, ref->key, prefix_len))
   {
     return 1;
   }
@@ -1072,16 +1057,19 @@ static int maxmin_in_range(bool max_fl, Field* field, COND *cond)
   /* If AND/OR condition */
   if (cond->type() == Item::COND_ITEM)
   {
-    List<Item>::iterator li(((Item_cond*) cond)->argument_list()->begin());
-    while (Item* item= li++)
+    List_iterator_fast<Item> li(*((Item_cond*) cond)->argument_list());
+    Item *item;
+    while ((item= li++))
     {
       if (maxmin_in_range(max_fl, field, item))
+      {
         return 1;
+      }
     }
     return 0;
   }
 
-  if (cond->used_tables() != field->getTable()->map)
+  if (cond->used_tables() != field->table->map)
   {
     return 0;
   }
@@ -1117,8 +1105,9 @@ static int maxmin_in_range(bool max_fl, Field* field, COND *cond)
   case Item_func::EQ_FUNC:
   case Item_func::EQUAL_FUNC:
     break;
-  default:
-    ; // assert(false); // Impossible; Olaf: Not really, assert is hit. BUG?
+  default:                                        // Keep compiler happy
+    assert(1);                               // Impossible
+    break;
   }
   return 0;
 }
