@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,21 +17,28 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/replication_services.h>
-#include <drizzled/plugin/transaction_applier.h>
+#include "drizzled/replication_services.h"
+#include "drizzled/plugin/transaction_applier.h"
+
+using namespace std;
 
 namespace drizzled
 {
 
-bool plugin::TransactionApplier::addPlugin(plugin::TransactionApplier *)
+bool plugin::TransactionApplier::addPlugin(plugin::TransactionApplier *applier)
 {
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  /** @TODO ReplicationServices should indicate error in some way here */
+  replication_services.attachApplier(applier);
   return false;
 }
 
-void plugin::TransactionApplier::removePlugin(plugin::TransactionApplier *)
+void plugin::TransactionApplier::removePlugin(plugin::TransactionApplier *applier)
 {
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  replication_services.detachApplier(applier);
 }
 
 } /* namespace drizzled */
