@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,23 +17,29 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/plugin/plugin.h>
-#include <drizzled/plugin/transaction_replicator.h>
-#include <drizzled/replication_services.h>
+#include "drizzled/plugin/plugin.h"
+#include "drizzled/plugin/transaction_replicator.h"
+#include "drizzled/replication_services.h"
 
-namespace drizzled {
+using namespace std;
+
+namespace drizzled
+{
 
 bool plugin::TransactionReplicator::addPlugin(plugin::TransactionReplicator *replicator)
 {
-  ReplicationServices::attachReplicator(replicator);
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  /** @TODO ReplicationServices should indicate error in some way here */
+  replication_services.attachReplicator(replicator);
   return false;
 }
 
 void plugin::TransactionReplicator::removePlugin(plugin::TransactionReplicator *replicator)
 {
-  ReplicationServices::detachReplicator(replicator);
+  ReplicationServices &replication_services= ReplicationServices::singleton();
+  replication_services.detachReplicator(replicator);
 }
 
 } /* namespace drizzled */
