@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,10 +21,10 @@
 #define DRIZZLED_PLUGIN_VERSION_H
 
 /* Machine comsumable version of the server which was built against. */
-#define DRIZZLE_VERSION_ID 70203
+#define DRIZZLE_VERSION_ID 2010031347
 
 /* A release comment about the branch that was built */
-#define DRIZZLE_RELEASE_COMMENT "drizzle-7.2"
+#define DRIZZLE_RELEASE_COMMENT "trunk"
 
 #endif /* DRIZZLED_PLUGIN_VERSION_H */
 
