@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,22 +17,28 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_PTHREAD_GLOBALS_H
+#define DRIZZLED_PTHREAD_GLOBALS_H
 
 #include <pthread.h>
-#include <boost/thread/mutex.hpp>
-#include <boost/thread/shared_mutex.hpp>
-#include <boost/thread/condition_variable.hpp>
-#include <boost/shared_ptr.hpp>
-#include <drizzled/visibility.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
-typedef boost::shared_ptr<boost::thread> thread_ptr;
+extern pthread_mutex_t LOCK_create_db;
+extern pthread_mutex_t LOCK_open;
+extern pthread_mutex_t LOCK_thread_count;
+extern pthread_mutex_t LOCK_status;
+extern pthread_mutex_t LOCK_global_read_lock;
+extern pthread_mutex_t LOCK_global_system_variables;
 
-extern boost::condition_variable_any COND_refresh;
-extern DRIZZLED_API boost::condition_variable COND_thread_count;
+extern pthread_rwlock_t LOCK_system_variables_hash;
+extern pthread_cond_t COND_refresh;
+extern pthread_cond_t COND_thread_count;
+extern pthread_cond_t COND_global_read_lock;
+extern pthread_attr_t connection_attrib;
 extern pthread_t signal_thread;
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_PTHREAD_GLOBALS_H */
