@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,10 +18,8 @@
  */
 
 
-#pragma once
-
-#include <drizzled/item/subselect.h>
-#include <drizzled/select_result_interceptor.h>
+#ifndef DRIZZLED_SELECT_SUBSELECT_H
+#define DRIZZLED_SELECT_SUBSELECT_H
 
 namespace drizzled
 {
@@ -34,8 +32,9 @@ protected:
 public:
   select_subselect(Item_subselect *item);
   bool send_data(List<Item> &items)=0;
-  bool send_eof() { return 0; }
+  bool send_eof() { return 0; };
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_SELECT_SUBSELECT_H */
