@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,31 +18,34 @@
  */
 
 
-#pragma once
+#ifndef DRIZZLED_SELECT_TO_FILE_H
+#define DRIZZLED_SELECT_TO_FILE_H
 
-#include <drizzled/select_result_interceptor.h>
-#include <drizzled/file_exchange.h>
-#include <boost/filesystem.hpp>
+namespace drizzled
+{
 
-namespace drizzled {
+namespace internal
+{
+typedef struct st_io_cache IO_CACHE;
+}
 
-class select_to_file :
-  public select_result_interceptor
+class select_to_file : public select_result_interceptor
 {
 protected:
   file_exchange *exchange;
   int file;
-  internal::io_cache_st *cache;
+  internal::IO_CACHE *cache;
   ha_rows row_count;
-  boost::filesystem::path path;
+  char path[FN_REFLEN];
 
 public:
   select_to_file(file_exchange *ex);
   virtual ~select_to_file();
-  void send_error(drizzled::error_t errcode,const char *err);
+  void send_error(uint32_t errcode,const char *err);
   bool send_eof();
   void cleanup();
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_SELECT_TO_FILE_H */
