@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,10 +18,8 @@
  */
 
 
-#pragma once
-
-#include <drizzled/select_result_interceptor.h>
-#include <drizzled/tmp_table_param.h>
+#ifndef DRIZZLED_SELECT_UNION_H
+#define DRIZZLED_SELECT_UNION_H
 
 namespace drizzled
 {
@@ -32,9 +30,7 @@ class select_union :public select_result_interceptor
 public:
   Table *table;
 
-  select_union() :table(0) { }
-  ~select_union() { }
-
+  select_union() :table(0) {}
   int prepare(List<Item> &list, Select_Lex_Unit *u);
   bool send_data(List<Item> &items);
   bool send_eof();
@@ -47,3 +43,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_SELECT_UNION_H */
