@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,30 +17,52 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_SQL_LEX_H
+#define DRIZZLED_SQL_LEX_H
 
 /**
   @defgroup Semantic_Analysis Semantic Analysis
 */
 #include <drizzled/message/table.pb.h>
-#include <drizzled/name_resolution_context.h>
-#include <drizzled/table_list.h>
-#include <drizzled/function/math/real.h>
-#include <drizzled/key_part_spec.h>
-#include <drizzled/index_hint.h>
-#include <drizzled/optimizer/explain_plan.h>
+
+#include "drizzled/plugin/function.h"
+#include "drizzled/name_resolution_context.h"
+#include "drizzled/item/subselect.h"
+#include "drizzled/table_list.h"
+#include "drizzled/function/math/real.h"
+#include "drizzled/alter_drop.h"
+#include "drizzled/alter_column.h"
+#include "drizzled/alter_info.h"
+#include "drizzled/key_part_spec.h"
+#include "drizzled/index_hint.h"
+#include "drizzled/statement.h"
+#include "drizzled/optimizer/explain_plan.h"
 
 #include <bitset>
 #include <string>
 
+namespace drizzled
+{
+
+class select_result_interceptor;
+
+/* YACC and LEX Definitions */
+
+/* These may not be declared yet */
+class Table_ident;
+class file_exchange;
+class Lex_Column;
+class Item_outer_ref;
+
+} /* namespace drizzled */
+
 /*
   The following hack is needed because mysql_yacc.cc does not define
   YYSTYPE before including this file
 */
 
 #ifdef DRIZZLE_SERVER
-/* set_var should change to set_var here ... */
-# include <drizzled/sys_var.h>
+# include <drizzled/set_var.h>
 # include <drizzled/item/func.h>
 # ifdef DRIZZLE_YACC
 #  define LEX_YYSTYPE void *
@@ -48,8 +70,7 @@
 #  if defined(DRIZZLE_LEX)
 #   include <drizzled/foreign_key.h>
 #   include <drizzled/lex_symbol.h>
-#   include <drizzled/comp_creator.h>
-#   include <drizzled/sql_yacc.hh>
+#   include <drizzled/sql_yacc.h>
 #   define LEX_YYSTYPE YYSTYPE *
 #  else
 #   define LEX_YYSTYPE void *
@@ -207,6 +228,9 @@ enum olap_type
     Base class for Select_Lex (Select_Lex) &
     Select_Lex_Unit (Select_Lex_Unit)
 */
+class LEX;
+class Select_Lex;
+class Select_Lex_Unit;
 class Select_Lex_Node {
 protected:
   Select_Lex_Node *next, **prev,   /* neighbor list */
@@ -224,8 +248,8 @@ public:
       UNCACHEABLE_EXPLAIN
       UNCACHEABLE_PREPARE
   */
-  std::bitset<8> uncacheable;
-  sub_select_type linkage;
+  uint8_t uncacheable;
+  enum sub_select_type linkage;
   bool no_table_names_allowed; /* used for global order by */
   bool no_error; /* suppress error message (convert it to warnings) */
 
@@ -234,10 +258,10 @@ public:
     return memory::sql_alloc(size);
   }
   static void *operator new(size_t size, memory::Root *mem_root)
-  { return mem_root->alloc(size); }
-  static void operator delete(void*, size_t)
+  { return (void*) alloc_root(mem_root, (uint32_t) size); }
+  static void operator delete(void *, size_t)
   {  }
-  static void operator delete(void*, memory::Root*)
+  static void operator delete(void *, memory::Root *)
   {}
   Select_Lex_Node(): linkage(UNSPECIFIED_TYPE) {}
   virtual ~Select_Lex_Node() {}
@@ -259,17 +283,18 @@ public:
   virtual uint32_t get_in_sum_expr();
   virtual TableList* get_table_list();
   virtual List<Item>* get_item_list();
+  virtual uint32_t get_table_join_options();
   virtual TableList *add_table_to_list(Session *session, Table_ident *table,
-                                       lex_string_t *alias,
-                                       const std::bitset<NUM_OF_TABLE_OPTIONS>& table_options,
-                                       thr_lock_type flags= TL_UNLOCK,
-                                       List<Index_hint> *hints= 0,
-                                       lex_string_t *option= 0);
+                                        LEX_STRING *alias,
+                                        uint32_t table_options,
+                                        thr_lock_type flags= TL_UNLOCK,
+                                        List<Index_hint> *hints= 0,
+                                        LEX_STRING *option= 0);
   virtual void set_lock_for_tables(thr_lock_type)
   {}
 
   friend class Select_Lex_Unit;
-  friend bool new_select(LEX *lex, bool move_down);
+  friend bool mysql_new_select(LEX *lex, bool move_down);
 private:
   void fast_exclude();
 };
@@ -278,6 +303,10 @@ private:
    Select_Lex_Unit - unit of selects (UNION, INTERSECT, ...) group
    Select_Lexs
 */
+class Session;
+class select_result;
+class JOIN;
+class select_union;
 class Select_Lex_Unit: public Select_Lex_Node {
 protected:
   TableList result_table_list;
@@ -353,7 +382,7 @@ public:
   inline void unclean() { cleaned= 0; }
   void reinit_exec_mechanism();
 
-  void print(String *str);
+  void print(String *str, enum_query_type query_type);
 
   bool add_fake_select_lex(Session *session);
   void init_prepare_fake_select_lex(Session *session);
@@ -361,9 +390,10 @@ public:
                      select_result_interceptor *old_result);
   void set_limit(Select_Lex *values);
   void set_session(Session *session_arg) { session= session_arg; }
-  inline bool is_union();
+  inline bool is_union ();
 
-  friend void lex_start(Session*);
+  friend void lex_start(Session *session);
+  friend int subselect_union_engine::exec();
 
   List<Item> *get_unit_column_types();
 };
@@ -371,70 +401,11 @@ public:
 /*
   Select_Lex - store information of parsed SELECT statment
 */
-class Select_Lex : public Select_Lex_Node
+class Select_Lex: public Select_Lex_Node
 {
 public:
-
-  Select_Lex() :
-    context(),
-    db(0),
-    where(0),
-    having(0),
-    cond_value(),
-    having_value(),
-    parent_lex(0),
-    olap(UNSPECIFIED_OLAP_TYPE),
-    table_list(),
-    group_list(),
-    item_list(),
-    interval_list(),
-    is_item_list_lookup(false),
-    join(0),
-    top_join_list(),
-    join_list(0),
-    embedding(0),
-    sj_nests(),
-    leaf_tables(0),
-    type(optimizer::ST_PRIMARY),
-    order_list(),
-    gorder_list(0),
-    select_limit(0),
-    offset_limit(0),
-    ref_pointer_array(0),
-    select_n_having_items(0),
-    cond_count(0),
-    between_count(0),
-    max_equal_elems(0),
-    select_n_where_fields(0),
-    parsing_place(NO_MATTER),
-    with_sum_func(0),
-    in_sum_expr(0),
-    select_number(0),
-    nest_level(0),
-    inner_sum_func_list(0),
-    with_wild(0),
-    braces(0),
-    having_fix_field(0),
-    inner_refs_list(),
-    n_sum_items(0),
-    n_child_sum_items(0),
-    explicit_limit(0),
-    is_cross(false),
-    subquery_in_having(0),
-    is_correlated(0),
-    exclude_from_table_unique_test(0),
-    non_agg_fields(),
-    cur_pos_in_select_list(0),
-    prev_join_using(0),
-    full_group_by_flag(),
-    current_index_hint_type(INDEX_HINT_IGNORE),
-    current_index_hint_clause(),
-    index_hints(0)
-  {
-  }
-
   Name_resolution_context context;
-  const char *db;
+  char *db;
   /* An Item representing the WHERE clause */
   Item *where;
   /* An Item representing the HAVING clause */
@@ -444,14 +415,14 @@ public:
   Item::cond_result having_value;
   /* point on lex in which it was created, used in view subquery detection */
   LEX *parent_lex;
-  olap_type olap;
+  enum olap_type olap;
   /* FROM clause - points to the beginning of the TableList::next_local list. */
   SQL_LIST table_list;
   SQL_LIST group_list; /* GROUP BY clause. */
   List<Item> item_list;  /* list of fields & expressions */
   List<String> interval_list;
   bool is_item_list_lookup;
-  Join *join; /* after Join::prepare it is pointer to corresponding JOIN */
+  JOIN *join; /* after JOIN::prepare it is pointer to corresponding JOIN */
   List<TableList> top_join_list; /* join list of the top level          */
   List<TableList> *join_list;    /* list for the currently parsed join  */
   TableList *embedding;          /* table embedding to the above list   */
@@ -462,7 +433,7 @@ public:
     by TableList::next_leaf, so leaf_tables points to the left-most leaf.
   */
   TableList *leaf_tables;
-  drizzled::optimizer::select_type type; /* type of select for EXPLAIN */
+  enum drizzled::optimizer::select_type type; /* type of select for EXPLAIN */
 
   SQL_LIST order_list;                /* ORDER clause */
   SQL_LIST *gorder_list;
@@ -487,6 +458,7 @@ public:
   enum_parsing_place parsing_place; /* where we are parsing expression */
   bool with_sum_func;   /* sum function indicator */
 
+  uint32_t table_join_options;
   uint32_t in_sum_expr;
   uint32_t select_number; /* number of select (used for EXPLAIN) */
   int8_t nest_level;     /* nesting level of select */
@@ -504,10 +476,6 @@ public:
 
   /* explicit LIMIT clause was used */
   bool explicit_limit;
-
-  /* explicit CROSS JOIN was used */
-  bool is_cross;
-
   /*
     there are subquery in HAVING clause => we can't close tables before
     query processing end even if we use temporary table
@@ -546,7 +514,6 @@ public:
           defined as SUM_FUNC_USED.
   */
   std::bitset<2> full_group_by_flag;
-
   void init_query();
   void init_select();
   Select_Lex_Unit* master_unit();
@@ -578,23 +545,24 @@ public:
   bool inc_in_sum_expr();
   uint32_t get_in_sum_expr();
 
-  void add_item_to_list(Session *session, Item *item);
-  void add_group_to_list(Session *session, Item *item, bool asc);
-  void add_order_to_list(Session *session, Item *item, bool asc);
+  bool add_item_to_list(Session *session, Item *item);
+  bool add_group_to_list(Session *session, Item *item, bool asc);
+  bool add_order_to_list(Session *session, Item *item, bool asc);
   TableList* add_table_to_list(Session *session,
                                Table_ident *table,
-                               lex_string_t *alias,
-                               const std::bitset<NUM_OF_TABLE_OPTIONS>& table_options,
+                               LEX_STRING *alias,
+                               uint32_t table_options,
                                thr_lock_type flags= TL_UNLOCK,
                                List<Index_hint> *hints= 0,
-                               lex_string_t *option= 0);
+                               LEX_STRING *option= 0);
   TableList* get_table_list();
-  void init_nested_join(Session&);
-  TableList *end_nested_join();
-  TableList *nest_last_join(Session*);
+  bool init_nested_join(Session *session);
+  TableList *end_nested_join(Session *session);
+  TableList *nest_last_join(Session *session);
   void add_joined_table(TableList *table);
   TableList *convert_right_join();
   List<Item>* get_item_list();
+  uint32_t get_table_join_options();
   void set_lock_for_tables(thr_lock_type lock_type);
   inline void init_order()
   {
@@ -603,7 +571,7 @@ public:
     order_list.next= (unsigned char**) &order_list.first;
   }
   /*
-    This method created for reiniting LEX in admin_table() and can be
+    This method created for reiniting LEX in mysql_admin_table() and can be
     used only if you are going remove all Select_Lex & units except belonger
     to LEX (LEX::unit & LEX::select, for other purposes there are
     Select_Lex_Unit::exclude_level & Select_Lex_Unit::exclude_tree
@@ -614,17 +582,19 @@ public:
   }
   bool test_limit();
 
-  friend void lex_start(Session*);
+  friend void lex_start(Session *session);
+  Select_Lex() : n_sum_items(0), n_child_sum_items(0) {}
   void make_empty_select()
   {
     init_query();
     init_select();
   }
-  void setup_ref_array(Session *session, uint32_t order_group_num);
-  void print(Session *session, String *str);
-  static void print_order(String *str, Order *order);
-
-  void print_limit(Session *session, String *str);
+  bool setup_ref_array(Session *session, uint32_t order_group_num);
+  void print(Session *session, String *str, enum_query_type query_type);
+  static void print_order(String *str,
+                          order_st *order,
+                          enum_query_type query_type);
+  void print_limit(Session *session, String *str, enum_query_type query_type);
   void fix_prepare_information(Session *session, Item **conds, Item **having_conds);
   /*
     Destroy the used execution plan (JOIN) of this subtree (this
@@ -637,49 +607,52 @@ public:
   */
   void cleanup_all_joins(bool full);
 
-  void set_index_hint_type(index_hint_type type, index_clause_map clause);
+  void set_index_hint_type(enum index_hint_type type, index_clause_map clause);
 
   /*
    Add a index hint to the tagged list of hints. The type and clause of the
    hint will be the current ones (set by set_index_hint())
   */
-  void add_index_hint(Session*, const char*);
+  bool add_index_hint (Session *session, char *str, uint32_t length);
 
   /* make a list to hold index hints */
   void alloc_index_hints (Session *session);
   /* read and clear the index hints */
-  List<Index_hint>* pop_index_hints()
+  List<Index_hint>* pop_index_hints(void)
   {
     List<Index_hint> *hints= index_hints;
     index_hints= NULL;
     return hints;
   }
 
-  void clear_index_hints() { index_hints= NULL; }
+  void clear_index_hints(void) { index_hints= NULL; }
 
 private:
   /* current index hint kind. used in filling up index_hints */
-  index_hint_type current_index_hint_type;
+  enum index_hint_type current_index_hint_type;
   index_clause_map current_index_hint_clause;
   /* a list of USE/FORCE/IGNORE INDEX */
   List<Index_hint> *index_hints;
 };
 
-inline bool Select_Lex_Unit::is_union()
+inline bool Select_Lex_Unit::is_union ()
 {
-  return first_select()->next_select() && first_select()->next_select()->linkage == UNION_TYPE;
+  return first_select()->next_select() &&
+    first_select()->next_select()->linkage == UNION_TYPE;
 }
 
 enum xa_option_words
 {
-  XA_NONE,
-  XA_JOIN,
-  XA_RESUME,
-  XA_ONE_PHASE,
-  XA_SUSPEND,
-  XA_FOR_MIGRATE
+  XA_NONE
+, XA_JOIN
+, XA_RESUME
+, XA_ONE_PHASE
+, XA_SUSPEND
+, XA_FOR_MIGRATE
 };
 
+extern const LEX_STRING null_lex_str;
+
 /*
   Class representing list of all tables used by statement.
   It also contains information about stored functions used by statement
@@ -705,6 +678,13 @@ public:
   */
   TableList **query_tables_own_last;
 
+  /*
+    These constructor and destructor serve for creation/destruction
+    of Query_tables_list instances which are used as backup storage.
+  */
+  Query_tables_list() {}
+  virtual ~Query_tables_list() {}
+
   /* Initializes (or resets) Query_tables_list object for "real" use. */
   void reset_query_tables_list(bool init);
 
@@ -759,7 +739,10 @@ enum enum_comment_state
 
 } /* namespace drizzled */
 
-namespace drizzled {
+#include "drizzled/lex_input_stream.h"
+
+namespace drizzled
+{
 
 /* The state of the lex parsing. This is saved in the Session struct */
 class LEX : public Query_tables_list
@@ -772,18 +755,18 @@ public:
   /* list of all Select_Lex */
   Select_Lex *all_selects_list;
 
-  /* This is the "scale" for DECIMAL (S,P) notation */
-  const char *length;
+  /* This is the "scale" for DECIMAL (S,P) notation */ 
+  char *length;
   /* This is the decimal precision in DECIMAL(S,P) notation */
-  const char *dec;
-
+  char *dec;
+  
   /**
-   * This is used kind of like the "ident" member variable below, as
+   * This is used kind of like the "ident" member variable below, as 
    * a place to store certain names of identifiers.  Unfortunately, it
    * is used differently depending on the Command (SELECT on a derived
    * table vs CREATE)
    */
-  lex_string_t name;
+  LEX_STRING name;
   /* The string literal used in a LIKE expression */
   String *wild;
   file_exchange *exchange;
@@ -794,13 +777,13 @@ public:
    * or a named savepoint.  It should probably be refactored out into
    * the eventual Command class built for the Keycache and Savepoint
    * commands.
-   */
-  lex_string_t ident;
+   */ 
+  LEX_STRING ident;
 
   unsigned char* yacc_yyss, *yacc_yyvs;
   /* The owning Session of this LEX */
   Session *session;
-  const charset_info_st *charset;
+  const CHARSET_INFO *charset;
   bool text_string_is_7bit;
   /* store original leaf_tables for INSERT SELECT and PS/SP */
   TableList *leaf_tables_insert;
@@ -811,7 +794,7 @@ public:
   List<Lex_Column>    columns;
   List<Item>	      *insert_list,field_list,value_list,update_list;
   List<List_item>     many_values;
-  SetVarVector  var_list;
+  List<set_var_base>  var_list;
   /*
     A stack of name resolution contexts for the query. This stack is used
     at parse time to set local name resolution contexts for various parts
@@ -891,17 +874,22 @@ public:
      statement in a session. It's re-used by doing lex_end, lex_start
      in sql_lex.cc
   */
-  virtual ~LEX();
+  virtual ~LEX()
+  {
+  }
 
   TableList *unlink_first_table(bool *link_to_local);
   void link_first_table_back(TableList *first, bool link_to_local);
   void first_lists_tables_same();
 
+  bool only_view_structure();
+  bool need_correct_ident();
+
   void cleanup_after_one_table_open();
 
-  void push_context(Name_resolution_context *context)
+  bool push_context(Name_resolution_context *context)
   {
-    context_stack.push_front(context);
+    return context_stack.push_front(context);
   }
 
   void pop_context()
@@ -909,10 +897,16 @@ public:
     context_stack.pop();
   }
 
+  bool copy_db_to(char **p_db, size_t *p_db_length) const;
+
   Name_resolution_context *current_context()
   {
-    return &context_stack.front();
+    return context_stack.head();
   }
+  /*
+    Restore the LEX and Session in case of a parse error.
+  */
+  static void cleanup_lex_after_parse_error(Session *session);
 
   /**
     @brief check if the statement is a single-level join
@@ -935,75 +929,12 @@ public:
     }
     return false;
   }
-  bool is_cross; // CROSS keyword was used
-  bool isCacheable()
-  {
-    return cacheable;
-  }
-  void setCacheable(bool val)
-  {
-    cacheable= val;
-  }
-
-  void reset()
-  {
-    sum_expr_used= false;
-    _exists= false;
-  }
-
-  void setSumExprUsed()
-  {
-    sum_expr_used= true;
-  }
-
-  bool isSumExprUsed()
-  {
-    return sum_expr_used;
-  }
-
-  void start(Session *session);
-  void end();
-
-  message::Table *table()
-  {
-    if (not _create_table)
-      _create_table= new message::Table;
-
-    return _create_table;
-  }
-
-  message::AlterTable *alter_table();
-
-  message::Table::Field *field()
-  {
-    return _create_field;
-  }
-
-  void setField(message::Table::Field *arg)
-  {
-    _create_field= arg;
-  }
-
-  void setExists()
-  {
-    _exists= true;
-  }
-
-  bool exists() const
-  {
-    return _exists;
-  }
-
-private:
-  bool cacheable;
-  bool sum_expr_used;
-  message::Table *_create_table;
-  message::AlterTable *_alter_table;
-  message::Table::Field *_create_field;
-  bool _exists;
 };
 
 extern void lex_start(Session *session);
+extern void lex_end(LEX *lex);
+extern void trim_whitespace(const CHARSET_INFO * const cs, LEX_STRING *str);
+extern bool is_lex_native_function(const LEX_STRING *name);
 
 /**
   @} (End of group Semantic_Analysis)
@@ -1012,3 +943,4 @@ extern void lex_start(Session *session);
 } /* namespace drizzled */
 
 #endif /* DRIZZLE_SERVER */
+#endif /* DRIZZLED_SQL_LEX_H */
