@@ -1,5 +1,4 @@
 /* Copyright (C) 2000-2006 MySQL AB
-   Copyright (C) 2011 Stewart Smith
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,35 +11,27 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 
 /* Copy data from a textfile to table */
 
-#include <config.h>
-
+#include "config.h"
 #include <drizzled/sql_load.h>
 #include <drizzled/error.h>
-#include <drizzled/catalog/local.h>
+#include <drizzled/data_home.h>
 #include <drizzled/session.h>
 #include <drizzled/sql_base.h>
-#include <drizzled/field/epoch.h>
-#include <drizzled/internal/my_sys.h>
-#include <drizzled/internal/iocache.h>
-#include <drizzled/plugin/storage_engine.h>
-#include <drizzled/sql_lex.h>
-#include <drizzled/copy_info.h>
-#include <drizzled/file_exchange.h>
-#include <drizzled/util/test.h>
-#include <drizzled/session/transactions.h>
+#include <drizzled/field/timestamp.h>
+#include "drizzled/internal/my_sys.h"
+#include "drizzled/internal/iocache.h"
+#include <drizzled/db.h>
 
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <algorithm>
 #include <climits>
-#include <boost/filesystem.hpp>
 
-namespace fs=boost::filesystem;
 using namespace std;
 namespace drizzled
 {
@@ -57,15 +48,15 @@ class READ_INFO {
   int	*stack,*stack_pos;
   bool	found_end_of_line,start_of_line,eof;
   bool  need_end_io_cache;
-  internal::io_cache_st cache;
+  internal::IO_CACHE cache;
 
 public:
   bool error,line_cuted,found_null,enclosed;
   unsigned char	*row_start,			/* Found row starts here */
 	*row_end;			/* Found row ends here */
-  const charset_info_st *read_charset;
+  const CHARSET_INFO *read_charset;
 
-  READ_INFO(int cursor, size_t tot_length, const charset_info_st * const cs,
+  READ_INFO(int cursor, size_t tot_length, const CHARSET_INFO * const cs,
 	    String &field_term,String &line_start,String &line_term,
 	    String &enclosed,int escape, bool is_fifo);
   ~READ_INFO();
@@ -82,24 +73,24 @@ public:
   */
   void end_io_cache()
   {
-    cache.end_io_cache();
+    internal::end_io_cache(&cache);
     need_end_io_cache = 0;
   }
 
   /*
     Either this method, or we need to make cache public
-    Arg must be set from load() since constructor does not see
+    Arg must be set from mysql_load() since constructor does not see
     either the table or Session value
   */
   void set_io_cache_arg(void* arg) { cache.arg = arg; }
 };
 
-static int read_fixed_length(Session *session, CopyInfo &info, TableList *table_list,
+static int read_fixed_length(Session *session, COPY_INFO &info, TableList *table_list,
                              List<Item> &fields_vars, List<Item> &set_fields,
                              List<Item> &set_values, READ_INFO &read_info,
 			     uint32_t skip_lines,
 			     bool ignore_check_option_errors);
-static int read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
+static int read_sep_field(Session *session, COPY_INFO &info, TableList *table_list,
                           List<Item> &fields_vars, List<Item> &set_fields,
                           List<Item> &set_values, READ_INFO &read_info,
 			  String &enclosed, uint32_t skip_lines,
@@ -110,7 +101,7 @@ static int read_sep_field(Session *session, CopyInfo &info, TableList *table_lis
   Execute LOAD DATA query
 
   SYNOPSYS
-    load()
+    mysql_load()
       session - current thread
       ex  - file_exchange object representing source cursor and its parsing rules
       table_list  - list of tables to which we are loading data
@@ -126,45 +117,45 @@ static int read_sep_field(Session *session, CopyInfo &info, TableList *table_lis
     true - error / false - success
 */
 
-int load(Session *session,file_exchange *ex,TableList *table_list,
+int mysql_load(Session *session,file_exchange *ex,TableList *table_list,
 	        List<Item> &fields_vars, List<Item> &set_fields,
                 List<Item> &set_values,
                 enum enum_duplicates handle_duplicates, bool ignore)
 {
+  char name[FN_REFLEN];
   int file;
   Table *table= NULL;
   int error;
   String *field_term=ex->field_term,*escaped=ex->escaped;
   String *enclosed=ex->enclosed;
   bool is_fifo=0;
-
-  assert(table_list->getSchemaName()); // This should never be null
-
+  char *db= table_list->db;			// This is never null
+  assert(db);
   /*
     If path for cursor is not defined, we will use the current database.
     If this is not set, we will use the directory where the table to be
     loaded is located
   */
-  util::string::ptr schema(session->schema());
-  const char *tdb= (schema and not schema->empty()) ? schema->c_str() : table_list->getSchemaName(); // Result should never be null
+  const char *tdb= session->db.empty() ? db  : session->db.c_str();		// Result is never null
   assert(tdb);
   uint32_t skip_lines= ex->skip_lines;
   bool transactional_table;
+  Session::killed_state killed_status= Session::NOT_KILLED;
 
   /* Escape and enclosed character may be a utf8 4-byte character */
   if (escaped->length() > 4 || enclosed->length() > 4)
   {
     my_error(ER_WRONG_FIELD_TERMINATORS,MYF(0),enclosed->c_ptr(), enclosed->length());
-    return true;
+    return(true);
   }
 
   if (session->openTablesLock(table_list))
-    return true;
+    return(true);
 
-  if (setup_tables_and_check_access(session, &session->lex().select_lex.context,
-                                    &session->lex().select_lex.top_join_list,
+  if (setup_tables_and_check_access(session, &session->lex->select_lex.context,
+                                    &session->lex->select_lex.top_join_list,
                                     table_list,
-                                    &session->lex().select_lex.leaf_tables, true))
+                                    &session->lex->select_lex.leaf_tables, true))
      return(-1);
 
   /*
@@ -177,17 +168,17 @@ int load(Session *session,file_exchange *ex,TableList *table_list,
   */
   if (unique_table(table_list, table_list->next_global))
   {
-    my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->getTableName());
-    return true;
+    my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->table_name);
+    return(true);
   }
 
   table= table_list->table;
   transactional_table= table->cursor->has_transactions();
 
-  if (!fields_vars.size())
+  if (!fields_vars.elements)
   {
     Field **field;
-    for (field= table->getFields(); *field ; field++)
+    for (field=table->field; *field ; field++)
       fields_vars.push_back(new Item_field(*field));
     table->setWriteSet();
     table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;
@@ -197,7 +188,7 @@ int load(Session *session,file_exchange *ex,TableList *table_list,
     */
     if (setup_fields(session, 0, set_fields, MARK_COLUMNS_WRITE, 0, 0) ||
         setup_fields(session, 0, set_values, MARK_COLUMNS_READ, 0, 0))
-      return true;
+      return(true);
   }
   else
   {						// Part field list
@@ -205,32 +196,30 @@ int load(Session *session,file_exchange *ex,TableList *table_list,
     if (setup_fields(session, 0, fields_vars, MARK_COLUMNS_WRITE, 0, 0) ||
         setup_fields(session, 0, set_fields, MARK_COLUMNS_WRITE, 0, 0) ||
         check_that_all_fields_are_given_values(session, table, table_list))
-      return true;
+      return(true);
     /*
       Check whenever TIMESTAMP field with auto-set feature specified
       explicitly.
     */
     if (table->timestamp_field)
     {
-      if (table->isWriteSet(table->timestamp_field->position()))
-      {
+      if (table->isWriteSet(table->timestamp_field->field_index))
         table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;
-      }
       else
       {
-        table->setWriteSet(table->timestamp_field->position());
+        table->setWriteSet(table->timestamp_field->field_index);
       }
     }
     /* Fix the expressions in SET clause */
     if (setup_fields(session, 0, set_values, MARK_COLUMNS_READ, 0, 0))
-      return true;
+      return(true);
   }
 
   table->mark_columns_needed_for_insert();
 
   size_t tot_length=0;
   bool use_blobs= 0, use_vars= 0;
-  List<Item>::iterator it(fields_vars.begin());
+  List_iterator_fast<Item> it(fields_vars);
   Item *item;
 
   while ((item= it++))
@@ -255,87 +244,79 @@ int load(Session *session,file_exchange *ex,TableList *table_list,
   {
     my_message(ER_BLOBS_AND_NO_TERMINATED,ER(ER_BLOBS_AND_NO_TERMINATED),
 	       MYF(0));
-    return true;
+    return(true);
   }
   if (use_vars && !field_term->length() && !enclosed->length())
   {
     my_error(ER_LOAD_FROM_FIXED_SIZE_ROWS_TO_VAR, MYF(0));
-    return true;
+    return(true);
   }
 
-  fs::path to_file(ex->file_name);
-  fs::path target_path(fs::system_complete(catalog::local_identifier().getPath()));
-  if (not to_file.has_root_directory())
   {
-    int count_elements= 0;
-    for (fs::path::iterator iter= to_file.begin();
-         iter != to_file.end();
-         ++iter, ++count_elements)
-    { }
-
-    if (count_elements == 1)
+#ifdef DONT_ALLOW_FULL_LOAD_DATA_PATHS
+    ex->file_name+=dirname_length(ex->file_name);
+#endif
+    if (!internal::dirname_length(ex->file_name))
     {
-      target_path /= tdb;
+      strcpy(name, drizzle_real_data_home);
+      strncat(name, tdb, FN_REFLEN-strlen(drizzle_real_data_home)-1);
+      (void) internal::fn_format(name, ex->file_name, name, "",
+		       MY_RELATIVE_PATH | MY_UNPACK_FILENAME);
     }
-    target_path /= to_file;
-  }
-  else
-  {
-    target_path= to_file;
-  }
-
-  if (not secure_file_priv.string().empty())
-  {
-    if (target_path.file_string().substr(0, secure_file_priv.file_string().size()) != secure_file_priv.file_string())
+    else
     {
-      /* Read only allowed from within dir specified by secure_file_priv */
-      my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--secure-file-priv");
-      return true;
+      (void) internal::fn_format(name, ex->file_name, drizzle_real_data_home, "",
+		       MY_RELATIVE_PATH | MY_UNPACK_FILENAME);
+
+      if (opt_secure_file_priv &&
+          strncmp(opt_secure_file_priv, name, strlen(opt_secure_file_priv)))
+      {
+        /* Read only allowed from within dir specified by secure_file_priv */
+        my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--secure-file-priv");
+        return(true);
+      }
+
+      struct stat stat_info;
+      if (stat(name,&stat_info))
+      {
+        my_error(ER_FILE_NOT_FOUND, MYF(0), name, errno);
+	return(true);
+      }
+
+      // if we are not in slave thread, the cursor must be:
+      if (!((stat_info.st_mode & S_IROTH) == S_IROTH &&  // readable by others
+            (stat_info.st_mode & S_IFLNK) != S_IFLNK && // and not a symlink
+            ((stat_info.st_mode & S_IFREG) == S_IFREG ||
+             (stat_info.st_mode & S_IFIFO) == S_IFIFO)))
+      {
+	my_error(ER_TEXTFILE_NOT_READABLE, MYF(0), name);
+	return(true);
+      }
+      if ((stat_info.st_mode & S_IFIFO) == S_IFIFO)
+	is_fifo = 1;
+    }
+    if ((file=internal::my_open(name,O_RDONLY,MYF(MY_WME))) < 0)
+    {
+      my_error(ER_CANT_OPEN_FILE, MYF(0), name, errno);
+      return(true);
     }
   }
 
-  struct stat stat_info;
-  if (stat(target_path.file_string().c_str(), &stat_info))
-  {
-    my_error(ER_FILE_NOT_FOUND, MYF(0), target_path.file_string().c_str(), errno);
-    return true;
-  }
-
-  // if we are not in slave thread, the cursor must be:
-  if (!((stat_info.st_mode & S_IROTH) == S_IROTH &&  // readable by others
-        (stat_info.st_mode & S_IFLNK) != S_IFLNK && // and not a symlink
-        ((stat_info.st_mode & S_IFREG) == S_IFREG ||
-         (stat_info.st_mode & S_IFIFO) == S_IFIFO)))
-  {
-    my_error(ER_TEXTFILE_NOT_READABLE, MYF(0), target_path.file_string().c_str());
-    return true;
-  }
-  if ((stat_info.st_mode & S_IFIFO) == S_IFIFO)
-    is_fifo = 1;
-
-
-  if ((file=internal::my_open(target_path.file_string().c_str(), O_RDONLY,MYF(MY_WME))) < 0)
-  {
-    my_error(ER_CANT_OPEN_FILE, MYF(0), target_path.file_string().c_str(), errno);
-    return true;
-  }
-  CopyInfo info;
+  COPY_INFO info;
   memset(&info, 0, sizeof(info));
   info.ignore= ignore;
   info.handle_duplicates=handle_duplicates;
   info.escape_char=escaped->length() ? (*escaped)[0] : INT_MAX;
 
-  identifier::Schema identifier(session->catalog().identifier(),
-                                *schema);
   READ_INFO read_info(file, tot_length,
-                      ex->cs ? ex->cs : plugin::StorageEngine::getSchemaCollation(identifier),
-		      *field_term, *ex->line_start, *ex->line_term, *enclosed,
+                      ex->cs ? ex->cs : plugin::StorageEngine::getSchemaCollation(session->db.c_str()),
+		      *field_term,*ex->line_start, *ex->line_term, *enclosed,
 		      info.escape_char, is_fifo);
   if (read_info.error)
   {
     if	(file >= 0)
       internal::my_close(file,MYF(0));			// no files in net reading
-    return true;				// Can't allocate buffers
+    return(true);				// Can't allocate buffers
   }
 
   /*
@@ -372,7 +353,7 @@ int load(Session *session,file_exchange *ex,TableList *table_list,
     table->cursor->ha_start_bulk_insert((ha_rows) 0);
     table->copy_blobs=1;
 
-    session->setAbortOnWarning(true);
+    session->abort_on_warning= true;
 
     if (!field_term->length() && !enclosed->length())
       error= read_fixed_length(session, info, table_list, fields_vars,
@@ -395,29 +376,31 @@ int load(Session *session,file_exchange *ex,TableList *table_list,
     internal::my_close(file,MYF(0));
   free_blobs(table);				/* if pack_blob was used */
   table->copy_blobs=0;
-  session->count_cuted_fields= CHECK_FIELD_ERROR_FOR_NULL;
-
+  session->count_cuted_fields= CHECK_FIELD_IGNORE;
+  /*
+     simulated killing in the middle of per-row loop
+     must be effective for binlogging
+  */
+  killed_status= (error == 0)? Session::NOT_KILLED : session->killed;
   if (error)
   {
     error= -1;				// Error on read
     goto err;
   }
-
-  char msg[FN_REFLEN];
-  snprintf(msg, sizeof(msg), ER(ER_LOAD_INFO), info.records, info.deleted,
-	   (info.records - info.copied), session->cuted_fields);
+  sprintf(name, ER(ER_LOAD_INFO), (uint32_t) info.records, (uint32_t) info.deleted,
+	  (uint32_t) (info.records - info.copied), (uint32_t) session->cuted_fields);
 
   if (session->transaction.stmt.hasModifiedNonTransData())
     session->transaction.all.markModifiedNonTransData();
 
-  session->my_ok(info.copied + info.deleted, 0, 0L, msg);
+  /* ok to client sent only after binlog write and engine commit */
+  session->my_ok(info.copied + info.deleted, 0, 0L, name);
 err:
   assert(transactional_table || !(info.copied || info.deleted) ||
               session->transaction.stmt.hasModifiedNonTransData());
   table->cursor->ha_release_auto_increment();
   table->auto_increment_field_not_null= false;
-  session->setAbortOnWarning(false);
-
+  session->abort_on_warning= 0;
   return(error);
 }
 
@@ -427,22 +410,25 @@ err:
 ****************************************************************************/
 
 static int
-read_fixed_length(Session *session, CopyInfo &info, TableList *table_list,
+read_fixed_length(Session *session, COPY_INFO &info, TableList *table_list,
                   List<Item> &fields_vars, List<Item> &set_fields,
                   List<Item> &set_values, READ_INFO &read_info,
                   uint32_t skip_lines, bool ignore_check_option_errors)
 {
-  List<Item>::iterator it(fields_vars.begin());
+  List_iterator_fast<Item> it(fields_vars);
   Item_field *sql_field;
   Table *table= table_list->table;
+  uint64_t id;
   bool err;
 
+  id= 0;
+
   while (!read_info.read_fixed_length())
   {
-    if (session->getKilled())
+    if (session->killed)
     {
       session->send_kill_message();
-      return 1;
+      return(1);
     }
     if (skip_lines)
     {
@@ -455,9 +441,9 @@ read_fixed_length(Session *session, CopyInfo &info, TableList *table_list,
       skip_lines--;
       continue;
     }
-    it= fields_vars.begin();
+    it.rewind();
     unsigned char *pos=read_info.row_start;
-#ifdef HAVE_VALGRIND
+#ifdef HAVE_purify
     read_info.row_end[0]=0;
 #endif
 
@@ -484,9 +470,8 @@ read_fixed_length(Session *session, CopyInfo &info, TableList *table_list,
         push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
                             ER_WARN_TOO_FEW_RECORDS,
                             ER(ER_WARN_TOO_FEW_RECORDS), session->row_count);
-
-        if (not field->maybe_null() and field->is_timestamp())
-            ((field::Epoch::pointer) field)->set_time();
+        if (!field->maybe_null() && field->type() == DRIZZLE_TYPE_TIMESTAMP)
+            ((Field_timestamp*) field)->set_time();
       }
       else
       {
@@ -513,15 +498,15 @@ read_fixed_length(Session *session, CopyInfo &info, TableList *table_list,
                           ER(ER_WARN_TOO_MANY_RECORDS), session->row_count);
     }
 
-    if (session->getKilled() ||
+    if (session->killed ||
         fill_record(session, set_fields, set_values,
                     ignore_check_option_errors))
-      return 1;
+      return(1);
 
     err= write_record(session, table, &info);
     table->auto_increment_field_not_null= false;
     if (err)
-      return 1;
+      return(1);
 
     /*
       We don't need to reset auto-increment field since we are restoring
@@ -544,26 +529,28 @@ read_fixed_length(Session *session, CopyInfo &info, TableList *table_list,
 
 
 static int
-read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
+read_sep_field(Session *session, COPY_INFO &info, TableList *table_list,
                List<Item> &fields_vars, List<Item> &set_fields,
                List<Item> &set_values, READ_INFO &read_info,
 	       String &enclosed, uint32_t skip_lines,
 	       bool ignore_check_option_errors)
 {
-  List<Item>::iterator it(fields_vars.begin());
+  List_iterator_fast<Item> it(fields_vars);
   Item *item;
   Table *table= table_list->table;
   uint32_t enclosed_length;
+  uint64_t id;
   bool err;
 
   enclosed_length=enclosed.length();
+  id= 0;
 
-  for (;;it= fields_vars.begin())
+  for (;;it.rewind())
   {
-    if (session->getKilled())
+    if (session->killed)
     {
       session->send_kill_message();
-      return 1;
+      return(1);
     }
 
     table->restoreRecordAsDefault();
@@ -597,19 +584,16 @@ read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
           {
             my_error(ER_WARN_NULL_TO_NOTNULL, MYF(0), field->field_name,
                      session->row_count);
-            return 1;
+            return(1);
           }
           field->set_null();
-          if (not field->maybe_null())
+          if (!field->maybe_null())
           {
-            if (field->is_timestamp())
-            {
-              ((field::Epoch::pointer) field)->set_time();
-            }
+            if (field->type() == DRIZZLE_TYPE_TIMESTAMP)
+              ((Field_timestamp*) field)->set_time();
             else if (field != table->next_number_field)
-            {
-              field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN, ER_WARN_NULL_TO_NOTNULL, 1);
-            }
+              field->set_warning(DRIZZLE_ERROR::WARN_LEVEL_WARN,
+                                 ER_WARN_NULL_TO_NOTNULL, 1);
           }
 	}
         else if (item->type() == Item::STRING_ITEM)
@@ -620,7 +604,7 @@ read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
         else
         {
           my_error(ER_LOAD_DATA_INVALID_COLUMN, MYF(0), item->full_name());
-          return 1;
+          return(1);
         }
 
 	continue;
@@ -637,12 +621,13 @@ read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
       }
       else if (item->type() == Item::STRING_ITEM)
       {
-        ((Item_user_var_as_out_param *)item)->set_value(str_ref((char*) pos, length), read_info.read_charset);
+        ((Item_user_var_as_out_param *)item)->set_value((char*) pos, length,
+                                                        read_info.read_charset);
       }
       else
       {
         my_error(ER_LOAD_DATA_INVALID_COLUMN, MYF(0), item->full_name());
-        return 1;
+        return(1);
       }
     }
     if (read_info.error)
@@ -655,7 +640,7 @@ read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
     if (item)
     {
       /* Have not read any field, thus input cursor is simply ended */
-      if (item == &fields_vars.front())
+      if (item == fields_vars.head())
 	break;
       for (; item ; item= it++)
       {
@@ -667,10 +652,10 @@ read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
           {
             my_error(ER_WARN_NULL_TO_NOTNULL, MYF(0),field->field_name,
                      session->row_count);
-            return 1;
+            return(1);
           }
-          if (not field->maybe_null() and field->is_timestamp())
-              ((field::Epoch::pointer) field)->set_time();
+          if (!field->maybe_null() && field->type() == DRIZZLE_TYPE_TIMESTAMP)
+              ((Field_timestamp*) field)->set_time();
           /*
             QQ: We probably should not throw warning for each field.
             But how about intention to always have the same number
@@ -690,20 +675,20 @@ read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
         else
         {
           my_error(ER_LOAD_DATA_INVALID_COLUMN, MYF(0), item->full_name());
-          return 1;
+          return(1);
         }
       }
     }
 
-    if (session->getKilled() ||
+    if (session->killed ||
         fill_record(session, set_fields, set_values,
                     ignore_check_option_errors))
-      return 1;
+      return(1);
 
     err= write_record(session, table, &info);
     table->auto_increment_field_not_null= false;
     if (err)
-      return 1;
+      return(1);
     /*
       We don't need to reset auto-increment field since we are restoring
       its default value at the beginning of each loop iteration.
@@ -716,8 +701,8 @@ read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
       push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
                           ER_WARN_TOO_MANY_RECORDS, ER(ER_WARN_TOO_MANY_RECORDS),
                           session->row_count);
-      if (session->getKilled())
-        return 1;
+      if (session->killed)
+        return(1);
     }
     session->row_count++;
   }
@@ -753,7 +738,7 @@ READ_INFO::unescape(char chr)
 
 
 READ_INFO::READ_INFO(int file_par, size_t tot_length,
-                     const charset_info_st * const cs,
+                     const CHARSET_INFO * const cs,
 		     String &field_term, String &line_start, String &line_term,
 		     String &enclosed_par, int escape, bool is_fifo)
   :cursor(file_par),escape_char(escape)
@@ -790,8 +775,8 @@ READ_INFO::READ_INFO(int file_par, size_t tot_length,
 
 
   /* Set of a stack for unget if long terminators */
-  size_t length= max(field_term_length,line_term_length)+1;
-  set_if_bigger(length, line_start.length());
+  uint32_t length= max(field_term_length,line_term_length)+1;
+  set_if_bigger(length,line_start.length());
   stack= stack_pos= (int*) memory::sql_alloc(sizeof(int)*length);
 
   if (!(buffer=(unsigned char*) calloc(1, buff_length+1)))
@@ -799,10 +784,10 @@ READ_INFO::READ_INFO(int file_par, size_t tot_length,
   else
   {
     end_of_buff=buffer+buff_length;
-    if (cache.init_io_cache((false) ? -1 : cursor, 0,
-                            (false) ? internal::READ_NET :
-                            (is_fifo ? internal::READ_FIFO : internal::READ_CACHE),0L,1,
-                            MYF(MY_WME)))
+    if (init_io_cache(&cache,(false) ? -1 : cursor, 0,
+		      (false) ? internal::READ_NET :
+		      (is_fifo ? internal::READ_FIFO : internal::READ_CACHE),0L,1,
+		      MYF(MY_WME)))
     {
       free((unsigned char*) buffer);
       error=1;
@@ -825,14 +810,14 @@ READ_INFO::~READ_INFO()
   if (!error)
   {
     if (need_end_io_cache)
-      cache.end_io_cache();
+      internal::end_io_cache(&cache);
     free(buffer);
     error=1;
   }
 }
 
 
-#define GET (stack_pos != stack ? *--stack_pos : cache.get())
+#define GET (stack_pos != stack ? *--stack_pos : my_b_get(&cache))
 #define PUSH(A) *(stack_pos++)=(A)
 
 
@@ -1003,7 +988,8 @@ int READ_INFO::read_field()
     /*
      ** We come here if buffer is too small. Enlarge it and continue
      */
-    new_buffer=(unsigned char*) realloc(buffer, buff_length+1+IO_SIZE);
+    if (!(new_buffer=(unsigned char*) realloc(buffer, buff_length+1+IO_SIZE)))
+      return (error=1);
     to=new_buffer + (to-buffer);
     buffer=new_buffer;
     buff_length+=IO_SIZE;
