@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,20 +17,30 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_SQL_LOAD_H
+#define DRIZZLED_SQL_LOAD_H
 
 #include <drizzled/definitions.h>
 #include <drizzled/sql_list.h>
 #include <drizzled/item.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
-int load(Session *session, file_exchange *ex, TableList *table_list,
+class TableList;
+class file_exchange;
+class Session;
+class Table;
+
+typedef struct st_copy_info COPY_INFO;
+
+int mysql_load(Session *session, file_exchange *ex, TableList *table_list,
                List<Item> &fields_vars, List<Item> &set_fields,
                List<Item> &set_values_list,
                enum enum_duplicates handle_duplicates, bool ignore);
     
-int write_record(Session *session, Table *table, CopyInfo *info);
+int write_record(Session *session, Table *table, COPY_INFO *info);
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_SQL_LOAD_H */
