@@ -11,25 +11,39 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* This file is originally from the mysql distribution. Coded by monty */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/definitions.h>
-#include <drizzled/internal/my_sys.h>
-#include <drizzled/internal/m_string.h>
-#include <drizzled/memory/root.h>
-#include <drizzled/charset.h>
+#include "drizzled/internal/my_sys.h"
+#include "drizzled/internal/m_string.h"
+#include "drizzled/charset.h"
+#include "drizzled/global_charset_info.h"
 
 #include <algorithm>
 
-#include <drizzled/sql_string.h>
+#include "drizzled/sql_string.h"
 
 using namespace std;
 
-namespace drizzled {
+namespace drizzled
+{
+
+// Converstion functions to and from std::string.
+
+std::string String_to_std_string(String const& s)
+{
+   return std::string(s.ptr(), s.length());
+}
+
+String* set_String_from_std_string(String* s, std::string const& cs)
+{
+   s->set_ascii(cs.c_str(), cs.length());
+   s->copy();
+   return s;
+}
 
 /*****************************************************************************
 ** String functions
@@ -44,7 +58,7 @@ String::String()
 { }
 
 
-String::String(size_t length_arg)
+String::String(uint32_t length_arg)
   : Ptr(NULL),
     str_length(0),
     Alloced_length(0),
@@ -54,16 +68,16 @@ String::String(size_t length_arg)
   (void) real_alloc(length_arg);
 }
 
-String::String(const char *str, const charset_info_st * const cs)
+String::String(const char *str, const CHARSET_INFO * const cs)
   : Ptr(const_cast<char *>(str)),
-    str_length(static_cast<size_t>(strlen(str))),
+    str_length(static_cast<uint32_t>(strlen(str))),
     Alloced_length(0),
     alloced(false),
     str_charset(cs)
 { }
 
 
-String::String(const char *str, size_t len, const charset_info_st * const cs)
+String::String(const char *str, uint32_t len, const CHARSET_INFO * const cs)
   : Ptr(const_cast<char *>(str)),
     str_length(len),
     Alloced_length(0),
@@ -72,7 +86,7 @@ String::String(const char *str, size_t len, const charset_info_st * const cs)
 { }
 
 
-String::String(char *str, size_t len, const charset_info_st * const cs)
+String::String(char *str, uint32_t len, const CHARSET_INFO * const cs)
   : Ptr(str),
     str_length(len),
     Alloced_length(len),
@@ -92,24 +106,25 @@ String::String(const String &str)
 
 void *String::operator new(size_t size, memory::Root *mem_root)
 {
-  return mem_root->alloc(size);
+  return alloc_root(mem_root, static_cast<uint32_t>(size));
 }
 
 String::~String() { free(); }
 
-void String::real_alloc(size_t arg_length)
+bool String::real_alloc(uint32_t arg_length)
 {
   arg_length=ALIGN_SIZE(arg_length+1);
   str_length=0;
   if (Alloced_length < arg_length)
   {
-    if (Alloced_length > 0)
-      free();
-    Ptr=(char*) malloc(arg_length);
+    free();
+    if (!(Ptr=(char*) malloc(arg_length)))
+      return true;
     Alloced_length=arg_length;
     alloced=1;
   }
   Ptr[0]=0;
+  return false;
 }
 
 
@@ -118,93 +133,103 @@ void String::real_alloc(size_t arg_length)
 ** (for C functions)
 */
 
-void String::realloc(size_t alloc_length)
+bool String::realloc(uint32_t alloc_length)
 {
-  size_t len=ALIGN_SIZE(alloc_length+1);
+  uint32_t len=ALIGN_SIZE(alloc_length+1);
   if (Alloced_length < len)
   {
     char *new_ptr;
     if (alloced)
     {
-      new_ptr= (char*) ::realloc(Ptr,len);
-      Ptr=new_ptr;
-      Alloced_length=len;
+      if ((new_ptr= (char*) ::realloc(Ptr,len)))
+      {
+	Ptr=new_ptr;
+	Alloced_length=len;
+      }
+      else
+	return true;				// Signal error
     }
-    else 
+    else if ((new_ptr= (char*) malloc(len)))
     {
-      new_ptr= (char*) malloc(len);
       if (str_length)				// Avoid bugs in memcpy on AIX
-        memcpy(new_ptr,Ptr,str_length);
+	memcpy(new_ptr,Ptr,str_length);
       new_ptr[str_length]=0;
       Ptr=new_ptr;
       Alloced_length=len;
       alloced=1;
     }
+    else
+      return true;			// Signal error
   }
   Ptr[alloc_length]=0;			// This make other funcs shorter
+  return false;
 }
 
-void String::set_int(int64_t num, bool unsigned_flag, const charset_info_st * const cs)
+bool String::set_int(int64_t num, bool unsigned_flag, const CHARSET_INFO * const cs)
 {
-  size_t l= 20 * cs->mbmaxlen + 1;
-  alloc(l);
-  str_length=(size_t) (cs->cset->int64_t10_to_str)(cs, Ptr, l, unsigned_flag ? 10 : -10,num);
+  uint32_t l=20*cs->mbmaxlen+1;
+  int base= unsigned_flag ? 10 : -10;
+
+  if (alloc(l))
+    return true;
+  str_length=(uint32_t) (cs->cset->int64_t10_to_str)(cs,Ptr,l,base,num);
   str_charset=cs;
+  return false;
 }
 
-void String::set_real(double num,size_t decimals, const charset_info_st * const cs)
+bool String::set_real(double num,uint32_t decimals, const CHARSET_INFO * const cs)
 {
   char buff[FLOATING_POINT_BUFFER];
+  uint32_t dummy_errors;
   size_t len;
 
   str_charset=cs;
   if (decimals >= NOT_FIXED_DEC)
   {
-    len= internal::my_gcvt(num, internal::MY_GCVT_ARG_DOUBLE, sizeof(buff) - 1, buff, NULL);
-    copy(buff, len, cs);
-    return;
+    len= internal::my_gcvt(num,
+                           internal::MY_GCVT_ARG_DOUBLE,
+                           sizeof(buff) - 1, buff, NULL);
+    return copy(buff, len, &my_charset_utf8_general_ci, cs, &dummy_errors);
   }
   len= internal::my_fcvt(num, decimals, buff, NULL);
-  copy(buff, len, cs);
+  return copy(buff, (uint32_t) len, &my_charset_utf8_general_ci, cs,
+              &dummy_errors);
 }
 
 
-void String::copy()
+bool String::copy()
 {
   if (!alloced)
   {
     Alloced_length=0;				// Force realloc
-    realloc(str_length);
+    return realloc(str_length);
   }
+  return false;
 }
 
-void String::copy(const String &str)
+bool String::copy(const String &str)
 {
-  alloc(str.str_length);
+  if (alloc(str.str_length))
+    return true;
   str_length=str.str_length;
   memmove(Ptr, str.Ptr, str_length);		// May be overlapping
   Ptr[str_length]=0;
-  str_charset= str.str_charset;
+  str_charset=str.str_charset;
+  return false;
 }
 
-void String::copy(const std::string& arg, const charset_info_st* cs) // Allocate new string
+bool String::copy(const char *str,uint32_t arg_length, const CHARSET_INFO * const cs)
 {
-  alloc(arg.size());
-  str_length= arg.size();
-  memcpy(Ptr, arg.c_str(), arg.size());
-  Ptr[arg.size()]= 0;
-  str_charset= cs;
-}
-
-void String::copy(const char *str,size_t arg_length, const charset_info_st* cs)
-{
-  alloc(arg_length);
+  if (alloc(arg_length))
+    return true;
   if ((str_length=arg_length))
     memcpy(Ptr,str,arg_length);
   Ptr[arg_length]=0;
   str_charset=cs;
+  return false;
 }
 
+
 /*
   Checks that the source string can be just copied to the destination string
   without conversion.
@@ -215,7 +240,7 @@ void String::copy(const char *str,size_t arg_length, const charset_info_st* cs)
   arg_length		Length of string to copy.
   from_cs		Character set to copy from
   to_cs			Character set to copy to
-  size_t *offset	Returns number of unaligned characters.
+  uint32_t *offset	Returns number of unaligned characters.
 
   RETURN
    0  No conversion needed
@@ -227,17 +252,48 @@ void String::copy(const char *str,size_t arg_length, const charset_info_st* cs)
   character_set_results is NULL.
 */
 
-bool String::needs_conversion(size_t arg_length, const charset_info_st* from_cs, const charset_info_st* to_cs)
+bool String::needs_conversion(uint32_t arg_length,
+			      const CHARSET_INFO * const from_cs,
+			      const CHARSET_INFO * const to_cs,
+			      uint32_t *offset)
 {
+  *offset= 0;
   if (!to_cs ||
-      to_cs == &my_charset_bin ||
-      to_cs == from_cs ||
+      (to_cs == &my_charset_bin) ||
+      (to_cs == from_cs) ||
       my_charset_same(from_cs, to_cs) ||
-      (from_cs == &my_charset_bin && not (arg_length % to_cs->mbminlen)))
+      ((from_cs == &my_charset_bin) &&
+       (!(*offset=(arg_length % to_cs->mbminlen)))))
     return false;
   return true;
 }
 
+
+
+
+bool String::set_or_copy_aligned(const char *str,uint32_t arg_length,
+                                 const CHARSET_INFO * const cs)
+{
+  /* How many bytes are in incomplete character */
+  uint32_t offset= (arg_length % cs->mbminlen);
+
+  assert(!offset); /* All characters are complete, just copy */
+
+  set(str, arg_length, cs);
+  return false;
+}
+
+	/* Copy with charset conversion */
+
+bool String::copy(const char *str, uint32_t arg_length,
+		          const CHARSET_INFO * const,
+				  const CHARSET_INFO * const to_cs, uint32_t *errors)
+{
+  *errors= 0;
+  return copy(str, arg_length, to_cs);
+}
+
+
 /*
   Set a string to the value of a latin1-string, keeping the original charset
 
@@ -257,14 +313,27 @@ bool String::needs_conversion(size_t arg_length, const charset_info_st* from_cs,
 
 */
 
-void String::set_ascii(const char *str, size_t arg_length)
+bool String::set_ascii(const char *str, uint32_t arg_length)
 {
   if (str_charset->mbminlen == 1)
   {
     set(str, arg_length, str_charset);
-    return;
+    return 0;
   }
-  copy(str, arg_length, str_charset);
+  uint32_t dummy_errors;
+  return copy(str, arg_length, &my_charset_utf8_general_ci, str_charset, &dummy_errors);
+}
+
+bool String::append(const String &s)
+{
+  if (s.length())
+  {
+    if (realloc(str_length+s.length()))
+      return true;
+    memcpy(Ptr+str_length,s.ptr(),s.length());
+    str_length+=s.length();
+  }
+  return false;
 }
 
 
@@ -272,31 +341,55 @@ void String::set_ascii(const char *str, size_t arg_length)
   Append an ASCII string to the a string of the current character set
 */
 
-void String::append(const char *s,size_t arg_length)
+bool String::append(const char *s,uint32_t arg_length)
 {
-  if (arg_length == 0)
-  {
-    return;
-  }
+  if (!arg_length)
+    return false;
 
   /*
     For an ASCII compatinble string we can just append.
   */
-  realloc(str_length + arg_length);
+  if (realloc(str_length+arg_length))
+    return true;
+  memcpy(Ptr+str_length,s,arg_length);
+  str_length+=arg_length;
+  return false;
+}
+
+
+/*
+  Append a 0-terminated ASCII string
+*/
+
+bool String::append(const char *s)
+{
+  return append(s, strlen(s));
+}
+
+
+/*
+  Append a string in the given charset to the string
+  with character set recoding
+*/
+
+bool String::append(const char *s,uint32_t arg_length, const CHARSET_INFO * const)
+{
+  if (realloc(str_length + arg_length))
+    return true;
   memcpy(Ptr + str_length, s, arg_length);
   str_length+= arg_length;
+
+  return false;
 }
 
-void String::append(str_ref s)
-{
-  append(s.data(), s.size());
-}
 
-void String::append_with_prefill(const char *s,size_t arg_length, size_t full_length, char fill_char)
+bool String::append_with_prefill(const char *s,uint32_t arg_length,
+		 uint32_t full_length, char fill_char)
 {
   int t_length= arg_length > full_length ? arg_length : full_length;
 
-  realloc(str_length + t_length);
+  if (realloc(str_length + t_length))
+    return true;
   t_length= full_length - arg_length;
   if (t_length > 0)
   {
@@ -304,39 +397,42 @@ void String::append_with_prefill(const char *s,size_t arg_length, size_t full_le
     str_length=str_length + t_length;
   }
   append(s, arg_length);
+  return false;
 }
 
-size_t String::numchars() const
+uint32_t String::numchars()
 {
   return str_charset->cset->numchars(str_charset, Ptr, Ptr+str_length);
 }
 
-int String::charpos(int i,size_t offset) const
+int String::charpos(int i,uint32_t offset)
 {
-  return i <= 0 ? i : str_charset->cset->charpos(str_charset, Ptr + offset, Ptr + str_length, i);
+  if (i <= 0)
+    return i;
+  return str_charset->cset->charpos(str_charset,Ptr+offset,Ptr+str_length,i);
 }
 
-int String::strstr(const String &s, size_t offset)
+int String::strstr(const String &s,uint32_t offset)
 {
-  if (s.length() + offset <= str_length)
+  if (s.length()+offset <= str_length)
   {
     if (!s.length())
       return ((int) offset);	// Empty string is always found
 
-    const char *str = Ptr+offset;
-    const char *search=s.ptr();
-    const char *last=Ptr+str_length-s.length()+1;
+    register const char *str = Ptr+offset;
+    register const char *search=s.ptr();
+    const char *end=Ptr+str_length-s.length()+1;
     const char *search_end=s.ptr()+s.length();
 skip:
-    while (str != last)
+    while (str != end)
     {
       if (*str++ == *search)
       {
-        const char* i= str; 
-        const char* j= search + 1;
-        while (j != search_end)
-          if (*i++ != *j++) goto skip;
-        return (int) (str - Ptr) - 1;
+	register char *i,*j;
+	i=(char*) str; j=(char*) search+1;
+	while (j != search_end)
+	  if (*i++ != *j++) goto skip;
+	return (int) (str-Ptr) -1;
       }
     }
   }
@@ -347,27 +443,27 @@ skip:
 ** Search string from end. Offset is offset to the end of string
 */
 
-int String::strrstr(const String &s,size_t offset)
+int String::strrstr(const String &s,uint32_t offset)
 {
   if (s.length() <= offset && offset <= str_length)
   {
     if (!s.length())
       return offset;				// Empty string is always found
-    const char *str = Ptr+offset-1;
-    const char *search=s.ptr()+s.length()-1;
+    register const char *str = Ptr+offset-1;
+    register const char *search=s.ptr()+s.length()-1;
 
-    const char *last=Ptr+s.length()-2;
+    const char *end=Ptr+s.length()-2;
     const char *search_end=s.ptr()-1;
 skip:
-    while (str != last)
+    while (str != end)
     {
       if (*str-- == *search)
       {
-        const char* i= str; 
-        const char* j= search-1;
-        while (j != search_end)
-          if (*i-- != *j--) goto skip;
-        return (int) (i-Ptr) + 1;
+	register char *i,*j;
+	i=(char*) str; j=(char*) search-1;
+	while (j != search_end)
+	  if (*i-- != *j--) goto skip;
+	return (int) (i-Ptr) +1;
       }
     }
   }
@@ -379,13 +475,13 @@ skip:
   If wrong parameter or not enough memory, do nothing
 */
 
-void String::replace(size_t offset,size_t arg_length,const String &to)
+bool String::replace(uint32_t offset,uint32_t arg_length,const String &to)
 {
-  replace(offset,arg_length,to.ptr(),to.length());
+  return replace(offset,arg_length,to.ptr(),to.length());
 }
 
-void String::replace(size_t offset,size_t arg_length,
-                     const char *to, size_t to_length)
+bool String::replace(uint32_t offset,uint32_t arg_length,
+                     const char *to, uint32_t to_length)
 {
   long diff = (long) to_length-(long) arg_length;
   if (offset+arg_length <= str_length)
@@ -401,7 +497,8 @@ void String::replace(size_t offset,size_t arg_length,
     {
       if (diff)
       {
-	realloc(str_length+(size_t) diff);
+	if (realloc(str_length+(uint32_t) diff))
+	  return true;
 	internal::bmove_upp((unsigned char*) Ptr+str_length+diff,
                             (unsigned char*) Ptr+str_length,
                             str_length-offset-arg_length);
@@ -409,8 +506,9 @@ void String::replace(size_t offset,size_t arg_length,
       if (to_length)
 	memcpy(Ptr+offset,to,to_length);
     }
-    str_length+=(size_t) diff;
+    str_length+=(uint32_t) diff;
   }
+  return false;
 }
 
 
@@ -434,7 +532,7 @@ void String::replace(size_t offset,size_t arg_length,
 */
 
 
-int sortcmp(const String *s,const String *t, const charset_info_st * const cs)
+int sortcmp(const String *s,const String *t, const CHARSET_INFO * const cs)
 {
  return cs->coll->strnncollsp(cs,
                               (unsigned char *) s->ptr(),s->length(),
@@ -462,13 +560,13 @@ int sortcmp(const String *s,const String *t, const charset_info_st * const cs)
 
 int stringcmp(const String *s,const String *t)
 {
-  size_t s_len= s->length(), t_len= t->length(), len= min(s_len,t_len);
+  uint32_t s_len= s->length(), t_len= t->length(), len= min(s_len,t_len);
   int cmp= memcmp(s->ptr(), t->ptr(), len);
   return (cmp) ? cmp : (int) (s_len - t_len);
 }
 
 
-String *copy_if_not_alloced(String *to,String *from,size_t from_length)
+String *copy_if_not_alloced(String *to,String *from,uint32_t from_length)
 {
   if (from->Alloced_length >= from_length)
     return from;
@@ -477,7 +575,8 @@ String *copy_if_not_alloced(String *to,String *from,size_t from_length)
     (void) from->realloc(from_length);
     return from;
   }
-  to->realloc(from_length);
+  if (to->realloc(from_length))
+    return from;				// Actually an error
   if ((to->str_length= min(from->str_length,from_length)))
     memcpy(to->Ptr,from->Ptr,to->str_length);
   to->str_charset=from->str_charset;
@@ -516,17 +615,17 @@ String *copy_if_not_alloced(String *to,String *from,size_t from_length)
 */
 
 
-size_t
-well_formed_copy_nchars(const charset_info_st * const to_cs,
-                        char *to, size_t to_length,
-                        const charset_info_st * const from_cs,
-                        const char *from, size_t from_length,
-                        size_t nchars,
+uint32_t
+well_formed_copy_nchars(const CHARSET_INFO * const to_cs,
+                        char *to, uint32_t to_length,
+                        const CHARSET_INFO * const from_cs,
+                        const char *from, uint32_t from_length,
+                        uint32_t nchars,
                         const char **well_formed_error_pos,
                         const char **cannot_convert_error_pos,
                         const char **from_end_pos)
 {
-  size_t res;
+  uint32_t res;
 
   assert((to_cs == &my_charset_bin) ||
          (from_cs == &my_charset_bin) ||
@@ -552,7 +651,7 @@ well_formed_copy_nchars(const charset_info_st * const to_cs,
   else
   {
     int well_formed_error;
-    size_t from_offset;
+    uint32_t from_offset;
 
     if ((from_offset= (from_length % to_cs->mbminlen)) &&
         (from_cs == &my_charset_bin))
@@ -562,7 +661,7 @@ well_formed_copy_nchars(const charset_info_st * const to_cs,
         INSERT INTO t1 (ucs2_column) VALUES (0x01);
         0x01 -> 0x0001
       */
-      size_t pad_length= to_cs->mbminlen - from_offset;
+      uint32_t pad_length= to_cs->mbminlen - from_offset;
       memset(to, 0, pad_length);
       memmove(to + pad_length, from, from_offset);
       nchars--;
@@ -573,7 +672,8 @@ well_formed_copy_nchars(const charset_info_st * const to_cs,
     }
 
     set_if_smaller(from_length, to_length);
-    res= to_cs->cset->well_formed_len(*to_cs, str_ref(from, from_length), nchars, &well_formed_error);
+    res= to_cs->cset->well_formed_len(to_cs, from, from + from_length,
+                                      nchars, &well_formed_error);
     memmove(to, from, res);
     *from_end_pos= from + res;
     *well_formed_error_pos= well_formed_error ? from + res : NULL;
@@ -585,34 +685,37 @@ well_formed_copy_nchars(const charset_info_st * const to_cs,
   return res;
 }
 
-void String::print(String& str) const
+
+
+
+void String::print(String *str)
 {
-  const char* last= Ptr + str_length;
-  for (const char* st= Ptr; st < last; st++)
+  char *st= (char*)Ptr, *end= st+str_length;
+  for (; st < end; st++)
   {
     unsigned char c= *st;
     switch (c)
     {
     case '\\':
-      str.append("\\\\", sizeof("\\\\")-1);
+      str->append("\\\\", sizeof("\\\\")-1);
       break;
     case '\0':
-      str.append("\\0", sizeof("\\0")-1);
+      str->append("\\0", sizeof("\\0")-1);
       break;
     case '\'':
-      str.append("\\'", sizeof("\\'")-1);
+      str->append("\\'", sizeof("\\'")-1);
       break;
     case '\n':
-      str.append("\\n", sizeof("\\n")-1);
+      str->append("\\n", sizeof("\\n")-1);
       break;
     case '\r':
-      str.append("\\r", sizeof("\\r")-1);
+      str->append("\\r", sizeof("\\r")-1);
       break;
     case '\032': // Ctrl-Z
-      str.append("\\Z", sizeof("\\Z")-1);
+      str->append("\\Z", sizeof("\\Z")-1);
       break;
     default:
-      str.append(c);
+      str->append(c);
     }
   }
 }
@@ -628,17 +731,24 @@ void String::print(String& str) const
 */
 
 /* Factor the extern out */
-extern const charset_info_st *system_charset_info;
+extern const CHARSET_INFO *system_charset_info, *files_charset_info;
 
-void String::append_identifier(const char *name, size_t in_length)
+void String::append_identifier(const char *name, uint32_t in_length)
 {
-  // The identifier must be quoted as it includes a quote character or it's a keyword
+  const char *name_end;
+  char quote_char;
+  int q= '`';
 
-  reserve(in_length * 2 + 2);
-  const char quote_char= '`';
-  append(&quote_char, 1);
+  /*
+    The identifier must be quoted as it includes a quote character or
+   it's a keyword
+  */
 
-  for (const char* name_end= name+in_length ; name < name_end ; name+= in_length)
+  reserve(in_length*2 + 2);
+  quote_char= (char) q;
+  append(&quote_char, 1, system_charset_info);
+
+  for (name_end= name+in_length ; name < name_end ; name+= in_length)
   {
     unsigned char chr= (unsigned char) *name;
     in_length= my_mbcharlen(system_charset_info, chr);
@@ -652,33 +762,63 @@ void String::append_identifier(const char *name, size_t in_length)
     if (!in_length)
       in_length= 1;
     if (in_length == 1 && chr == (unsigned char) quote_char)
-      append(&quote_char, 1);
-    append(name, in_length);
+      append(&quote_char, 1, system_charset_info);
+    append(name, in_length, system_charset_info);
   }
-  append(&quote_char, 1);
+  append(&quote_char, 1, system_charset_info);
 }
 
-void String::append_identifier(str_ref v)
+
+/*
+  Exchange state of this object and argument.
+
+  SYNOPSIS
+    String::swap()
+
+  RETURN
+    Target string will contain state of this object and vice versa.
+*/
+
+void String::swap(String &s)
 {
-  append_identifier(v.data(), v.size());
+  std::swap(Ptr, s.Ptr);
+  std::swap(str_length, s.str_length);
+  std::swap(Alloced_length, s.Alloced_length);
+  std::swap(alloced, s.alloced);
+  std::swap(str_charset, s.str_charset);
 }
 
-bool check_if_only_end_space(const charset_info_st * const cs, char *str, char *end)
+void String::q_append(const uint32_t n)
+{
+  int4store(Ptr + str_length, n);
+  str_length += 4;
+}
+void String::q_append(double d)
+{
+  float8store(Ptr + str_length, d);
+  str_length += 8;
+}
+void String::q_append(double *d)
+{
+  float8store(Ptr + str_length, *d);
+  str_length += 8;
+}
+void String::q_append(const char *data, uint32_t data_len)
+{
+  memcpy(Ptr + str_length, data, data_len);
+  str_length += data_len;
+}
+
+void String::write_at_position(int position, uint32_t value)
+{
+  int4store(Ptr + position,value);
+}
+bool check_if_only_end_space(const CHARSET_INFO * const cs, char *str,
+                             char *end)
 {
   return str+ cs->cset->scan(cs, str, end, MY_SEQ_SPACES) == end;
 }
 
-std::ostream& operator<<(std::ostream& output, const String &str)
-{
-  output << "String:(";
-  output <<  const_cast<String&>(str).c_str();
-  output << ", ";
-  output << str.length();
-  output << ")";
-
-  return output;  // for multiple << operators.
-}
-
 } /* namespace drizzled */
 
 bool operator==(const drizzled::String &s1, const drizzled::String &s2)
