@@ -11,35 +11,32 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
   UNION  of select's
   UNION's  were introduced by Monty and Sinisa <sinisa@mysql.com>
 */
-#include <config.h>
-
+#include "config.h"
 #include <drizzled/sql_select.h>
 #include <drizzled/error.h>
 #include <drizzled/item/type_holder.h>
 #include <drizzled/sql_base.h>
 #include <drizzled/sql_union.h>
-#include <drizzled/select_union.h>
-#include <drizzled/sql_lex.h>
-#include <drizzled/session.h>
-#include <drizzled/item/subselect.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
 bool drizzle_union(Session *session, LEX *, select_result *result,
 		   Select_Lex_Unit *unit, uint64_t setup_tables_done_option)
 {
-  bool res= unit->prepare(session, result, SELECT_NO_UNLOCK | setup_tables_done_option);
-  if (not res)
+  bool res;
+  if (!(res= unit->prepare(session, result, SELECT_NO_UNLOCK |
+                           setup_tables_done_option)))
     res= unit->exec();
   if (res)
-    unit->cleanup();
-  return res;
+    res|= unit->cleanup();
+  return(res);
 }
 
 
@@ -62,18 +59,17 @@ bool select_union::send_data(List<Item> &values)
     unit->offset_limit_cnt--;
     return 0;
   }
-  fill_record(session, table->getFields(), values, true);
+  fill_record(session, table->field, values, true);
   if (session->is_error())
     return 1;
 
-  if ((error= table->cursor->insertRecord(table->getInsertRecord())))
+  if ((error= table->cursor->ha_write_row(table->record[0])))
   {
     /* create_myisam_from_heap will generate error if needed */
-    if (table->cursor->is_fatal_error(error, HA_CHECK_DUP))
-    {
-      my_error(ER_USE_SQL_BIG_RESULT, MYF(0));
-      return true;
-    }
+    if (table->cursor->is_fatal_error(error, HA_CHECK_DUP) &&
+        create_myisam_from_heap(session, table, tmp_table_param.start_recinfo,
+                                &tmp_table_param.recinfo, error, 1))
+      return 1;
   }
   return 0;
 }
@@ -125,18 +121,14 @@ select_union::create_result_table(Session *session_arg, List<Item> *column_types
 {
   assert(table == NULL);
   tmp_table_param.init();
-  tmp_table_param.field_count= column_types->size();
+  tmp_table_param.field_count= column_types->elements;
 
   if (! (table= create_tmp_table(session_arg, &tmp_table_param, *column_types,
-                                 (Order*) NULL, is_union_distinct, 1,
+                                 (order_st*) NULL, is_union_distinct, 1,
                                  options, HA_POS_ERROR, (char*) table_alias)))
-  {
     return true;
-  }
-
   table->cursor->extra(HA_EXTRA_WRITE_CACHE);
   table->cursor->extra(HA_EXTRA_IGNORE_DUP_KEY);
-
   return false;
 }
 
@@ -171,7 +163,7 @@ void select_union::cleanup()
 void
 Select_Lex_Unit::init_prepare_fake_select_lex(Session *session_arg)
 {
-  session_arg->lex().current_select= fake_select_lex;
+  session_arg->lex->current_select= fake_select_lex;
   fake_select_lex->table_list.link_in_list((unsigned char *)&result_table_list,
 					   (unsigned char **)
 					   &result_table_list.next_local);
@@ -179,12 +171,12 @@ Select_Lex_Unit::init_prepare_fake_select_lex(Session *session_arg)
     fake_select_lex->context.first_name_resolution_table=
     fake_select_lex->get_table_list();
 
-  for (Order *order= (Order *) global_parameters->order_list.first;
+  for (order_st *order= (order_st *) global_parameters->order_list.first;
        order;
        order= order->next)
     order->item= &order->item_ptr;
 
-  for (Order *order= (Order *)global_parameters->order_list.first;
+  for (order_st *order= (order_st *)global_parameters->order_list.first;
        order;
        order=order->next)
   {
@@ -197,7 +189,7 @@ Select_Lex_Unit::init_prepare_fake_select_lex(Session *session_arg)
 bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
                               uint64_t additional_options)
 {
-  Select_Lex *lex_select_save= session_arg->lex().current_select;
+  Select_Lex *lex_select_save= session_arg->lex->current_select;
   Select_Lex *sl, *first_sl= first_select();
   select_result *tmp_result;
   bool is_union_select;
@@ -223,18 +215,18 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
 	offset_limit_cnt= 0;
 	if (result->prepare(sl->join->fields_list, this))
 	{
-	  return true;
+	  return(true);
 	}
 	sl->join->select_options|= SELECT_DESCRIBE;
 	sl->join->reinit();
       }
     }
-    return false;
+    return(false);
   }
   prepared= 1;
   saved_error= false;
 
-  session_arg->lex().current_select= sl= first_sl;
+  session_arg->lex->current_select= sl= first_sl;
   found_rows_for_union= first_sl->options & OPTION_FOUND_ROWS;
   is_union_select= is_union() || fake_select_lex;
 
@@ -242,7 +234,8 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
 
   if (is_union_select)
   {
-    tmp_result= union_result= new select_union;
+    if (!(tmp_result= union_result= new select_union))
+      goto err;
     if (describe)
       tmp_result= sel_result;
   }
@@ -255,7 +248,7 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
   {
     bool can_skip_order_by;
     sl->options|=  SELECT_NO_UNLOCK;
-    Join *join= new Join(session_arg, sl->item_list,
+    JOIN *join= new JOIN(session_arg, sl->item_list,
 			 sl->options | session_arg->options | additional_options,
 			 tmp_result);
     /*
@@ -268,7 +261,7 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
     if (!join)
       goto err;
 
-    session_arg->lex().current_select= sl;
+    session_arg->lex->current_select= sl;
 
     can_skip_order_by= is_union_select && !(sl->braces && sl->explicit_limit);
 
@@ -277,11 +270,11 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
                                sl->with_wild,
                                sl->where,
                                (can_skip_order_by ? 0 :
-                                sl->order_list.size()) +
-                               sl->group_list.size(),
+                                sl->order_list.elements) +
+                               sl->group_list.elements,
                                can_skip_order_by ?
-                               (Order*) NULL : (Order *)sl->order_list.first,
-                               (Order*) sl->group_list.first,
+                               (order_st*) NULL : (order_st *)sl->order_list.first,
+                               (order_st*) sl->group_list.first,
                                sl->having,
                                sl, this);
     /* There are no * in the statement anymore (for PS) */
@@ -304,10 +297,11 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
         field object without table.
       */
       assert(!empty_table);
-      empty_table= (Table*) session->mem.calloc(sizeof(Table));
-      types.clear();
-      List<Item>::iterator it(sl->item_list.begin());
-      while (Item* item_tmp= it++)
+      empty_table= (Table*) session->calloc(sizeof(Table));
+      types.empty();
+      List_iterator_fast<Item> it(sl->item_list);
+      Item *item_tmp;
+      while ((item_tmp= it++))
       {
 	/* Error's in 'new' will be detected after loop */
 	types.push_back(new Item_type_holder(session_arg, item_tmp));
@@ -318,19 +312,19 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
     }
     else
     {
-      if (types.size() != sl->item_list.size())
+      if (types.elements != sl->item_list.elements)
       {
 	my_message(ER_WRONG_NUMBER_OF_COLUMNS_IN_SELECT,
 		   ER(ER_WRONG_NUMBER_OF_COLUMNS_IN_SELECT),MYF(0));
 	goto err;
       }
-      List<Item>::iterator it(sl->item_list.begin());
-      List<Item>::iterator tp(types.begin());
+      List_iterator_fast<Item> it(sl->item_list);
+      List_iterator_fast<Item> tp(types);
       Item *type, *item_tmp;
       while ((type= tp++, item_tmp= it++))
       {
         if (((Item_type_holder*)type)->join_types(session_arg, item_tmp))
-	  return true;
+	  return(true);
       }
     }
   }
@@ -341,7 +335,7 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
       Check that it was possible to aggregate
       all collations together for UNION.
     */
-    List<Item>::iterator tp(types.begin());
+    List_iterator_fast<Item> tp(types);
     Item *type;
     uint64_t create_options;
 
@@ -355,51 +349,57 @@ bool Select_Lex_Unit::prepare(Session *session_arg, select_result *sel_result,
       }
     }
 
-    create_options= first_sl->options | session_arg->options | TMP_TABLE_ALL_COLUMNS;
+    create_options= (first_sl->options | session_arg->options |
+                     TMP_TABLE_ALL_COLUMNS);
 
-    if (union_result->create_result_table(session, &types, test(union_distinct), create_options, ""))
+    if (union_result->create_result_table(session, &types, test(union_distinct),
+                                          create_options, ""))
       goto err;
     memset(&result_table_list, 0, sizeof(result_table_list));
-    result_table_list.setSchemaName("");
+    result_table_list.db= (char*) "";
     result_table_list.alias= "union";
-    result_table_list.setTableName("union");
+    result_table_list.table_name= (char *) "union";
     result_table_list.table= table= union_result->table;
 
-    session_arg->lex().current_select= lex_select_save;
-    if (item_list.is_empty())
-      table->fill_item_list(item_list);
+    session_arg->lex->current_select= lex_select_save;
+    if (!item_list.elements)
+    {
+      saved_error= table->fill_item_list(&item_list);
+      if (saved_error)
+        goto err;
+    }
     else
     {
       /*
         We're in execution of a prepared statement or stored procedure:
         reset field items to point at fields from the created temporary table.
       */
-      assert(false);
+      assert(1);
     }
   }
 
-  session_arg->lex().current_select= lex_select_save;
+  session_arg->lex->current_select= lex_select_save;
 
   return(saved_error || session_arg->is_fatal_error);
 
 err:
-  session_arg->lex().current_select= lex_select_save;
-  return true;
+  session_arg->lex->current_select= lex_select_save;
+  return(true);
 }
 
 
 bool Select_Lex_Unit::exec()
 {
-  Select_Lex *lex_select_save= session->lex().current_select;
+  Select_Lex *lex_select_save= session->lex->current_select;
   Select_Lex *select_cursor=first_select();
   uint64_t add_rows=0;
   ha_rows examined_rows= 0;
 
-  if (executed && uncacheable.none() && ! describe)
-    return false;
+  if (executed && !uncacheable && !describe)
+    return(false);
   executed= 1;
 
-  if (uncacheable.any() || ! item || ! item->assigned() || describe)
+  if (uncacheable || !item || !item->assigned() || describe)
   {
     if (item)
       item->reset_value_registration();
@@ -420,7 +420,7 @@ bool Select_Lex_Unit::exec()
     for (Select_Lex *sl= select_cursor; sl; sl= sl->next_select())
     {
       ha_rows records_at_start= 0;
-      session->lex().current_select= sl;
+      session->lex->current_select= sl;
 
       if (optimized)
 	saved_error= sl->join->reinit();
@@ -456,7 +456,7 @@ bool Select_Lex_Unit::exec()
         if (sl == union_distinct)
 	{
 	  if (table->cursor->ha_disable_indexes(HA_KEY_SWITCH_ALL))
-	    return true;
+	    return(true);
 	  table->no_keyread=1;
 	}
 	saved_error= sl->join->error;
@@ -468,14 +468,14 @@ bool Select_Lex_Unit::exec()
 	  examined_rows+= session->examined_row_count;
 	  if (union_result->flush())
 	  {
-	    session->lex().current_select= lex_select_save;
-	    return 1;
+	    session->lex->current_select= lex_select_save;
+	    return(1);
 	  }
 	}
       }
       if (saved_error)
       {
-	session->lex().current_select= lex_select_save;
+	session->lex->current_select= lex_select_save;
 	return(saved_error);
       }
       /* Needed for the following test and for records_at_start in next loop */
@@ -483,7 +483,7 @@ bool Select_Lex_Unit::exec()
       if (error)
       {
         table->print_error(error, MYF(0));
-        return 1;
+        return(1);
       }
       if (found_rows_for_union && !sl->braces &&
           select_limit_cnt != HA_POS_ERROR)
@@ -508,31 +508,36 @@ bool Select_Lex_Unit::exec()
     {
       set_limit(global_parameters);
       init_prepare_fake_select_lex(session);
-      Join *join= fake_select_lex->join;
+      JOIN *join= fake_select_lex->join;
       if (!join)
       {
 	/*
 	  allocate JOIN for fake select only once (prevent
-	  select_query automatic allocation)
-          TODO: The above is nonsense. select_query() will not allocate the
+	  mysql_select automatic allocation)
+          TODO: The above is nonsense. mysql_select() will not allocate the
           join if one already exists. There must be some other reason why we
           don't let it allocate the join. Perhaps this is because we need
           some special parameter values passed to join constructor?
 	*/
-	fake_select_lex->join= new Join(session, item_list, fake_select_lex->options, result);
-  fake_select_lex->join->no_const_tables= true;
+	if (!(fake_select_lex->join= new JOIN(session, item_list,
+					      fake_select_lex->options, result)))
+	{
+	  fake_select_lex->table_list.empty();
+	  return(true);
+	}
+        fake_select_lex->join->no_const_tables= true;
 
 	/*
 	  Fake Select_Lex should have item list for correctref_array
 	  allocation.
 	*/
 	fake_select_lex->item_list= item_list;
-        saved_error= select_query(session, &fake_select_lex->ref_pointer_array,
+        saved_error= mysql_select(session, &fake_select_lex->ref_pointer_array,
                               &result_table_list,
                               0, item_list, NULL,
-                              global_parameters->order_list.size(),
-                              (Order*)global_parameters->order_list.first,
-                              (Order*) NULL, NULL,
+                              global_parameters->order_list.elements,
+                              (order_st*)global_parameters->order_list.first,
+                              (order_st*) NULL, NULL,
                               fake_select_lex->options | SELECT_NO_UNLOCK,
                               result, this, fake_select_lex);
       }
@@ -550,12 +555,12 @@ bool Select_Lex_Unit::exec()
             to reset them back, we re-do all of the actions (yes it is ugly):
           */
 	        join->reset(session, item_list, fake_select_lex->options, result);
-          saved_error= select_query(session, &fake_select_lex->ref_pointer_array,
+          saved_error= mysql_select(session, &fake_select_lex->ref_pointer_array,
                                 &result_table_list,
                                 0, item_list, NULL,
-                                global_parameters->order_list.size(),
-                                (Order*)global_parameters->order_list.first,
-                                (Order*) NULL, NULL,
+                                global_parameters->order_list.elements,
+                                (order_st*)global_parameters->order_list.first,
+                                (order_st*) NULL, NULL,
                                 fake_select_lex->options | SELECT_NO_UNLOCK,
                                 result, this, fake_select_lex);
         }
@@ -567,7 +572,7 @@ bool Select_Lex_Unit::exec()
         }
       }
 
-      fake_select_lex->table_list.clear();
+      fake_select_lex->table_list.empty();
       if (!saved_error)
       {
 	session->limit_found_rows = (uint64_t)table->cursor->stats.records + add_rows;
@@ -579,7 +584,7 @@ bool Select_Lex_Unit::exec()
       */
     }
   }
-  session->lex().current_select= lex_select_save;
+  session->lex->current_select= lex_select_save;
   return(saved_error);
 }
 
@@ -590,13 +595,16 @@ bool Select_Lex_Unit::cleanup()
 
   if (cleaned)
   {
-    return false;
+    return(false);
   }
   cleaned= 1;
 
   if (union_result)
   {
-    safe_delete(union_result);
+    delete union_result;
+    union_result=0; // Safety
+    if (table)
+      table->free_tmp_table(session);
     table= 0; // Safety
   }
 
@@ -605,17 +613,17 @@ bool Select_Lex_Unit::cleanup()
 
   if (fake_select_lex)
   {
-    Join *join;
+    JOIN *join;
     if ((join= fake_select_lex->join))
     {
       join->tables_list= 0;
       join->tables= 0;
     }
     error|= fake_select_lex->cleanup();
-    if (fake_select_lex->order_list.size())
+    if (fake_select_lex->order_list.elements)
     {
-      Order *ord;
-      for (ord= (Order*)fake_select_lex->order_list.first; ord; ord= ord->next)
+      order_st *ord;
+      for (ord= (order_st*)fake_select_lex->order_list.first; ord; ord= ord->next)
         (*ord->item)->cleanup();
     }
   }
@@ -698,15 +706,16 @@ bool Select_Lex::cleanup()
   {
     assert((Select_Lex*)join->select_lex == this);
     error= join->destroy();
-    safe_delete(join);
+    delete join;
+    join= 0;
   }
   for (Select_Lex_Unit *lex_unit= first_inner_unit(); lex_unit ;
        lex_unit= lex_unit->next_unit())
   {
     error= (bool) ((uint32_t) error | (uint32_t) lex_unit->cleanup());
   }
-  non_agg_fields.clear();
-  inner_refs_list.clear();
+  non_agg_fields.empty();
+  inner_refs_list.empty();
   return(error);
 }
 
