@@ -11,48 +11,38 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 
 /*
   Single table and multi table updates of tables.
+  Multi-table updates were introduced by Sinisa & Monty
 */
+#include "config.h"
+#include "drizzled/sql_select.h"
+#include "drizzled/error.h"
+#include "drizzled/probes.h"
+#include "drizzled/sql_base.h"
+#include "drizzled/field/timestamp.h"
+#include "drizzled/sql_parse.h"
+#include "drizzled/optimizer/range.h"
+#include "drizzled/records.h"
+#include "drizzled/internal/my_sys.h"
+#include "drizzled/internal/iocache.h"
 
-#include <config.h>
-
-#include <drizzled/sql_select.h>
-#include <drizzled/error.h>
-#include <drizzled/probes.h>
-#include <drizzled/sql_base.h>
-#include <drizzled/field/epoch.h>
-#include <drizzled/sql_parse.h>
-#include <drizzled/optimizer/range.h>
-#include <drizzled/records.h>
-#include <drizzled/internal/my_sys.h>
-#include <drizzled/internal/iocache.h>
-#include <drizzled/transaction_services.h>
-#include <drizzled/filesort.h>
-#include <drizzled/plugin/storage_engine.h>
-#include <drizzled/key.h>
-#include <drizzled/sql_lex.h>
-#include <drizzled/diagnostics_area.h>
-#include <drizzled/util/test.h>
-#include <drizzled/statistics_variables.h>
-#include <drizzled/session/transactions.h>
-
-#include <boost/dynamic_bitset.hpp>
 #include <list>
 
 using namespace std;
 
-namespace drizzled {
+namespace drizzled
+{
 
 /**
   Re-read record if more columns are needed for error message.
 
   If we got a duplicate key error, we want to write an error
   message containing the value of the duplicate key. If we do not have
-  all fields of the key value in getInsertRecord(), we need to re-read the
+  all fields of the key value in record[0], we need to re-read the
   record with a proper read_set.
 
   @param[in] error   error number
@@ -61,16 +51,18 @@ namespace drizzled {
 
 static void prepare_record_for_error_message(int error, Table *table)
 {
-  Field **field_p= NULL;
-  Field *field= NULL;
-  uint32_t keynr= 0;
+  Field **field_p;
+  Field *field;
+  uint32_t keynr;
+  MyBitmap unique_map; /* Fields in offended unique. */
+  my_bitmap_map unique_map_buf[bitmap_buffer_size(MAX_FIELDS)];
 
   /*
     Only duplicate key errors print the key value.
     If storage engine does always read all columns, we have the value alraedy.
   */
   if ((error != HA_ERR_FOUND_DUPP_KEY) ||
-      ! (table->cursor->getEngine()->check_flag(HTON_BIT_PARTIAL_COLUMN_READ)))
+      !(table->cursor->getEngine()->check_flag(HTON_BIT_PARTIAL_COLUMN_READ)))
     return;
 
   /*
@@ -81,35 +73,31 @@ static void prepare_record_for_error_message(int error, Table *table)
     return;
 
   /* Create unique_map with all fields used by that index. */
-  boost::dynamic_bitset<> unique_map(table->getShare()->sizeFields()); /* Fields in offended unique. */
-  table->mark_columns_used_by_index_no_reset(keynr, unique_map);
+  unique_map.init(unique_map_buf, table->s->fields);
+  table->mark_columns_used_by_index_no_reset(keynr, &unique_map);
 
   /* Subtract read_set and write_set. */
-  unique_map-= *table->read_set;
-  unique_map-= *table->write_set;
+  bitmap_subtract(&unique_map, table->read_set);
+  bitmap_subtract(&unique_map, table->write_set);
 
   /*
     If the unique index uses columns that are neither in read_set
     nor in write_set, we must re-read the record.
     Otherwise no need to do anything.
   */
-  if (unique_map.none())
+  if (unique_map.isClearAll())
     return;
 
   /* Get identifier of last read record into table->cursor->ref. */
-  table->cursor->position(table->getInsertRecord());
+  table->cursor->position(table->record[0]);
   /* Add all fields used by unique index to read_set. */
-  *table->read_set|= unique_map;
+  bitmap_union(table->read_set, &unique_map);
   /* Read record that is identified by table->cursor->ref. */
-  (void) table->cursor->rnd_pos(table->getUpdateRecord(), table->cursor->ref);
+  (void) table->cursor->rnd_pos(table->record[1], table->cursor->ref);
   /* Copy the newly read columns into the new record. */
-  for (field_p= table->getFields(); (field= *field_p); field_p++)
-  {
-    if (unique_map.test(field->position()))
-    {
-      field->copy_from_tmp(table->getShare()->rec_buff_length);
-    }
-  }
+  for (field_p= table->field; (field= *field_p); field_p++)
+    if (unique_map.isBitSet(field->field_index))
+      field->copy_from_tmp(table->s->rec_buff_length);
 
   return;
 }
@@ -119,7 +107,7 @@ static void prepare_record_for_error_message(int error, Table *table)
   Process usual UPDATE
 
   SYNOPSIS
-    update_query()
+    mysql_update()
     session			thread handler
     fields		fields for update
     values		values of fields for update
@@ -134,29 +122,30 @@ static void prepare_record_for_error_message(int error, Table *table)
     1  - error
 */
 
-int update_query(Session *session, TableList *table_list,
+int mysql_update(Session *session, TableList *table_list,
                  List<Item> &fields, List<Item> &values, COND *conds,
-                 uint32_t order_num, Order *order,
+                 uint32_t order_num, order_st *order,
                  ha_rows limit, enum enum_duplicates,
                  bool ignore)
 {
   bool		using_limit= limit != HA_POS_ERROR;
   bool		used_key_is_modified;
   bool		transactional_table;
-  int		error= 0;
+  bool		can_compare_record;
+  int		error;
   uint		used_index= MAX_KEY, dup_key_found;
   bool          need_sort= true;
   ha_rows	updated, found;
   key_map	old_covering_keys;
   Table		*table;
   optimizer::SqlSelect *select= NULL;
-  ReadRecord	info;
-  Select_Lex    *select_lex= &session->lex().select_lex;
+  READ_RECORD	info;
+  Select_Lex    *select_lex= &session->lex->select_lex;
   uint64_t     id;
   List<Item> all_fields;
-  Session::killed_state_t killed_status= Session::NOT_KILLED;
+  Session::killed_state killed_status= Session::NOT_KILLED;
 
-  DRIZZLE_UPDATE_START(session->getQueryString()->c_str());
+  DRIZZLE_UPDATE_START(session->query.c_str());
   if (session->openTablesLock(table_list))
   {
     DRIZZLE_UPDATE_DONE(1, 0, 0);
@@ -167,53 +156,40 @@ int update_query(Session *session, TableList *table_list,
   table= table_list->table;
 
   /* Calculate "table->covering_keys" based on the WHERE */
-  table->covering_keys= table->getShare()->keys_in_use;
+  table->covering_keys= table->s->keys_in_use;
   table->quick_keys.reset();
 
-  if (prepare_update(session, table_list, &conds, order_num, order))
-  {
-    DRIZZLE_UPDATE_DONE(1, 0, 0);
-    return 1;
-  }
+  if (mysql_prepare_update(session, table_list, &conds, order_num, order))
+    goto abort;
 
   old_covering_keys= table->covering_keys;		// Keys used in WHERE
   /* Check the fields we are going to modify */
   if (setup_fields_with_no_wrap(session, 0, fields, MARK_COLUMNS_WRITE, 0, 0))
-  {
-    DRIZZLE_UPDATE_DONE(1, 0, 0);
-    return 1;
-  }
-
+    goto abort;
   if (table->timestamp_field)
   {
     // Don't set timestamp column if this is modified
     if (table->timestamp_field->isWriteSet())
-    {
       table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;
-    }
     else
     {
       if (table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_UPDATE ||
           table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_BOTH)
-      {
-        table->setWriteSet(table->timestamp_field->position());
-      }
+        table->setWriteSet(table->timestamp_field->field_index);
     }
   }
 
   if (setup_fields(session, 0, values, MARK_COLUMNS_READ, 0, 0))
   {
     free_underlaid_joins(session, select_lex);
-    DRIZZLE_UPDATE_DONE(1, 0, 0);
-
-    return 1;
+    goto abort;
   }
 
-  if (select_lex->inner_refs_list.size() &&
+  if (select_lex->inner_refs_list.elements &&
     fix_inner_refs(session, all_fields, select_lex, select_lex->ref_pointer_array))
   {
     DRIZZLE_UPDATE_DONE(1, 0, 0);
-    return 1;
+    return -1;
   }
 
   if (conds)
@@ -233,9 +209,7 @@ int update_query(Session *session, TableList *table_list,
       table->timestamp_field &&
       (table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_UPDATE ||
        table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_BOTH))
-  {
-    *table->read_set|= *table->write_set;
-  }
+    bitmap_union(table->read_set, table->write_set);
   // Don't count on usage of 'only index' when calculating which key to use
   table->covering_keys.reset();
 
@@ -251,13 +225,10 @@ int update_query(Session *session, TableList *table_list,
      * Resetting the Diagnostic area to prevent
      * lp bug# 439719
      */
-    session->main_da().reset_diagnostics_area();
+    session->main_da.reset_diagnostics_area();
     free_underlaid_joins(session, select_lex);
-    if (error || session->is_error())
-    {
-      DRIZZLE_UPDATE_DONE(1, 0, 0);
-      return 1;
-    }
+    if (error)
+      goto abort;				// Error in where
     DRIZZLE_UPDATE_DONE(0, 0, 0);
     session->my_ok();				// No matching records
     return 0;
@@ -281,7 +252,7 @@ int update_query(Session *session, TableList *table_list,
   {
     used_index= select->quick->index;
     used_key_is_modified= (!select->quick->unique_key_range() &&
-                          select->quick->is_keys_used(*table->write_set));
+                          select->quick->is_keys_used(table->write_set));
   }
   else
   {
@@ -289,7 +260,7 @@ int update_query(Session *session, TableList *table_list,
     if (used_index == MAX_KEY)                  // no index for sort order
       used_index= table->cursor->key_used_on_scan;
     if (used_index != MAX_KEY)
-      used_key_is_modified= is_key_used(table, used_index, *table->write_set);
+      used_key_is_modified= is_key_used(table, used_index, table->write_set);
   }
 
 
@@ -318,20 +289,26 @@ int update_query(Session *session, TableList *table_list,
         NOTE: filesort will call table->prepare_for_position()
       */
       uint32_t         length= 0;
-      SortField  *sortorder;
+      SORT_FIELD  *sortorder;
       ha_rows examined_rows;
-      FileSort filesort(*session);
 
-      table->sort.io_cache= new internal::io_cache_st;
-      sortorder=make_unireg_sortorder(order, &length, NULL);
+      table->sort.io_cache = new internal::IO_CACHE;
+      memset(table->sort.io_cache, 0, sizeof(internal::IO_CACHE));
 
-      if ((table->sort.found_records= filesort.run(table, sortorder, length, select, limit, 1, examined_rows)) == HA_POS_ERROR)
-        goto err;
+      if (!(sortorder=make_unireg_sortorder(order, &length, NULL)) ||
+          (table->sort.found_records= filesort(session, table, sortorder, length,
+                                               select, limit, 1,
+                                               &examined_rows))
+          == HA_POS_ERROR)
+      {
+	goto err;
+      }
       /*
 	Filesort has already found and selected the rows we want to update,
 	so we don't need the where clause
       */
-      safe_delete(select);
+      delete select;
+      select= 0;
     }
     else
     {
@@ -341,11 +318,10 @@ int update_query(Session *session, TableList *table_list,
 	update these in a separate loop based on the pointer.
       */
 
-      internal::io_cache_st tempfile;
-      if (tempfile.open_cached_file(drizzle_tmpdir.c_str(),TEMP_PREFIX, DISK_BUFFER_SIZE, MYF(MY_WME)))
-      {
+      internal::IO_CACHE tempfile;
+      if (open_cached_file(&tempfile, drizzle_tmpdir,TEMP_PREFIX,
+			   DISK_BUFFER_SIZE, MYF(MY_WME)))
 	goto err;
-      }
 
       /* If quick select is used, initialize it before retrieving rows. */
       if (select && select->quick && select->quick->reset())
@@ -364,28 +340,23 @@ int update_query(Session *session, TableList *table_list,
       */
 
       if (used_index == MAX_KEY || (select && select->quick))
-      {
-        if ((error= info.init_read_record(session, table, select, 0, true)))
-          goto err;
-      }
+        init_read_record(&info,session,table,select,0,1);
       else
-      {
-        if ((error= info.init_read_record_idx(session, table, 1, used_index)))
-          goto err;
-      }
+        init_read_record_idx(&info, session, table, 1, used_index);
 
       session->set_proc_info("Searching rows for update");
       ha_rows tmp_limit= limit;
 
-      while (not(error= info.read_record(&info)) && not session->getKilled())
+      while (!(error=info.read_record(&info)) && !session->killed)
       {
 	if (!(select && select->skip_record()))
 	{
           if (table->cursor->was_semi_consistent_read())
 	    continue;  /* repeat the read of the same row if it still exists */
 
-	  table->cursor->position(table->getInsertRecord());
-	  if (tempfile.write(table->cursor->ref, table->cursor->ref_length))
+	  table->cursor->position(table->record[0]);
+	  if (my_b_write(&tempfile,table->cursor->ref,
+			 table->cursor->ref_length))
 	  {
 	    error=1;
 	    break;
@@ -399,26 +370,27 @@ int update_query(Session *session, TableList *table_list,
 	else
 	  table->cursor->unlock_row();
       }
-      if (session->getKilled() && not error)
+      if (session->killed && !error)
 	error= 1;				// Aborted
       limit= tmp_limit;
       table->cursor->try_semi_consistent_read(0);
-      info.end_read_record();
+      end_read_record(&info);
 
       /* Change select to use tempfile */
       if (select)
       {
-	safe_delete(select->quick);
+	delete select->quick;
 	if (select->free_cond)
 	  delete select->cond;
+	select->quick=0;
 	select->cond=0;
       }
       else
       {
-	select= new optimizer::SqlSelect();
+	select= new optimizer::SqlSelect;
 	select->head=table;
       }
-      if (tempfile.reinit_io_cache(internal::READ_CACHE,0L,0,0))
+      if (reinit_io_cache(&tempfile,internal::READ_CACHE,0L,0,0))
 	error=1;
       // Read row ptrs from this cursor
       memcpy(select->file, &tempfile, sizeof(tempfile));
@@ -435,10 +407,7 @@ int update_query(Session *session, TableList *table_list,
   if (select && select->quick && select->quick->reset())
     goto err;
   table->cursor->try_semi_consistent_read(1);
-  if ((error= info.init_read_record(session, table, select, 0, true)))
-  {
-    goto err;
-  }
+  init_read_record(&info,session,table,select,0,1);
 
   updated= found= 0;
   /*
@@ -454,7 +423,7 @@ int update_query(Session *session, TableList *table_list,
   session->set_proc_info("Updating");
 
   transactional_table= table->cursor->has_transactions();
-  session->setAbortOnWarning(test(!ignore));
+  session->abort_on_warning= test(!ignore);
 
   /*
     Assure that we can use position()
@@ -463,9 +432,17 @@ int update_query(Session *session, TableList *table_list,
   if (table->cursor->getEngine()->check_flag(HTON_BIT_PARTIAL_COLUMN_READ))
     table->prepare_for_position();
 
-  while (not (error=info.read_record(&info)) && not session->getKilled())
+  /*
+    We can use compare_record() to optimize away updates if
+    the table handler is returning all columns OR if
+    if all updated columns are read
+  */
+  can_compare_record= (!(table->cursor->getEngine()->check_flag(HTON_BIT_PARTIAL_COLUMN_READ)) ||
+                       bitmap_is_subset(table->write_set, table->read_set));
+
+  while (!(error=info.read_record(&info)) && !session->killed)
   {
-    if (not (select && select->skip_record()))
+    if (!(select && select->skip_record()))
     {
       if (table->cursor->was_semi_consistent_read())
         continue;  /* repeat the read of the same row if it still exists */
@@ -476,24 +453,21 @@ int update_query(Session *session, TableList *table_list,
 
       found++;
 
-      if (! table->records_are_comparable() || table->compare_records())
+      if (!can_compare_record || table->compare_record())
       {
         /* Non-batched update */
-        error= table->cursor->updateRecord(table->getUpdateRecord(),
-                                            table->getInsertRecord());
-
-        table->auto_increment_field_not_null= false;
-
+        error= table->cursor->ha_update_row(table->record[1],
+                                            table->record[0]);
         if (!error || error == HA_ERR_RECORD_IS_THE_SAME)
-        {
+	{
           if (error != HA_ERR_RECORD_IS_THE_SAME)
             updated++;
           else
             error= 0;
-        }
-        else if (! ignore ||
+	}
+	else if (! ignore ||
                  table->cursor->is_fatal_error(error, HA_CHECK_DUP_KEY))
-        {
+	{
           /*
             If (ignore && error is ignorable) we don't have to
             do anything; otherwise...
@@ -504,10 +478,10 @@ int update_query(Session *session, TableList *table_list,
             flags|= ME_FATALERROR; /* Other handler errors are fatal */
 
           prepare_record_for_error_message(error, table);
-          table->print_error(error,MYF(flags));
-          error= 1;
-          break;
-        }
+	  table->print_error(error,MYF(flags));
+	  error= 1;
+	  break;
+	}
       }
 
       if (!--limit && using_limit)
@@ -523,12 +497,14 @@ int update_query(Session *session, TableList *table_list,
   dup_key_found= 0;
   /*
     Caching the killed status to pass as the arg to query event constuctor;
-
+    The cached value can not change whereas the killed status can
+    (externally) since this point and change of the latter won't affect
+    binlogging.
     It's assumed that if an error was set in combination with an effective
     killed status then the error is due to killing.
   */
-  killed_status= session->getKilled(); // get the status of the volatile
-
+  killed_status= session->killed; // get the status of the volatile
+  // simulated killing after the loop must be ineffective for binlogging
   error= (killed_status == Session::NOT_KILLED)?  error : 1;
 
   updated-= dup_key_found;
@@ -537,7 +513,7 @@ int update_query(Session *session, TableList *table_list,
   if (!transactional_table && updated > 0)
     session->transaction.stmt.markModifiedNonTransData();
 
-  info.end_read_record();
+  end_read_record(&info);
   delete select;
   session->set_proc_info("end");
   table->cursor->extra(HA_EXTRA_NO_IGNORE_DUP_KEY);
@@ -547,7 +523,9 @@ int update_query(Session *session, TableList *table_list,
     last one without error. error > 0 means an error (e.g. unique key
     violation and no IGNORE or REPLACE). error == 0 is also an error (if
     preparing the record or invoking before triggers fails). See
-    autocommitOrRollback(error>=0) and return(error>=0) below.
+    ha_autocommit_or_rollback(error>=0) and return(error>=0) below.
+    Sometimes we want to binlog even if we updated no rows, in case user used
+    it to be sure master and slave are in same state.
   */
   if ((error < 0) || session->transaction.stmt.hasModifiedNonTransData())
   {
@@ -564,26 +542,22 @@ int update_query(Session *session, TableList *table_list,
   if (error < 0)
   {
     char buff[STRING_BUFFER_USUAL_SIZE];
-    snprintf(buff, sizeof(buff), ER(ER_UPDATE_INFO), (ulong) found, (ulong) updated,
+    sprintf(buff, ER(ER_UPDATE_INFO), (ulong) found, (ulong) updated,
 	    (ulong) session->cuted_fields);
     session->row_count_func= updated;
     /**
      * Resetting the Diagnostic area to prevent
      * lp bug# 439719
      */
-    session->main_da().reset_diagnostics_area();
-    session->my_ok((ulong) session->rowCount(), found, id, buff);
-    session->status_var.updated_row_count+= session->rowCount();
+    session->main_da.reset_diagnostics_area();
+    session->my_ok((ulong) session->row_count_func, found, id, buff);
   }
-  session->count_cuted_fields= CHECK_FIELD_ERROR_FOR_NULL;		/* calc cuted fields */
-  session->setAbortOnWarning(false);
+  session->count_cuted_fields= CHECK_FIELD_IGNORE;		/* calc cuted fields */
+  session->abort_on_warning= 0;
   DRIZZLE_UPDATE_DONE((error >= 0 || session->is_error()), found, updated);
   return ((error >= 0 || session->is_error()) ? 1 : 0);
 
 err:
-  if (error != 0)
-    table->print_error(error,MYF(0));
-
   delete select;
   free_underlaid_joins(session, select_lex);
   if (table->key_read)
@@ -591,8 +565,9 @@ err:
     table->key_read=0;
     table->cursor->extra(HA_EXTRA_NO_KEYREAD);
   }
-  session->setAbortOnWarning(false);
+  session->abort_on_warning= 0;
 
+abort:
   DRIZZLE_UPDATE_DONE(1, 0, 0);
   return 1;
 }
@@ -601,7 +576,7 @@ err:
   Prepare items in UPDATE statement
 
   SYNOPSIS
-    prepare_update()
+    mysql_prepare_update()
     session			- thread handler
     table_list		- global/local table list
     conds		- conditions
@@ -612,26 +587,33 @@ err:
     false OK
     true  error
 */
-bool prepare_update(Session *session, TableList *table_list,
-			 Item **conds, uint32_t order_num, Order *order)
+bool mysql_prepare_update(Session *session, TableList *table_list,
+			 Item **conds, uint32_t order_num, order_st *order)
 {
   List<Item> all_fields;
-  Select_Lex *select_lex= &session->lex().select_lex;
+  Select_Lex *select_lex= &session->lex->select_lex;
 
-  session->lex().allow_sum_func= 0;
+  session->lex->allow_sum_func= 0;
 
-  if (setup_tables_and_check_access(session, &select_lex->context, &select_lex->top_join_list, table_list, &select_lex->leaf_tables, false) ||
-      session->setup_conds(table_list, conds))
-      return true;
-  select_lex->setup_ref_array(session, order_num);
-  if (setup_order(session, select_lex->ref_pointer_array, table_list, all_fields, all_fields, order))
+  if (setup_tables_and_check_access(session, &select_lex->context,
+                                    &select_lex->top_join_list,
+                                    table_list,
+                                    &select_lex->leaf_tables,
+                                    false) ||
+      session->setup_conds(table_list, conds) ||
+      select_lex->setup_ref_array(session, order_num) ||
+      setup_order(session, select_lex->ref_pointer_array,
+		  table_list, all_fields, all_fields, order))
     return true;
 
   /* Check that we are not using table that we are updating in a sub select */
-  if (unique_table(table_list, table_list->next_global))
   {
-    my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->getTableName());
-    return true;
+    TableList *duplicate;
+    if ((duplicate= unique_table(table_list, table_list->next_global)))
+    {
+      my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->table_name);
+      return true;
+    }
   }
 
   return false;
