@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,27 +18,26 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/change_schema.h>
-#include <drizzled/schema.h>
-#include <drizzled/sql_lex.h>
-#include <drizzled/catalog/instance.h>
+#include <drizzled/db.h>
 
 #include <string>
 
 using namespace std;
 
-namespace drizzled {
+namespace drizzled
+{
 
 bool statement::ChangeSchema::execute()
 {
-  identifier::Schema identifier(session().catalog().identifier(),
-                                str_ref(lex().select_lex.db));
-  if (not schema::change(session(), identifier))
+  Select_Lex *select_lex= &session->lex->select_lex;
+
+  if (not mysql_change_db(session, select_lex->db))
   {
-    session().my_ok();
+    session->my_ok();
   }
 
   return false;
