@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,35 +18,36 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_STATEMENT_CREATE_SCHEMA_H
+#define DRIZZLED_STATEMENT_CREATE_SCHEMA_H
 
-#include <drizzled/session.h>
 #include <drizzled/statement.h>
 #include <drizzled/message/schema.pb.h>
-#include <uuid/uuid.h>
 
-namespace drizzled {
-namespace statement {
+namespace drizzled
+{
+class Session;
+
+namespace statement
+{
 
 class CreateSchema : public Statement
 {
 public:
-  CreateSchema(Session *in_session) :
-    Statement(in_session),
-    is_if_not_exists(false)
+  CreateSchema(Session *in_session)
+    :
+      Statement(in_session),
+      is_if_not_exists(false)
   {
-    set_command(SQLCOM_CREATE_DB);
   }
 
   bool execute();
   bool is_if_not_exists;
   message::Schema schema_message;
-
-  bool validateSchemaOptions();
-private:
-  bool check(const identifier::Schema &identifier);
 };
 
 } /* end namespace statement */
+
 } /* end namespace drizzled */
 
+#endif /* DRIZZLED_STATEMENT_CREATE_SCHEMA_H */
