@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,30 +18,31 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_STATEMENT_SAVEPOINT_H
+#define DRIZZLED_STATEMENT_SAVEPOINT_H
 
 #include <drizzled/statement.h>
-#include <drizzled/sql_lex.h>
 
-namespace drizzled {
-namespace statement {
+namespace drizzled
+{
+class Session;
+
+namespace statement
+{
 
 class Savepoint : public Statement
 {
 public:
-  Savepoint(Session *in_session, str_ref ident) :
-    Statement(in_session)
-  {
-    set_command(SQLCOM_SAVEPOINT);
-    lex().ident= ident;
-  }
+  Savepoint(Session *in_session)
+    :
+      Statement(in_session)
+  {}
 
   bool execute();
-  bool isTransactional()
-  {
-    return true;
-  }
 };
 
 } /* namespace statement */
+
 } /* namespace drizzled */
+
+#endif /* DRIZZLED_STATEMENT_SAVEPOINT_H */
