@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/show_warnings.h>
@@ -27,7 +27,8 @@
 
 using namespace std;
 
-namespace drizzled {
+namespace drizzled
+{
 
 bool statement::ShowWarnings::execute()
 {
@@ -35,7 +36,8 @@ bool statement::ShowWarnings::execute()
   warning_levels.set(DRIZZLE_ERROR::WARN_LEVEL_NOTE);
   warning_levels.set(DRIZZLE_ERROR::WARN_LEVEL_WARN);
   warning_levels.set(DRIZZLE_ERROR::WARN_LEVEL_ERROR);
-  return show_warnings(&session(), warning_levels);
+  bool res= mysqld_show_warnings(session, warning_levels);
+  return res;
 }
 
 } /* namespace drizzled */
