@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,21 +18,27 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_STATEMENT_TRUNCATE_H
+#define DRIZZLED_STATEMENT_TRUNCATE_H
 
 #include <drizzled/statement.h>
 
-namespace drizzled {
-namespace statement {
+
+namespace drizzled
+{
+
+class Session;
+
+namespace statement
+{
 
 class Truncate : public Statement
 {
 public:
-  Truncate(Session *in_session) :
-    Statement(in_session)
-  {
-    set_command(SQLCOM_TRUNCATE);
-  }
+  Truncate(Session *in_session)
+    :
+      Statement(in_session)
+  {}
 
   bool execute();
 };
@@ -41,3 +47,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_STATEMENT_TRUNCATE_H */
