@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,107 +18,71 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+/* Structs that defines the Table */
+
+#ifndef DRIZZLED_TABLE_H
+#define DRIZZLED_TABLE_H
 
 #include <string>
-#include <boost/dynamic_bitset.hpp>
 
-#include <drizzled/order.h>
-#include <drizzled/filesort_info.h>
-#include <drizzled/natural_join_column.h>
-#include <drizzled/field_iterator.h>
-#include <drizzled/cursor.h>
-#include <drizzled/lex_string.h>
-#include <drizzled/table/instance.h>
-#include <drizzled/atomics.h>
+#include "drizzled/order.h"
+#include "drizzled/filesort_info.h"
+#include "drizzled/natural_join_column.h"
+#include "drizzled/field_iterator.h"
+#include "drizzled/cursor.h"
+#include "drizzled/lex_string.h"
+#include "drizzled/table_list.h"
+#include "drizzled/table_share.h"
+#include "drizzled/atomics.h"
+#include "drizzled/query_id.h"
 
-#include <drizzled/visibility.h>
+namespace drizzled
+{
 
-namespace drizzled {
+class Item;
+class Item_subselect;
+class Select_Lex_Unit;
+class Select_Lex;
+class COND_EQUAL;
+class SecurityContext;
+class TableList;
+class Field_timestamp;
+class Field_blob;
+
+extern uint64_t refresh_version;
+
+typedef enum enum_table_category TABLE_CATEGORY;
+typedef struct st_columndef MI_COLUMNDEF;
+
+bool create_myisam_from_heap(Session *session, Table *table,
+                             MI_COLUMNDEF *start_recinfo,
+                             MI_COLUMNDEF **recinfo,
+                             int error, bool ignore_last_dupp_key_error);
 
 /**
  * Class representing a set of records, either in a temporary, 
  * normal, or derived table.
  */
-class DRIZZLED_API Table 
+class Table 
 {
+public:
+
+  TableShare *s; /**< Pointer to the shared metadata about the table */
   Field **field; /**< Pointer to fields collection */
 
-public:
-  Field **getFields() const
-  {
-    return field;
-  }
-
-  Field *getField(uint32_t arg) const
-  {
-    return field[arg];
-  }
-
-  void setFields(Field **arg)
-  {
-    field= arg;
-  }
-
-  void setFieldAt(Field *arg, uint32_t arg_pos)
-  {
-    field[arg_pos]= arg;
-  }
-
   Cursor *cursor; /**< Pointer to the storage engine's Cursor managing this table */
-
-private:
   Table *next;
-
-public:
-  Table *getNext() const
-  {
-    return next;
-  }
-
-  Table **getNextPtr()
-  {
-    return &next;
-  }
-
-  void setNext(Table *arg)
-  {
-    next= arg;
-  }
-
-  void unlink()
-  {
-    getNext()->setPrev(getPrev());		/* remove from used chain */
-    getPrev()->setNext(getNext());
-  }
-
-private:
   Table *prev;
-public:
-  Table *getPrev() const
-  {
-    return prev;
-  }
 
-  Table **getPrevPtr()
-  {
-    return &prev;
-  }
-
-  void setPrev(Table *arg)
-  {
-    prev= arg;
-  }
-
-  boost::dynamic_bitset<> *read_set; /* Active column sets */
-  boost::dynamic_bitset<> *write_set; /* Active column sets */
+  MyBitmap *read_set; /* Active column sets */
+  MyBitmap *write_set; /* Active column sets */
 
   uint32_t tablenr;
   uint32_t db_stat; /**< information about the cursor as in Cursor.h */
 
-  boost::dynamic_bitset<> def_read_set; /**< Default read set of columns */
-  boost::dynamic_bitset<> def_write_set; /**< Default write set of columns */
-  boost::dynamic_bitset<> tmp_set; /* Not sure about this... */
+  MyBitmap def_read_set; /**< Default read set of columns */
+  MyBitmap def_write_set; /**< Default write set of columns */
+  MyBitmap tmp_set; /* Not sure about this... */
 
   Session *in_use; /**< Pointer to the current session using this object */
   Session *getSession()
@@ -126,52 +90,23 @@ public:
     return in_use;
   }
 
-  unsigned char *getInsertRecord() const
-  {
-    return record[0];
-  }
-
-  unsigned char *getUpdateRecord()
-  {
-    return record[1];
-  }
-
   unsigned char *record[2]; /**< Pointer to "records" */
-  std::vector<unsigned char> insert_values; /* used by INSERT ... UPDATE */
-  KeyInfo  *key_info; /**< data of keys in database */
+  unsigned char *insert_values; /* used by INSERT ... UPDATE */
+  KEY  *key_info; /**< data of keys in database */
   Field *next_number_field; /**< Set if next_number is activated. @TODO What the heck is the difference between this and the next member? */
   Field *found_next_number_field; /**< Points to the "next-number" field (autoincrement field) */
-  field::Epoch *timestamp_field; /**< Points to the auto-setting timestamp field, if any */
+  Field_timestamp *timestamp_field; /**< Points to the auto-setting timestamp field, if any */
 
   TableList *pos_in_table_list; /* Element referring to this table */
-  Order *group;
-  
-  const char *getAlias() const
-  {
-    return _alias.c_str();
-  }
-
-  void clearAlias()
-  {
-    _alias.clear();
-  }
-
-  void setAlias(const char *arg)
-  {
-    _alias= arg;
-  }
-
-private:
-  std::string _alias; /**< alias or table name if no alias */
-public:
-
+  order_st *group;
+  const char *alias; /**< alias or table name if no alias */
   unsigned char *null_flags;
 
   uint32_t lock_position; /**< Position in DRIZZLE_LOCK.table */
   uint32_t lock_data_start; /**< Start pos. in DRIZZLE_LOCK.locks */
   uint32_t lock_count; /**< Number of locks */
   uint32_t used_fields;
-  uint32_t status; /* What's in getInsertRecord() */
+  uint32_t status; /* What's in record[0] */
   /* number of select if it is derived table */
   uint32_t derived_select_number;
   int current_lock; /**< Type of lock on table */
@@ -191,9 +126,7 @@ public:
   bool null_row;
 
   bool force_index;
-  bool distinct;
-  bool const_table;
-  bool no_rows;
+  bool distinct,const_table,no_rows;
   bool key_read;
   bool no_keyread;
   /*
@@ -263,7 +196,7 @@ public:
     statement then the variable contains TIMESTAMP_NO_AUTO_SET (i.e. 0).
 
     Value of this variable is set for each statement in open_table() and
-    if needed cleared later in statement processing code (see update_query()
+    if needed cleared later in statement processing code (see mysql_update()
     as example).
   */
   timestamp_auto_set_type timestamp_field_type;
@@ -291,10 +224,8 @@ public:
     The set is implemented as a bitmap.
   */
   key_map keys_in_use_for_query;
-
   /* Map of keys that can be used to calculate GROUP BY without sorting */
   key_map keys_in_use_for_group_by;
-
   /* Map of keys that can be used to calculate ORDER BY without sorting */
   key_map keys_in_use_for_order_by;
 
@@ -310,106 +241,67 @@ public:
   uint32_t quick_key_parts[MAX_KEY];
   uint32_t quick_n_ranges[MAX_KEY];
 
-private:
   memory::Root mem_root;
-
-  void init_mem_root()
-  {
-    if (not mem_root.alloc_root_inited())
-      mem_root.init(TABLE_ALLOC_BLOCK_SIZE);
-  }
-public:
-  memory::Root& mem()
-  {
-    init_mem_root();
-    return mem_root;
-  }
-
-  unsigned char* alloc(size_t arg)
-  {
-    init_mem_root();
-    return mem_root.alloc(arg);
-  }
-
-  char* strdup(const char* str_arg, size_t len_arg)
-  {
-    init_mem_root();
-    return mem_root.strdup(str_arg, len_arg);
-  }
-
-  filesort_info sort;
+  filesort_info_st sort;
 
   Table();
-  virtual ~Table();
 
   int report_error(int error);
   /**
    * Free information allocated by openfrm
    *
    * @param If true if we also want to free table_share
-   * @note this should all be the destructor
    */
-  int delete_table(bool free_share= false);
+  int closefrm(bool free_share);
 
   void resetTable(Session *session, TableShare *share, uint32_t db_stat_arg);
 
   /* SHARE methods */
-  virtual const TableShare *getShare() const= 0; /* Get rid of this long term */
-  virtual TableShare *getMutableShare()= 0; /* Get rid of this long term */
-  virtual bool hasShare() const= 0; /* Get rid of this long term */
-  virtual void setShare(TableShare *new_share)= 0; /* Get rid of this long term */
+  inline const TableShare *getShare() const { assert(s); return s; } /* Get rid of this long term */
+  inline void setShare(TableShare *new_share) { s= new_share; } /* Get rid of this long term */
+  inline uint32_t sizeKeys() { return s->keys; }
+  inline uint32_t sizeFields() { return s->fields; }
+  inline uint32_t getRecordLength() { return s->reclength; }
+  inline uint32_t sizeBlobFields() { return s->blob_fields; }
+  inline uint32_t *getBlobField() { return s->blob_field; }
+  inline uint32_t getNullBytes() { return s->null_bytes; }
+  inline uint32_t getNullFields() { return s->null_fields; }
+  inline unsigned char *getDefaultValues() { return s->default_values; }
 
-  virtual void release(void)= 0;
+  inline bool isDatabaseLowByteFirst() { return s->db_low_byte_first; } /* Portable row format */
+  inline bool isNameLock() { return s->name_lock; }
+  inline bool isReplaceWithNameLock() { return s->replace_with_name_lock; }
+  inline bool isWaitingOnCondition() { return s->waiting_on_cond; } /* Protection against free */
 
-  uint32_t sizeKeys() { return getMutableShare()->sizeKeys(); }
-  uint32_t sizeFields() { return getMutableShare()->sizeFields(); }
-  uint32_t getRecordLength() const { return getShare()->getRecordLength(); }
-  uint32_t sizeBlobFields() { return getMutableShare()->blob_fields; }
-  uint32_t *getBlobField() { return &getMutableShare()->blob_field[0]; }
-
-public:
-  virtual bool hasVariableWidth() const
+  uint32_t index_flags(uint32_t idx) const
   {
-    return getShare()->hasVariableWidth(); // We should calculate this.
+    return s->storage_engine->index_flags(s->key_info[idx].algorithm);
   }
 
-  virtual void setVariableWidth(void);
-
-  Field_blob *getBlobFieldAt(uint32_t arg) const
+  inline plugin::StorageEngine *getEngine() const	/* table_type for handler */
   {
-    if (arg < getShare()->blob_fields)
-      return (Field_blob*) field[getShare()->blob_field[arg]]; /*NOTE: Using 'Table.field' NOT SharedTable.field. */
-
-    return NULL;
-  }
-  inline uint8_t getBlobPtrSize() const { return getShare()->sizeBlobPtr(); }
-  inline uint32_t getNullBytes() const { return getShare()->null_bytes; }
-  inline uint32_t getNullFields() const { return getShare()->null_fields; }
-  inline unsigned char *getDefaultValues() { return  getMutableShare()->getDefaultValues(); }
-  inline const char *getSchemaName()  const { return getShare()->getSchemaName(); }
-  inline const char *getTableName()  const { return getShare()->getTableName(); }
-
-  inline bool isDatabaseLowByteFirst() const { return getShare()->db_low_byte_first; } /* Portable row format */
-  inline bool isNameLock() const { return open_placeholder; }
-
-  uint32_t index_flags(uint32_t idx) const;
-
-  inline plugin::StorageEngine *getEngine() const   /* table_type for handler */
-  {
-    return getShare()->getEngine();
+    return s->storage_engine;
   }
 
-  Cursor &getCursor() const /* table_type for handler */
+  Cursor &getCursor() const	/* table_type for handler */
   {
     assert(cursor);
     return *cursor;
   }
 
+  /* For TMP tables, should be pulled out as a class */
+  void updateCreateInfo(message::Table *table_proto);
+  void setup_tmp_table_column_bitmaps(unsigned char *bitmaps);
+  bool create_myisam_tmp_table(KEY *keyinfo,
+                               MI_COLUMNDEF *start_recinfo,
+                               MI_COLUMNDEF **recinfo,
+                               uint64_t options);
+  void free_tmp_table(Session *session);
+  bool open_tmp_table();
   size_t max_row_length(const unsigned char *data);
   uint32_t find_shortest_key(const key_map *usable_keys);
   bool compare_record(Field **ptr);
-  bool records_are_comparable();
-  bool compare_records();
+  bool compare_record();
   /* TODO: the (re)storeRecord's may be able to be further condensed */
   void storeRecord();
   void storeRecordAsInsert();
@@ -418,10 +310,9 @@ public:
   void restoreRecordAsDefault();
   void emptyRecord();
 
-
   /* See if this can be blown away */
-  inline uint32_t getDBStat () const { return db_stat; }
-
+  inline uint32_t getDBStat () { return db_stat; }
+  inline uint32_t setDBStat () { return db_stat; }
   /**
    * Create Item_field for each column in the table.
    *
@@ -438,10 +329,10 @@ public:
    * @retval
    *  true when out of memory
    */
-  void fill_item_list(List<Item>&) const;
+  bool fill_item_list(List<Item> *item_list) const;
   void clear_column_bitmaps(void);
   void prepare_for_position(void);
-  void mark_columns_used_by_index_no_reset(uint32_t index, boost::dynamic_bitset<>& bitmap);
+  void mark_columns_used_by_index_no_reset(uint32_t index, MyBitmap *map);
   void mark_columns_used_by_index_no_reset(uint32_t index);
   void mark_columns_used_by_index(uint32_t index);
   void restore_column_maps_after_mark_index();
@@ -449,15 +340,19 @@ public:
   void mark_columns_needed_for_update(void);
   void mark_columns_needed_for_delete(void);
   void mark_columns_needed_for_insert(void);
-  void column_bitmaps_set(boost::dynamic_bitset<>& read_set_arg,
-                          boost::dynamic_bitset<>& write_set_arg);
+  inline void column_bitmaps_set(MyBitmap *read_set_arg,
+                                 MyBitmap *write_set_arg)
+  {
+    read_set= read_set_arg;
+    write_set= write_set_arg;
+  }
 
-  void restore_column_map(const boost::dynamic_bitset<>& old);
+  void restore_column_map(my_bitmap_map *old);
 
-  const boost::dynamic_bitset<> use_all_columns(boost::dynamic_bitset<>& map);
+  my_bitmap_map *use_all_columns(MyBitmap *bitmap);
   inline void use_all_columns()
   {
-    column_bitmaps_set(getMutableShare()->all_set, getMutableShare()->all_set);
+    column_bitmaps_set(&s->all_set, &s->all_set);
   }
 
   inline void default_column_bitmaps()
@@ -467,66 +362,68 @@ public:
   }
 
   /* Both of the below should go away once we can move this bit to the field objects */
-  inline bool isReadSet(uint32_t index) const
+  inline bool isReadSet(uint32_t index)
   {
-    return read_set->test(index);
+    return read_set->isBitSet(index);
   }
 
   inline void setReadSet(uint32_t index)
   {
-    read_set->set(index);
+    read_set->setBit(index);
   }
 
   inline void setReadSet()
   {
-    read_set->set();
+    read_set->setAll();
   }
 
   inline void clearReadSet(uint32_t index)
   {
-    read_set->reset(index);
+    read_set->clearBit(index);
   }
 
   inline void clearReadSet()
   {
-    read_set->reset();
+    read_set->clearAll();
   }
 
   inline bool isWriteSet(uint32_t index)
   {
-    return write_set->test(index);
+    return write_set->isBitSet(index);
   }
 
   inline void setWriteSet(uint32_t index)
   {
-    write_set->set(index);
+    write_set->setBit(index);
   }
 
   inline void setWriteSet()
   {
-    write_set->set();
+    write_set->setAll();
   }
 
   inline void clearWriteSet(uint32_t index)
   {
-    write_set->reset(index);
+    write_set->clearBit(index);
   }
 
   inline void clearWriteSet()
   {
-    write_set->reset();
+    write_set->clearAll();
   }
 
   /* Is table open or should be treated as such by name-locking? */
-  bool is_name_opened() const
+  inline bool is_name_opened()
   {
     return db_stat || open_placeholder;
   }
-
   /*
     Is this instance of the table should be reopen or represents a name-lock?
   */
-  bool needs_reopen_or_name_lock() const;
+  inline bool needs_reopen_or_name_lock()
+  { 
+    return s->version != refresh_version;
+  }
 
   /**
     clean/setup table fields and map.
@@ -540,20 +437,24 @@ public:
   {
     null_row= 1;
     status|= STATUS_NULL_ROW;
-    memset(null_flags, 255, getShare()->null_bytes);
+    memset(null_flags, 255, s->null_bytes);
   }
 
+  bool rename_temporary_table(const char *db, const char *table_name);
   void free_io_cache();
   void filesort_free_buffers(bool full= false);
   void intern_close_table();
 
-  void print_error(int error, myf errflag) const;
+  void print_error(int error, myf errflag)
+  {
+    s->storage_engine->print_error(error, errflag, *this);
+  }
 
   /**
     @return
     key if error because of duplicated keys
   */
-  uint32_t get_dup_key(int error) const
+  uint32_t get_dup_key(int error)
   {
     cursor->errkey  = (uint32_t) -1;
     if (error == HA_ERR_FOUND_DUPP_KEY || error == HA_ERR_FOREIGN_DUPLICATE_KEY ||
@@ -569,7 +470,27 @@ public:
   */
   bool operator<(const Table &right) const
   {
-    return getShare()->getCacheKey() < right.getShare()->getCacheKey();
+    int result= strcmp(this->getShare()->getSchemaName(), right.getShare()->getSchemaName());
+
+    if (result <  0)
+      return true;
+
+    if (result >  0)
+      return false;
+
+    result= strcmp(this->getShare()->getTableName(), right.getShare()->getTableName());
+
+    if (result <  0)
+      return true;
+
+    if (result >  0)
+      return false;
+
+    if (this->getShare()->getTableProto()->type()  < 
+        right.getShare()->getTableProto()->type())
+      return true;
+
+    return false;
   }
 
   static bool compare(const Table *a, const Table *b)
@@ -579,207 +500,65 @@ public:
 
   friend std::ostream& operator<<(std::ostream& output, const Table &table)
   {
-    if (table.getShare())
-    {
-      output << "Table:(";
-      output << table.getShare()->getSchemaName();
-      output << ", ";
-      output <<  table.getShare()->getTableName();
-      output << ", ";
-      output <<  table.getShare()->getTableTypeAsString();
-      output << ")";
-    }
-    else
-    {
-      output << "Table:(has no share)";
-    }
+    output << "Table:(";
+    output << table.getShare()->getSchemaName();
+    output << ", ";
+    output <<  table.getShare()->getTableName();
+    output << ", ";
+    output <<  table.getShare()->getTableTypeAsString();
+    output << ")";
 
     return output;  // for multiple << operators.
   }
 
-public:
-  virtual bool isPlaceHolder(void) const
-  {
-    return false;
-  }
 };
 
-/**
- * @class
- *  ForeignKeyInfo
- *
- * @brief
- *  This class defines the information for foreign keys.
- */
-class ForeignKeyInfo
+Table *create_virtual_tmp_table(Session *session, List<CreateField> &field_list);
+
+typedef struct st_foreign_key_info
 {
-public:
-    /**
-     * @brief
-     *  This is the constructor with all properties set.
-     *
-     * @param[in] in_foreign_id The id of the foreign key
-     * @param[in] in_referenced_db The referenced database name of the foreign key
-     * @param[in] in_referenced_table The referenced table name of the foreign key
-     * @param[in] in_update_method The update method of the foreign key.
-     * @param[in] in_delete_method The delete method of the foreign key.
-     * @param[in] in_referenced_key_name The name of referenced key
-     * @param[in] in_foreign_fields The foreign fields
-     * @param[in] in_referenced_fields The referenced fields
-     */
-    ForeignKeyInfo(lex_string_t *in_foreign_id,
-                   lex_string_t *in_referenced_db,
-                   lex_string_t *in_referenced_table,
-                   lex_string_t *in_update_method,
-                   lex_string_t *in_delete_method,
-                   lex_string_t *in_referenced_key_name,
-                   List<lex_string_t> in_foreign_fields,
-                   List<lex_string_t> in_referenced_fields)
-    :
-      foreign_id(in_foreign_id),
-      referenced_db(in_referenced_db),
-      referenced_table(in_referenced_table),
-      update_method(in_update_method),
-      delete_method(in_delete_method),
-      referenced_key_name(in_referenced_key_name),
-      foreign_fields(in_foreign_fields),
-      referenced_fields(in_referenced_fields)
-    {}
+  LEX_STRING *forein_id;
+  LEX_STRING *referenced_db;
+  LEX_STRING *referenced_table;
+  LEX_STRING *update_method;
+  LEX_STRING *delete_method;
+  LEX_STRING *referenced_key_name;
+  List<LEX_STRING> foreign_fields;
+  List<LEX_STRING> referenced_fields;
+} FOREIGN_KEY_INFO;
 
-    /**
-     * @brief
-     *  This is the default constructor. All properties are set to default values for their types.
-     */
-    ForeignKeyInfo()
-    : foreign_id(NULL), referenced_db(NULL), referenced_table(NULL),
-      update_method(NULL), delete_method(NULL), referenced_key_name(NULL)
-    {}
 
-    /**
-     * @brief
-     *  Gets the foreign id.
-     *
-     * @ retval  the foreign id
-     */
-    const lex_string_t *getForeignId() const
-    {
-        return foreign_id;
-    }
 
-    /**
-     * @brief
-     *  Gets the name of the referenced database.
-     *
-     * @ retval  the name of the referenced database
-     */
-    const lex_string_t *getReferencedDb() const
-    {
-        return referenced_db;
-    }
+class TableList;
 
-    /**
-     * @brief
-     *  Gets the name of the referenced table.
-     *
-     * @ retval  the name of the referenced table
-     */
-    const lex_string_t *getReferencedTable() const
-    {
-        return referenced_table;
-    }
+#define JOIN_TYPE_LEFT	1
+#define JOIN_TYPE_RIGHT	2
 
-    /**
-     * @brief
-     *  Gets the update method.
-     *
-     * @ retval  the update method
-     */
-    const lex_string_t *getUpdateMethod() const
-    {
-        return update_method;
-    }
+struct st_lex;
+class select_union;
+class Tmp_Table_Param;
 
-    /**
-     * @brief
-     *  Gets the delete method.
-     *
-     * @ retval  the delete method
-     */
-    const lex_string_t *getDeleteMethod() const
-    {
-        return delete_method;
-    }
+struct open_table_list_st
+{
+  std::string	db;
+  std::string	table;
+  uint32_t in_use;
+  uint32_t locked;
 
-    /**
-     * @brief
-     *  Gets the name of the referenced key.
-     *
-     * @ retval  the name of the referenced key
-     */
-    const lex_string_t *getReferencedKeyName() const
-    {
-        return referenced_key_name;
-    }
+  open_table_list_st() :
+    in_use(0),
+    locked(0)
+  { }
 
-    /**
-     * @brief
-     *  Gets the foreign fields.
-     *
-     * @ retval  the foreign fields
-     */
-    const List<lex_string_t> &getForeignFields() const
-    {
-        return foreign_fields;
-    }
-
-    /**
-     * @brief
-     *  Gets the referenced fields.
-     *
-     * @ retval  the referenced fields
-     */
-    const List<lex_string_t> &getReferencedFields() const
-    {
-        return referenced_fields;
-    }
-private:
-    /**
-     * The foreign id.
-     */
-    lex_string_t *foreign_id;
-    /**
-     * The name of the reference database.
-     */
-    lex_string_t *referenced_db;
-    /**
-     * The name of the reference table.
-     */
-    lex_string_t *referenced_table;
-    /**
-     * The update method.
-     */
-    lex_string_t *update_method;
-    /**
-     * The delete method.
-     */
-    lex_string_t *delete_method;
-    /**
-     * The name of the referenced key.
-     */
-    lex_string_t *referenced_key_name;
-    /**
-     * The foreign fields.
-     */
-    List<lex_string_t> foreign_fields;
-    /**
-     * The referenced fields.
-     */
-    List<lex_string_t> referenced_fields;
 };
 
-#define JOIN_TYPE_LEFT  1
-#define JOIN_TYPE_RIGHT 2
-
+TableShare *alloc_table_share(TableList *table_list, char *key,
+                               uint32_t key_length);
+int open_table_def(Session& session, TableShare *share);
+void open_table_error(TableShare *share, int error, int db_errno, int errarg);
+int open_table_from_share(Session *session, TableShare *share, const char *alias,
+                          uint32_t db_stat, uint32_t ha_open_flags,
+                          Table *outparam);
 void free_blobs(Table *table);
 int set_zone(int nr,int min_zone,int max_zone);
 uint32_t convert_period_to_month(uint32_t period);
@@ -787,20 +566,36 @@ uint32_t convert_month_to_period(uint32_t month);
 
 int test_if_number(char *str,int *res,bool allow_wildcards);
 void change_byte(unsigned char *,uint,char,char);
+
+namespace optimizer { class SqlSelect; }
+
+ha_rows filesort(Session *session,
+                 Table *form,
+                 st_sort_field *sortorder,
+                 uint32_t s_length,
+                 optimizer::SqlSelect *select,
+                 ha_rows max_rows,
+                 bool sort_positions,
+                 ha_rows *examined_rows);
+
+void filesort_free_buffers(Table *table, bool full);
 void change_double_for_sort(double nr,unsigned char *to);
+double my_double_round(double value, int64_t dec, bool dec_unsigned,
+                       bool truncate);
 int get_quick_record(optimizer::SqlSelect *select);
 
 void find_date(char *pos,uint32_t *vek,uint32_t flag);
-TYPELIB* convert_strings_to_array_type(char** typelibs, char** end);
-TYPELIB* typelib(memory::Root&, List<String>&);
+TYPELIB *convert_strings_to_array_type(char * *typelibs, char * *end);
+TYPELIB *typelib(memory::Root *mem_root, List<String> &strings);
 ulong get_form_pos(int file, unsigned char *head, TYPELIB *save_names);
+ulong next_io_size(ulong pos);
 void append_unescaped(String *res, const char *pos, uint32_t length);
 
-bool check_column_name(const char*);
-bool check_table_name(str_ref);
+int rename_file_ext(const char * from,const char * to,const char * ext);
+bool check_column_name(const char *name);
+bool check_db_name(LEX_STRING *org_name);
+bool check_table_name(const char *name, uint32_t length);
 
 } /* namespace drizzled */
 
-#include <drizzled/table/singular.h>
-#include <drizzled/table/concurrent.h>
-
+#endif /* DRIZZLED_TABLE_H */
