@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,9 +17,9 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
 
-#include <drizzled/sql_list.h>
+#ifndef DRIZZLED_TABLE_PROTO_H
+#define DRIZZLED_TABLE_PROTO_H
 
 namespace drizzled {
 /*
@@ -31,11 +31,21 @@ namespace drizzled {
 #define COLUMN_COMMENT_MAXLEN 1024
 #define INDEX_COMMENT_MAXLEN 1024
 
-bool rea_create_table(Session *session,
-                      const identifier::Table &identifier,
-                      message::Table &table_proto,
-                      HA_CREATE_INFO *create_info,
-                      List<CreateField> &create_field,
-                      uint32_t key_count,KeyInfo *key_info);
+int parse_table_proto(Session& session,
+                      message::Table &table,
+                      TableShare *share);
+
+int rename_table_proto_file(const char *from, const char* to);
+
+int delete_table_proto_file(const char *file_name);
+
+int rea_create_table(Session *session,
+                     TableIdentifier &identifier,
+                     message::Table &table_proto,
+                     HA_CREATE_INFO *create_info,
+                     List<CreateField> &create_field,
+                     uint32_t key_count,KEY *key_info);
 
 } /* namespace drizzled */
+
+#endif /* DRIZZLED_TABLE_PROTO_H */
