@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  Authors:
  *
@@ -34,45 +34,46 @@
  * their single parameter.
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <boost/foreach.hpp>
-#include <drizzled/charset.h>
-#include <drizzled/type/decimal.h>
-#include <drizzled/calendar.h>
-#include <drizzled/temporal.h>
-#include <drizzled/temporal_format.h>
-#include <drizzled/time_functions.h>
+#include "drizzled/charset_info.h"
+#include "drizzled/my_decimal.h"
+#include "drizzled/calendar.h"
+#include "drizzled/temporal.h"
+#ifdef NOTYETIMPLEMENTED
+#include "drizzled/temporal_interval.h"
+#endif
+#include "drizzled/temporal_format.h"
+#include "drizzled/time_functions.h"
 #include "time.h"
 
-#include <drizzled/util/gmtime.h>
-
 #include <time.h>
 
-#include <cstdio>
 #include <ostream>
 #include <iomanip>
 #include <vector>
 #include <string.h>
 
-namespace drizzled {
+namespace drizzled 
+{
 
 extern std::vector<TemporalFormat *> known_datetime_formats;
 extern std::vector<TemporalFormat *> known_date_formats;
 extern std::vector<TemporalFormat *> known_time_formats;
 
-Temporal::Temporal() :
-  _calendar(GREGORIAN),
-  _years(0),
-  _months(0),
-  _days(0),
-  _hours(0),
-  _minutes(0),
-  _seconds(0),
-  _epoch_seconds(0),
-  _useconds(0),
-  _nseconds(0),
-  _overflow(false)
+Temporal::Temporal()
+:
+  _calendar(GREGORIAN)
+, _years(0)
+, _months(0)
+, _days(0)
+, _hours(0)
+, _minutes(0)
+, _seconds(0)
+, _epoch_seconds(0)
+, _useconds(0)
+, _nseconds(0)
+, _overflow(false)
 {}
 
 uint64_t Temporal::_cumulative_seconds_in_time() const
@@ -82,44 +83,6 @@ uint64_t Temporal::_cumulative_seconds_in_time() const
       + _seconds);
 }
 
-#if defined(TARGET_OS_SOLARIS)
-/* @TODO: Replace this with Boost.DateTime */
-static time_t timegm(struct tm *my_time)
-{
-	time_t local_secs, gm_secs;
-	struct tm gm__rec, *gm_time;
-
-	// Interpret 't' as the local time and convert it to seconds since the Epoch
-	local_secs = mktime(my_time);
-	if (local_secs == -1)
-  {
-		my_time->tm_hour--;
-		local_secs = mktime (my_time);
-		if (local_secs == -1)
-			return -1; 
-		local_secs += 3600;
-	}
-	
-	// Get the gmtime based on the local seconds since the Epoch
-	gm_time = util::gmtime(local_secs, &gm__rec);
-	gm_time->tm_isdst = 0;
-	
-	// Interpret gmtime as the local time and convert it to seconds since the Epoch
-	gm_secs = mktime (gm_time);
-	if (gm_secs == -1)
-  {
-		gm_time->tm_hour--;
-		gm_secs = mktime (gm_time);
-		if (gm_secs == -1)
-			return -1; 
-		gm_secs += 3600;
-	}
-	
-	// Return the local time adjusted by the difference from GM time.
-	return (local_secs - (gm_secs - local_secs));
-}
-#endif
-
 void Temporal::set_epoch_seconds()
 {
   /* 
@@ -146,28 +109,58 @@ void Temporal::set_epoch_seconds()
 
 bool Date::from_string(const char *from, size_t from_len)
 {
-  _useconds= 0; // We may not match on it, so we need to make sure we zero it out.
-  BOOST_FOREACH(TemporalFormat* it, known_date_formats)
+  /* 
+   * Loop through the known date formats and see if 
+   * there is a match.
+   */
+  bool matched= false;
+  TemporalFormat *current_format;
+  std::vector<TemporalFormat *>::iterator current= known_date_formats.begin();
+
+  while (current != known_date_formats.end())
   {
-    if (not it->matches(from, from_len, this))
-      continue;
-    set_epoch_seconds();
-    return is_valid();
+    current_format= *current;
+    if (current_format->matches(from, from_len, this))
+    {
+      matched= true;
+      break;
+    }
+    current++;
   }
-  return false;
+
+  if (! matched)
+    return false;
+
+  set_epoch_seconds();
+  return is_valid();
 }
 
 bool DateTime::from_string(const char *from, size_t from_len)
 {
-  BOOST_FOREACH(TemporalFormat* it, known_datetime_formats)
-  {
-    if (not it->matches(from, from_len, this))
-      continue;
-    set_epoch_seconds();
-    return is_valid();
-  }
-  return false;
+  /* 
+   * Loop through the known datetime formats and see if 
+   * there is a match.
+   */
+  bool matched= false;
+  TemporalFormat *current_format;
+  std::vector<TemporalFormat *>::iterator current= known_datetime_formats.begin();
 
+  while (current != known_datetime_formats.end())
+  {
+    current_format= *current;
+    if (current_format->matches(from, from_len, this))
+    {
+      matched= true;
+      break;
+    }
+    current++;
+  }
+
+  if (! matched)
+    return false;
+
+  set_epoch_seconds();
+  return is_valid();
 }
 
 /*
@@ -802,7 +795,199 @@ Date& Date::operator+=(const DateTime &rhs)
 
   return *this;
 }
+#ifdef NOTYETIMPLEMENTED
+Date& Date::operator+=(const TemporalIntervalYear &rhs)
+{
+  /* Simple one...add the years and adjust for any leaps */
+  int64_t new_years= _years;
+  new_years+= rhs._years;
+  if (new_years > DRIZZLE_MAX_YEARS_SQL)
+  {
+    /* 
+     * Set everything to zero. We got an overflow.
+     * @TODO Exceptions would be great here...
+     */
+    _reset();
+    _overflow= true;
+    return *this;
+  }
+  _years= (uint32_t) new_years;
+  if (_months == 2 && _days == 29 && days_in_gregorian_year_month(_years, _months) != 366)
+    _days= 28;
+  return *this;
+} 
 
+Date& Date::operator-=(const TemporalIntervalYear &rhs)
+{
+  /* Simple one...subtract the years and adjust for any leaps */
+  int64_t new_years= _years;
+  new_years-= rhs._years;
+  if (new_years < 0)
+  {
+    /* 
+     * Set everything to zero. We got an overflow.
+     * @TODO Exceptions would be great here...
+     */
+    _reset();
+    _overflow= true;
+    return *this;
+  }
+  _years= (uint32_t) new_years;
+  if (_months == 2 && _days == 29 && days_in_gregorian_year_month(_years, _months) != 366)
+    _days= 28;
+  return *this;
+} 
+
+Date& Date::operator+=(const TemporalIntervalDayOrWeek &rhs)
+{
+  /* Simple one...add the days */
+  int64_t julian_day= julian_day_number_from_gregorian_date(_years, _months, _days) + rhs._days;
+  gregorian_date_from_julian_day_number(julian_day, &_years, &_months, &_days);
+  return *this;
+} 
+
+Date& Date::operator-=(const TemporalIntervalDayOrWeek &rhs)
+{
+  /* Simple one...subtract the days */
+  int64_t julian_day= julian_day_number_from_gregorian_date(_years, _months, _days) - rhs._days;
+  gregorian_date_from_julian_day_number(julian_day, &_years, &_months, &_days);
+  return *this;
+} 
+
+Date& Date::operator+=(const TemporalIntervalYearMonth &rhs)
+{
+  /* Simple one...add the months in the period adjust */
+  int64_t period= (_years * 12) + (rhs._years * 12) + (_months - 1) + rhs._months;
+  int64_t new_years= (period / 12);
+  if (new_years > DRIZZLE_MAX_YEARS_SQL)
+  {
+    /* 
+     * Set everything to zero. We got an overflow.
+     * @TODO Exceptions would be great here...
+     */
+    _reset();
+    _overflow= true;
+    return *this;
+  }
+  _years= (uint32_t) new_years;
+  _months= (uint32_t) (period % 12) + 1;
+  
+  /* Adjust day if the new month doesn't have enough days */
+  uint32_t days_in_new_month= days_in_gregorian_year_month(_years, _months);
+  if (_days > days_in_new_month)
+    _days= days_in_new_month;
+  return *this;
+} 
+
+Date& Date::operator-=(const TemporalIntervalYearMonth &rhs)
+{
+  /* Simple one...subtract the months in the period and adjust */
+  int64_t period= (_years * 12) - (rhs._years * 12) + (_months - 1) - rhs._months;
+  int64_t new_years= (period / 12);
+  if (new_years < 0)
+  {
+    /* 
+     * Set everything to zero. We got an overflow.
+     * @TODO Exceptions would be great here...
+     */
+    _reset();
+    _overflow= true;
+    return *this;
+  }
+  _years= (uint32_t) (period / 12);
+  _months= (uint32_t) (period % 12) + 1;
+  
+  /* Adjust day if the new month doesn't have enough days */
+  uint32_t days_in_new_month= days_in_gregorian_year_month(_years, _months);
+  if (_days > days_in_new_month)
+    _days= days_in_new_month;
+  return *this;
+} 
+
+Date& Date::operator+=(const TemporalIntervalDayOrLess &rhs)
+{
+  /* 
+   * Convert the temporal and the interval into a number of 
+   * microseconds, then add them together and convert the
+   * resulting microseconds back into a broken-down temporal
+   * component.
+   */
+  int64_t new_seconds;
+  int64_t new_microseconds;
+  int64_t extra_sec;
+  int64_t new_days;
+  new_microseconds= _useconds + rhs._useconds;
+  extra_sec= new_microseconds / INT64_C(1000000);
+  new_microseconds= new_microseconds % INT64_C(1000000);
+
+  new_seconds= ((_days - 1) * 3600 * 24) + (_hours * 3600) + (_minutes * 60) + _seconds;
+  new_seconds+= (rhs._days * 3600 * 24) + (rhs._hours * 3600) + (rhs._minutes * 60) + rhs._seconds;
+  new_seconds+= extra_sec;
+
+  if (new_microseconds < 0)
+  {
+    new_microseconds+= INT64_C(1000000);
+    new_seconds--;
+  }
+  
+  new_days= new_seconds / (3600 * 24L);
+  new_seconds-= new_days * 3600 * 24L;
+  if (new_seconds < 0)
+  {
+    new_days--;
+    new_seconds+= 3600 * 24L;
+  }
+  _useconds= (uint32_t) new_microseconds;
+  _seconds= (uint32_t) (new_seconds % 60);
+  _minutes= (uint32_t) ((new_seconds / 60) % 60);
+  _hours= (uint32_t) (new_seconds / 3600);
+  int64_t julian_day= julian_day_number_from_gregorian_date(_years, _months, 1) + new_days;
+  gregorian_date_from_julian_day_number(julian_day, &_years, &_months, &_days);
+  return *this;
+}
+
+Date& Date::operator-=(const TemporalIntervalDayOrLess &rhs)
+{
+  /* 
+   * Convert the temporal and the interval into a number of 
+   * microseconds, then subtract them from each other and convert 
+   * the resulting microseconds back into a broken-down temporal
+   * component.
+   */
+  int64_t new_seconds;
+  int64_t new_microseconds;
+  int64_t extra_sec;
+  int64_t new_days;
+  new_microseconds= _useconds - rhs._useconds;
+  extra_sec= new_microseconds / INT64_C(1000000);
+  new_microseconds= new_microseconds % INT64_C(1000000);
+
+  new_seconds= ((_days - 1) * 3600 * 24) + (_hours * 3600) + (_minutes * 60) + _seconds;
+  new_seconds-= (rhs._days * 3600 * 24) + (rhs._hours * 3600) + (rhs._minutes * 60) + rhs._seconds;
+  new_seconds+= extra_sec;
+
+  if (new_microseconds < 0)
+  {
+    new_microseconds+= INT64_C(1000000);
+    new_seconds--;
+  }
+  
+  new_days= new_seconds / (3600 * 24L);
+  new_seconds-= new_days * 3600 * 24L;
+  if (new_seconds < 0)
+  {
+    new_days--;
+    new_seconds+= 3600 * 24L;
+  }
+  _useconds= (uint32_t) new_microseconds;
+  _seconds= (uint32_t) (new_seconds % 60);
+  _minutes= (uint32_t) ((new_seconds / 60) % 60);
+  _hours= (uint32_t) (new_seconds / 3600);
+  int64_t julian_day= julian_day_number_from_gregorian_date(_years, _months, 1) + new_days;
+  gregorian_date_from_julian_day_number(julian_day, &_years, &_months, &_days);
+  return *this;
+}
+#endif /* NOTYETIMPLEMENTED */
 /*
  * Comparison operators between a Date and a Timestamp
  */
@@ -834,11 +1019,11 @@ bool Date::operator<=(const Timestamp& rhs)
 }
 bool Date::operator>(const Timestamp& rhs)
 {
-  return ! (*this <= rhs);
+  return ! (*this < rhs);
 }
 bool Date::operator>=(const Timestamp& rhs)
 {
-  return ! (*this < rhs);
+  return ! (*this <= rhs);
 }
 /*
  * Comparison operators between a Timestamp and a Date
@@ -871,11 +1056,11 @@ bool Timestamp::operator<=(const Date& rhs)
 }
 bool Timestamp::operator>(const Date& rhs)
 {
-  return ! (*this <= rhs);
+  return ! (*this < rhs);
 }
 bool Timestamp::operator>=(const Date& rhs)
 {
-  return ! (*this < rhs);
+  return ! (*this <= rhs);
 }
 /*
  * Comparison operators between a Timestamp and a DateTime
@@ -924,11 +1109,11 @@ bool Timestamp::operator<=(const DateTime& rhs)
 }
 bool Timestamp::operator>(const DateTime& rhs)
 {
-  return ! (*this <= rhs);
+  return ! (*this < rhs);
 }
 bool Timestamp::operator>=(const DateTime& rhs)
 {
-  return ! (*this < rhs);
+  return ! (*this <= rhs);
 }
 /*
  * Comparison operators between two Timestamps
@@ -951,11 +1136,11 @@ bool Timestamp::operator<=(const Timestamp& rhs)
 }
 bool Timestamp::operator>(const Timestamp& rhs)
 {
-  return ! (*this <= rhs);
+  return ! (*this < rhs);
 }
 bool Timestamp::operator>=(const Timestamp& rhs)
 {
-  return ! (*this < rhs);
+  return ! (*this <= rhs);
 }
 
 /**
@@ -977,23 +1162,43 @@ std::ostream& operator<<(std::ostream& os, const Timestamp& subject)
 
 bool Time::from_string(const char *from, size_t from_len)
 {
-  BOOST_FOREACH(TemporalFormat* it, known_time_formats)
+  /*
+   * Loop through the known time formats and see if
+   * there is a match.
+   */
+  bool matched= false;
+  TemporalFormat *current_format;
+  std::vector<TemporalFormat *>::iterator current= known_time_formats.begin();
+
+  while (current != known_time_formats.end())
   {
-    if (not it->matches(from, from_len, this))
-      continue;
-    return is_fuzzy_valid();
+    current_format= *current;
+    if (current_format->matches(from, from_len, this))
+    {
+      matched= true;
+      break;
+    }
+    current++;
   }
-  return false;
+
+  if (! matched)
+    return false;
+  else
+    return is_valid();
 }
 
 int Time::to_string(char *to, size_t to_len) const
 {
-  return snprintf(to, to_len, "%02" PRIu32 ":%02" PRIu32 ":%02" PRIu32, _hours, _minutes, _seconds);
+  return snprintf(to, to_len,
+		  "%02" PRIu32 ":%02" PRIu32 ":%02" PRIu32,
+		  _hours, _minutes, _seconds);
 }
 
 int Date::to_string(char *to, size_t to_len) const
 {
-  return snprintf(to, to_len, "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32, _years, _months, _days);
+  return snprintf(to, to_len,
+		  "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32,
+		  _years, _months, _days);
 }
 
 int DateTime::to_string(char *to, size_t to_len) const
@@ -1002,28 +1207,34 @@ int DateTime::to_string(char *to, size_t to_len) const
   if (_useconds == 0)
   {
     return snprintf(to, to_len,
-		    "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32 " %02" PRIu32 ":%02" PRIu32 ":%02" PRIu32,
-		    _years, _months, _days, _hours, _minutes, _seconds);
+		    "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32
+		          " %02" PRIu32 ":%02" PRIu32 ":%02" PRIu32,
+		    _years, _months, _days,
+		    _hours, _minutes, _seconds);
   }
   else
   {
     return snprintf(to, to_len,
-		    "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32 " %02" PRIu32 ":%02" PRIu32 ":%02" PRIu32 ".%06" PRIu32,
-		    _years, _months, _days, _hours, _minutes, _seconds, _useconds);
+		    "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32
+		       " %02" PRIu32 ":%02" PRIu32 ":%02" PRIu32 ".%06" PRIu32,
+		    _years, _months, _days,
+		    _hours, _minutes, _seconds, _useconds);
   }
 }
 
 int MicroTimestamp::to_string(char *to, size_t to_len) const
 {
   return snprintf(to, to_len,
-                  "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32 " %02" PRIu32 ":%02" PRIu32 ":%02" PRIu32 ".%06" PRIu32,
-                  _years, _months, _days, _hours, _minutes, _seconds, _useconds);
+		  "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32
+		      " %02" PRIu32 ":%02" PRIu32 ":%02" PRIu32 ".%06" PRIu32,
+		  _years, _months, _days,
+		  _hours, _minutes, _seconds, _useconds);
 }
 
-void Time::to_decimal(type::Decimal *to) const
+void Time::to_decimal(my_decimal *to) const
 {
   int64_t time_portion= (((_hours * 100L) + _minutes) * 100L) + _seconds;
-  (void) int2_class_decimal(E_DEC_FATAL_ERROR, time_portion, false, to);
+  (void) int2my_decimal(E_DEC_FATAL_ERROR, time_portion, false, to);
   if (_useconds > 0)
   {
     to->buf[(to->intg-1) / 9 + 1]= _useconds * 1000;
@@ -1031,17 +1242,17 @@ void Time::to_decimal(type::Decimal *to) const
   }
 }
 
-void Date::to_decimal(type::Decimal *to) const
+void Date::to_decimal(my_decimal *to) const
 {
   int64_t date_portion= (((_years * 100L) + _months) * 100L) + _days;
-  (void) int2_class_decimal(E_DEC_FATAL_ERROR, date_portion, false, to);
+  (void) int2my_decimal(E_DEC_FATAL_ERROR, date_portion, false, to);
 }
 
-void DateTime::to_decimal(type::Decimal *to) const
+void DateTime::to_decimal(my_decimal *to) const
 {
   int64_t date_portion= (((_years * 100L) + _months) * 100L) + _days;
   int64_t time_portion= (((((date_portion * 100L) + _hours) * 100L) + _minutes) * 100L) + _seconds;
-  (void) int2_class_decimal(E_DEC_FATAL_ERROR, time_portion, false, to);
+  (void) int2my_decimal(E_DEC_FATAL_ERROR, time_portion, false, to);
   if (_useconds > 0)
   {
     to->buf[(to->intg-1) / 9 + 1]= _useconds * 1000;
@@ -1051,23 +1262,23 @@ void DateTime::to_decimal(type::Decimal *to) const
 
 void Date::to_int64_t(int64_t *to) const
 {
-  *to= (_years * INT32_C(10000)) + (_months * INT32_C(100)) + _days;
+  *to= (_years * INT32_C(10000)) 
+     + (_months * INT32_C(100)) 
+     + _days;
 }
 
 void Date::to_int32_t(int32_t *to) const
 {
-  *to= (_years * INT32_C(10000)) + (_months * INT32_C(100)) + _days;
+  *to= (_years * INT32_C(10000)) 
+     + (_months * INT32_C(100)) 
+     + _days;
 }
 
 void Time::to_int32_t(int32_t *to) const
 {
-  *to= (_hours * INT32_C(10000)) + (_minutes * INT32_C(100)) + _seconds;
-}
-
-// We fill the structure based on just int
-void Time::to_uint64_t(uint64_t &to) const
-{
-  to= (_hours * 60 * 60) + (_minutes * 60) + _seconds;
+  *to= (_hours * INT32_C(10000)) 
+     + (_minutes * INT32_C(100)) 
+     + _seconds;
 }
 
 void DateTime::to_int64_t(int64_t *to) const
@@ -1130,9 +1341,9 @@ bool Date::from_int32_t(const int32_t from)
 bool Time::from_int32_t(const int32_t from)
 {
   uint32_t copy_from= (uint32_t) from;
-  _hours= copy_from / INT32_C(10000);
-  _minutes= (copy_from % INT32_C(10000)) / INT32_C(100);
-  _seconds= copy_from % INT32_C(100); /* Masks off all but last 2 digits */
+  _hours= copy_from % INT32_C(10000);
+  _minutes= copy_from % INT32_C(100);
+  _seconds= copy_from & 3; /* Masks off all but last 2 digits */
   return is_valid();
 }
 
@@ -1171,7 +1382,7 @@ bool DateTime::from_int64_t(const int64_t from, bool convert)
     else if (copy_from <  DRIZZLE_YY_PART_YEAR * 10000000000LL + 101000000LL)
       return false;
     else if (copy_from <= 991231235959LL)
-      copy_from= copy_from + 19000000000000LL;    /* YYMMDDHHMMSS, 1970-1999 */
+      copy_from= copy_from + 19000000000000LL;		/* YYMMDDHHMMSS, 1970-1999 */
   }
 
   part1= (int64_t) (copy_from / 1000000LL);
@@ -1226,7 +1437,7 @@ bool Time::from_time_t(const time_t from)
   struct tm broken_time;
   struct tm *result;
 
-  result= util::gmtime(from, &broken_time);
+  result= gmtime_r(&from, &broken_time);
   if (result != NULL)
   {
     _years= 0;
@@ -1250,7 +1461,7 @@ bool Date::from_time_t(const time_t from)
   struct tm broken_time;
   struct tm *result;
 
-  result= util::gmtime(from, &broken_time);
+  result= gmtime_r(&from, &broken_time);
   if (result != NULL)
   {
     _years= 1900 + broken_time.tm_year;
@@ -1269,38 +1480,12 @@ bool Date::from_time_t(const time_t from)
     return false;
 }
 
-bool DateTime::from_timeval(struct timeval &timeval_arg)
-{
-  struct tm broken_time;
-  struct tm *result;
-
-  result= util::gmtime(timeval_arg.tv_sec, &broken_time);
-  if (result != NULL)
-  {
-    _years= 1900 + broken_time.tm_year;
-    _months= 1 + broken_time.tm_mon; /* Month is NOT ordinal for struct tm! */
-    _days= broken_time.tm_mday; /* Day IS ordinal for struct tm */
-    _hours= broken_time.tm_hour;
-    _minutes= broken_time.tm_min;
-    _seconds= broken_time.tm_sec;
-    _epoch_seconds= timeval_arg.tv_sec;
-    /* Set hires precision to zero */
-    _useconds= timeval_arg.tv_usec;
-    _nseconds= 0;
-    return is_valid();
-  }
-  else 
-  {
-    return false;
-  }
-}
-
 bool DateTime::from_time_t(const time_t from)
 {
   struct tm broken_time;
   struct tm *result;
 
-  result= util::gmtime(from, &broken_time);
+  result= gmtime_r(&from, &broken_time);
   if (result != NULL)
   {
     _years= 1900 + broken_time.tm_year;
@@ -1316,32 +1501,28 @@ bool DateTime::from_time_t(const time_t from)
     return is_valid();
   }
   else 
-  {
     return false;
-  }
 }
 
-void Date::to_time_t(time_t &to) const
+void Date::to_time_t(time_t *to) const
 {
   if (in_unix_epoch())
   {
-    to= _epoch_seconds;
+    *to= _epoch_seconds;
   }
   else
-  {
-    to= 0;
-  }
+    *to= 0;
 }
 
-void Timestamp::to_time_t(time_t &to) const
+void Timestamp::to_time_t(time_t *to) const
 {
-  to= _epoch_seconds;
+  *to= _epoch_seconds;
 }
 
-void MicroTimestamp::to_timeval(struct timeval &to) const
+void MicroTimestamp::to_timeval(struct timeval *to) const
 {
-  to.tv_sec= _epoch_seconds;
-  to.tv_usec= _useconds;
+  to->tv_sec= _epoch_seconds;
+  to->tv_usec= _useconds;
 }
 
 void NanoTimestamp::to_timespec(struct timespec *to) const
@@ -1353,7 +1534,7 @@ void NanoTimestamp::to_timespec(struct timespec *to) const
 bool Date::is_valid() const
 {
   return (_years >= DRIZZLE_MIN_YEARS_SQL && _years <= DRIZZLE_MAX_YEARS_SQL)
-      && (_months >= 1 && _months <= DRIZZLE_MAX_MONTHS)
+      && (_months >= 1 && _months <= 12)
       && (_days >= 1 && _days <= days_in_gregorian_year_month(_years, _months));
 }
 
@@ -1362,39 +1543,25 @@ bool Time::is_valid() const
   return (_years == 0)
       && (_months == 0)
       && (_days == 0)
-      && (_hours <= DRIZZLE_MAX_HOURS)
-      && (_minutes <= DRIZZLE_MAX_MINUTES)
-      && (_seconds <= DRIZZLE_MAX_SECONDS); /* No Leap second... TIME is for elapsed time... */
-}
-
-bool Time::is_fuzzy_valid() const
-{
-  if (is_valid())
-    return true;
-
-  return (_years >= DRIZZLE_MIN_YEARS_SQL && _years <= DRIZZLE_MAX_YEARS_SQL)
-      && (_months >= 1 && _months <= DRIZZLE_MAX_MONTHS)
-      && (_days >= 1 && _days <= days_in_gregorian_year_month(_years, _months))
-      && (_hours <= DRIZZLE_MAX_HOURS)
-      && (_minutes <= DRIZZLE_MAX_MINUTES)
-      && (_seconds <= DRIZZLE_MAX_SECONDS); /* No Leap second... TIME is for elapsed time... */
+      && (_hours <= 23)
+      && (_minutes <= 59)
+      && (_seconds <= 59); /* No Leap second... TIME is for elapsed time... */
 }
 
 bool DateTime::is_valid() const
 {
   return (_years >= DRIZZLE_MIN_YEARS_SQL && _years <= DRIZZLE_MAX_YEARS_SQL)
-      && (_months >= 1 && _months <= DRIZZLE_MAX_MONTHS)
+      && (_months >= 1 && _months <= 12)
       && (_days >= 1 && _days <= days_in_gregorian_year_month(_years, _months))
-      && (_hours <= DRIZZLE_MAX_HOURS)
-      && (_minutes <= DRIZZLE_MAX_MINUTES)
-      && (_seconds <= DRIZZLE_MAX_SECONDS_WITH_LEAP); /* Leap second... */
+      && (_hours <= 23)
+      && (_minutes <= 59)
+      && (_seconds <= 61); /* Leap second... */
 }
 
 bool Timestamp::is_valid() const
 {
-  return DateTime::is_valid()
-      && in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds)
-      && (_seconds <= DRIZZLE_MAX_SECONDS);
+  return DateTime::is_valid() 
+      && in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds);
 }
 
 bool MicroTimestamp::is_valid() const
