@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008-2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2008-2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,10 +39,10 @@
  * @see drizzled/calendar.cc
  */
 
-#pragma once
+#ifndef DRIZZLED_TEMPORAL_H
+#define DRIZZLED_TEMPORAL_H
 
 #define DRIZZLE_MAX_SECONDS 59
-#define DRIZZLE_MAX_SECONDS_WITH_LEAP 61
 #define DRIZZLE_MAX_MINUTES 59
 #define DRIZZLE_MAX_HOURS 23
 #define DRIZZLE_MAX_DAYS 31
@@ -66,17 +66,28 @@
 
 #define DRIZZLE_YY_PART_YEAR  70
 
-#include <drizzled/calendar.h>
-#include <drizzled/common_fwd.h>
+#include "drizzled/calendar.h"
+
 #include <cassert>
 #include <ostream>
 
-namespace drizzled {
+/* Outside forward declarations */
+class my_decimal;
+
+namespace drizzled
+{
+
+/* Forward declaration needed */
+class TemporalInterval;
+class TemporalIntervalYear;
+class TemporalIntervalDayOrLess;
+class TemporalIntervalDayOrWeek;
+class TemporalIntervalYearMonth;
 
 /**
  * Base class for all temporal data classes.
  */
-class DRIZZLED_API Temporal
+class Temporal
 {
 protected:
   enum calendar _calendar;
@@ -184,11 +195,16 @@ public:
   friend class TemporalFormat;
 };
 
+/* Forward declaration needed */
+class DateTime;
+class Timestamp;
+class Time;
+
 /**
  * Class representing temporal components in a valid
  * SQL date range, with no time component
  */
-class DRIZZLED_API Date: public Temporal
+class Date: public Temporal
 {
 public:
   Date() :Temporal() {}
@@ -268,6 +284,22 @@ public:
   Date& operator-=(const DateTime &rhs);
 
 
+  /**
+   * Operator overload for adding/subtracting a TemporalInterval
+   * instance to this temporal.
+   *
+   * @param TemporalInterval instance to add/subtract to/from
+   */
+  Date& operator+=(const TemporalIntervalYear &rhs);
+  Date& operator+=(const TemporalIntervalDayOrLess &rhs);
+  Date& operator+=(const TemporalIntervalDayOrWeek &rhs);
+  Date& operator+=(const TemporalIntervalYearMonth &rhs);
+  Date& operator-=(const TemporalIntervalYear &rhs);
+  Date& operator-=(const TemporalIntervalDayOrLess &rhs);
+  Date& operator-=(const TemporalIntervalDayOrWeek &rhs);
+  Date& operator-=(const TemporalIntervalYearMonth &rhs);
+
+
   /**
    * Operator overload for when a DateTime instance is
    * assigned to a Date.  We do a copy of the DateTime's
@@ -297,7 +329,7 @@ public:
    *
    * @param C-String to fill.
    * @param Length of to C-String
-   * @returns length of string written (including trailing '\0').
+   * @returns length of string written (not including trailing '\0').
    *          If output was truncated, returns length that would have
    *          been outputted.
    */
@@ -317,7 +349,7 @@ public:
    * successful.
    *
    * @param String to convert from
-   * @param Length of supplied string (not including trailing '\0').
+   * @param Length of supplied string
    */
   virtual bool from_string(const char *from, size_t from_len);
 
@@ -403,7 +435,7 @@ public:
    *
    * @param Pointer to a time_t to convert to
    */
-  virtual void to_time_t(time_t &to) const;
+  virtual void to_time_t(time_t *to) const;
 
   /**
    * Attempts to populate the Date instance based
@@ -417,12 +449,12 @@ public:
   virtual bool from_time_t(const time_t from);
 
   /**
-   * Fills a supplied type::Decimal with a representation of
+   * Fills a supplied my_decimal with a representation of
    * the Date value.
    *
-   * @param Pointer to the type::Decimal to fill
+   * @param Pointer to the my_decimal to fill
    */
-  virtual void to_decimal(type::Decimal *to) const;
+  virtual void to_decimal(my_decimal *to) const;
 
   friend class TemporalInterval;
   friend class Timestamp;
@@ -435,7 +467,7 @@ class DateTime;
  * Class representing temporal components having only
  * a time component, with no date structure
  */
-class DRIZZLED_API Time: public Temporal
+class Time: public Temporal
 {
 public:
   Time() :Temporal() {}
@@ -469,10 +501,8 @@ public:
   bool is_valid_datetime() const {return false;}
   bool is_valid_time() const {return is_valid();}
   bool is_valid_timestamp() const {return false;}
-
   /** Returns whether the temporal value is valid date. */
   bool is_valid() const;
-  bool is_fuzzy_valid() const;
 
   /**
    * Fills a supplied char string with a
@@ -515,15 +545,6 @@ public:
    */
   void to_int32_t(int32_t *to) const;
 
-  /**
-   * Fills a supplied 8-byte integer pointer with an
-   * integer representation of the Time
-   * value. It is assume seconds past unix epoch
-   *
-   * @param Integer to fill.
-   */
-  void to_uint64_t(uint64_t &to) const;
-
   /**
    * Attempts to populate the Time instance based
    * on the contents of a supplied 4-byte integer.
@@ -552,12 +573,12 @@ public:
   bool from_time_t(const time_t from);
 
   /**
-   * Fills a supplied type::Decimal with a representation of
+   * Fills a supplied my_decimal with a representation of
    * the Time value.
    *
-   * @param Pointer to the type::Decimal to fill
+   * @param Pointer to the my_decimal to fill
    */
-  void to_decimal(type::Decimal *to) const;
+  void to_decimal(my_decimal *to) const;
 
   friend class Date;
   friend class DateTime;
@@ -567,7 +588,7 @@ public:
  * Class representing temporal components in a valid
  * SQL datetime range, including a time component
  */
-class DRIZZLED_API DateTime: public Date
+class DateTime: public Date
 {
 public:
   DateTime() :Date() {}
@@ -638,7 +659,6 @@ public:
    * @param time_t to convert from
    */
   bool from_time_t(const time_t from);
-  bool from_timeval(struct timeval &_timeval);
 
   /**
    * Attempts to populate the DateTime instance based
@@ -667,12 +687,12 @@ public:
   void to_tm(struct tm *to) const;
 
   /**
-   * Fills a supplied type::Decimal with a representation of
+   * Fills a supplied my_decimal with a representation of
    * the DateTime value.
    *
-   * @param Pointer to the type::Decimal to fill
+   * @param Pointer to the my_decimal to fill
    */
-  void to_decimal(type::Decimal *to) const;
+  void to_decimal(my_decimal *to) const;
 
   friend class Timestamp;
 };
@@ -680,7 +700,7 @@ public:
 /**
  * Class representing temporal components in the UNIX epoch
  */
-class DRIZZLED_API Timestamp: public DateTime
+class Timestamp: public DateTime
 {
 public:
   Timestamp() :DateTime() {}
@@ -734,7 +754,7 @@ public:
    *
    * @param Pointer to a time_t to convert to
    */
-  void to_time_t(time_t &to) const;
+  void to_time_t(time_t *to) const;
 };
 
 /**
@@ -746,7 +766,7 @@ std::ostream& operator<<(std::ostream& os, const Timestamp& subject);
  * Class representing temporal components in the UNIX epoch
  * with an additional microsecond component.
  */
-class DRIZZLED_API MicroTimestamp: public Timestamp
+class MicroTimestamp: public Timestamp
 {
 public:
   MicroTimestamp() :Timestamp() {}
@@ -782,14 +802,14 @@ public:
    *
    * @param timeval to fill.
    */
-  void to_timeval(struct timeval &to) const;
+  void to_timeval(struct timeval *to) const;
 };
 
 /**
  * Class representing temporal components in the UNIX epoch
  * with an additional nanosecond component.
  */
-class DRIZZLED_API NanoTimestamp: public Timestamp
+class NanoTimestamp: public Timestamp
 {
 public:
   NanoTimestamp() :Timestamp() {}
@@ -811,3 +831,4 @@ public:
 
 } /* end namespace drizzled */
 
+#endif /* DRIZZLED_TEMPORAL_H */
