@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
 Read and write locks for Posix threads. All tread must acquire
@@ -46,22 +46,30 @@ To allow concurrent read/writes locks, with 'WRITE_CONCURRENT_INSERT' one
 should put a pointer to the following functions in the lock structure:
 (If the pointer is zero (default), the function is not called)
 
+check_status:
+	 Before giving a lock of type TL_WRITE_CONCURRENT_INSERT,
+         we check if this function exists and returns 0.
+	 If not, then the lock is upgraded to TL_WRITE_LOCK
+	 In MyISAM this is a simple check if the insert can be done
+	 at the end of the datafile.
+update_status:
+	Before a write lock is released, this function is called.
+	In MyISAM this functions updates the count and length of the datafile
+get_status:
+	When one gets a lock this functions is called.
+	In MyISAM this stores the number of rows and size of the datafile
+	for concurrent reads.
 
 The lock algorithm allows one to have one TL_WRITE_ALLOW_READ,
 TL_WRITE_CONCURRENT_INSERT lock at the same time as multiple read locks.
 
 */
 
-#include <config.h>
-#include <drizzled/internal/my_sys.h>
-#include <drizzled/internal/thread_var.h>
-#include <drizzled/statistics_variables.h>
-#include <drizzled/pthread_globals.h>
-
-#include <drizzled/session.h>
+#include "config.h"
+#include "drizzled/internal/my_sys.h"
 
 #include "thr_lock.h"
-#include <drizzled/internal/m_string.h>
+#include "drizzled/internal/m_string.h"
 #include <errno.h>
 #include <list>
 
@@ -78,22 +86,37 @@ TL_WRITE_CONCURRENT_INSERT lock at the same time as multiple read locks.
 
 #include <drizzled/util/test.h>
 
-#include <boost/interprocess/sync/lock_options.hpp>
-
 using namespace std;
 
-namespace drizzled {
+namespace drizzled
+{
 
+bool thr_lock_inited= false;
+uint32_t locks_immediate = 0L, locks_waited = 0L;
 uint64_t table_lock_wait_timeout;
 static enum thr_lock_type thr_upgraded_concurrent_insert_lock = TL_WRITE;
 
 
-uint64_t max_write_lock_count= UINT64_MAX;
+static list<THR_LOCK *> thr_lock_thread_list;          /* List of threads in use */
+
+uint64_t max_write_lock_count= ~(uint64_t) 0L;
+
+static inline pthread_cond_t *get_cond(void)
+{
+  return &my_thread_var->suspend;
+}
 
 /*
 ** For the future (now the thread specific cond is alloced by my_pthread.c)
 */
 
+bool init_thr_lock()
+{
+  thr_lock_inited= true;
+
+  return false;
+}
+
 static inline bool
 thr_lock_owner_equal(THR_LOCK_OWNER *rhs, THR_LOCK_OWNER *lhs)
 {
@@ -105,28 +128,45 @@ thr_lock_owner_equal(THR_LOCK_OWNER *rhs, THR_LOCK_OWNER *lhs)
 
 void thr_lock_init(THR_LOCK *lock)
 {
+  memset(lock, 0, sizeof(*lock));
+  pthread_mutex_init(&lock->mutex,MY_MUTEX_INIT_FAST);
   lock->read.last= &lock->read.data;
   lock->read_wait.last= &lock->read_wait.data;
   lock->write_wait.last= &lock->write_wait.data;
   lock->write.last= &lock->write.data;
+
+  pthread_mutex_lock(&internal::THR_LOCK_lock);		/* Add to locks in use */
+  thr_lock_thread_list.push_front(lock);
+  pthread_mutex_unlock(&internal::THR_LOCK_lock);
 }
 
 
-void THR_LOCK_INFO::init()
+void thr_lock_delete(THR_LOCK *lock)
 {
-  thread_id= internal::my_thread_var2()->id;
-  n_cursors= 0;
+  pthread_mutex_destroy(&lock->mutex);
+  pthread_mutex_lock(&internal::THR_LOCK_lock);
+  thr_lock_thread_list.remove(lock);
+  pthread_mutex_unlock(&internal::THR_LOCK_lock);
+}
+
+
+void thr_lock_info_init(THR_LOCK_INFO *info)
+{
+  internal::st_my_thread_var *tmp= my_thread_var;
+  info->thread= tmp->pthread_self;
+  info->thread_id= tmp->id;
+  info->n_cursors= 0;
 }
 
 	/* Initialize a lock instance */
 
-void THR_LOCK_DATA::init(THR_LOCK *lock_arg, void *param_arg)
+void thr_lock_data_init(THR_LOCK *lock,THR_LOCK_DATA *data, void *param)
 {
-  lock= lock_arg;
-  type= TL_UNLOCK;
-  owner= NULL;                               /* no owner yet */
-  status_param= param_arg;
-  cond= NULL;
+  data->lock= lock;
+  data->type= TL_UNLOCK;
+  data->owner= NULL;                               /* no owner yet */
+  data->status_param= param;
+  data->cond= NULL;
 }
 
 
@@ -144,48 +184,41 @@ have_old_read_lock(THR_LOCK_DATA *data, THR_LOCK_OWNER *owner)
 static void wake_up_waiters(THR_LOCK *lock);
 
 
-static enum enum_thr_lock_result wait_for_lock(Session &session, struct st_lock_list *wait, THR_LOCK_DATA *data)
+static enum enum_thr_lock_result
+wait_for_lock(struct st_lock_list *wait, THR_LOCK_DATA *data,
+              bool in_wait_list)
 {
-  boost::condition_variable_any *cond= &session.getThreadVar()->suspend;
+  internal::st_my_thread_var *thread_var= my_thread_var;
+  pthread_cond_t *cond= &thread_var->suspend;
+  struct timespec wait_timeout;
   enum enum_thr_lock_result result= THR_LOCK_ABORTED;
   bool can_deadlock= test(data->owner->info->n_cursors);
 
+  if (!in_wait_list)
   {
     (*wait->last)=data;				/* Wait for lock */
     data->prev= wait->last;
     wait->last= &data->next;
   }
 
-  current_global_counters.locks_waited++;
+  statistic_increment(locks_waited, &internal::THR_LOCK_lock);
 
   /* Set up control struct to allow others to abort locks */
-  session.getThreadVar()->current_mutex= data->lock->native_handle();
-  session.getThreadVar()->current_cond=  &session.getThreadVar()->suspend;
-  data->cond= &session.getThreadVar()->suspend;;
+  thread_var->current_mutex= &data->lock->mutex;
+  thread_var->current_cond=  cond;
+  data->cond= cond;
 
-  while (not session.getThreadVar()->abort)
+  if (can_deadlock)
+    set_timespec(wait_timeout, table_lock_wait_timeout);
+  while (!thread_var->abort || in_wait_list)
   {
-    boost::mutex::scoped_lock scoped(*data->lock->native_handle(), boost::adopt_lock_t());
-
-    if (can_deadlock)
-    {
-      boost::xtime xt; 
-      xtime_get(&xt, boost::TIME_UTC); 
-      xt.sec += table_lock_wait_timeout; 
-      if (not cond->timed_wait(scoped, xt))
-      {
-        result= THR_LOCK_WAIT_TIMEOUT;
-        scoped.release();
-        break;
-      }
-    }
-    else
-    {
-      cond->wait(scoped);
-    }
+    int rc= (can_deadlock ?
+             pthread_cond_timedwait(cond, &data->lock->mutex,
+                                    &wait_timeout) :
+             pthread_cond_wait(cond, &data->lock->mutex));
     /*
       We must break the wait if one of the following occurs:
-      - the connection has been aborted (!session.getThreadVar()->abort), but
+      - the connection has been aborted (!thread_var->abort), but
         this is not a delayed insert thread (in_wait_list). For a delayed
         insert thread the proper action at shutdown is, apparently, to
         acquire the lock and complete the insert.
@@ -196,25 +229,24 @@ static enum enum_thr_lock_result wait_for_lock(Session &session, struct st_lock_
       Order of checks below is important to not report about timeout
       if the predicate is true.
     */
-    if (data->cond == NULL)
+    if (data->cond == 0)
     {
-      scoped.release();
       break;
     }
-    scoped.release();
+    if (rc == ETIMEDOUT || rc == ETIME)
+    {
+      result= THR_LOCK_WAIT_TIMEOUT;
+      break;
+    }
   }
   if (data->cond || data->type == TL_UNLOCK)
   {
     if (data->cond)                             /* aborted or timed out */
     {
       if (((*data->prev)=data->next))		/* remove from wait-list */
-      {
 	data->next->prev= data->prev;
-      }
       else
-      {
 	wait->last=data->prev;
-      }
       data->type= TL_UNLOCK;                    /* No lock */
       wake_up_waiters(data->lock);
     }
@@ -222,21 +254,25 @@ static enum enum_thr_lock_result wait_for_lock(Session &session, struct st_lock_
   else
   {
     result= THR_LOCK_SUCCESS;
+    if (data->lock->get_status)
+      (*data->lock->get_status)(data->status_param, 0);
   }
-  data->lock->unlock();
+  pthread_mutex_unlock(&data->lock->mutex);
 
   /* The following must be done after unlock of lock->mutex */
-  boost::mutex::scoped_lock scopedLock(session.getThreadVar()->mutex);
-  session.getThreadVar()->current_mutex= NULL;
-  session.getThreadVar()->current_cond= NULL;
-
+  pthread_mutex_lock(&thread_var->mutex);
+  thread_var->current_mutex= NULL;
+  thread_var->current_cond= NULL;
+  pthread_mutex_unlock(&thread_var->mutex);
   return(result);
 }
 
 
-static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data, THR_LOCK_OWNER *owner, enum thr_lock_type lock_type)
+static enum enum_thr_lock_result
+thr_lock(THR_LOCK_DATA *data, THR_LOCK_OWNER *owner,
+         enum thr_lock_type lock_type)
 {
-  THR_LOCK *lock= data->lock;
+  THR_LOCK *lock=data->lock;
   enum enum_thr_lock_result result= THR_LOCK_SUCCESS;
   struct st_lock_list *wait_queue;
   THR_LOCK_DATA *lock_owner;
@@ -245,7 +281,7 @@ static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data,
   data->cond=0;					/* safety */
   data->type=lock_type;
   data->owner= owner;                           /* Must be reset ! */
-  lock->lock();
+  pthread_mutex_lock(&lock->mutex);
   if ((int) lock_type <= (int) TL_READ_NO_INSERT)
   {
     /* Request for READ lock */
@@ -271,7 +307,9 @@ static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data,
 	lock->read.last= &data->next;
 	if (lock_type == TL_READ_NO_INSERT)
 	  lock->read_no_write_count++;
-        current_global_counters.locks_immediate++;
+	if (lock->get_status)
+	  (*lock->get_status)(data->status_param, 0);
+	statistic_increment(locks_immediate,&internal::THR_LOCK_lock);
 	goto end;
       }
       if (lock->write.data->type == TL_WRITE_ONLY)
@@ -289,9 +327,11 @@ static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data,
       (*lock->read.last)=data;			/* Add to running FIFO */
       data->prev=lock->read.last;
       lock->read.last= &data->next;
+      if (lock->get_status)
+	(*lock->get_status)(data->status_param, 0);
       if (lock_type == TL_READ_NO_INSERT)
 	lock->read_no_write_count++;
-      current_global_counters.locks_immediate++;
+      statistic_increment(locks_immediate,&internal::THR_LOCK_lock);
       goto end;
     }
     /*
@@ -303,7 +343,7 @@ static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data,
   }
   else						/* Request for WRITE lock */
   {
-    if (lock_type == TL_WRITE_CONCURRENT_INSERT)
+    if (lock_type == TL_WRITE_CONCURRENT_INSERT && ! lock->check_status)
       data->type=lock_type= thr_upgraded_concurrent_insert_lock;
 
     if (lock->write.data)			/* If there is a write lock */
@@ -338,7 +378,9 @@ static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data,
 	(*lock->write.last)=data;	/* Add to running fifo */
 	data->prev=lock->write.last;
 	lock->write.last= &data->next;
-        current_global_counters.locks_immediate++;
+	if (data->lock->get_status)
+	  (*data->lock->get_status)(data->status_param, 0);
+	statistic_increment(locks_immediate,&internal::THR_LOCK_lock);
 	goto end;
       }
     }
@@ -346,6 +388,17 @@ static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data,
     {
       if (!lock->write_wait.data)
       {						/* no scheduled write locks */
+        bool concurrent_insert= 0;
+	if (lock_type == TL_WRITE_CONCURRENT_INSERT)
+        {
+          concurrent_insert= 1;
+          if ((*lock->check_status)(data->status_param))
+          {
+            concurrent_insert= 0;
+            data->type=lock_type= thr_upgraded_concurrent_insert_lock;
+          }
+        }
+
 	if (!lock->read.data ||
 	    (lock_type <= TL_WRITE_CONCURRENT_INSERT &&
 	     ((lock_type != TL_WRITE_CONCURRENT_INSERT &&
@@ -355,7 +408,9 @@ static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data,
 	  (*lock->write.last)=data;		/* Add as current write lock */
 	  data->prev=lock->write.last;
 	  lock->write.last= &data->next;
-          current_global_counters.locks_immediate++;
+	  if (data->lock->get_status)
+	    (*data->lock->get_status)(data->status_param, concurrent_insert);
+	  statistic_increment(locks_immediate,&internal::THR_LOCK_lock);
 	  goto end;
 	}
       }
@@ -374,20 +429,17 @@ static enum enum_thr_lock_result thr_lock(Session &session, THR_LOCK_DATA *data,
     result= THR_LOCK_DEADLOCK;
     goto end;
   }
-
   /* Can't get lock yet;  Wait for it */
-  return(wait_for_lock(session, wait_queue, data));
-
+  return(wait_for_lock(wait_queue, data, 0));
 end:
-  lock->unlock();
-
+  pthread_mutex_unlock(&lock->mutex);
   return(result);
 }
 
 
 static void free_all_read_locks(THR_LOCK *lock, bool using_concurrent_insert)
 {
-  THR_LOCK_DATA *data= lock->read_wait.data;
+  THR_LOCK_DATA *data=lock->read_wait.data;
 
   /* move all locks from read_wait list to read list */
   (*lock->read.last)=data;
@@ -399,7 +451,7 @@ static void free_all_read_locks(THR_LOCK *lock, bool using_concurrent_insert)
 
   do
   {
-    boost::condition_variable_any *cond= data->cond;
+    pthread_cond_t *cond=data->cond;
     if ((int) data->type == (int) TL_READ_NO_INSERT)
     {
       if (using_concurrent_insert)
@@ -419,21 +471,21 @@ static void free_all_read_locks(THR_LOCK *lock, bool using_concurrent_insert)
       }
       lock->read_no_write_count++;
     }
-    data->cond= NULL;				/* Mark thread free */
-    cond->notify_one();
+    data->cond=0;				/* Mark thread free */
+    pthread_cond_signal(cond);
   } while ((data=data->next));
   *lock->read_wait.last=0;
   if (!lock->read_wait.data)
     lock->write_lock_count=0;
 }
 
-/* Unlock lock and free next thread on same lock */
+	/* Unlock lock and free next thread on same lock */
 
 static void thr_unlock(THR_LOCK_DATA *data)
 {
   THR_LOCK *lock=data->lock;
   enum thr_lock_type lock_type=data->type;
-  lock->lock();
+  pthread_mutex_lock(&lock->mutex);
 
   if (((*data->prev)=data->next))		/* remove from lock-list */
     data->next->prev= data->prev;
@@ -442,14 +494,21 @@ static void thr_unlock(THR_LOCK_DATA *data)
   else
     lock->write.last=data->prev;
   if (lock_type >= TL_WRITE_CONCURRENT_INSERT)
-  { }
+  {
+    if (lock->update_status)
+      (*lock->update_status)(data->status_param);
+  }
   else
-  { }
+  {
+    if (lock->restore_status)
+      (*lock->restore_status)(data->status_param);
+  }
   if (lock_type == TL_READ_NO_INSERT)
     lock->read_no_write_count--;
   data->type=TL_UNLOCK;				/* Mark unlocked */
   wake_up_waiters(lock);
-  lock->unlock();
+  pthread_mutex_unlock(&lock->mutex);
+  return;
 }
 
 
@@ -495,11 +554,13 @@ static void wake_up_waiters(THR_LOCK *lock)
 	  data->prev=lock->write.last;
 	  data->next=0;
 	  lock->write.last= &data->next;
-
+	  if (data->type == TL_WRITE_CONCURRENT_INSERT &&
+	      (*lock->check_status)(data->status_param))
+	    data->type=TL_WRITE;			/* Upgrade lock */
 	  {
-            boost::condition_variable_any *cond= data->cond;
-	    data->cond= NULL;				/* Mark thread free */
-            cond->notify_one(); /* Start waiting thred */
+	    pthread_cond_t *cond=data->cond;
+	    data->cond=0;				/* Mark thread free */
+	    pthread_cond_signal(cond);	/* Start waiting thread */
 	  }
 	  if (data->type != TL_WRITE_ALLOW_WRITE ||
 	      !lock->write_wait.data ||
@@ -523,8 +584,20 @@ static void wake_up_waiters(THR_LOCK *lock)
 	       lock_type != TL_WRITE_ALLOW_WRITE) ||
 	      !lock->read_no_write_count))
     {
+      /*
+	For DELAYED, ALLOW_READ, WRITE_ALLOW_WRITE or CONCURRENT_INSERT locks
+	start WRITE locks together with the READ locks
+      */
+      if (lock_type == TL_WRITE_CONCURRENT_INSERT &&
+	  (*lock->check_status)(data->status_param))
+      {
+	data->type=TL_WRITE;			/* Upgrade lock */
+	if (lock->read_wait.data)
+	  free_all_read_locks(lock,0);
+	goto end;
+      }
       do {
-        boost::condition_variable_any *cond= data->cond;
+	pthread_cond_t *cond=data->cond;
 	if (((*data->prev)=data->next))		/* remove from wait-list */
 	  data->next->prev= data->prev;
 	else
@@ -533,8 +606,8 @@ static void wake_up_waiters(THR_LOCK *lock)
 	data->prev=lock->write.last;
 	lock->write.last= &data->next;
 	data->next=0;				/* Only one write lock */
-	data->cond= NULL;				/* Mark thread free */
-        cond->notify_one(); /* Start waiting thread */
+	data->cond=0;				/* Mark thread free */
+	pthread_cond_signal(cond);	/* Start waiting thread */
       } while (lock_type == TL_WRITE_ALLOW_WRITE &&
 	       (data=lock->write_wait.data) &&
 	       data->type == TL_WRITE_ALLOW_WRITE);
@@ -544,9 +617,7 @@ static void wake_up_waiters(THR_LOCK *lock)
 			     lock_type == TL_WRITE_ALLOW_WRITE));
     }
     else if (!data && lock->read_wait.data)
-    {
       free_all_read_locks(lock,0);
-    }
   }
 end:
   return;
@@ -584,7 +655,7 @@ static void sort_locks(THR_LOCK_DATA **data,uint32_t count)
 
 
 enum enum_thr_lock_result
-thr_multi_lock(Session &session, THR_LOCK_DATA **data, uint32_t count, THR_LOCK_OWNER *owner)
+thr_multi_lock(THR_LOCK_DATA **data, uint32_t count, THR_LOCK_OWNER *owner)
 {
   THR_LOCK_DATA **pos,**end;
   if (count > 1)
@@ -592,13 +663,60 @@ thr_multi_lock(Session &session, THR_LOCK_DATA **data, uint32_t count, THR_LOCK_
   /* lock everything */
   for (pos=data,end=data+count; pos < end ; pos++)
   {
-    enum enum_thr_lock_result result= thr_lock(session, *pos, owner, (*pos)->type);
+    enum enum_thr_lock_result result= thr_lock(*pos, owner, (*pos)->type);
     if (result != THR_LOCK_SUCCESS)
     {						/* Aborted */
       thr_multi_unlock(data,(uint32_t) (pos-data));
       return(result);
     }
   }
+  /*
+    Ensure that all get_locks() have the same status
+    If we lock the same table multiple times, we must use the same
+    status_param!
+  */
+#if !defined(DONT_USE_RW_LOCKS)
+  if (count > 1)
+  {
+    THR_LOCK_DATA *last_lock= end[-1];
+    pos=end-1;
+    do
+    {
+      pos--;
+      if (last_lock->lock == (*pos)->lock &&
+	  last_lock->lock->copy_status)
+      {
+	if (last_lock->type <= TL_READ_NO_INSERT)
+	{
+	  THR_LOCK_DATA **read_lock;
+	  /*
+	    If we are locking the same table with read locks we must ensure
+	    that all tables share the status of the last write lock or
+	    the same read lock.
+	  */
+	  for (;
+	       (*pos)->type <= TL_READ_NO_INSERT &&
+		 pos != data &&
+		 pos[-1]->lock == (*pos)->lock ;
+	       pos--) ;
+
+	  read_lock = pos+1;
+	  do
+	  {
+	    (last_lock->lock->copy_status)((*read_lock)->status_param,
+					   (*pos)->status_param);
+	  } while (*(read_lock++) != last_lock);
+	  last_lock= (*pos);			/* Point at last write lock */
+	}
+	else
+	  (*last_lock->lock->copy_status)((*pos)->status_param,
+					  last_lock->status_param);
+      }
+      else
+	last_lock=(*pos);
+    } while (pos != data);
+  }
+#endif
   return(THR_LOCK_SUCCESS);
 }
 
@@ -613,17 +731,7 @@ void thr_multi_unlock(THR_LOCK_DATA **data,uint32_t count)
     if ((*pos)->type != TL_UNLOCK)
       thr_unlock(*pos);
   }
-}
-
-void DrizzleLock::unlock(uint32_t count)
-{
-  THR_LOCK_DATA **pos,**end;
-
-  for (pos= getLocks(),end= getLocks()+count; pos < end ; pos++)
-  {
-    if ((*pos)->type != TL_UNLOCK)
-      thr_unlock(*pos);
-  }
+  return;
 }
 
 /*
@@ -631,28 +739,31 @@ void DrizzleLock::unlock(uint32_t count)
   TL_WRITE_ONLY to abort any new accesses to the lock
 */
 
-void THR_LOCK::abort_locks()
+void thr_abort_locks(THR_LOCK *lock)
 {
-  boost::mutex::scoped_lock scopedLock(mutex);
+  THR_LOCK_DATA *data;
+  pthread_mutex_lock(&lock->mutex);
 
-  for (THR_LOCK_DATA *local_data= read_wait.data; local_data ; local_data= local_data->next)
+  for (data=lock->read_wait.data; data ; data=data->next)
   {
-    local_data->type= TL_UNLOCK;			/* Mark killed */
+    data->type= TL_UNLOCK;			/* Mark killed */
     /* It's safe to signal the cond first: we're still holding the mutex. */
-    local_data->cond->notify_one();
-    local_data->cond= NULL;				/* Removed from list */
+    pthread_cond_signal(data->cond);
+    data->cond= NULL;				/* Removed from list */
   }
-  for (THR_LOCK_DATA *local_data= write_wait.data; local_data ; local_data= local_data->next)
+  for (data=lock->write_wait.data; data ; data=data->next)
   {
-    local_data->type= TL_UNLOCK;
-    local_data->cond->notify_one();
-    local_data->cond= NULL;
+    data->type=TL_UNLOCK;
+    pthread_cond_signal(data->cond);
+    data->cond= NULL;
   }
-  read_wait.last= &read_wait.data;
-  write_wait.last= &write_wait.data;
-  read_wait.data= write_wait.data=0;
-  if (write.data)
-    write.data->type=TL_WRITE_ONLY;
+  lock->read_wait.last= &lock->read_wait.data;
+  lock->write_wait.last= &lock->write_wait.data;
+  lock->read_wait.data=lock->write_wait.data=0;
+  if (lock->write.data)
+    lock->write.data->type=TL_WRITE_ONLY;
+  pthread_mutex_unlock(&lock->mutex);
+  return;
 }
 
 
@@ -662,45 +773,46 @@ void THR_LOCK::abort_locks()
   This is used to abort all locks for a specific thread
 */
 
-bool THR_LOCK::abort_locks_for_thread(uint64_t thread_id_arg)
+bool thr_abort_locks_for_thread(THR_LOCK *lock, uint64_t thread_id)
 {
+  THR_LOCK_DATA *data;
   bool found= false;
 
-  boost::mutex::scoped_lock scopedLock(mutex);
-  for (THR_LOCK_DATA *local_data= read_wait.data; local_data ; local_data= local_data->next)
+  pthread_mutex_lock(&lock->mutex);
+  for (data= lock->read_wait.data; data ; data= data->next)
   {
-    if (local_data->owner->info->thread_id == thread_id_arg)
+    if (data->owner->info->thread_id == thread_id)
     {
-      local_data->type= TL_UNLOCK;			/* Mark killed */
+      data->type= TL_UNLOCK;			/* Mark killed */
       /* It's safe to signal the cond first: we're still holding the mutex. */
       found= true;
-      local_data->cond->notify_one();
-      local_data->cond= 0;				/* Removed from list */
+      pthread_cond_signal(data->cond);
+      data->cond= 0;				/* Removed from list */
 
-      if (((*local_data->prev)= local_data->next))
-	local_data->next->prev= local_data->prev;
+      if (((*data->prev)= data->next))
+	data->next->prev= data->prev;
       else
-	read_wait.last= local_data->prev;
+	lock->read_wait.last= data->prev;
     }
   }
-  for (THR_LOCK_DATA *local_data= write_wait.data; local_data ; local_data= local_data->next)
+  for (data= lock->write_wait.data; data ; data= data->next)
   {
-    if (local_data->owner->info->thread_id == thread_id_arg)
+    if (data->owner->info->thread_id == thread_id)
     {
-      local_data->type= TL_UNLOCK;
+      data->type= TL_UNLOCK;
       found= true;
-      local_data->cond->notify_one();
-      local_data->cond= NULL;
+      pthread_cond_signal(data->cond);
+      data->cond= NULL;
 
-      if (((*local_data->prev)= local_data->next))
-	local_data->next->prev= local_data->prev;
+      if (((*data->prev)= data->next))
+	data->next->prev= data->prev;
       else
-	write_wait.last= local_data->prev;
+	lock->write_wait.last= data->prev;
     }
   }
-  wake_up_waiters(this);
-
-  return found;
+  wake_up_waiters(lock);
+  pthread_mutex_unlock(&lock->mutex);
+  return(found);
 }
 
 } /* namespace drizzled */
