@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,16 +17,16 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_VERSION_H
+#define DRIZZLED_VERSION_H
 
 #include <string>
 
-#include <drizzled/visibility.h>
-
 namespace drizzled
 {
 
-DRIZZLED_API const std::string& version();
+const std::string& version();
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_VERSION_H */
