@@ -18,34 +18,45 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <plugin/collation_dictionary/dictionary.h>
+#include "config.h"
+#include "plugin/collation_dictionary/dictionary.h"
 
 using namespace drizzled;
 
 static CharacterSetsTool *character_sets;
 static CollationsTool *collations;
 
-static int init(drizzled::module::Context &context)
+static int init(drizzled::plugin::Registry &registry)
 {
-  character_sets= new CharacterSetsTool;
-  collations= new CollationsTool;
+  character_sets= new(std::nothrow)CharacterSetsTool;
+  collations= new(std::nothrow)CollationsTool;
 
-  context.add(character_sets);
-  context.add(collations);
+  registry.add(character_sets);
+  registry.add(collations);
   
   return 0;
 }
 
+static int finalize(drizzled::plugin::Registry &registry)
+{
+  registry.remove(character_sets);
+  registry.remove(collations);
+  delete character_sets;
+  delete collations;
+
+  return 0;
+}
+
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
   "collation_dictionary",
   "1.0",
   "Brian Aker",
-  N_("Character set and collation dictionaries"),
+  "Data Dictionary for schema, table, column, indexes, etc",
   PLUGIN_LICENSE_GPL,
   init,
+  finalize,
   NULL,
   NULL
 }
