@@ -18,10 +18,12 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_COLLATION_DICTIONARY_DICTIONARY_H
+#define PLUGIN_COLLATION_DICTIONARY_DICTIONARY_H
 
-#include <drizzled/plugin/table_function.h>
+#include "drizzled/plugin/table_function.h"
 
 #include <plugin/collation_dictionary/character_sets.h>
 #include <plugin/collation_dictionary/collations.h>
 
+#endif /* PLUGIN_COLLATION_DICTIONARY_DICTIONARY_H */
