@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,89 +36,89 @@
  * the event will not be passed along to the applier.
  */
 
-#include <config.h>
+#include "config.h"
 #include <drizzled/gettext.h>
 #include <drizzled/plugin/transaction_applier.h>
 #include <drizzled/message/transaction.pb.h>
 #include <drizzled/plugin.h>
+#include <drizzled/plugin/registry.h>
 
-#include <drizzled/item/string.h>
 #include "filtered_replicator.h"
-#include <boost/program_options.hpp>
-#include <drizzled/module/option_map.h>
+
 #include <vector>
 #include <string>
-namespace po= boost::program_options;
+
 using namespace std;
 using namespace drizzled;
 
-namespace drizzle_plugin
-{
-
-static string sysvar_filtered_replicator_sch_filters;
-static string sysvar_filtered_replicator_tab_filters;
+static bool sysvar_filtered_replicator_enabled= false;
+static char *sysvar_filtered_replicator_sch_filters= NULL;
+static char *sysvar_filtered_replicator_tab_filters= NULL;
+static char *sysvar_filtered_replicator_sch_regex= NULL;
+static char *sysvar_filtered_replicator_tab_regex= NULL;
 
 FilteredReplicator::FilteredReplicator(string name_arg,
-                                       const std::string &sch_filter,
-                                       const std::string &tab_filter,
-                                       const std::string &sch_regex,
-                                       const std::string &tab_regex) :
-  plugin::TransactionReplicator(name_arg),
-  schemas_to_filter(),
-  tables_to_filter(),
-  _sch_filter(sch_filter),
-  _tab_filter(tab_filter),
-  _sch_regex(sch_regex),
-  _tab_regex(tab_regex),
-  sch_re(NULL),
-  tab_re(NULL)
+                                       const char *in_sch_filters,
+                                       const char *in_tab_filters)
+  :
+    plugin::TransactionReplicator(name_arg),
+    schemas_to_filter(),
+    tables_to_filter(),
+    sch_filter_string(in_sch_filters),
+    tab_filter_string(in_tab_filters),
+    sch_regex_enabled(false),
+    tab_regex_enabled(false),
+    sch_re(NULL),
+    tab_re(NULL)
 {
   /* 
    * Add each of the specified schemas to the vector of schemas
    * to filter.
    */
-  if (not _sch_filter.empty())
+  if (in_sch_filters)
   {
-    populateFilter(_sch_filter, schemas_to_filter);
+    populateFilter(sch_filter_string, schemas_to_filter);
   }
 
   /* 
    * Add each of the specified tables to the vector of tables
    * to filter.
    */
-  if (not _tab_filter.empty())
+  if (in_tab_filters)
   {
-    populateFilter(_tab_filter, tables_to_filter);
+    populateFilter(tab_filter_string, tables_to_filter);
   }
 
   /* 
    * Compile the regular expression for schema's to filter
    * if one is specified.
    */
-  if (not _sch_regex.empty())
+  if (sysvar_filtered_replicator_sch_regex)
   {
     const char *error= NULL;
     int32_t error_offset= 0;
-    sch_re= pcre_compile(_sch_regex.c_str(),
+    sch_re= pcre_compile(sysvar_filtered_replicator_sch_regex,
                          0,
                          &error,
                          &error_offset,
                          NULL);
+    sch_regex_enabled= true;
   }
 
   /* 
    * Compile the regular expression for table's to filter
    * if one is specified.
    */
-  if (not _tab_regex.empty())
+  if (sysvar_filtered_replicator_tab_regex)
   {
     const char *error= NULL;
     int32_t error_offset= 0;
-    tab_re= pcre_compile(_tab_regex.c_str(),
+    tab_re= pcre_compile(sysvar_filtered_replicator_tab_regex,
                          0,
                          &error,
                          &error_offset,
                          NULL);
+    tab_regex_enabled= true;
   }
 
   pthread_mutex_init(&sch_vector_lock, NULL);
@@ -127,22 +127,9 @@ FilteredReplicator::FilteredReplicator(string name_arg,
   pthread_mutex_init(&sysvar_tab_lock, NULL);
 }
 
-FilteredReplicator::~FilteredReplicator()
+bool FilteredReplicator::isEnabled() const
 {
-  if (sch_re)
-  {
-    pcre_free(sch_re);
-  }
-  if (tab_re)
-  {
-    pcre_free(tab_re);
-  }
-
-  pthread_mutex_destroy(&sch_vector_lock);
-  pthread_mutex_destroy(&tab_vector_lock);
-  pthread_mutex_destroy(&sysvar_sch_lock);
-  pthread_mutex_destroy(&sysvar_tab_lock);
-
+  return sysvar_filtered_replicator_enabled;
 }
 
 void FilteredReplicator::parseStatementTableMetadata(const message::Statement &in_statement,
@@ -154,59 +141,59 @@ void FilteredReplicator::parseStatementTableMetadata(const message::Statement &i
     case message::Statement::INSERT:
     {
       const message::TableMetadata &metadata= in_statement.insert_header().table_metadata();
-      in_schema_name= metadata.schema_name();
-      in_table_name= metadata.table_name();
+      in_schema_name.assign(metadata.schema_name());
+      in_table_name.assign(metadata.table_name());
       break;
     }
     case message::Statement::UPDATE:
     {
       const message::TableMetadata &metadata= in_statement.update_header().table_metadata();
-      in_schema_name= metadata.schema_name();
-      in_table_name= metadata.table_name();
+      in_schema_name.assign(metadata.schema_name());
+      in_table_name.assign(metadata.table_name());
       break;
     }
     case message::Statement::DELETE:
     {
       const message::TableMetadata &metadata= in_statement.delete_header().table_metadata();
-      in_schema_name= metadata.schema_name();
-      in_table_name= metadata.table_name();
+      in_schema_name.assign(metadata.schema_name());
+      in_table_name.assign(metadata.table_name());
       break;
     }
     case message::Statement::CREATE_SCHEMA:
     {
-      in_schema_name= in_statement.create_schema_statement().schema().name();
+      in_schema_name.assign(in_statement.create_schema_statement().schema().name());
       in_table_name.clear();
       break;
     }
     case message::Statement::ALTER_SCHEMA:
     {
-      in_schema_name= in_statement.alter_schema_statement().after().name();
+      in_schema_name.assign(in_statement.alter_schema_statement().after().name());
       in_table_name.clear();
       break;
     }
     case message::Statement::DROP_SCHEMA:
     {
-      in_schema_name= in_statement.drop_schema_statement().schema_name();
+      in_schema_name.assign(in_statement.drop_schema_statement().schema_name());
       in_table_name.clear();
       break;
     }
     case message::Statement::CREATE_TABLE:
     {
-      in_schema_name= in_statement.create_table_statement().table().schema();
-      in_table_name= in_statement.create_table_statement().table().name();
+      // in_schema_name.assign(in_statement.create_table_statement().table().name());
+      in_table_name.assign(in_statement.create_table_statement().table().name());
       break;
     }
     case message::Statement::ALTER_TABLE:
     {
-      in_schema_name= in_statement.alter_table_statement().after().schema();
-      in_table_name= in_statement.alter_table_statement().after().name();
+      // in_schema_name.assign(in_statement.alter_table_statement().table().name());
+      in_table_name.assign(in_statement.alter_table_statement().after().name());
       break;
     }
     case message::Statement::DROP_TABLE:
     {
       const message::TableMetadata &metadata= in_statement.drop_table_statement().table_metadata();
-      in_schema_name= metadata.schema_name();
-      in_table_name= metadata.table_name();
+      in_schema_name.assign(metadata.schema_name());
+      in_table_name.assign(metadata.table_name());
       break;
     }
     default:
@@ -218,16 +205,24 @@ void FilteredReplicator::parseStatementTableMetadata(const message::Statement &i
     }
   }  
 }
+void FilteredReplicator::enable()
+{
+  sysvar_filtered_replicator_enabled= true;
+}
 
-plugin::ReplicationReturnCode
-FilteredReplicator::replicate(plugin::TransactionApplier *in_applier,
-                              Session &in_session,
-                              message::Transaction &to_replicate)
+void FilteredReplicator::disable()
+{
+  sysvar_filtered_replicator_enabled= false;
+}
+
+void FilteredReplicator::replicate(plugin::TransactionApplier *in_applier, 
+                                   message::Transaction &to_replicate)
 {
   string schema_name;
   string table_name;
 
   size_t num_statements= to_replicate.statement_size();
+  size_t x;
 
   /* 
    * We build a new transaction message containing only Statement
@@ -239,7 +234,7 @@ FilteredReplicator::replicate(plugin::TransactionApplier *in_applier,
    */
   message::Transaction filtered_transaction;
 
-  for (size_t x= 0; x < num_statements; ++x)
+  for (x= 0; x < num_statements; ++x)
   {
     schema_name.clear();
     table_name.clear();
@@ -271,10 +266,13 @@ FilteredReplicator::replicate(plugin::TransactionApplier *in_applier,
      * also keep all entries in the vectors of schemas and tables to filter in
      * lowercase.
      */
-    boost::to_lower(schema_name);
-    boost::to_lower(table_name);
+    std::transform(schema_name.begin(), schema_name.end(),
+                  schema_name.begin(), ::tolower);
+    std::transform(table_name.begin(), table_name.end(),
+                  table_name.begin(), ::tolower);
 
-    if (! isSchemaFiltered(schema_name) && ! isTableFiltered(table_name))
+    if (! isSchemaFiltered(schema_name) &&
+        ! isTableFiltered(table_name))
     {
       message::Statement *s= filtered_transaction.add_statement();
       *s= statement; /* copy contruct */
@@ -290,19 +288,19 @@ FilteredReplicator::replicate(plugin::TransactionApplier *in_applier,
      */
     message::TransactionContext *tc= filtered_transaction.mutable_transaction_context();
     *tc= to_replicate.transaction_context(); /* copy construct */
-    return in_applier->apply(in_session, filtered_transaction);
+    in_applier->apply(filtered_transaction);
   }
-  return plugin::SUCCESS;
 }
 
 void FilteredReplicator::populateFilter(std::string input,
-                                        std::vector<string> &filter)
+                                        vector<string> &filter)
 {
   /*
    * Convert the input string to lowercase so that all entries in the vector
    * will be in lowercase.
    */
-  boost::to_lower(input);
+  std::transform(input.begin(), input.end(),
+                 input.begin(), ::tolower);
   string::size_type last_pos= input.find_first_not_of(',', 0);
   string::size_type pos= input.find_first_of(',', last_pos);
 
@@ -317,7 +315,9 @@ void FilteredReplicator::populateFilter(std::string input,
 bool FilteredReplicator::isSchemaFiltered(const string &schema_name)
 {
   pthread_mutex_lock(&sch_vector_lock);
-  std::vector<string>::iterator it= find(schemas_to_filter.begin(), schemas_to_filter.end(), schema_name);
+  vector<string>::iterator it= find(schemas_to_filter.begin(),
+                                    schemas_to_filter.end(),
+                                    schema_name);
   if (it != schemas_to_filter.end())
   {
     pthread_mutex_unlock(&sch_vector_lock);
@@ -330,11 +330,20 @@ bool FilteredReplicator::isSchemaFiltered(const string &schema_name)
    * we check to see if this schema name matches the regular expression that
    * has been specified. 
    */
-  if (not _sch_regex.empty())
+  if (sch_regex_enabled)
   {
-    int32_t result= pcre_exec(sch_re, NULL, schema_name.c_str(), schema_name.length(), 0, 0, NULL, 0);
+    int32_t result= pcre_exec(sch_re,
+                              NULL,
+                              schema_name.c_str(),
+                              schema_name.length(),
+                              0,
+                              0,
+                              NULL,
+                              0);
     if (result >= 0)
+    {
       return true;
+    }
   }
 
   return false;
@@ -343,7 +352,9 @@ bool FilteredReplicator::isSchemaFiltered(const string &schema_name)
 bool FilteredReplicator::isTableFiltered(const string &table_name)
 {
   pthread_mutex_lock(&tab_vector_lock);
-  std::vector<string>::iterator it= find(tables_to_filter.begin(), tables_to_filter.end(), table_name);
+  vector<string>::iterator it= find(tables_to_filter.begin(),
+                                    tables_to_filter.end(),
+                                    table_name);
   if (it != tables_to_filter.end())
   {
     pthread_mutex_unlock(&tab_vector_lock);
@@ -356,11 +367,20 @@ bool FilteredReplicator::isTableFiltered(const string &table_name)
    * we check to see if this table name matches the regular expression that
    * has been specified. 
    */
-  if (not _tab_regex.empty())
+  if (tab_regex_enabled)
   {
-    int32_t result= pcre_exec(tab_re, NULL, table_name.c_str(), table_name.length(), 0, 0, NULL, 0);
+    int32_t result= pcre_exec(tab_re,
+                              NULL,
+                              table_name.c_str(),
+                              table_name.length(),
+                              0,
+                              0,
+                              NULL,
+                              0);
     if (result >= 0)
+    {
       return true;
+    }
   }
 
   return false;
@@ -381,7 +401,8 @@ void FilteredReplicator::parseQuery(const string &sql,
    * Convert the type string to uppercase here so that it doesn't
    * matter what case the user entered the statement in.
    */
-  boost::to_upper(type);
+  std::transform(type.begin(), type.end(),
+                 type.begin(), ::toupper);
 
   if (type.compare("DROP") == 0)
   {
@@ -398,11 +419,11 @@ void FilteredReplicator::parseQuery(const string &sql,
     {
       /* the name must be the fifth word */
       pos= sql.find_first_of(' ', 21);
-      target_name= sql.substr(21, pos - 21);
+      target_name.assign(sql.substr(21, pos - 21));
     }
     else
     {
-      target_name= cmp_str;
+      target_name.assign(cmp_str);
     }
     /*
      * Determine whether the name is a concatenation of the schema
@@ -415,15 +436,15 @@ void FilteredReplicator::parseQuery(const string &sql,
       /*
        * There is a schema name here...
        */
-      schema_name= target_name.substr(0, pos);
+      schema_name.assign(target_name.substr(0, pos));
       /*
        * The rest of the name string is the table name.
        */
-      table_name= target_name.substr(pos + 1);
+      table_name.assign(target_name.substr(pos + 1));
     }
     else
     {
-      table_name= target_name;
+      table_name.assign(target_name);
     }
   }
   else if (type.compare("CREATE") == 0)
@@ -446,15 +467,15 @@ void FilteredReplicator::parseQuery(const string &sql,
       /*
        * There is a schema name here...
        */
-      schema_name= target_name.substr(0, pos);
+      schema_name.assign(target_name.substr(0, pos));
       /*
        * The rest of the name string is the table name.
        */
-      table_name= target_name.substr(pos + 1);
+      table_name.assign(target_name.substr(pos + 1));
     }
     else
     {
-      table_name= target_name;
+      table_name.assign(target_name);
     }
   }
   else
@@ -468,10 +489,9 @@ void FilteredReplicator::setSchemaFilter(const string &input)
 {
   pthread_mutex_lock(&sch_vector_lock);
   pthread_mutex_lock(&sysvar_sch_lock);
-  _sch_filter= input;
+  sch_filter_string.assign(input);
   schemas_to_filter.clear();
-  populateFilter(_sch_filter, schemas_to_filter);
-  pthread_mutex_unlock(&sysvar_sch_lock);
+  populateFilter(sch_filter_string, schemas_to_filter);
   pthread_mutex_unlock(&sch_vector_lock);
 }
 
@@ -479,18 +499,50 @@ void FilteredReplicator::setTableFilter(const string &input)
 {
   pthread_mutex_lock(&tab_vector_lock);
   pthread_mutex_lock(&sysvar_tab_lock);
-  _tab_filter= input;
+  tab_filter_string.assign(input);
   tables_to_filter.clear();
-  populateFilter(_tab_filter, tables_to_filter);
-  pthread_mutex_unlock(&sysvar_tab_lock);
+  populateFilter(tab_filter_string, tables_to_filter);
   pthread_mutex_unlock(&tab_vector_lock);
 }
 
 static FilteredReplicator *filtered_replicator= NULL; /* The singleton replicator */
 
-static int filtered_schemas_validate(Session*, set_var *var)
+static int init(plugin::Registry &registry)
 {
-  const char *input= var->value->str_value.ptr();
+  if (sysvar_filtered_replicator_enabled)
+  {
+    filtered_replicator= new(std::nothrow) 
+      FilteredReplicator("filtered_replicator",
+                         sysvar_filtered_replicator_sch_filters,
+                         sysvar_filtered_replicator_tab_filters);
+    if (filtered_replicator == NULL)
+    {
+      return 1;
+    }
+    registry.add(filtered_replicator);
+  }
+  return 0;
+}
+
+static int deinit(plugin::Registry &registry)
+{
+  if (filtered_replicator)
+  {
+    registry.remove(filtered_replicator);
+    delete filtered_replicator;
+  }
+  return 0;
+}
+
+static int check_filtered_schemas(Session *, 
+                                  drizzle_sys_var *,
+                                  void *,
+                                  drizzle_value *value)
+{
+  char buff[STRING_BUFFER_USUAL_SIZE];
+  int len= sizeof(buff);
+  const char *input= value->val_str(value, buff, &len);
+
   if (input && filtered_replicator)
   {
     filtered_replicator->setSchemaFilter(input);
@@ -499,73 +551,111 @@ static int filtered_schemas_validate(Session*, set_var *var)
   return 1;
 }
 
-
-static int filtered_tables_validate(Session*, set_var *var)
+static void set_filtered_schemas(Session *,
+                                 drizzle_sys_var *,
+                                 void *var_ptr,
+                                 const void *save)
 {
-  const char *input= var->value->str_value.ptr();
+  if (filtered_replicator)
+  {
+    if (*(bool *)save != true)
+    {
+      /* update the value of the system variable */
+      filtered_replicator->updateSchemaSysvar((const char **) var_ptr);
+    }
+  }
+}
+
+static int check_filtered_tables(Session *, 
+                                 drizzle_sys_var *,
+                                 void *save,
+                                 drizzle_value *value)
+{
+  char buff[STRING_BUFFER_USUAL_SIZE];
+  int len= sizeof(buff);
+  const char *input= value->val_str(value, buff, &len);
+
   if (input && filtered_replicator)
   {
     filtered_replicator->setTableFilter(input);
+    *(bool *) save= (bool) true;
     return 0;
   }
+  *(bool *) save= (bool) false;
   return 1;
 }
 
-
-static int init(module::Context &context)
+static void set_filtered_tables(Session *,
+                                drizzle_sys_var *,
+                                void *var_ptr,
+                                const void *save)
 {
-  const module::option_map &vm= context.getOptions();
-  
-  filtered_replicator= new FilteredReplicator("filtered_replicator",
-                                              vm["filteredschemas"].as<string>(),
-                                              vm["filteredtables"].as<string>(),
-                                              vm["schemaregex"].as<string>(),
-                                              vm["tableregex"].as<string>());
-
-  context.add(filtered_replicator);
-  context.registerVariable(new sys_var_std_string("filteredschemas",
-                                                  sysvar_filtered_replicator_sch_filters,
-                                                  filtered_schemas_validate));
-  context.registerVariable(new sys_var_std_string("filteredtables",
-                                                  sysvar_filtered_replicator_tab_filters,
-                                                  filtered_tables_validate));
-
-  context.registerVariable(new sys_var_const_string_val("schemaregex",
-                                                        vm["schemaregex"].as<string>()));
-  context.registerVariable(new sys_var_const_string_val("tableregex",
-                                                        vm["tableregex"].as<string>()));
-
-  return 0;
+  if (filtered_replicator)
+  {
+    if (*(bool *)save != false)
+    {
+      /* update the value of the system variable */
+      filtered_replicator->updateTableSysvar((const char **) var_ptr);
+    }
+  }
 }
 
-static void init_options(drizzled::module::option_context &context)
-{
-  context("filteredschemas",
-          po::value<string>(&sysvar_filtered_replicator_sch_filters)->default_value(""),
-          N_("Comma-separated list of schemas to exclude"));
-  context("filteredtables",
-          po::value<string>(&sysvar_filtered_replicator_tab_filters)->default_value(""),
-          N_("Comma-separated list of tables to exclude"));
-  context("schemaregex", 
-          po::value<string>()->default_value(""),
-          N_("Regular expression to apply to schemas to exclude"));
-  context("tableregex", 
-          po::value<string>()->default_value(""),
-          N_("Regular expression to apply to tables to exclude"));
-}
+static DRIZZLE_SYSVAR_BOOL(enable,
+                           sysvar_filtered_replicator_enabled,
+                           PLUGIN_VAR_NOCMDARG,
+                           N_("Enable filtered replicator"),
+                           NULL, /* check func */
+                           NULL, /* update func */
+                           false /*default */);
+static DRIZZLE_SYSVAR_STR(filteredschemas,
+                          sysvar_filtered_replicator_sch_filters,
+                          PLUGIN_VAR_OPCMDARG,
+                          N_("List of schemas to filter"),
+                          check_filtered_schemas,
+                          set_filtered_schemas,
+                          "");
+static DRIZZLE_SYSVAR_STR(filteredtables,
+                          sysvar_filtered_replicator_tab_filters,
+                          PLUGIN_VAR_OPCMDARG,
+                          N_("List of tables to filter"),
+                          check_filtered_tables,
+                          set_filtered_tables,
+                          "");
+static DRIZZLE_SYSVAR_STR(schemaregex,
+                          sysvar_filtered_replicator_sch_regex,
+                          PLUGIN_VAR_READONLY,
+                          N_("Regular expression to apply to schemas to filter"),
+                          NULL,
+                          NULL,
+                          NULL);
+static DRIZZLE_SYSVAR_STR(tableregex,
+                          sysvar_filtered_replicator_tab_regex,
+                          PLUGIN_VAR_READONLY,
+                          N_("Regular expression to apply to tables to filter"),
+                          NULL,
+                          NULL,
+                          NULL);
 
-} /* namespace drizzle_plugin */
+static drizzle_sys_var* filtered_replicator_system_variables[]= {
+  DRIZZLE_SYSVAR(enable),
+  DRIZZLE_SYSVAR(filteredschemas),
+  DRIZZLE_SYSVAR(filteredtables),
+  DRIZZLE_SYSVAR(schemaregex),
+  DRIZZLE_SYSVAR(tableregex),
+  NULL
+};
 
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
   "filtered_replicator",
   "0.2",
-  "Padraig O Sullivan",
-  N_("Replicates events filtered by schema or table name"),
+  "Padraig O'Sullivan",
+  N_("Filtered Replicator"),
   PLUGIN_LICENSE_GPL,
-  drizzle_plugin::init,
-  NULL,
-  drizzle_plugin::init_options
+  init, /* Plugin Init */
+  deinit, /* Plugin Deinit */
+  filtered_replicator_system_variables, /* system variables */
+  NULL    /* config options */
 }
 DRIZZLE_DECLARE_PLUGIN_END;
