@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (C) 2005, 2010, Innobase Oy. All Rights Reserved.
+Copyright (c) 2005, 2009, Innobase Oy. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -11,8 +11,8 @@ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
-St, Fifth Floor, Boston, MA 02110-1301 USA
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place, Suite 330, Boston, MA 02111-1307 USA
 
 *****************************************************************************/
 
@@ -21,14 +21,15 @@ St, Fifth Floor, Boston, MA 02110-1301 USA
 Smart ALTER TABLE
 *******************************************************/
 
-#include <config.h>
+#include "config.h"
 #include <drizzled/error.h>
-#include <drizzled/charset.h>
+#include "drizzled/charset_info.h"
 #include <drizzled/field.h>
 #include <drizzled/table.h>
 #include <drizzled/field/varstring.h>
-#include <drizzled/internal/my_sys.h>
+#include "drizzled/internal/my_sys.h"
 
+extern "C" {
 #include "log0log.h"
 #include "row0merge.h"
 #include "srv0srv.h"
@@ -36,6 +37,7 @@ Smart ALTER TABLE
 #include "trx0roll.h"
 #include "ha_prototypes.h"
 #include "handler0alter.h"
+}
 
 #include "ha_innodb.h"
 #include "handler0vars.h"
@@ -101,10 +103,8 @@ innobase_col_to_mysql(
 #ifdef UNIV_DEBUG
 	case DATA_MYSQL:
 		ut_ad(flen >= len);
-		ut_ad(DATA_MBMAXLEN(col->mbminmaxlen)
-		      >= DATA_MBMINLEN(col->mbminmaxlen));
-		ut_ad(DATA_MBMAXLEN(col->mbminmaxlen)
-		      > DATA_MBMINLEN(col->mbminmaxlen) || flen == len);
+		ut_ad(col->mbmaxlen >= col->mbminlen);
+		ut_ad(col->mbmaxlen > col->mbminlen || flen == len);
 		memcpy(dest, data, len);
 		break;
 
@@ -129,8 +129,8 @@ innobase_col_to_mysql(
 }
 
 /*************************************************************//**
-Copies an InnoDB record to table->getInsertRecord(). */
-UNIV_INTERN
+Copies an InnoDB record to table->record[0]. */
+extern "C" UNIV_INTERN
 void
 innobase_rec_to_mysql(
 /*==================*/
@@ -140,13 +140,13 @@ innobase_rec_to_mysql(
 	const ulint*		offsets)	/*!< in: rec_get_offsets(
 						rec, index, ...) */
 {
-	uint	n_fields	= table->getShare()->sizeFields();
+	uint	n_fields	= table->s->fields;
 	uint	i;
 
 	ut_ad(n_fields == dict_table_get_n_user_cols(index->table));
 
 	for (i = 0; i < n_fields; i++) {
-		Field*		field	= table->getField(i);
+		Field*		field	= table->field[i];
 		ulint		ipos;
 		ulint		ilen;
 		const unsigned char*	ifield;
@@ -179,22 +179,21 @@ null_field:
 }
 
 /*************************************************************//**
-Resets table->getInsertRecord(). */
-UNIV_INTERN
+Resets table->record[0]. */
+extern "C" UNIV_INTERN
 void
 innobase_rec_reset(
 /*===============*/
 	Table*			table)		/*!< in/out: MySQL table */
 {
-	uint	n_fields	= table->getShare()->sizeFields();
+	uint	n_fields	= table->s->fields;
 	uint	i;
 
 	for (i = 0; i < n_fields; i++) {
-		table->getField(i)->set_default();
+		table->field[i]->set_default();
 	}
 }
 
-#if 0 // This is a part of the fast index code.
 /******************************************************************//**
 Removes the filename encoding of a database and table name. */
 static
@@ -222,7 +221,6 @@ innobase_convert_tablename(
 	}
 }
 
-
 /*******************************************************************//**
 This function checks that index keys are sensible.
 @return	0 or error number */
@@ -230,10 +228,9 @@ static
 int
 innobase_check_index_keys(
 /*======================*/
-	const KeyInfo*	key_info,	/*!< in: Indexes to be created */
-	ulint		num_of_keys,	/*!< in: Number of indexes to
+	const KEY*	key_info,	/*!< in: Indexes to be created */
+	ulint		num_of_keys)	/*!< in: Number of indexes to
 					be created */
-	const dict_table_t*	table)	/*!< in: Existing indexes */
 {
 	ulint		key_num;
 
@@ -241,31 +238,18 @@ innobase_check_index_keys(
 	ut_ad(num_of_keys);
 
 	for (key_num = 0; key_num < num_of_keys; key_num++) {
-		const KeyInfo&	key = key_info[key_num];
+		const KEY&	key = key_info[key_num];
 
 		/* Check that the same index name does not appear
 		twice in indexes to be created. */
 
 		for (ulint i = 0; i < key_num; i++) {
-			const KeyInfo&	key2 = key_info[i];
+			const KEY&	key2 = key_info[i];
 
 			if (0 == strcmp(key.name, key2.name)) {
-				my_error(ER_WRONG_NAME_FOR_INDEX, MYF(0),
-					 key.name);
-
-				return(ER_WRONG_NAME_FOR_INDEX);
-			}
-		}
-
-		/* Check that the same index name does not already exist. */
-
-		for (const dict_index_t* index
-			     = dict_table_get_first_index(table);
-		     index; index = dict_table_get_next_index(index)) {
-
-			if (0 == strcmp(key.name, index->name)) {
-				my_error(ER_WRONG_NAME_FOR_INDEX, MYF(0),
-					 key.name);
+				errmsg_printf(ERRMSG_LVL_ERROR, "InnoDB: key name `%s` appears"
+						" twice in CREATE INDEX\n",
+						key.name);
 
 				return(ER_WRONG_NAME_FOR_INDEX);
 			}
@@ -273,10 +257,10 @@ innobase_check_index_keys(
 
 		/* Check that MySQL does not try to create a column
 		prefix index field on an inappropriate data type and
-		that the same column does not appear twice in the index. */
+		that the same colum does not appear twice in the index. */
 
 		for (ulint i = 0; i < key.key_parts; i++) {
-			const KeyPartInfo&	key_part1
+			const KEY_PART_INFO&	key_part1
 				= key.key_part[i];
 			const Field*		field
 				= key_part1.field;
@@ -304,13 +288,19 @@ innobase_check_index_keys(
 					}
 				}
 
-				my_error(ER_WRONG_KEY_COLUMN, MYF(0),
-					 field->field_name);
+				errmsg_printf(ERRMSG_LVL_ERROR, "InnoDB: MySQL is trying to"
+						" create a column prefix"
+						" index field on an"
+						" inappropriate data type."
+						" column `%s`,"
+						" index `%s`.\n",
+						field->field_name,
+						key.name);
 				return(ER_WRONG_KEY_COLUMN);
 			}
 
 			for (ulint j = 0; j < i; j++) {
-				const KeyPartInfo&	key_part2
+				const KEY_PART_INFO&	key_part2
 					= key.key_part[j];
 
 				if (strcmp(key_part1.field->field_name,
@@ -318,8 +308,11 @@ innobase_check_index_keys(
 					continue;
 				}
 
-				my_error(ER_WRONG_KEY_COLUMN, MYF(0),
-					 key_part1.field->field_name);
+				errmsg_printf(ERRMSG_LVL_ERROR, "InnoDB: column `%s`"
+						" is not allowed to occur"
+						" twice in index `%s`.\n",
+						key_part1.field->field_name,
+						key.name);
 				return(ER_WRONG_KEY_COLUMN);
 			}
 		}
@@ -334,7 +327,7 @@ static
 void
 innobase_create_index_field_def(
 /*============================*/
-	KeyPartInfo*		key_part,	/*!< in: MySQL key definition */
+	KEY_PART_INFO*		key_part,	/*!< in: MySQL key definition */
 	mem_heap_t*		heap,		/*!< in: memory heap */
 	merge_index_field_t*	index_field)	/*!< out: index field
 						definition for key_part */
@@ -374,7 +367,7 @@ static
 void
 innobase_create_index_def(
 /*======================*/
-	KeyInfo*			key,		/*!< in: key definition */
+	KEY*			key,		/*!< in: key definition */
 	bool			new_primary,	/*!< in: TRUE=generating
 						a new primary key
 						on the table */
@@ -491,7 +484,6 @@ ENDIF
 
 
 @return	key definitions or NULL */
-
 static
 merge_index_def_t*
 innobase_create_key_def(
@@ -500,7 +492,7 @@ innobase_create_key_def(
 	const dict_table_t*table,		/*!< in: table definition */
 	mem_heap_t*	heap,		/*!< in: heap where space for key
 					definitions are allocated */
-	KeyInfo*		key_info,	/*!< in: Indexes to be created */
+	KEY*		key_info,	/*!< in: Indexes to be created */
 	ulint&		n_keys)		/*!< in/out: Number of indexes to
 					be created */
 {
@@ -516,17 +508,16 @@ innobase_create_key_def(
 	/* If there is a primary key, it is always the first index
 	defined for the table. */
 
-	new_primary = !system_charset_info->strcasecmp(key_info->name, "PRIMARY");
+	new_primary = !my_strcasecmp(system_charset_info,
+				     key_info->name, "PRIMARY");
 
 	/* If there is a UNIQUE INDEX consisting entirely of NOT NULL
-	columns and if the index does not contain column prefix(es)
-	(only prefix/part of the column is indexed), MySQL will treat the
-	index as a PRIMARY KEY unless the table already has one. */
+	columns, MySQL will treat it as a PRIMARY KEY unless the
+	table already has one. */
 
 	if (!new_primary && (key_info->flags & HA_NOSAME)
-	    && (!(key_info->flags & HA_KEY_HAS_PART_KEY_SEG))
 	    && row_table_got_default_clust_index(table)) {
-		uint    key_part = key_info->key_parts;
+		uint	key_part = key_info->key_parts;
 
 		new_primary = TRUE;
 
@@ -555,8 +546,8 @@ innobase_create_key_def(
 		UNIQUE INDEX, it must be converted to a secondary index. */
 
 		if (dict_index_get_nth_col(index, 0)->mtype == DATA_SYS
-		    || !system_charset_info->strcasecmp(index->name, "PRIMARY")) 
-    {
+		    || !my_strcasecmp(system_charset_info,
+				      index->name, "PRIMARY")) {
 			index = dict_table_get_next_index(index);
 		}
 
@@ -605,7 +596,6 @@ innobase_create_temporary_tablename(
 	return(name);
 }
 
-
 /*******************************************************************//**
 Create indexes.
 @return	0 or error number */
@@ -613,9 +603,8 @@ UNIV_INTERN
 int
 ha_innobase::add_index(
 /*===================*/
-                       Session *session,
 	Table*	i_table,	/*!< in: Table where indexes are created */
-	KeyInfo*	key_info,	/*!< in: Indexes to be created */
+	KEY*	key_info,	/*!< in: Indexes to be created */
 	uint	num_of_keys)	/*!< in: Number of indexes to be created */
 {
 	dict_index_t**	index;		/*!< Index to be created */
@@ -628,7 +617,7 @@ ha_innobase::add_index(
 	ulint		num_created	= 0;
 	ibool		dict_locked	= FALSE;
 	ulint		new_primary;
-	int		error;
+	ulint		error;
 
 	ut_a(i_table);
 	ut_a(key_info);
@@ -638,7 +627,7 @@ ha_innobase::add_index(
 		return(HA_ERR_WRONG_COMMAND);
 	}
 
-	update_session(session);
+	update_session();
 
 	heap = mem_heap_create(1024);
 
@@ -655,24 +644,14 @@ ha_innobase::add_index(
 	innodb_table = indexed_table
 		= dict_table_get(prebuilt->table->name, FALSE);
 
-	if (UNIV_UNLIKELY(!innodb_table)) {
-		error = HA_ERR_NO_SUCH_TABLE;
-		goto err_exit;
-	}
+	/* Check that index keys are sensible */
 
-	/* Check if the index name is reserved. */
-	if (innobase_index_name_is_reserved(trx, key_info, num_of_keys)) {
-		error = -1;
-	} else {
-		/* Check that index keys are sensible */
-		error = innobase_check_index_keys(key_info, num_of_keys,
-						  innodb_table);
-	}
+	error = innobase_check_index_keys(key_info, num_of_keys);
 
 	if (UNIV_UNLIKELY(error)) {
 err_exit:
 		mem_heap_free(heap);
-		trx_general_rollback_for_mysql(trx, NULL);
+		trx_general_rollback_for_mysql(trx, FALSE, NULL);
 		trx_free_for_mysql(trx);
 		trx_commit_for_mysql(prebuilt->trx);
 		return(error);
@@ -713,8 +692,6 @@ err_exit:
 	row_mysql_lock_data_dictionary(trx);
 	dict_locked = TRUE;
 
-	ut_d(dict_table_check_for_dup_indexes(innodb_table, FALSE));
-
 	/* If a new primary key is defined for the table we need
 	to drop the original table and rebuild all indexes. */
 
@@ -747,8 +724,6 @@ err_exit:
 					user_session);
 			}
 
-			ut_d(dict_table_check_for_dup_indexes(innodb_table,
-							      FALSE));
 			row_mysql_unlock_data_dictionary(trx);
 			goto err_exit;
 		}
@@ -774,11 +749,10 @@ err_exit:
 	ut_ad(error == DB_SUCCESS);
 
 	/* Commit the data dictionary transaction in order to release
-	the table locks on the system tables.  This means that if
-	MySQL crashes while creating a new primary key inside
-	row_merge_build_indexes(), indexed_table will not be dropped
-	by trx_rollback_active().  It will have to be recovered or
-	dropped by the database administrator. */
+	the table locks on the system tables.  Unfortunately, this
+	means that if MySQL crashes while creating a new primary key
+	inside row_merge_build_indexes(), indexed_table will not be
+	dropped on crash recovery.  Thus, it will become orphaned. */
 	trx_commit_for_mysql(trx);
 
 	row_mysql_unlock_data_dictionary(trx);
@@ -808,6 +782,17 @@ err_exit:
 					index, num_of_idx, i_table);
 
 error_handling:
+#ifdef UNIV_DEBUG
+	/* TODO: At the moment we can't handle the following statement
+	in our debugging code below:
+
+	alter table t drop index b, add index (b);
+
+	The fix will have to parse the SQL and note that the index
+	being added has the same name as the the one being dropped and
+	ignore that in the dup index check.*/
+	//dict_table_check_for_dup_indexes(prebuilt->table);
+#endif
 
 	/* After an error, remove all those index definitions from the
 	dictionary which were defined. */
@@ -820,8 +805,6 @@ error_handling:
 		row_mysql_lock_data_dictionary(trx);
 		dict_locked = TRUE;
 
-		ut_d(dict_table_check_for_dup_indexes(prebuilt->table, TRUE));
-
 		if (!new_primary) {
 			error = row_merge_rename_indexes(trx, indexed_table);
 
@@ -868,7 +851,6 @@ error_handling:
 		indexed_table->n_mysql_handles_opened++;
 
 		error = row_merge_drop_table(trx, innodb_table);
-		innodb_table = indexed_table;
 		goto convert_error;
 
 	case DB_TOO_BIG_RECORD:
@@ -882,12 +864,8 @@ error:
 		prebuilt->trx->error_info = NULL;
 		/* fall through */
 	default:
-		trx->error_state = DB_SUCCESS;
-
 		if (new_primary) {
-			if (indexed_table != innodb_table) {
-				row_merge_drop_table(trx, indexed_table);
-			}
+			row_merge_drop_table(trx, indexed_table);
 		} else {
 			if (!dict_locked) {
 				row_mysql_lock_data_dictionary(trx);
@@ -899,14 +877,6 @@ error:
 		}
 
 convert_error:
-		if (error == DB_SUCCESS) {
-			/* Build index is successful. We will need to
-			rebuild index translation table.  Reset the
-			index entry count in the translation table
-			to zero, so that translation table will be rebuilt */
-			share->idx_trans_tbl.index_count = 0;
-		}
-
 		error = convert_error_code_to_mysql(error,
 						    innodb_table->flags,
 						    user_session);
@@ -919,7 +889,6 @@ convert_error:
 	}
 
 	if (dict_locked) {
-		ut_d(dict_table_check_for_dup_indexes(innodb_table, FALSE));
 		row_mysql_unlock_data_dictionary(trx);
 	}
 
@@ -938,7 +907,6 @@ UNIV_INTERN
 int
 ha_innobase::prepare_drop_index(
 /*============================*/
-                                Session *session,
 	Table*	i_table,	/*!< in: Table where indexes are dropped */
 	uint*	key_num,	/*!< in: Key nums to be dropped */
 	uint	num_of_keys)	/*!< in: Number of keys to be dropped */
@@ -954,7 +922,7 @@ ha_innobase::prepare_drop_index(
 		return(HA_ERR_WRONG_COMMAND);
 	}
 
-	update_session(session);
+	update_session();
 
 	trx_search_latch_release_if_reserved(prebuilt->trx);
 	trx = prebuilt->trx;
@@ -962,7 +930,6 @@ ha_innobase::prepare_drop_index(
 	/* Test and mark all the indexes to be dropped */
 
 	row_mysql_lock_data_dictionary(trx);
-	ut_d(dict_table_check_for_dup_indexes(prebuilt->table, FALSE));
 
 	/* Check that none of the indexes have previously been flagged
 	for deletion. */
@@ -976,7 +943,7 @@ ha_innobase::prepare_drop_index(
 	}
 
 	for (n_key = 0; n_key < num_of_keys; n_key++) {
-		const KeyInfo*	key;
+		const KEY*	key;
 		dict_index_t*	index;
 
 		key = i_table->key_info + key_num[n_key];
@@ -1008,13 +975,12 @@ ha_innobase::prepare_drop_index(
 		index->to_be_dropped = TRUE;
 	}
 
-	/* If FOREIGN_KEY_CHECKS = 1 you may not drop an index defined
+	/* If FOREIGN_KEY_CHECK = 1 you may not drop an index defined
 	for a foreign key constraint because InnoDB requires that both
-	tables contain indexes for the constraint. Such index can
-	be dropped only if FOREIGN_KEY_CHECKS is set to 0.
-	Note that CREATE INDEX id ON table does a CREATE INDEX and
-	DROP INDEX, and we can ignore here foreign keys because a
-	new index for the foreign key has already been created.
+	tables contain indexes for the constraint.  Note that CREATE
+	INDEX id ON table does a CREATE INDEX and DROP INDEX, and we
+	can ignore here foreign keys because a new index for the
+	foreign key has already been created.
 
 	We check for the foreign key constraints after marking the
 	candidate indexes for deletion, because when we check for an
@@ -1129,7 +1095,6 @@ func_exit:
 		} while (index);
 	}
 
-	ut_d(dict_table_check_for_dup_indexes(prebuilt->table, FALSE));
 	row_mysql_unlock_data_dictionary(trx);
 
 	return(err);
@@ -1142,7 +1107,6 @@ UNIV_INTERN
 int
 ha_innobase::final_drop_index(
 /*==========================*/
-                              Session *session,
 	Table*	)		/*!< in: Table where indexes are dropped */
 {
 	dict_index_t*	index;		/*!< Index to be dropped */
@@ -1153,7 +1117,7 @@ ha_innobase::final_drop_index(
 		return(HA_ERR_WRONG_COMMAND);
 	}
 
-	update_session(session);
+	update_session();
 
 	trx_search_latch_release_if_reserved(prebuilt->trx);
 	trx_start_if_not_started(prebuilt->trx);
@@ -1174,7 +1138,6 @@ ha_innobase::final_drop_index(
 		prebuilt->table->flags, user_session);
 
 	row_mysql_lock_data_dictionary(trx);
-	ut_d(dict_table_check_for_dup_indexes(prebuilt->table, FALSE));
 
 	if (UNIV_UNLIKELY(err)) {
 
@@ -1211,12 +1174,11 @@ ha_innobase::final_drop_index(
 		ut_a(!index->to_be_dropped);
 	}
 
-	/* We will need to rebuild index translation table. Set
-	valid index entry count in the translation table to zero */
-	share->idx_trans_tbl.index_count = 0;
+#ifdef UNIV_DEBUG
+	dict_table_check_for_dup_indexes(prebuilt->table);
+#endif
 
 func_exit:
-	ut_d(dict_table_check_for_dup_indexes(prebuilt->table, FALSE));
 	trx_commit_for_mysql(trx);
 	trx_commit_for_mysql(prebuilt->trx);
 	row_mysql_unlock_data_dictionary(trx);
@@ -1236,4 +1198,3 @@ func_exit:
 
 	return(err);
 }
-#endif
