@@ -18,9 +18,10 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INNOBASE_HANDLER_STATUS_FUNCTION_H
+#define PLUGIN_INNOBASE_HANDLER_STATUS_FUNCTION_H
 
-#include <drizzled/plugin/table_function.h>
+#include "drizzled/plugin/table_function.h"
 
 class InnodbStatusTool : public drizzled::plugin::TableFunction
 {
@@ -48,3 +49,4 @@ public:
   }
 };
 
+#endif /* PLUGIN_INNOBASE_HANDLER_STATUS_FUNCTION_H */
