@@ -11,17 +11,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* Create a MyISAM table */
 
 #include "myisam_priv.h"
-#include <drizzled/internal/my_bit.h>
-#include <drizzled/internal/my_sys.h>
+#include "drizzled/internal/my_bit.h"
+#include "drizzled/internal/my_sys.h"
 
-#include <drizzled/util/test.h>
-#include <drizzled/charset.h>
-#include <drizzled/error.h>
+#include "drizzled/util/test.h"
+#include "drizzled/global_charset_info.h"
+#include "drizzled/error.h"
 
 #include <cassert>
 #include <algorithm>
@@ -29,36 +29,6 @@
 using namespace std;
 using namespace drizzled;
 
-/*
-  Next highest power of two
-
-  SYNOPSIS
-    my_round_up_to_next_power()
-    v		Value to check
-
-  RETURN
-    Next or equal power of 2
-    Note: 0 will return 0
-
-  NOTES
-    Algorithm by Sean Anderson, according to:
-    http://graphics.stanford.edu/~seander/bithacks.html
-    (Orignal code public domain)
-
-    Comments shows how this works with 01100000000000000000000000001011
-*/
-
-static inline uint32_t my_round_up_to_next_power(uint32_t v)
-{
-  v--;			/* 01100000000000000000000000001010 */
-  v|= v >> 1;		/* 01110000000000000000000000001111 */
-  v|= v >> 2;		/* 01111100000000000000000000001111 */
-  v|= v >> 4;		/* 01111111110000000000000000001111 */
-  v|= v >> 8;		/* 01111111111111111100000000001111 */
-  v|= v >> 16;		/* 01111111111111111111111111111111 */
-  return v+1;		/* 10000000000000000000000000000000 */
-}
-
 /*
   Old options is used when recreating database, from myisamchk
 */
@@ -295,6 +265,7 @@ int mi_create(const char *name,uint32_t keys,MI_KEYDEF *keydefs,
 	case HA_KEYTYPE_ULONG_INT:
 	case HA_KEYTYPE_LONGLONG:
 	case HA_KEYTYPE_ULONGLONG:
+	case HA_KEYTYPE_UINT24:
 	  keyseg->flag|= HA_SWAP_KEY;
           break;
         case HA_KEYTYPE_VARTEXT1:
@@ -422,7 +393,7 @@ int mi_create(const char *name,uint32_t keys,MI_KEYDEF *keydefs,
   /* There are only 16 bits for the total header length. */
   if (info_length > 65535)
   {
-    my_printf_error(EE_OK, "MyISAM table '%s' has too many columns and/or "
+    my_printf_error(0, "MyISAM table '%s' has too many columns and/or "
                     "indexes and/or unique constraints.",
                     MYF(0), name + internal::dirname_length(name));
     errno= HA_WRONG_CREATE_OPTION;
@@ -497,7 +468,7 @@ int mi_create(const char *name,uint32_t keys,MI_KEYDEF *keydefs,
   if (! (flags & HA_DONT_TOUCH_DATA))
     share.state.create_time= (long) time((time_t*) 0);
 
-  THR_LOCK_myisam.lock();
+  pthread_mutex_lock(&THR_LOCK_myisam);
 
   /*
     NOTE: For test_if_reopen() we need a real path name. Hence we need
@@ -556,7 +527,7 @@ int mi_create(const char *name,uint32_t keys,MI_KEYDEF *keydefs,
   */
   if (test_if_reopen(filename))
   {
-    my_printf_error(EE_OK, "MyISAM table '%s' is in use "
+    my_printf_error(0, "MyISAM table '%s' is in use "
                     "(most likely by a MERGE table). Try FLUSH TABLES.",
                     MYF(0), name + internal::dirname_length(name));
     errno= HA_ERR_TABLE_EXIST;
@@ -700,14 +671,14 @@ int mi_create(const char *name,uint32_t keys,MI_KEYDEF *keydefs,
       goto err;
   }
   errpos=0;
-  THR_LOCK_myisam.unlock();
+  pthread_mutex_unlock(&THR_LOCK_myisam);
   if (internal::my_close(file,MYF(0)))
     goto err;
   free((char*) rec_per_key_part);
   return(0);
 
 err:
-  THR_LOCK_myisam.unlock();
+  pthread_mutex_unlock(&THR_LOCK_myisam);
   save_errno=errno;
   switch (errpos) {
   case 3:
