@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
   Functions to handle space-packed-records and blobs
@@ -32,7 +32,7 @@
 #include <sys/mman.h>
 #endif
 #include <drizzled/util/test.h>
-#include <drizzled/error.h>
+#include "drizzled/error.h"
 
 #include <cassert>
 #include <algorithm>
@@ -142,6 +142,9 @@ void mi_remap_file(MI_INFO *info, internal::my_off_t size)
 size_t mi_mmap_pread(MI_INFO *info, unsigned char *Buffer,
                     size_t Count, internal::my_off_t offset, myf MyFlags)
 {
+  if (info->s->concurrent_insert)
+    pthread_rwlock_rdlock(&info->s->mmap_lock);
+
   /*
     The following test may fail in the following cases:
     - We failed to remap a memory area (fragmented memory?)
@@ -152,10 +155,14 @@ size_t mi_mmap_pread(MI_INFO *info, unsigned char *Buffer,
   if (info->s->mmaped_length >= offset + Count)
   {
     memcpy(Buffer, info->s->file_map + offset, Count);
+    if (info->s->concurrent_insert)
+      pthread_rwlock_unlock(&info->s->mmap_lock);
     return 0;
   }
   else
   {
+    if (info->s->concurrent_insert)
+      pthread_rwlock_unlock(&info->s->mmap_lock);
     return my_pread(info->dfile, Buffer, Count, offset, MyFlags);
   }
 }
@@ -189,6 +196,8 @@ size_t mi_nommap_pread(MI_INFO *info, unsigned char *Buffer,
 size_t mi_mmap_pwrite(MI_INFO *info, const unsigned char *Buffer,
                       size_t Count, internal::my_off_t offset, myf MyFlags)
 {
+  if (info->s->concurrent_insert)
+    pthread_rwlock_rdlock(&info->s->mmap_lock);
 
   /*
     The following test may fail in the following cases:
@@ -200,11 +209,15 @@ size_t mi_mmap_pwrite(MI_INFO *info, const unsigned char *Buffer,
   if (info->s->mmaped_length >= offset + Count)
   {
     memcpy(info->s->file_map + offset, Buffer, Count);
+    if (info->s->concurrent_insert)
+      pthread_rwlock_unlock(&info->s->mmap_lock);
     return 0;
   }
   else
   {
     info->s->nonmmaped_inserts++;
+    if (info->s->concurrent_insert)
+      pthread_rwlock_unlock(&info->s->mmap_lock);
     return my_pwrite(info->dfile, Buffer, Count, offset, MyFlags);
   }
 
@@ -717,11 +730,12 @@ int _mi_write_part_record(MI_INFO *info,
     if (info->update & HA_STATE_EXTEND_BLOCK)
     {
       info->update&= ~HA_STATE_EXTEND_BLOCK;
-      if (info->rec_cache.block_write(*record - head_length,
+      if (my_block_write(&info->rec_cache,(unsigned char*) *record-head_length,
 			 length+extra_length+del_length,filepos))
       goto err;
     }
-    else if (info->rec_cache.write(*record-head_length, length+extra_length+del_length))
+    else if (my_b_write(&info->rec_cache,(unsigned char*) *record-head_length,
+			length+extra_length+del_length))
       goto err;
   }
   else
@@ -996,7 +1010,7 @@ uint32_t _mi_rec_pack(MI_INFO *info, register unsigned char *to,
       }
       else if (type == FIELD_VARCHAR)
       {
-        uint32_t pack_length= ha_varchar_packlength(rec->length -1);
+        uint32_t pack_length= HA_VARCHAR_PACKLENGTH(rec->length -1);
 	uint32_t tmp_length;
         if (pack_length == 1)
         {
@@ -1116,7 +1130,7 @@ bool _mi_rec_check(MI_INFO *info,const unsigned char *record, unsigned char *rec
       }
       else if (type == FIELD_VARCHAR)
       {
-        uint32_t pack_length= ha_varchar_packlength(rec->length -1);
+        uint32_t pack_length= HA_VARCHAR_PACKLENGTH(rec->length -1);
 	uint32_t tmp_length;
         if (pack_length == 1)
         {
@@ -1189,7 +1203,7 @@ ulong _mi_rec_unpack(register MI_INFO *info, register unsigned char *to, unsigne
     {
       if (type == FIELD_VARCHAR)
       {
-        uint32_t pack_length= ha_varchar_packlength(rec_length-1);
+        uint32_t pack_length= HA_VARCHAR_PACKLENGTH(rec_length-1);
         if (pack_length == 1)
         {
           length= (uint) *(unsigned char*) from;
@@ -1404,7 +1418,7 @@ int _mi_read_dynamic_record(MI_INFO *info, internal::my_off_t filepos, unsigned
         goto panic;
       if (info->opt_flag & WRITE_CACHE_USED &&
 	  info->rec_cache.pos_in_file < filepos + MI_BLOCK_INFO_HEADER_LENGTH &&
-	  info->rec_cache.flush())
+	  flush_io_cache(&info->rec_cache))
 	goto err;
       info->rec_cache.seek_not_done=1;
       if ((b_type= _mi_get_block_info(&block_info, file, filepos))
@@ -1452,7 +1466,7 @@ int _mi_read_dynamic_record(MI_INFO *info, internal::my_off_t filepos, unsigned
       {
         if (info->opt_flag & WRITE_CACHE_USED &&
             info->rec_cache.pos_in_file < filepos + block_info.data_len &&
-            info->rec_cache.flush())
+            flush_io_cache(&info->rec_cache))
           goto err;
         /*
           What a pity that this method is not called 'file_pread' and that
@@ -1504,7 +1518,8 @@ int _mi_cmp_dynamic_unique(MI_INFO *info, MI_UNIQUEDEF *def,
   if (info->s->base.blobs)
   {
     void * rec_buff_ptr= mi_get_rec_buff_ptr(info, info->rec_buff);
-    free(rec_buff_ptr);
+    if (rec_buff_ptr != NULL)
+      free(rec_buff_ptr);
     info->rec_buff=rec_buff;
   }
   free(old_record);
@@ -1524,7 +1539,7 @@ int _mi_cmp_dynamic_record(register MI_INFO *info, register const unsigned char
   if (info->opt_flag & WRITE_CACHE_USED)
   {
     info->update&= ~(HA_STATE_WRITE_AT_END | HA_STATE_EXTEND_BLOCK);
-    if (info->rec_cache.flush())
+    if (flush_io_cache(&info->rec_cache))
       return(-1);
   }
   info->rec_cache.seek_not_done=1;
@@ -1703,7 +1718,7 @@ int _mi_read_rnd_dynamic_record(MI_INFO *info, unsigned char *buf,
     {
       if (info->opt_flag & WRITE_CACHE_USED &&
 	  info->rec_cache.pos_in_file < filepos + MI_BLOCK_INFO_HEADER_LENGTH &&
-	  info->rec_cache.flush())
+	  flush_io_cache(&info->rec_cache))
 	return(errno);
       info->rec_cache.seek_not_done=1;
       b_type=_mi_get_block_info(&block_info,info->dfile,filepos);
@@ -1778,7 +1793,7 @@ int _mi_read_rnd_dynamic_record(MI_INFO *info, unsigned char *buf,
         if (info->opt_flag & WRITE_CACHE_USED &&
             info->rec_cache.pos_in_file <
             block_info.filepos + block_info.data_len &&
-            info->rec_cache.flush())
+            flush_io_cache(&info->rec_cache))
           goto err;
 	/* lseek(info->dfile,filepos,SEEK_SET); */
 	if (internal::my_read(info->dfile,(unsigned char*) to,block_info.data_len,MYF(MY_NABP)))
