@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* open a isam-database */
 
@@ -19,15 +19,13 @@
 
 #include <string.h>
 #include <algorithm>
-#include <memory>
-#include <boost/scoped_ptr.hpp>
-#include <boost/scoped_array.hpp>
 
-#include <drizzled/internal/m_string.h>
-#include <drizzled/util/test.h>
-#include <drizzled/charset.h>
-#include <drizzled/memory/multi_malloc.h>
-#include <drizzled/identifier.h>
+#include "drizzled/charset_info.h"
+#include "drizzled/internal/m_string.h"
+#include "drizzled/util/test.h"
+#include "drizzled/global_charset_info.h"
+#include "drizzled/charset.h"
+#include "drizzled/memory/multi_malloc.h"
 
 
 using namespace std;
@@ -78,21 +76,18 @@ MI_INFO *test_if_reopen(char *filename)
   have an open count of 0.
 ******************************************************************************/
 
-MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32_t open_flags)
+MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
 {
   int lock_error,kfile,open_mode,save_errno,have_rtree=0;
   uint32_t i,j,len,errpos,head_length,base_pos,offset,info_length,keys,
-    key_parts,unique_key_parts,uniques;
+    key_parts,unique_key_parts,fulltext_keys,uniques;
   char name_buff[FN_REFLEN], org_name[FN_REFLEN], index_name[FN_REFLEN],
        data_name[FN_REFLEN], rp_buff[PATH_MAX];
   unsigned char *disk_cache= NULL;
   unsigned char *disk_pos, *end_pos;
   MI_INFO info,*m_info,*old_info;
-  boost::scoped_ptr<MYISAM_SHARE> share_buff_ap(new MYISAM_SHARE);
-  MYISAM_SHARE &share_buff= *share_buff_ap.get();
-  MYISAM_SHARE *share;
-  boost::scoped_array<ulong> rec_per_key_part_ap(new ulong[HA_MAX_POSSIBLE_KEY*MI_MAX_KEY_SEG]);
-  ulong *rec_per_key_part= rec_per_key_part_ap.get();
+  MYISAM_SHARE share_buff,*share;
+  ulong rec_per_key_part[HA_MAX_POSSIBLE_KEY*MI_MAX_KEY_SEG];
   internal::my_off_t key_root[HA_MAX_POSSIBLE_KEY],key_del[MI_MAX_KEY_BLOCK_SIZE];
   uint64_t max_key_file_length, max_data_file_length;
 
@@ -102,16 +97,12 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
   head_length=sizeof(share_buff.state.header);
   memset(&info, 0, sizeof(info));
 
-  (void)internal::fn_format(org_name,
-                            identifier.getPath().c_str(), 
-                            "",
-                            MI_NAME_IEXT,
-                            MY_UNPACK_FILENAME);
+  (void)internal::fn_format(org_name,name,"",MI_NAME_IEXT, MY_UNPACK_FILENAME);
   if (!realpath(org_name,rp_buff))
     internal::my_load_path(rp_buff,org_name, NULL);
   rp_buff[FN_REFLEN-1]= '\0';
   strcpy(name_buff,rp_buff);
-  THR_LOCK_myisam.lock();
+  pthread_mutex_lock(&THR_LOCK_myisam);
   if (!(old_info=test_if_reopen(name_buff)))
   {
     share= &share_buff;
@@ -119,6 +110,7 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
     share_buff.state.rec_per_key_part=rec_per_key_part;
     share_buff.state.key_root=key_root;
     share_buff.state.key_del=key_del;
+    share_buff.key_cache= dflt_key_cache;
 
     if ((kfile=internal::my_open(name_buff,(open_mode=O_RDWR),MYF(0))) < 0)
     {
@@ -141,12 +133,12 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
       goto err;
     }
     share->options= mi_uint2korr(share->state.header.options);
-    static const uint64_t OLD_FILE_OPTIONS= HA_OPTION_PACK_RECORD |
-	    HA_OPTION_PACK_KEYS |
-	    HA_OPTION_COMPRESS_RECORD | HA_OPTION_READ_ONLY_DATA |
-	    HA_OPTION_TEMP_COMPRESS_RECORD |
-	    HA_OPTION_TMP_TABLE;
-    if (share->options & ~OLD_FILE_OPTIONS)
+    if (share->options &
+	~(HA_OPTION_PACK_RECORD | HA_OPTION_PACK_KEYS |
+	  HA_OPTION_COMPRESS_RECORD | HA_OPTION_READ_ONLY_DATA |
+	  HA_OPTION_TEMP_COMPRESS_RECORD |
+          HA_OPTION_TMP_TABLE
+          ))
     {
       errno=HA_ERR_OLD_FILE;
       goto err;
@@ -182,6 +174,7 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
     len=mi_uint2korr(share->state.header.state_info_length);
     keys=    (uint) share->state.header.keys;
     uniques= (uint) share->state.header.uniques;
+    fulltext_keys= (uint) share->state.header.fulltext_keys;
     key_parts= mi_uint2korr(share->state.header.key_parts);
     unique_key_parts= mi_uint2korr(share->state.header.unique_key_parts);
     share->state_diff_length=len-MI_STATE_INFO_SIZE;
@@ -254,6 +247,8 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
            &share->state.key_root,keys*sizeof(uint64_t),
            &share->state.key_del,
            (share->state.header.max_block_size_index*sizeof(uint64_t)),
+           &share->key_root_lock,sizeof(pthread_rwlock_t)*keys,
+           &share->mmap_lock,sizeof(pthread_rwlock_t),
            NULL))
       goto err;
     errpos=4;
@@ -394,7 +389,17 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
     disk_cache= NULL;
     mi_setup_functions(share);
     share->is_log_table= false;
-    if (myisam_concurrent_insert)
+    thr_lock_init(&share->lock);
+    pthread_mutex_init(&share->intern_lock,MY_MUTEX_INIT_FAST);
+    for (i=0; i<keys; i++)
+      pthread_rwlock_init(&share->key_root_lock[i], NULL);
+    pthread_rwlock_init(&share->mmap_lock, NULL);
+    if (!thr_lock_inited)
+    {
+      /* Probably a single threaded program; Don't use concurrent inserts */
+      myisam_concurrent_insert=0;
+    }
+    else if (myisam_concurrent_insert)
     {
       share->concurrent_insert=
 	((share->options & (HA_OPTION_READ_ONLY_DATA | HA_OPTION_TMP_TABLE |
@@ -403,7 +408,11 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
 	 (open_flags & HA_OPEN_TMP_TABLE) || have_rtree) ? 0 : 1;
       if (share->concurrent_insert)
       {
-        assert(0);
+	share->lock.get_status= mi_get_status;
+	share->lock.copy_status= mi_copy_status;
+	share->lock.update_status= mi_update_status;
+        share->lock.restore_status= mi_restore_status;
+	share->lock.check_status= mi_check_status;
       }
     }
   }
@@ -429,7 +438,7 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
                      share->base.max_key_length),
          &info.lastkey,share->base.max_key_length*3+1,
          &info.first_mbr_key, share->base.max_key_length,
-         &info.filename, identifier.getPath().length()+1,
+         &info.filename,strlen(name)+1,
          &info.rtree_recursion_state,have_rtree ? 1024 : 0,
          NULL))
     goto err;
@@ -438,7 +447,7 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
   if (!have_rtree)
     info.rtree_recursion_state= NULL;
 
-  strcpy(info.filename, identifier.getPath().c_str());
+  strcpy(info.filename,name);
   memcpy(info.blobs,share->blobs,sizeof(MI_BLOB)*share->base.blobs);
   info.lastkey2=info.lastkey+share->base.max_key_length;
 
@@ -459,6 +468,7 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
   info.bulk_insert=0;
   info.errkey= -1;
   info.page_changed=1;
+  pthread_mutex_lock(&share->intern_lock);
   info.read_record=share->read_record;
   share->reopen++;
   share->write_flag=MYF(MY_NABP | MY_WAIT_IF_FULL);
@@ -488,27 +498,30 @@ MI_INFO *mi_open(const drizzled::identifier::Table &identifier, int mode, uint32
 
   share->delay_key_write= 1;
   info.state= &share->state.state;	/* Change global values by default */
+  pthread_mutex_unlock(&share->intern_lock);
 
   /* Allocate buffer for one record */
 
   /* prerequisites: memset(info, 0) && info->s=share; are met. */
-  if (!mi_alloc_rec_buff(&info, SIZE_MAX, &info.rec_buff))
+  if (!mi_alloc_rec_buff(&info, -1, &info.rec_buff))
     goto err;
   memset(info.rec_buff, 0, mi_get_rec_buff_len(&info, info.rec_buff));
 
   *m_info=info;
+  thr_lock_data_init(&share->lock,&m_info->lock,(void*) m_info);
   myisam_open_list.push_front(m_info);
 
-  THR_LOCK_myisam.unlock();
+  pthread_mutex_unlock(&THR_LOCK_myisam);
   return(m_info);
 
 err:
-  free(disk_cache);
+  if (disk_cache != NULL)
+    free(disk_cache);
   save_errno=errno ? errno : HA_ERR_END_OF_FILE;
   if ((save_errno == HA_ERR_CRASHED) ||
       (save_errno == HA_ERR_CRASHED_ON_USAGE) ||
       (save_errno == HA_ERR_CRASHED_ON_REPAIR))
-    mi_report_error(save_errno, identifier.getPath().c_str());
+    mi_report_error(save_errno, name);
   switch (errpos) {
   case 6:
     free((unsigned char*) m_info);
@@ -530,7 +543,7 @@ err:
   default:
     break;
   }
-  THR_LOCK_myisam.unlock();
+  pthread_mutex_unlock(&THR_LOCK_myisam);
   errno=save_errno;
   return (NULL);
 } /* mi_open */
@@ -546,7 +559,7 @@ unsigned char *mi_alloc_rec_buff(MI_INFO *info, size_t length, unsigned char **b
     unsigned char *newptr = *buf;
 
     /* to simplify initial init of info->rec_buf in mi_open and mi_extra */
-    if (length == SIZE_MAX)
+    if (length == (ulong) -1)
     {
       if (info->s->options & HA_OPTION_COMPRESS_RECORD)
         length= max(info->s->base.pack_reclength, info->s->max_pack_length);
@@ -652,7 +665,7 @@ static void setup_key_functions(register MI_KEYDEF *keyinfo)
         cannot represent blank like ASCII does. In these cases we have
         to use _mi_seq_search() for the search.
       */
-      if (not keyinfo->seg->charset || keyinfo->seg->charset->use_strnxfrm() ||
+      if (!keyinfo->seg->charset || use_strnxfrm(keyinfo->seg->charset) ||
           (keyinfo->seg->flag & HA_NULL_PART) ||
           (keyinfo->seg->charset->mbminlen > 1))
         keyinfo->bin_search=_mi_seq_search;
@@ -767,7 +780,7 @@ static unsigned char *mi_state_info_read(unsigned char *ptr, MI_STATE_INFO *stat
   state->state.empty	= mi_sizekorr(ptr);	ptr +=8;
   state->state.key_empty= mi_sizekorr(ptr);	ptr +=8;
   state->auto_increment=mi_uint8korr(ptr);	ptr +=8;
-  state->state.checksum=(ha_checksum) mi_uint8korr(ptr);	ptr +=8;
+  state->state.checksum=(internal::ha_checksum) mi_uint8korr(ptr);	ptr +=8;
   state->process= mi_uint4korr(ptr);		ptr +=4;
   state->unique = mi_uint4korr(ptr);		ptr +=4;
   state->status = mi_uint4korr(ptr);		ptr +=4;
