@@ -11,13 +11,13 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* key handling functions */
 
 #include "myisam_priv.h"
-#include <drizzled/charset.h>
-#include <drizzled/internal/m_string.h>
+#include "drizzled/charset_info.h"
+#include "drizzled/internal/m_string.h"
 #include <drizzled/util/test.h>
 
 using namespace drizzled;
@@ -44,7 +44,7 @@ int _mi_check_index(MI_INFO *info, int inx)
     info->update= ((info->update & (HA_STATE_CHANGED | HA_STATE_ROW_CHANGED)) |
                    HA_STATE_NEXT_FOUND | HA_STATE_PREV_FOUND);
   }
-  if (info->opt_flag & WRITE_CACHE_USED && info->rec_cache.flush())
+  if (info->opt_flag & WRITE_CACHE_USED && flush_io_cache(&info->rec_cache))
     return(-1);
   return(inx);
 } /* mi_check_index */
@@ -175,7 +175,7 @@ int _mi_bin_search(MI_INFO *info, register MI_KEYDEF *keyinfo, unsigned char *pa
 {
   (void)buff;
   register int start,mid,end,save_end;
-  int flag= 0;
+  int flag;
   uint32_t totlength,nod_flag,not_used[2];
 
   totlength=keyinfo->keylength+(nod_flag=mi_test_if_nod(page));
@@ -234,7 +234,7 @@ int _mi_seq_search(MI_INFO *info, register MI_KEYDEF *keyinfo, unsigned char *pa
                    unsigned char *key, uint32_t key_len, uint32_t comp_flag, unsigned char **ret_pos,
                    unsigned char *buff, bool *last_key)
 {
-  int flag= 0;
+  int flag=0;
   uint32_t nod_flag,length=0,not_used[2];
   unsigned char t_buff[MI_MAX_KEY_BUFF],*end;
 
@@ -1680,7 +1680,7 @@ _mi_calc_bin_pack_key_length(MI_KEYDEF *keyinfo,uint32_t nod_flag,unsigned char
   uint32_t length,key_length,ref_length;
 
   s_temp->totlength=key_length=_mi_keylength(keyinfo,key)+nod_flag;
-#ifdef HAVE_VALGRIND
+#ifdef HAVE_purify
   s_temp->n_length= s_temp->n_ref_length=0;	/* For valgrind */
 #endif
   s_temp->key=key;
