@@ -11,15 +11,14 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* Write a row to a MyISAM table */
 
 #include "myisam_priv.h"
 
-#include <drizzled/internal/m_string.h>
+#include "drizzled/internal/m_string.h"
 #include <drizzled/util/test.h>
-#include <drizzled/tree.h>
 
 using namespace drizzled;
 
@@ -98,21 +97,27 @@ int mi_write(MI_INFO *info, unsigned char *record)
     {
       bool local_lock_tree= (lock_tree &&
                                 !(info->bulk_insert &&
-                                  info->bulk_insert[i].is_inited()));
+                                  is_tree_inited(&info->bulk_insert[i])));
       if (local_lock_tree)
       {
+	pthread_rwlock_wrlock(&share->key_root_lock[i]);
 	share->keyinfo[i].version++;
       }
       {
         if (share->keyinfo[i].ck_insert(info,i,buff,
 			_mi_make_key(info,i,buff,record,filepos)))
         {
+          if (local_lock_tree)
+            pthread_rwlock_unlock(&share->key_root_lock[i]);
           goto err;
         }
       }
 
       /* The above changed info->lastkey2. Inform mi_rnext_same(). */
       info->update&= ~HA_STATE_RNEXT_SAME;
+
+      if (local_lock_tree)
+        pthread_rwlock_unlock(&share->key_root_lock[i]);
     }
   }
   if (share->calc_checksum)
@@ -140,8 +145,8 @@ int mi_write(MI_INFO *info, unsigned char *record)
     not critical to use outdated share->is_log_table value (2) locking
     mutex here for every write is too expensive.
   */
-  if (share->is_log_table) // Log table do not exist in Drizzle
-    assert(0);
+  if (share->is_log_table)
+    mi_update_status((void*) info);
 
   return(0);
 
@@ -161,13 +166,22 @@ err:
     {
       if (mi_is_key_active(share->state.key_map, i))
       {
+	bool local_lock_tree= (lock_tree &&
+                                  !(info->bulk_insert &&
+                                    is_tree_inited(&info->bulk_insert[i])));
+	if (local_lock_tree)
+	  pthread_rwlock_wrlock(&share->key_root_lock[i]);
 	{
 	  uint32_t key_length=_mi_make_key(info,i,buff,record,filepos);
 	  if (_mi_ck_delete(info,i,buff,key_length))
 	  {
+	    if (local_lock_tree)
+	      pthread_rwlock_unlock(&share->key_root_lock[i]);
 	    break;
 	  }
 	}
+	if (local_lock_tree)
+	  pthread_rwlock_unlock(&share->key_root_lock[i]);
       }
     }
   }
@@ -189,7 +203,7 @@ err2:
 
 int _mi_ck_write(MI_INFO *info, uint32_t keynr, unsigned char *key, uint32_t key_length)
 {
-  if (info->bulk_insert && info->bulk_insert[keynr].is_inited())
+  if (info->bulk_insert && is_tree_inited(&info->bulk_insert[keynr]))
   {
     return(_mi_ck_write_tree(info, keynr, key, key_length));
   }
@@ -717,9 +731,9 @@ int _mi_ck_write_tree(register MI_INFO *info, uint32_t keynr, unsigned char *key
 {
   int error;
 
-  error= info->bulk_insert[keynr].tree_insert(key,
+  error= tree_insert(&info->bulk_insert[keynr], key,
          key_length + info->s->rec_reflength,
-         info->bulk_insert[keynr].getCustomArg()) ? 0 : HA_ERR_OUT_OF_MEM ;
+         info->bulk_insert[keynr].custom_arg) ? 0 : HA_ERR_OUT_OF_MEM ;
 
   return(error);
 } /* _mi_ck_write_tree */
@@ -750,6 +764,7 @@ static int keys_free(unsigned char *key, TREE_FREE mode, bulk_insert_param *para
   case free_init:
     if (param->info->s->concurrent_insert)
     {
+      pthread_rwlock_wrlock(&param->info->s->key_root_lock[param->keynr]);
       param->info->s->keyinfo[param->keynr].version++;
     }
     return 0;
@@ -760,6 +775,8 @@ static int keys_free(unsigned char *key, TREE_FREE mode, bulk_insert_param *para
     return _mi_ck_write_btree(param->info,param->keynr,lastkey,
 			      keylen - param->info->s->rec_reflength);
   case free_end:
+    if (param->info->s->concurrent_insert)
+      pthread_rwlock_unlock(&param->info->s->key_root_lock[param->keynr]);
     return 0;
   }
   return -1;
@@ -798,7 +815,8 @@ int mi_init_bulk_insert(MI_INFO *info, uint32_t cache_size, ha_rows rows)
   else
     cache_size/=total_keylength*16;
 
-  info->bulk_insert=(Tree *) malloc((sizeof(Tree)*share->base.keys+
+  info->bulk_insert=(TREE *)
+    malloc((sizeof(TREE)*share->base.keys+
            sizeof(bulk_insert_param)*num_keys));
 
   if (!info->bulk_insert)
@@ -812,13 +830,14 @@ int mi_init_bulk_insert(MI_INFO *info, uint32_t cache_size, ha_rows rows)
       params->info=info;
       params->keynr=i;
       /* Only allocate a 16'th of the buffer at a time */
-      info->bulk_insert[i].init_tree(cache_size * key[i].maxlength,
+      init_tree(&info->bulk_insert[i],
+                cache_size * key[i].maxlength,
                 cache_size * key[i].maxlength, 0,
 		(qsort_cmp2)keys_compare, false,
 		(tree_element_free) keys_free, (void *)params++);
     }
     else
-     info->bulk_insert[i].setRoot(0);
+     info->bulk_insert[i].root=0;
   }
 
   return(0);
@@ -828,8 +847,8 @@ void mi_flush_bulk_insert(MI_INFO *info, uint32_t inx)
 {
   if (info->bulk_insert)
   {
-    if (info->bulk_insert[inx].is_inited())
-      info->bulk_insert[inx].reset_tree();
+    if (is_tree_inited(&info->bulk_insert[inx]))
+      reset_tree(&info->bulk_insert[inx]);
   }
 }
 
@@ -840,9 +859,9 @@ void mi_end_bulk_insert(MI_INFO *info)
     uint32_t i;
     for (i=0 ; i < info->s->base.keys ; i++)
     {
-      if (info->bulk_insert[i].is_inited())
+      if (is_tree_inited(& info->bulk_insert[i]))
       {
-        info->bulk_insert[i].delete_tree();
+        delete_tree(& info->bulk_insert[i]);
       }
     }
     free((void *)info->bulk_insert);
