@@ -11,17 +11,24 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* This file should be included when using myisam_funktions */
 
-#pragma once
+#ifndef PLUGIN_MYISAM_MYISAM_H
+#define PLUGIN_MYISAM_MYISAM_H
 
-#include <drizzled/common_fwd.h>
 #include <drizzled/key_map.h>
+
 #include <drizzled/base.h>
+#ifndef _m_ctype_h
+#include "drizzled/charset_info.h"
+#endif
+#ifndef _keycache_h
+#include "keycache.h"
+#endif
 #include <plugin/myisam/my_handler.h>
-#include <drizzled/internal/iocache.h>
+#include "drizzled/internal/iocache.h"
 
 /*
   Limit max keys according to HA_MAX_POSSIBLE_KEY
@@ -111,9 +118,8 @@
                             (_to_)= (mi_get_mask_all_keys_active(_maxkeys_) & \
                                      (_from_))
 
-typedef uint32_t ha_checksum;
-
 	/* Param to/from mi_status */
+
 typedef struct st_mi_isaminfo		/* Struct from h_info */
 {
   drizzled::ha_rows records;			/* Records in database */
@@ -156,20 +162,6 @@ typedef struct st_mi_create_info
   uint32_t old_options;
   uint8_t language;
   bool with_auto_increment;
-
-  st_mi_create_info():
-    index_file_name(0),
-    data_file_name(0),
-    max_rows(0),
-    reloc_rows(0),
-    auto_increment(0),
-    data_file_length(0),
-    key_file_length(0),
-    old_options(0),
-    language(0),
-    with_auto_increment(0)
-  { }
-
 } MI_CREATE_INFO;
 
 struct st_myisam_info;			/* For referense */
@@ -269,8 +261,7 @@ extern uint32_t data_pointer_size;
 
 extern int mi_close(struct st_myisam_info *file);
 extern int mi_delete(struct st_myisam_info *file,const unsigned char *buff);
-extern struct st_myisam_info *mi_open(const drizzled::identifier::Table &identifier,
-                                      int mode,
+extern struct st_myisam_info *mi_open(const char *name,int mode,
 				      uint32_t wait_if_locked);
 extern int mi_panic(enum drizzled::ha_panic_function function);
 extern int mi_rfirst(struct st_myisam_info *file,unsigned char *buf,int inx);
@@ -324,6 +315,7 @@ extern uint32_t mi_get_pointer_length(uint64_t file_length, uint32_t def);
 
 #define T_AUTO_INC              1
 #define T_AUTO_REPAIR           2              /* QQ to be removed */
+#define T_BACKUP_DATA           4
 #define T_CALC_CHECKSUM         8
 #define T_CHECK                 16             /* QQ to be removed */
 #define T_CHECK_ONLY_CHANGED    32             /* QQ to be removed */
@@ -397,7 +389,7 @@ typedef struct st_mi_check_param
   drizzled::internal::my_off_t new_file_pos,key_file_blocks;
   drizzled::internal::my_off_t keydata,totaldata,key_blocks,start_check_pos;
   drizzled::ha_rows total_records,total_deleted;
-  ha_checksum record_checksum,glob_crc;
+  drizzled::internal::ha_checksum record_checksum,glob_crc;
   uint64_t use_buffers;
   size_t read_buffer_length, write_buffer_length,
          sort_buffer_length, sort_key_blocks;
@@ -410,7 +402,7 @@ typedef struct st_mi_check_param
   char temp_filename[FN_REFLEN],*isam_file_name;
   int tmpfile_createflag;
   drizzled::myf myf_rw;
-  drizzled::internal::io_cache_st read_cache;
+  drizzled::internal::IO_CACHE read_cache;
 
   /*
     The next two are used to collect statistics, see update_key_parts for
@@ -419,7 +411,7 @@ typedef struct st_mi_check_param
   uint64_t unique_count[MI_MAX_KEY_SEG+1];
   uint64_t notnull_count[MI_MAX_KEY_SEG+1];
 
-  ha_checksum key_crc[HA_MAX_POSSIBLE_KEY];
+  drizzled::internal::ha_checksum key_crc[HA_MAX_POSSIBLE_KEY];
   ulong rec_per_key_part[MI_MAX_KEY_SEG*HA_MAX_POSSIBLE_KEY];
   void *session;
   const char *db_name, *table_name;
@@ -456,6 +448,8 @@ int mi_repair(MI_CHECK *param, register MI_INFO *info,
 int mi_sort_index(MI_CHECK *param, register MI_INFO *info, char * name);
 int mi_repair_by_sort(MI_CHECK *param, register MI_INFO *info,
 		      const char * name, int rep_quick);
+int mi_repair_parallel(MI_CHECK *param, register MI_INFO *info,
+		      const char * name, int rep_quick);
 int change_to_newfile(const char * filename, const char * old_ext,
 		      const char * new_ext, uint32_t raid_chunks,
 		      drizzled::myf myflags);
@@ -472,6 +466,7 @@ int movepoint(MI_INFO *info,unsigned char *record,drizzled::internal::my_off_t o
 	      drizzled::internal::my_off_t newpos, uint32_t prot_key);
 int write_data_suffix(SORT_INFO *sort_info, bool fix_datafile);
 int test_if_almost_full(MI_INFO *info);
+int recreate_table(MI_CHECK *param, MI_INFO **org_info, char *filename);
 bool mi_test_if_sort_rep(MI_INFO *info, drizzled::ha_rows rows, uint64_t key_map,
 			    bool force);
 
@@ -480,3 +475,4 @@ void mi_flush_bulk_insert(MI_INFO *info, uint32_t inx);
 void mi_end_bulk_insert(MI_INFO *info);
 int mi_preload(MI_INFO *info, uint64_t key_map, bool ignore_leaves);
 
+#endif /* PLUGIN_MYISAM_MYISAM_H */
