@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
   Creates a index for a database by reading keys, sorting them and outputing
@@ -51,40 +51,40 @@ static ha_rows  find_all_keys(MI_SORT_PARAM *info,uint32_t keys,
 			      unsigned char **sort_keys,
 			      DYNAMIC_ARRAY *buffpek,
 			      size_t *maxbuffer,
-			      internal::io_cache_st *tempfile,
-			      internal::io_cache_st *tempfile_for_exceptions);
+			      internal::IO_CACHE *tempfile,
+			      internal::IO_CACHE *tempfile_for_exceptions);
 static int  write_keys(MI_SORT_PARAM *info,unsigned char **sort_keys,
-                             uint32_t count, BUFFPEK *buffpek,internal::io_cache_st *tempfile);
+                             uint32_t count, BUFFPEK *buffpek,internal::IO_CACHE *tempfile);
 static int  write_key(MI_SORT_PARAM *info, unsigned char *key,
-			    internal::io_cache_st *tempfile);
+			    internal::IO_CACHE *tempfile);
 static int  write_index(MI_SORT_PARAM *info,unsigned char * *sort_keys,
                               uint32_t count);
 static int  merge_many_buff(MI_SORT_PARAM *info,uint32_t keys,
 			    unsigned char * *sort_keys,
 			    BUFFPEK *buffpek,size_t *maxbuffer,
-			    internal::io_cache_st *t_file);
-static uint32_t  read_to_buffer(internal::io_cache_st *fromfile,BUFFPEK *buffpek,
+			    internal::IO_CACHE *t_file);
+static uint32_t  read_to_buffer(internal::IO_CACHE *fromfile,BUFFPEK *buffpek,
                                   uint32_t sort_length);
 static int  merge_buffers(MI_SORT_PARAM *info,uint32_t keys,
-                                internal::io_cache_st *from_file, internal::io_cache_st *to_file,
+                                internal::IO_CACHE *from_file, internal::IO_CACHE *to_file,
                                 unsigned char * *sort_keys, BUFFPEK *lastbuff,
                                 BUFFPEK *Fb, BUFFPEK *Tb);
 static int  merge_index(MI_SORT_PARAM *,uint,unsigned char **,BUFFPEK *, int,
-                              internal::io_cache_st *);
+                              internal::IO_CACHE *);
 static int  write_keys_varlen(MI_SORT_PARAM *info,unsigned char **sort_keys,
                        uint32_t count, BUFFPEK *buffpek,
-                       internal::io_cache_st *tempfile);
-static uint32_t  read_to_buffer_varlen(internal::io_cache_st *fromfile,BUFFPEK *buffpek,
+                       internal::IO_CACHE *tempfile);
+static uint32_t  read_to_buffer_varlen(internal::IO_CACHE *fromfile,BUFFPEK *buffpek,
                                 uint32_t sort_length);
-static int  write_merge_key(MI_SORT_PARAM *info, internal::io_cache_st *to_file,
+static int  write_merge_key(MI_SORT_PARAM *info, internal::IO_CACHE *to_file,
                      unsigned char *key, uint32_t sort_length, uint32_t count);
 static int  write_merge_key_varlen(MI_SORT_PARAM *info,
-                            internal::io_cache_st *to_file,
+                            internal::IO_CACHE *to_file,
                             unsigned char* key, uint32_t sort_length,
                             uint32_t count);
 
 inline int
-my_var_write(MI_SORT_PARAM *info, internal::io_cache_st *to_file, unsigned char *bufs);
+my_var_write(MI_SORT_PARAM *info, internal::IO_CACHE *to_file, unsigned char *bufs);
 
 /*
   Creates a index of sorted keys
@@ -103,10 +103,13 @@ my_var_write(MI_SORT_PARAM *info, internal::io_cache_st *to_file, unsigned char
 int _create_index_by_sort(MI_SORT_PARAM *info,bool no_messages,
 			  size_t sortbuff_size)
 {
-  size_t skr;
-  uint32_t memavl,keys;
+  int error;
+  size_t maxbuffer, skr;
+  uint32_t memavl,old_memavl,keys,sort_length;
   DYNAMIC_ARRAY buffpek;
-  internal::io_cache_st tempfile, tempfile_for_exceptions;
+  ha_rows records;
+  unsigned char **sort_keys;
+  internal::IO_CACHE tempfile, tempfile_for_exceptions;
 
   if (info->keyinfo->flag & HA_VAR_LENGTH_KEY)
   {
@@ -121,16 +124,15 @@ int _create_index_by_sort(MI_SORT_PARAM *info,bool no_messages,
     info->write_key=write_merge_key;
   }
 
-  tempfile.clear();
-  tempfile_for_exceptions.clear();
+  my_b_clear(&tempfile);
+  my_b_clear(&tempfile_for_exceptions);
   memset(&buffpek, 0, sizeof(buffpek));
-  unsigned char** sort_keys= (unsigned char **) NULL; 
-  int error= 1;
-  size_t maxbuffer=1;
+  sort_keys= (unsigned char **) NULL; error= 1;
+  maxbuffer=1;
 
   memavl=max(sortbuff_size,(size_t)MIN_SORT_MEMORY);
-  ha_rows records= info->sort_info->max_records;
-  uint32_t sort_length= info->key_length;
+  records=	info->sort_info->max_records;
+  sort_length=	info->key_length;
 
   while (memavl >= MIN_SORT_MEMORY)
   {
@@ -141,21 +143,33 @@ int _create_index_by_sort(MI_SORT_PARAM *info,bool no_messages,
     else
       do
       {
-        skr=maxbuffer;
-        if (memavl < sizeof(BUFFPEK)* maxbuffer ||
-          (keys=(memavl-sizeof(BUFFPEK)* maxbuffer) / (sort_length+sizeof(char*))) <= 1 ||
-          keys < maxbuffer)
-        {
-          mi_check_print_error(info->sort_info->param,
-            "myisam_sort_buffer_size is too small");
-          goto err;
-        }
+	skr=maxbuffer;
+	if (memavl < sizeof(BUFFPEK)* maxbuffer ||
+	    (keys=(memavl-sizeof(BUFFPEK)* maxbuffer)/
+             (sort_length+sizeof(char*))) <= 1 ||
+            keys < maxbuffer)
+	{
+	  mi_check_print_error(info->sort_info->param,
+			       "myisam_sort_buffer_size is too small");
+	  goto err;
+	}
       }
       while ((maxbuffer= (size_t)(records/(keys-1)+1)) != skr);
 
-    sort_keys=(unsigned char **)malloc(keys*(sort_length+sizeof(char*)));
-    buffpek.init(sizeof(BUFFPEK), maxbuffer, maxbuffer/2);
-    break;
+    if ((sort_keys=(unsigned char **)malloc(keys*(sort_length+sizeof(char*)))))
+    {
+      if (my_init_dynamic_array(&buffpek, sizeof(BUFFPEK), maxbuffer,
+			     maxbuffer/2))
+      {
+	free((unsigned char*) sort_keys);
+        sort_keys= 0;
+      }
+      else
+	break;
+    }
+    old_memavl=memavl;
+    if ((memavl=memavl/4*3) < MIN_SORT_MEMORY && old_memavl > MIN_SORT_MEMORY)
+      memavl=MIN_SORT_MEMORY;
   }
   if (memavl < MIN_SORT_MEMORY)
   {
@@ -185,52 +199,55 @@ int _create_index_by_sort(MI_SORT_PARAM *info,bool no_messages,
     {
       if (!no_messages)
 	printf("  - Merging %u keys\n", (uint32_t) records);
-      if (merge_many_buff(info,keys,sort_keys, (BUFFPEK*)buffpek.buffer, &maxbuffer, &tempfile))
+      if (merge_many_buff(info,keys,sort_keys,
+                  dynamic_element(&buffpek,0,BUFFPEK *),&maxbuffer,&tempfile))
 	goto err;
     }
-    if (tempfile.flush() ||
-	tempfile.reinit_io_cache(internal::READ_CACHE,0L,0,0))
+    if (internal::flush_io_cache(&tempfile) ||
+	internal::reinit_io_cache(&tempfile,internal::READ_CACHE,0L,0,0))
       goto err;
     if (!no_messages)
       printf("  - Last merge and dumping keys\n");
-    if (merge_index(info,keys,sort_keys, (BUFFPEK*)buffpek.buffer, maxbuffer, &tempfile))
+    if (merge_index(info,keys,sort_keys,dynamic_element(&buffpek,0,BUFFPEK *),
+                    maxbuffer,&tempfile))
       goto err;
   }
 
   if (flush_pending_blocks(info))
     goto err;
 
-  if (tempfile_for_exceptions.inited())
+  if (my_b_inited(&tempfile_for_exceptions))
   {
     MI_INFO *idx=info->sort_info->info;
     uint32_t     keyno=info->key;
     uint32_t     key_length, ref_length=idx->s->rec_reflength;
 
-    if (not no_messages)
+    if (!no_messages)
       printf("  - Adding exceptions\n");
-
-    if (tempfile_for_exceptions.flush() || tempfile_for_exceptions.reinit_io_cache(internal::READ_CACHE,0L,0,0))
-    {
+    if (flush_io_cache(&tempfile_for_exceptions) ||
+	reinit_io_cache(&tempfile_for_exceptions,internal::READ_CACHE,0L,0,0))
       goto err;
-    }
 
-    while (not tempfile_for_exceptions.read(&key_length, sizeof(key_length))
-        && not tempfile_for_exceptions.read(sort_keys, key_length))
+    while (!my_b_read(&tempfile_for_exceptions,(unsigned char*)&key_length,
+		      sizeof(key_length))
+        && !my_b_read(&tempfile_for_exceptions,(unsigned char*)sort_keys,
+		      (uint) key_length))
     {
-      if (_mi_ck_write(idx,keyno,(unsigned char*) sort_keys,key_length-ref_length))
-        goto err;
+	if (_mi_ck_write(idx,keyno,(unsigned char*) sort_keys,key_length-ref_length))
+	  goto err;
     }
   }
 
   error =0;
 
 err:
-  free(sort_keys);
-  buffpek.free();
-  tempfile.close_cached_file();
-  tempfile_for_exceptions.close_cached_file();
+  if (sort_keys)
+    free((unsigned char*) sort_keys);
+  delete_dynamic(&buffpek);
+  close_cached_file(&tempfile);
+  close_cached_file(&tempfile_for_exceptions);
 
-  return error ? -1 : 0;
+  return(error ? -1 : 0);
 } /* _create_index_by_sort */
 
 
@@ -239,8 +256,8 @@ err:
 static ha_rows  find_all_keys(MI_SORT_PARAM *info, uint32_t keys,
 			      unsigned char **sort_keys,
 			      DYNAMIC_ARRAY *buffpek,
-			      size_t *maxbuffer, internal::io_cache_st *tempfile,
-			      internal::io_cache_st *tempfile_for_exceptions)
+			      size_t *maxbuffer, internal::IO_CACHE *tempfile,
+			      internal::IO_CACHE *tempfile_for_exceptions)
 {
   int error;
   uint32_t idx;
@@ -259,8 +276,9 @@ static ha_rows  find_all_keys(MI_SORT_PARAM *info, uint32_t keys,
 
     if (++idx == keys)
     {
-      if (info->write_keys(info,sort_keys,idx-1,(BUFFPEK *)buffpek->alloc(), tempfile))
-        return HA_POS_ERROR;
+      if (info->write_keys(info,sort_keys,idx-1,(BUFFPEK *)alloc_dynamic(buffpek),
+		     tempfile))
+      return(HA_POS_ERROR);
 
       sort_keys[0]=(unsigned char*) (sort_keys+keys);
       memcpy(sort_keys[0],sort_keys[idx-1],(size_t) info->key_length);
@@ -270,11 +288,12 @@ static ha_rows  find_all_keys(MI_SORT_PARAM *info, uint32_t keys,
   }
   if (error > 0)
     return(HA_POS_ERROR);
-  if (buffpek->size())
+  if (buffpek->elements)
   {
-    if (info->write_keys(info,sort_keys,idx,(BUFFPEK *)buffpek->alloc(), tempfile))
-      return HA_POS_ERROR;
-    *maxbuffer=buffpek->size() - 1;
+    if (info->write_keys(info,sort_keys,idx,(BUFFPEK *)alloc_dynamic(buffpek),
+		   tempfile))
+      return(HA_POS_ERROR);
+    *maxbuffer=buffpek->elements-1;
   }
   else
     *maxbuffer=0;
@@ -283,6 +302,177 @@ static ha_rows  find_all_keys(MI_SORT_PARAM *info, uint32_t keys,
 } /* find_all_keys */
 
 
+/* Search after all keys and place them in a temp. file */
+
+pthread_handler_t thr_find_all_keys(void *arg)
+{
+  MI_SORT_PARAM *sort_param= (MI_SORT_PARAM*) arg;
+  int error;
+  uint32_t memavl,old_memavl,keys,sort_length;
+  uint32_t idx;
+  size_t maxbuffer;
+  unsigned char **sort_keys=0;
+
+  error=1;
+
+  if (internal::my_thread_init())
+    goto err;
+
+  {
+    if (sort_param->sort_info->got_error)
+      goto err;
+
+    if (sort_param->keyinfo->flag & HA_VAR_LENGTH_KEY)
+    {
+      sort_param->write_keys=     write_keys_varlen;
+      sort_param->read_to_buffer= read_to_buffer_varlen;
+      sort_param->write_key=      write_merge_key_varlen;
+    }
+    else
+    {
+      sort_param->write_keys=     write_keys;
+      sort_param->read_to_buffer= read_to_buffer;
+      sort_param->write_key=      write_merge_key;
+    }
+
+    my_b_clear(&sort_param->tempfile);
+    my_b_clear(&sort_param->tempfile_for_exceptions);
+    memset(&sort_param->buffpek, 0, sizeof(sort_param->buffpek));
+    memset(&sort_param->unique, 0,  sizeof(sort_param->unique));
+    sort_keys= (unsigned char **) NULL;
+
+    memavl=       max(sort_param->sortbuff_size, (uint32_t)MIN_SORT_MEMORY);
+    idx=          (uint)sort_param->sort_info->max_records;
+    sort_length=  sort_param->key_length;
+    maxbuffer=    1;
+
+    while (memavl >= MIN_SORT_MEMORY)
+    {
+      if ((internal::my_off_t) (idx+1)*(sort_length+sizeof(char*)) <=
+          (internal::my_off_t) memavl)
+        keys= idx+1;
+      else
+      {
+        uint32_t skr;
+        do
+        {
+          skr= maxbuffer;
+          if (memavl < sizeof(BUFFPEK)*maxbuffer ||
+              (keys=(memavl-sizeof(BUFFPEK)*maxbuffer)/
+               (sort_length+sizeof(char*))) <= 1 ||
+              keys < maxbuffer)
+          {
+            mi_check_print_error(sort_param->sort_info->param,
+                                 "myisam_sort_buffer_size is too small");
+            goto err;
+          }
+        }
+        while ((maxbuffer= (idx/(keys-1)+1)) != skr);
+      }
+      if ((sort_keys= (unsigned char**)
+           malloc(keys*(sort_length+sizeof(char*)))))
+      {
+        if (my_init_dynamic_array(&sort_param->buffpek, sizeof(BUFFPEK),
+                                  maxbuffer, maxbuffer/2))
+        {
+          free((unsigned char*) sort_keys);
+          sort_keys= (unsigned char **) NULL; /* for err: label */
+        }
+        else
+          break;
+      }
+      old_memavl= memavl;
+      if ((memavl= memavl/4*3) < MIN_SORT_MEMORY &&
+          old_memavl > MIN_SORT_MEMORY)
+        memavl= MIN_SORT_MEMORY;
+    }
+    if (memavl < MIN_SORT_MEMORY)
+    {
+      mi_check_print_error(sort_param->sort_info->param,
+                           "MyISAM sort buffer too small");
+      goto err;
+    }
+
+    if (sort_param->sort_info->param->testflag & T_VERBOSE)
+      printf("Key %d - Allocating buffer for %d keys\n",
+             sort_param->key + 1, keys);
+    sort_param->sort_keys= sort_keys;
+
+    idx= error= 0;
+    sort_keys[0]= (unsigned char*) (sort_keys+keys);
+
+    while (!(error= sort_param->sort_info->got_error) &&
+           !(error= (*sort_param->key_read)(sort_param, sort_keys[idx])))
+    {
+      if (sort_param->real_key_length > sort_param->key_length)
+      {
+        if (write_key(sort_param, sort_keys[idx],
+                      &sort_param->tempfile_for_exceptions))
+          goto err;
+        continue;
+      }
+
+      if (++idx == keys)
+      {
+        if (sort_param->write_keys(sort_param, sort_keys, idx - 1,
+                                   (BUFFPEK*) alloc_dynamic(&sort_param->buffpek),
+                                   &sort_param->tempfile))
+          goto err;
+        sort_keys[0]= (unsigned char*) (sort_keys+keys);
+        memcpy(sort_keys[0], sort_keys[idx - 1], (size_t) sort_param->key_length);
+        idx= 1;
+      }
+      sort_keys[idx]= sort_keys[idx - 1] + sort_param->key_length;
+    }
+    if (error > 0)
+      goto err;
+    if (sort_param->buffpek.elements)
+    {
+      if (sort_param->write_keys(sort_param, sort_keys, idx,
+                                 (BUFFPEK*) alloc_dynamic(&sort_param->buffpek),
+                                 &sort_param->tempfile))
+        goto err;
+      sort_param->keys= (sort_param->buffpek.elements - 1) * (keys - 1) + idx;
+    }
+    else
+      sort_param->keys= idx;
+
+    sort_param->sort_keys_length= keys;
+    goto ok;
+
+err:
+    sort_param->sort_info->got_error= 1; /* no need to protect with a mutex */
+    if (sort_keys)
+      free((unsigned char*) sort_keys);
+    sort_param->sort_keys= 0;
+    delete_dynamic(& sort_param->buffpek);
+    close_cached_file(&sort_param->tempfile);
+    close_cached_file(&sort_param->tempfile_for_exceptions);
+
+ok:
+    free_root(&sort_param->wordroot, MYF(0));
+    /*
+      Detach from the share if the writer is involved. Avoid others to
+      be blocked. This includes a flush of the write buffer. This will
+      also indicate EOF to the readers.
+    */
+    if (sort_param->sort_info->info->rec_cache.share)
+      remove_io_thread(&sort_param->sort_info->info->rec_cache);
+
+    /* Readers detach from the share if any. Avoid others to be blocked. */
+    if (sort_param->read_cache.share)
+      remove_io_thread(&sort_param->read_cache);
+
+    pthread_mutex_lock(&sort_param->sort_info->mutex);
+    if (!--sort_param->sort_info->threads_running)
+      pthread_cond_signal(&sort_param->sort_info->cond);
+    pthread_mutex_unlock(&sort_param->sort_info->mutex);
+  }
+  internal::my_thread_end();
+  return NULL;
+}
+
+
 int thr_write_keys(MI_SORT_PARAM *sort_param)
 {
   SORT_INFO *sort_info= sort_param->sort_info;
@@ -304,13 +494,14 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
     {
       got_error=1;
       void * rec_buff_ptr= mi_get_rec_buff_ptr(info, sinfo->rec_buff);
-      free(rec_buff_ptr);
+      if (rec_buff_ptr != NULL)
+        free(rec_buff_ptr);
       continue;
     }
     if (!got_error)
     {
       mi_set_key_active(share->state.key_map, sinfo->key);
-      if (!sinfo->buffpek.size())
+      if (!sinfo->buffpek.elements)
       {
         if (param->testflag & T_VERBOSE)
         {
@@ -328,14 +519,17 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
     }
     free((unsigned char*) sinfo->sort_keys);
     void * rec_buff_ptr= mi_get_rec_buff_ptr(info, sinfo->rec_buff);
-    free(rec_buff_ptr);
+    if (rec_buff_ptr != NULL)
+      free(rec_buff_ptr);
     sinfo->sort_keys=0;
   }
 
-  for (i= 0, sinfo= sort_param; i < sort_info->total_keys; i++,
-	 sinfo->buffpek.free(),
-	 sinfo->tempfile.close_cached_file(),
-	 sinfo->tempfile_for_exceptions.close_cached_file(),
+  for (i= 0, sinfo= sort_param ;
+       i < sort_info->total_keys ;
+       i++,
+	 delete_dynamic(&sinfo->buffpek),
+	 close_cached_file(&sinfo->tempfile),
+	 close_cached_file(&sinfo->tempfile_for_exceptions),
 	 sinfo++)
   {
     if (got_error)
@@ -352,9 +546,9 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
       sinfo->read_to_buffer=read_to_buffer;
       sinfo->write_key=write_merge_key;
     }
-    if (sinfo->buffpek.size())
+    if (sinfo->buffpek.elements)
     {
-      size_t maxbuffer=sinfo->buffpek.size() - 1;
+      size_t maxbuffer=sinfo->buffpek.elements-1;
       if (!mergebuf)
       {
         length=param->sort_buffer_length;
@@ -375,21 +569,24 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
       {
         if (param->testflag & T_VERBOSE)
           printf("Key %d  - Merging %u keys\n",sinfo->key+1, sinfo->keys);
-        if (merge_many_buff(sinfo, keys, (unsigned char **)mergebuf, (BUFFPEK*)sinfo->buffpek.buffer,
+        if (merge_many_buff(sinfo, keys, (unsigned char **)mergebuf,
+			    dynamic_element(&sinfo->buffpek, 0, BUFFPEK *),
 			    &maxbuffer, &sinfo->tempfile))
         {
           got_error=1;
           continue;
         }
       }
-      if (sinfo->tempfile.flush() || sinfo->tempfile.reinit_io_cache(internal::READ_CACHE,0L,0,0))
+      if (flush_io_cache(&sinfo->tempfile) ||
+          reinit_io_cache(&sinfo->tempfile,internal::READ_CACHE,0L,0,0))
       {
         got_error=1;
         continue;
       }
       if (param->testflag & T_VERBOSE)
         printf("Key %d  - Last merge and dumping keys\n", sinfo->key+1);
-      if (merge_index(sinfo, keys, (unsigned char **)mergebuf, (BUFFPEK*)sinfo->buffpek.buffer,
+      if (merge_index(sinfo, keys, (unsigned char **)mergebuf,
+                      dynamic_element(&sinfo->buffpek,0,BUFFPEK *),
                       maxbuffer,&sinfo->tempfile) ||
 	  flush_pending_blocks(sinfo))
       {
@@ -397,25 +594,31 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
         continue;
       }
     }
-    if (sinfo->tempfile_for_exceptions.inited())
+    if (my_b_inited(&sinfo->tempfile_for_exceptions))
     {
       uint32_t key_length;
 
       if (param->testflag & T_VERBOSE)
         printf("Key %d  - Dumping 'long' keys\n", sinfo->key+1);
 
-      if (sinfo->tempfile_for_exceptions.flush() || sinfo->tempfile_for_exceptions.reinit_io_cache(internal::READ_CACHE,0L,0,0))
+      if (flush_io_cache(&sinfo->tempfile_for_exceptions) ||
+          reinit_io_cache(&sinfo->tempfile_for_exceptions,internal::READ_CACHE,0L,0,0))
       {
         got_error=1;
         continue;
       }
 
-      while (!got_error && not sinfo->tempfile_for_exceptions.read(&key_length, sizeof(key_length)))
+      while (!got_error &&
+	     !my_b_read(&sinfo->tempfile_for_exceptions,(unsigned char*)&key_length,
+			sizeof(key_length)))
       {
         unsigned char ft_buf[10];
-        if (key_length > sizeof(ft_buf) || sinfo->tempfile_for_exceptions.read(ft_buf, (uint)key_length) 
-          || _mi_ck_write(info, sinfo->key, (unsigned char*)ft_buf, key_length - info->s->rec_reflength))
-          got_error= 1;
+        if (key_length > sizeof(ft_buf) ||
+            my_b_read(&sinfo->tempfile_for_exceptions, (unsigned char*)ft_buf,
+                      (uint)key_length) ||
+            _mi_ck_write(info, sinfo->key, (unsigned char*)ft_buf,
+                         key_length - info->s->rec_reflength))
+          got_error=1;
       }
     }
   }
@@ -426,21 +629,24 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
         /* Write all keys in memory to file for later merge */
 
 static int  write_keys(MI_SORT_PARAM *info, register unsigned char **sort_keys,
-                             uint32_t count, BUFFPEK *buffpek, internal::io_cache_st *tempfile)
+                             uint32_t count, BUFFPEK *buffpek, internal::IO_CACHE *tempfile)
 {
   unsigned char **end;
   uint32_t sort_length=info->key_length;
 
-  internal::my_qsort2((unsigned char*) sort_keys,count,sizeof(unsigned char*),(qsort2_cmp) info->key_cmp, info);
-  if (not tempfile->inited() && tempfile->open_cached_file(P_tmpdir, "ST", DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
+  internal::my_qsort2((unsigned char*) sort_keys,count,sizeof(unsigned char*),(qsort2_cmp) info->key_cmp,
+            info);
+  if (!my_b_inited(tempfile) &&
+      open_cached_file(tempfile, P_tmpdir, "ST",
+                       DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
     return(1);
 
-  buffpek->file_pos= tempfile->tell();
+  buffpek->file_pos=my_b_tell(tempfile);
   buffpek->count=count;
 
   for (end=sort_keys+count ; sort_keys != end ; sort_keys++)
   {
-    if (tempfile->write(*sort_keys, sort_length))
+    if (my_b_write(tempfile,(unsigned char*) *sort_keys,(uint) sort_length))
       return(1);
   }
   return(0);
@@ -448,15 +654,15 @@ static int  write_keys(MI_SORT_PARAM *info, register unsigned char **sort_keys,
 
 
 inline int
-my_var_write(MI_SORT_PARAM *info, internal::io_cache_st *to_file, unsigned char *bufs)
+my_var_write(MI_SORT_PARAM *info, internal::IO_CACHE *to_file, unsigned char *bufs)
 {
   int err;
   uint16_t len = _mi_keylength(info->keyinfo, (unsigned char*) bufs);
 
   /* The following is safe as this is a local file */
-  if ((err= to_file->write(&len, sizeof(len))))
+  if ((err= my_b_write(to_file, (unsigned char*)&len, sizeof(len))))
     return (err);
-  if ((err= to_file->write(bufs, len)))
+  if ((err= my_b_write(to_file,bufs, (uint) len)))
     return (err);
   return (0);
 }
@@ -465,16 +671,19 @@ my_var_write(MI_SORT_PARAM *info, internal::io_cache_st *to_file, unsigned char
 static int  write_keys_varlen(MI_SORT_PARAM *info,
 				    register unsigned char **sort_keys,
                                     uint32_t count, BUFFPEK *buffpek,
-				    internal::io_cache_st *tempfile)
+				    internal::IO_CACHE *tempfile)
 {
   unsigned char **end;
   int err;
 
-  internal::my_qsort2((unsigned char*) sort_keys,count,sizeof(unsigned char*),(qsort2_cmp) info->key_cmp, info);
-  if (not tempfile->inited() && tempfile->open_cached_file(P_tmpdir, "ST", DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
+  internal::my_qsort2((unsigned char*) sort_keys,count,sizeof(unsigned char*),(qsort2_cmp) info->key_cmp,
+            info);
+  if (!my_b_inited(tempfile) &&
+      open_cached_file(tempfile, P_tmpdir, "ST",
+                       DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
     return(1);
 
-  buffpek->file_pos= tempfile->tell();
+  buffpek->file_pos=my_b_tell(tempfile);
   buffpek->count=count;
   for (end=sort_keys+count ; sort_keys != end ; sort_keys++)
   {
@@ -486,15 +695,17 @@ static int  write_keys_varlen(MI_SORT_PARAM *info,
 
 
 static int  write_key(MI_SORT_PARAM *info, unsigned char *key,
-			    internal::io_cache_st *tempfile)
+			    internal::IO_CACHE *tempfile)
 {
   uint32_t key_length=info->real_key_length;
 
-  if (not tempfile->inited() && tempfile->open_cached_file(P_tmpdir, "ST", DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
+  if (!my_b_inited(tempfile) &&
+      open_cached_file(tempfile, P_tmpdir, "ST",
+                       DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
     return(1);
 
-  if (tempfile->write(&key_length, sizeof(key_length)) ||
-      tempfile->write(key, key_length))
+  if (my_b_write(tempfile,(unsigned char*)&key_length,sizeof(key_length)) ||
+      my_b_write(tempfile,(unsigned char*)key,(uint) key_length))
     return(1);
   return(0);
 } /* write_key */
@@ -520,22 +731,24 @@ static int  write_index(MI_SORT_PARAM *info, register unsigned char **sort_keys,
 
 static int  merge_many_buff(MI_SORT_PARAM *info, uint32_t keys,
 			    unsigned char **sort_keys, BUFFPEK *buffpek,
-			    size_t *maxbuffer, internal::io_cache_st *t_file)
+			    size_t *maxbuffer, internal::IO_CACHE *t_file)
 {
   uint32_t i;
-  internal::io_cache_st t_file2, *from_file, *to_file, *temp;
+  internal::IO_CACHE t_file2, *from_file, *to_file, *temp;
   BUFFPEK *lastbuff;
 
   if (*maxbuffer < MERGEBUFF2)
     return(0);
-  if (t_file->flush() || t_file2.open_cached_file(P_tmpdir, "ST", DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
+  if (flush_io_cache(t_file) ||
+      open_cached_file(&t_file2, P_tmpdir, "ST",
+                       DISK_BUFFER_SIZE, info->sort_info->param->myf_rw))
     return(1);
 
   from_file= t_file ; to_file= &t_file2;
   while (*maxbuffer >= MERGEBUFF2)
   {
-    from_file->reinit_io_cache(internal::READ_CACHE,0L,0,0);
-    to_file->reinit_io_cache(internal::WRITE_CACHE,0L,0,0);
+    reinit_io_cache(from_file,internal::READ_CACHE,0L,0,0);
+    reinit_io_cache(to_file,internal::WRITE_CACHE,0L,0,0);
     lastbuff=buffpek;
     for (i=0 ; i <= *maxbuffer-MERGEBUFF*3/2 ; i+=MERGEBUFF)
     {
@@ -546,13 +759,13 @@ static int  merge_many_buff(MI_SORT_PARAM *info, uint32_t keys,
     if (merge_buffers(info,keys,from_file,to_file,sort_keys,lastbuff++,
                       buffpek+i,buffpek+ *maxbuffer))
       break;
-    if (to_file->flush())
+    if (flush_io_cache(to_file))
       break;
     temp=from_file; from_file=to_file; to_file=temp;
     *maxbuffer= (int) (lastbuff-buffpek)-1;
   }
 cleanup:
-  to_file->close_cached_file();                   /* This holds old result */
+  close_cached_file(to_file);                   /* This holds old result */
   if (to_file == t_file)
     *t_file=t_file2;                            /* Copy result file */
 
@@ -573,7 +786,7 @@ cleanup:
     -1	Error
 */
 
-static uint32_t  read_to_buffer(internal::io_cache_st *fromfile, BUFFPEK *buffpek,
+static uint32_t  read_to_buffer(internal::IO_CACHE *fromfile, BUFFPEK *buffpek,
                                   uint32_t sort_length)
 {
   register uint32_t count;
@@ -592,7 +805,7 @@ static uint32_t  read_to_buffer(internal::io_cache_st *fromfile, BUFFPEK *buffpe
   return (count*sort_length);
 } /* read_to_buffer */
 
-static uint32_t  read_to_buffer_varlen(internal::io_cache_st *fromfile, BUFFPEK *buffpek,
+static uint32_t  read_to_buffer_varlen(internal::IO_CACHE *fromfile, BUFFPEK *buffpek,
                                          uint32_t sort_length)
 {
   register uint32_t count;
@@ -625,7 +838,7 @@ static uint32_t  read_to_buffer_varlen(internal::io_cache_st *fromfile, BUFFPEK
 
 
 static int  write_merge_key_varlen(MI_SORT_PARAM *info,
-					 internal::io_cache_st *to_file, unsigned char* key,
+					 internal::IO_CACHE *to_file, unsigned char* key,
                                          uint32_t sort_length, uint32_t count)
 {
   uint32_t idx;
@@ -642,11 +855,12 @@ static int  write_merge_key_varlen(MI_SORT_PARAM *info,
 }
 
 
-static int  write_merge_key(MI_SORT_PARAM*,
-				  internal::io_cache_st *to_file, unsigned char *key,
+static int  write_merge_key(MI_SORT_PARAM *info,
+				  internal::IO_CACHE *to_file, unsigned char *key,
 				  uint32_t sort_length, uint32_t count)
 {
-  return to_file->write(key, sort_length*count);
+  (void)info;
+  return my_b_write(to_file, key, (size_t) sort_length*count);
 }
 
 /*
@@ -674,8 +888,8 @@ class compare_functor
 */
 
 static int
-merge_buffers(MI_SORT_PARAM *info, uint32_t keys, internal::io_cache_st *from_file,
-              internal::io_cache_st *to_file, unsigned char **sort_keys, BUFFPEK *lastbuff,
+merge_buffers(MI_SORT_PARAM *info, uint32_t keys, internal::IO_CACHE *from_file,
+              internal::IO_CACHE *to_file, unsigned char **sort_keys, BUFFPEK *lastbuff,
               BUFFPEK *Fb, BUFFPEK *Tb)
 {
   int error;
@@ -692,7 +906,7 @@ merge_buffers(MI_SORT_PARAM *info, uint32_t keys, internal::io_cache_st *from_fi
   maxcount=keys/((uint) (Tb-Fb) +1);
   assert(maxcount > 0);
   if (to_file)
-    to_start_filepos= to_file->tell();
+    to_start_filepos=my_b_tell(to_file);
   strpos=(unsigned char*) sort_keys;
   sort_length=info->key_length;
 
@@ -791,9 +1005,9 @@ err:
 
 static int
 merge_index(MI_SORT_PARAM *info, uint32_t keys, unsigned char **sort_keys,
-            BUFFPEK *buffpek, int maxbuffer, internal::io_cache_st *tempfile)
+            BUFFPEK *buffpek, int maxbuffer, internal::IO_CACHE *tempfile)
 {
-  if (merge_buffers(info,keys,tempfile,(internal::io_cache_st*) 0,sort_keys,buffpek,buffpek,
+  if (merge_buffers(info,keys,tempfile,(internal::IO_CACHE*) 0,sort_keys,buffpek,buffpek,
                     buffpek+maxbuffer))
     return(1);
   return(0);
