@@ -18,13 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-
-#include <drizzled/current_session.h>
-#include <drizzled/error.h>
+#include "config.h"
 #include <drizzled/session.h>
-#include <drizzled/statistics_variables.h>
-#include <drizzled/system_variables.h>
 
 #include <assert.h>
 #include <stdio.h>
@@ -41,10 +36,7 @@
 #include "vio.h"
 #include "net_serv.h"
 
-namespace drizzle_plugin {
-
 using namespace std;
-using namespace drizzled;
 
 /*
   The following handles the differences when this is linked between the
@@ -60,90 +52,217 @@ using namespace drizzled;
 #define COMP_HEADER_SIZE 3		/* compression header extra size */
 
 #define MAX_PACKET_LENGTH (256L*256L*256L-1)
+const char  *not_error_sqlstate= "00000";
 
-static bool net_write_buff(NET*, const void*, uint32_t len);
+static bool net_write_buff(NET *net, const unsigned char *packet, uint32_t len);
 static int drizzleclient_net_real_write(NET *net, const unsigned char *packet, size_t len);
 
 /** Init with packet info. */
 
-void NET::init(int sock, uint32_t buffer_length)
+bool drizzleclient_net_init(NET *net, Vio* vio, uint32_t buffer_length)
 {
-  vio= new Vio(sock);
-  max_packet= (uint32_t) buffer_length;
-  max_packet_size= max(buffer_length, drizzled::global_system_variables.max_allowed_packet);
+  net->vio = vio;
+  net->max_packet= (uint32_t) buffer_length;
+  net->max_packet_size= max(buffer_length, drizzled::global_system_variables.max_allowed_packet);
 
-  buff= (unsigned char*) malloc((size_t) max_packet + NET_HEADER_SIZE + COMP_HEADER_SIZE);
-  buff_end= buff + max_packet;
-  error_= 0;
-  pkt_nr= compress_pkt_nr= 0;
-  write_pos= read_pos= buff;
-  compress= 0; 
-  where_b= remain_in_buf= 0;
-  last_errno= 0;
-  vio->fastsend();
+  if (!(net->buff=(unsigned char*) malloc((size_t) net->max_packet+
+                                          NET_HEADER_SIZE + COMP_HEADER_SIZE)))
+    return(1);
+  net->buff_end=net->buff+net->max_packet;
+  net->error=0; net->return_status=0;
+  net->pkt_nr=net->compress_pkt_nr=0;
+  net->write_pos=net->read_pos = net->buff;
+  net->last_error[0]=0;
+  net->compress=0; net->reading_or_writing=0;
+  net->where_b = net->remain_in_buf=0;
+  net->last_errno=0;
+  net->unused= 0;
+
+  if (vio != 0)                    /* If real connection */
+  {
+    net->fd  = drizzleclient_vio_fd(vio);            /* For perl DBI/DBD */
+    drizzleclient_vio_fastsend(vio);
+  }
+  return(0);
 }
 
-void NET::end()
+bool drizzleclient_net_init_sock(NET * net, int sock, int flags,
+                                 uint32_t buffer_length)
 {
-  free(buff);
-  buff= NULL;
+
+  Vio *drizzleclient_vio_tmp= drizzleclient_vio_new(sock, VIO_TYPE_TCPIP, flags);
+  if (drizzleclient_vio_tmp == NULL)
+    return true;
+  else
+    if (drizzleclient_net_init(net, drizzleclient_vio_tmp, buffer_length))
+    {
+      /* Only delete the temporary vio if we didn't already attach it to the
+       * NET object.
+       */
+      if (drizzleclient_vio_tmp && (net->vio != drizzleclient_vio_tmp))
+        drizzleclient_vio_delete(drizzleclient_vio_tmp);
+      else
+      {
+        (void) shutdown(sock, SHUT_RDWR);
+        (void) close(sock);
+      }
+      return true;
+    }
+  return false;
 }
 
-void NET::close()
+void drizzleclient_net_end(NET *net)
 {
-  drizzled::safe_delete(vio);
+  if (net->buff != NULL)
+    free(net->buff);
+  net->buff= NULL;
+  return;
 }
 
-bool NET::peer_addr(char *buf, size_t buflen, uint16_t& port)
+void drizzleclient_net_close(NET *net)
 {
-  return vio->peer_addr(buf, buflen, port);
+  if (net->vio != NULL)
+  {
+    drizzleclient_vio_delete(net->vio);
+    net->vio= 0;
+  }
 }
 
-void NET::keepalive(bool flag)
+bool drizzleclient_net_peer_addr(NET *net, char *buf, uint16_t *port, size_t buflen)
 {
-  vio->keepalive(flag);
+  return drizzleclient_vio_peer_addr(net->vio, buf, port, buflen);
 }
 
-int NET::get_sd() const
+void drizzleclient_net_keepalive(NET *net, bool flag)
 {
-  return vio->get_fd();
+  drizzleclient_vio_keepalive(net->vio, flag);
+}
+
+int drizzleclient_net_get_sd(NET *net)
+{
+  return net->vio->sd;
+}
+
+bool drizzleclient_net_more_data(NET *net)
+{
+  return (net->vio == 0 || net->vio->read_pos < net->vio->read_end);
 }
 
 /** Realloc the packet buffer. */
 
 static bool drizzleclient_net_realloc(NET *net, size_t length)
 {
+  unsigned char *buff;
+  size_t pkt_length;
+
   if (length >= net->max_packet_size)
   {
     /* @todo: 1 and 2 codes are identical. */
-    net->error_= 3;
-    net->last_errno= ER_NET_PACKET_TOO_LARGE;
-    my_error(ER_NET_PACKET_TOO_LARGE, MYF(0));
-    return 1;
+    net->error= 1;
+    net->last_errno= CR_NET_PACKET_TOO_LARGE;
+    return(1);
   }
-  size_t pkt_length = (length + IO_SIZE - 1) & ~(IO_SIZE - 1);
+  pkt_length = (length+IO_SIZE-1) & ~(IO_SIZE-1);
   /*
     We must allocate some extra bytes for the end 0 and to be able to
     read big compressed blocks
   */
-  unsigned char* buff= (unsigned char*)realloc((char*) net->buff, pkt_length + NET_HEADER_SIZE + COMP_HEADER_SIZE);
-  net->buff=net->write_pos= buff;
-  net->buff_end= buff + (net->max_packet= (uint32_t) pkt_length);
-  return 0;
+  if (!(buff= (unsigned char*) realloc((char*) net->buff, pkt_length +
+                               NET_HEADER_SIZE + COMP_HEADER_SIZE)))
+  {
+    /* @todo: 1 and 2 codes are identical. */
+    net->error= 1;
+    net->last_errno= CR_OUT_OF_MEMORY;
+    /* In the server the error is reported by MY_WME flag. */
+    return(1);
+  }
+  net->buff=net->write_pos=buff;
+  net->buff_end=buff+(net->max_packet= (uint32_t) pkt_length);
+  return(0);
 }
 
-bool NET::flush()
+
+/**
+   Check if there is any data to be read from the socket.
+
+   @param sd   socket descriptor
+
+   @retval
+   0  No data to read
+   @retval
+   1  Data or EOF to read
+   @retval
+   -1   Don't know if data is ready or not
+*/
+
+static bool net_data_is_ready(int sd)
 {
-  bool error= false;
-  if (buff != write_pos)
+  struct pollfd ufds;
+  int res;
+
+  ufds.fd= sd;
+  ufds.events= POLLIN | POLLPRI;
+  if (!(res= poll(&ufds, 1, 0)))
+    return 0;
+  if (res < 0 || !(ufds.revents & (POLLIN | POLLPRI)))
+    return 0;
+  return 1;
+}
+
+/**
+   Remove unwanted characters from connection
+   and check if disconnected.
+
+   Read from socket until there is nothing more to read. Discard
+   what is read.
+
+   If there is anything when to read 'drizzleclient_net_clear' is called this
+   normally indicates an error in the protocol.
+
+   When connection is properly closed (for TCP it means with
+   a FIN packet), then select() considers a socket "ready to read",
+   in the sense that there's EOF to read, but read() returns 0.
+
+   @param net            NET handler
+   @param clear_buffer           if <> 0, then clear all data from comm buff
+*/
+
+void drizzleclient_net_clear(NET *net, bool clear_buffer)
+{
+  if (clear_buffer)
   {
-    error= drizzleclient_net_real_write(this, buff, write_pos - buff) ? true : false;
-    write_pos= buff;
+    while (net_data_is_ready(net->vio->sd) > 0)
+    {
+      /* The socket is ready */
+      if (drizzleclient_vio_read(net->vio, net->buff,
+                   (size_t) net->max_packet) <= 0)
+      {
+        net->error= 2;
+        break;
+      }
+    }
+  }
+  net->pkt_nr=net->compress_pkt_nr=0;        /* Ready for new command */
+  net->write_pos=net->buff;
+  return;
+}
+
+
+/** Flush write_buffer if not empty. */
+
+bool drizzleclient_net_flush(NET *net)
+{
+  bool error= 0;
+  if (net->buff != net->write_pos)
+  {
+    error=drizzleclient_net_real_write(net, net->buff,
+                         (size_t) (net->write_pos - net->buff)) ? 1 : 0;
+    net->write_pos=net->buff;
   }
   /* Sync packet number if using compression */
-  if (compress)
-    pkt_nr= compress_pkt_nr;
-  return error;
+  if (net->compress)
+    net->pkt_nr=net->compress_pkt_nr;
+  return(error);
 }
 
 
@@ -161,10 +280,9 @@ bool NET::flush()
    If compression is used the original package is modified!
 */
 
-static bool
-drizzleclient_net_write(NET* net, const void* packet0, size_t len)
+bool
+drizzleclient_net_write(NET *net,const unsigned char *packet,size_t len)
 {
-  const unsigned char* packet= reinterpret_cast<const unsigned char*>(packet0);
   unsigned char buff[NET_HEADER_SIZE];
   if (unlikely(!net->vio)) /* nowhere to write */
     return 0;
@@ -178,7 +296,8 @@ drizzleclient_net_write(NET* net, const void* packet0, size_t len)
     const uint32_t z_size = MAX_PACKET_LENGTH;
     int3store(buff, z_size);
     buff[3]= (unsigned char) net->pkt_nr++;
-    if (net_write_buff(net, buff, NET_HEADER_SIZE) || net_write_buff(net, packet, z_size))
+    if (net_write_buff(net, buff, NET_HEADER_SIZE) ||
+        net_write_buff(net, packet, z_size))
       return 1;
     packet += z_size;
     len-=     z_size;
@@ -186,7 +305,9 @@ drizzleclient_net_write(NET* net, const void* packet0, size_t len)
   /* Write last packet */
   int3store(buff,len);
   buff[3]= (unsigned char) net->pkt_nr++;
-  return net_write_buff(net, buff, NET_HEADER_SIZE) || net_write_buff(net, packet, len);
+  if (net_write_buff(net, buff, NET_HEADER_SIZE))
+    return 1;
+  return net_write_buff(net,packet,len) ? 1 : 0;
 }
 
 /**
@@ -216,7 +337,7 @@ drizzleclient_net_write(NET* net, const void* packet0, size_t len)
    1    error
 */
 
-static bool
+bool
 drizzleclient_net_write_command(NET *net,unsigned char command,
                   const unsigned char *header, size_t head_len,
                   const unsigned char *packet, size_t len)
@@ -249,9 +370,9 @@ drizzleclient_net_write_command(NET *net,unsigned char command,
   }
   int3store(buff,length);
   buff[3]= (unsigned char) net->pkt_nr++;
-  return (net_write_buff(net, buff, header_size) ||
+  return((net_write_buff(net, buff, header_size) ||
           (head_len && net_write_buff(net, header, head_len)) ||
-          net_write_buff(net, packet, len) || net->flush());
+          net_write_buff(net, packet, len) || drizzleclient_net_flush(net)) ? 1 : 0 );
 }
 
 /**
@@ -281,9 +402,8 @@ drizzleclient_net_write_command(NET *net,unsigned char command,
 */
 
 static bool
-net_write_buff(NET* net, const void* packet0, uint32_t len)
+net_write_buff(NET *net, const unsigned char *packet, uint32_t len)
 {
-  const unsigned char* packet= reinterpret_cast<const unsigned char*>(packet0);
   uint32_t left_length;
   if (net->compress && net->max_packet > MAX_PACKET_LENGTH)
     left_length= MAX_PACKET_LENGTH - (net->write_pos - net->buff);
@@ -343,34 +463,56 @@ net_write_buff(NET* net, const void* packet0, uint32_t len)
 static int
 drizzleclient_net_real_write(NET *net, const unsigned char *packet, size_t len)
 {
+  size_t length;
+  const unsigned char *pos, *end;
+  uint32_t retry_count= 0;
+
   /* Backup of the original SO_RCVTIMEO timeout */
 
-  if (net->error_ == 2)
+  if (net->error == 2)
     return(-1);                /* socket can't be used */
 
+  net->reading_or_writing=2;
   if (net->compress)
   {
+    size_t complen;
+    unsigned char *b;
     const uint32_t header_length=NET_HEADER_SIZE+COMP_HEADER_SIZE;
-    unsigned char* b= (unsigned char*) malloc(len + NET_HEADER_SIZE + COMP_HEADER_SIZE);
+    if (!(b= (unsigned char*) malloc(len + NET_HEADER_SIZE +
+                             COMP_HEADER_SIZE)))
+    {
+      net->error= 2;
+      net->last_errno= CR_OUT_OF_MEMORY;
+      /* In the server, the error is reported by MY_WME flag. */
+      net->reading_or_writing= 0;
+      return(1);
+    }
     memcpy(b+header_length,packet,len);
 
-    size_t complen= len * 120 / 100 + 12;
-    unsigned char* compbuf= new unsigned char[complen];
-    uLongf tmp_complen= complen;
-    int res= compress((Bytef*) compbuf, &tmp_complen,
-      (Bytef*) (b+header_length),
-      len);
-    complen= tmp_complen;
+    complen= len * 120 / 100 + 12;
+    unsigned char * compbuf= (unsigned char *) malloc(complen);
+    if (compbuf != NULL)
+    {
+      uLongf tmp_complen= complen;
+      int res= compress((Bytef*) compbuf, &tmp_complen,
+                        (Bytef*) (b+header_length),
+                        len);
+      complen= tmp_complen;
 
-    delete[] compbuf;
+      free(compbuf);
 
-    if (res != Z_OK || complen >= len)
-      complen= 0;
+      if ((res != Z_OK) || (complen >= len))
+        complen= 0;
+      else
+      {
+        size_t tmplen= complen;
+        complen= len;
+        len= tmplen;
+      }
+    }
     else
     {
-      size_t tmplen= complen;
-      complen= len;
-      len= tmplen;
+      complen=0;
     }
     int3store(&b[NET_HEADER_SIZE],complen);
     int3store(b,len);
@@ -379,16 +521,13 @@ drizzleclient_net_real_write(NET *net, const unsigned char *packet, size_t len)
     packet= b;
   }
 
-  uint32_t retry_count= 0;
-  const unsigned char* pos= packet;
-  const unsigned char* end= pos + len;
+  pos= packet;
+  end=pos+len;
   /* Loop until we have read everything */
   while (pos != end)
   {
     assert(pos);
-    // TODO - see bug comment below - will we crash now?
-    size_t length;
-    if ((long) (length= net->vio->write( pos, (size_t) (end-pos))) <= 0)
+    if ((long) (length= drizzleclient_vio_write(net->vio, pos, (size_t) (end-pos))) <= 0)
     {
      /*
       * We could end up here with net->vio == NULL
@@ -398,22 +537,22 @@ drizzleclient_net_real_write(NET *net, const unsigned char *packet, size_t len)
       if (net->vio == NULL)
         break;
       
-      const bool interrupted= net->vio->should_retry();
+      const bool interrupted= drizzleclient_vio_should_retry(net->vio);
       /*
         If we read 0, or we were interrupted this means that
         we need to switch to blocking mode and wait until the timeout
         on the socket kicks in.
       */
-      if (interrupted || length == 0)
+      if ((interrupted || length == 0))
       {
         bool old_mode;
-        while (net->vio->blocking(true, &old_mode) < 0)
+
+        while (drizzleclient_vio_blocking(net->vio, true, &old_mode) < 0)
         {
-          if (net->vio->should_retry() && retry_count++ < net->retry_count)
+          if (drizzleclient_vio_should_retry(net->vio) && retry_count++ < net->retry_count)
             continue;
-          net->error_= 2;                     /* Close socket */
-          net->last_errno= ER_NET_PACKET_TOO_LARGE;
-          my_error(ER_NET_PACKET_TOO_LARGE, MYF(0));
+          net->error= 2;                     /* Close socket */
+          net->last_errno= CR_NET_PACKET_TOO_LARGE;
           goto end;
         }
         retry_count=0;
@@ -425,25 +564,23 @@ drizzleclient_net_real_write(NET *net, const unsigned char *packet, size_t len)
           continue;
       }
 
-      if (net->vio->get_errno() == EINTR)
+      if (drizzleclient_vio_errno(net->vio) == EINTR)
       {
         continue;
       }
-      net->error_= 2;                /* Close socket */
-      net->last_errno= interrupted ? CR_NET_WRITE_INTERRUPTED : CR_NET_ERROR_ON_WRITE;
+      net->error= 2;                /* Close socket */
+      net->last_errno= (interrupted ? CR_NET_WRITE_INTERRUPTED :
+                        CR_NET_ERROR_ON_WRITE);
       break;
     }
-    pos+= length;
-
-    /* If this is an error we may not have a current_session any more */
-    if (current_session)
-      current_session->status_var.bytes_sent+= length;
+    pos+=length;
   }
 end:
-  if (net->compress)
-    free((char*)packet);
+  if ((net->compress) && (packet != NULL))
+    free((char*) packet);
+  net->reading_or_writing=0;
 
-  return (int) (pos != end);
+  return(((int) (pos != end)));
 }
 
 
@@ -459,46 +596,51 @@ end:
 static uint32_t
 my_real_read(NET *net, size_t *complen)
 {
-  size_t length= 0;
-  uint32_t retry_count=0;
+  unsigned char *pos;
+  size_t length;
+  uint32_t i,retry_count=0;
   size_t len=packet_error;
-  uint32_t remain= net->compress ? NET_HEADER_SIZE+COMP_HEADER_SIZE : NET_HEADER_SIZE;
+  uint32_t remain= (net->compress ? NET_HEADER_SIZE+COMP_HEADER_SIZE :
+                    NET_HEADER_SIZE);
 
   *complen = 0;
 
+  net->reading_or_writing= 1;
   /* Read timeout is set in drizzleclient_net_set_read_timeout */
 
-  unsigned char* pos = net->buff + net->where_b;        /* net->packet -4 */
+  pos = net->buff + net->where_b;        /* net->packet -4 */
 
-  for (uint32_t i= 0; i < 2 ; i++)
+  for (i= 0; i < 2 ; i++)
   {
     while (remain > 0)
     {
       /* First read is done with non blocking mode */
-      if ((long) (length= net->vio->read(pos, remain)) <= 0L)
+      if ((long) (length= drizzleclient_vio_read(net->vio, pos, remain)) <= 0L)
       {
         if (net->vio == NULL)
           goto end;
 
-        const bool interrupted = net->vio->should_retry();
+        const bool interrupted = drizzleclient_vio_should_retry(net->vio);
 
         if (interrupted)
         {                    /* Probably in MIT threads */
           if (retry_count++ < net->retry_count)
             continue;
         }
-        if (net->vio->get_errno() == EINTR)
+        if (drizzleclient_vio_errno(net->vio) == EINTR)
         {
           continue;
         }
         len= packet_error;
-        net->error_= 2;                /* Close socket */
-        net->last_errno= net->vio->was_interrupted() ? CR_NET_READ_INTERRUPTED : CR_NET_READ_ERROR;
+        net->error= 2;                /* Close socket */
+        net->last_errno= (drizzleclient_vio_was_interrupted(net->vio) ?
+                          CR_NET_READ_INTERRUPTED :
+                          CR_NET_READ_ERROR);
+        ER(net->last_errno);
         goto end;
       }
       remain -= (uint32_t) length;
       pos+= length;
-      current_session->status_var.bytes_received+= length;
     }
     if (i == 0)
     {                    /* First parts is packet length */
@@ -508,7 +650,6 @@ my_real_read(NET *net, size_t *complen)
       {
         len= packet_error;
         /* Not a NET error on the client. XXX: why? */
-        my_error(ER_NET_PACKETS_OUT_OF_ORDER, MYF(0));
         goto end;
       }
       net->compress_pkt_nr= ++net->pkt_nr;
@@ -530,14 +671,6 @@ my_real_read(NET *net, size_t *complen)
       {
         if (drizzleclient_net_realloc(net,helping))
         {
-          /* Clear the buffer so libdrizzle doesn't keep retrying */
-          while (len > 0)
-          {
-            length= read(net->vio->get_fd(), net->buff, min((size_t)net->max_packet, len));
-            assert((long)length > 0L);
-            len-= length;
-          }
-            
           len= packet_error;          /* Return error and close connection */
           goto end;
         }
@@ -548,7 +681,9 @@ my_real_read(NET *net, size_t *complen)
   }
 
 end:
-  return len;
+  net->reading_or_writing= 0;
+
+  return(len);
 }
 
 
@@ -568,12 +703,12 @@ end:
    net->read_pos points to the read data.
 */
 
-static uint32_t
+uint32_t
 drizzleclient_net_read(NET *net)
 {
   size_t len, complen;
 
-  if (not net->compress)
+  if (!net->compress)
   {
     len = my_real_read(net,&complen);
     if (len == MAX_PACKET_LENGTH)
@@ -581,25 +716,19 @@ drizzleclient_net_read(NET *net)
       /* First packet of a multi-packet.  Concatenate the packets */
       uint32_t save_pos = net->where_b;
       size_t total_length= 0;
-
       do
       {
         net->where_b += len;
         total_length += len;
         len = my_real_read(net,&complen);
       } while (len == MAX_PACKET_LENGTH);
-
       if (len != packet_error)
-      {
         len+= total_length;
-      }
       net->where_b = save_pos;
     }
     net->read_pos = net->buff + net->where_b;
-
     if (len != packet_error)
       net->read_pos[len]=0;        /* Safeguard for drizzleclient_use_result */
-
     return len;
   }
   else
@@ -697,7 +826,7 @@ drizzleclient_net_read(NET *net)
 
           if (error != Z_OK)
           {
-            net->error_= 2;            /* caller will close socket */
+            net->error= 2;            /* caller will close socket */
             net->last_errno= CR_NET_UNCOMPRESS_ERROR;
           }
           else
@@ -708,9 +837,7 @@ drizzleclient_net_read(NET *net)
         }
       }
       else
-      {
         complen= packet_len;
-      }
 
     }
     buf_length+= complen;
@@ -724,41 +851,39 @@ drizzleclient_net_read(NET *net)
     net->read_pos[len]=0;        /* Safeguard for drizzleclient_use_result */
   }
   return len;
-}
+  }
 
-void NET::set_read_timeout(uint32_t timeout)
+
+void drizzleclient_net_set_read_timeout(NET *net, uint32_t timeout)
 {
-  read_timeout_= timeout;
+  net->read_timeout= timeout;
 #ifndef __sun
-  if (vio)
-    vio->timeout(0, timeout);
+  if (net->vio)
+    drizzleclient_vio_timeout(net->vio, 0, timeout);
 #endif
   return;
 }
 
-void NET::set_write_timeout(uint32_t timeout)
+
+void drizzleclient_net_set_write_timeout(NET *net, uint32_t timeout)
 {
-  write_timeout_= timeout;
+  net->write_timeout= timeout;
 #ifndef __sun
-  if (vio)
-    vio->timeout(1, timeout);
+  if (net->vio)
+    drizzleclient_vio_timeout(net->vio, 1, timeout);
 #endif
   return;
 }
+/**
+  Clear possible error state of struct NET
 
-bool NET::write(const void* data, size_t size)
+  @param net  clear the state of the argument
+*/
+
+void drizzleclient_drizzleclient_net_clear_error(NET *net)
 {
-  return drizzleclient_net_write(this, data, size);
+  net->last_errno= 0;
+  net->last_error[0]= '\0';
+  strcpy(net->sqlstate, not_error_sqlstate);
 }
 
-bool NET::write_command(unsigned char command, data_ref header, data_ref body)
-{
-  return drizzleclient_net_write_command(this, command, header.data(), header.size(), body.data(), body.size());
-}
-
-uint32_t NET::read()
-{
-  return drizzleclient_net_read(this);
-}
-
-} /* namespace drizzle_plugin */
