@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2010 Sun Microsystems, Inc.
+ *  Copyright (C) 2010 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_REGISTRY_DICTIONARY_MODULES_H
+#define PLUGIN_REGISTRY_DICTIONARY_MODULES_H
 
 class ModulesTool : public drizzled::plugin::TableFunction
 {
@@ -28,8 +29,8 @@ public:
 
   class Generator : public drizzled::plugin::TableFunction::Generator 
   {
-    drizzled::module::Registry::ModuleList modules;
-    drizzled::module::Registry::ModuleList::iterator it;
+    std::vector<drizzled::plugin::Module *> modules;
+    std::vector<drizzled::plugin::Module *>::iterator it;
 
   public:
     Generator(drizzled::Field **arg);
@@ -44,3 +45,4 @@ public:
   }
 };
 
+#endif /* PLUGIN_REGISTRY_DICTIONARY_MODULES_H */
