@@ -18,7 +18,7 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <plugin/schema_engine/schema.h>
 
@@ -29,23 +29,39 @@ using namespace drizzled;
 
 static drizzled::plugin::StorageEngine *schema_plugin= NULL;
 
-static int init(drizzled::module::Context &context)
+static int init(drizzled::plugin::Registry &registry)
 {
-  schema_plugin= new Schema;
-  context.add(schema_plugin);
+  schema_plugin= new(std::nothrow) Schema();
+
+  if (not schema_plugin)
+  {
+    return 1;
+  }
+
+  registry.add(schema_plugin);
+
+  return 0;
+}
+
+static int finalize(drizzled::plugin::Registry &registry)
+{
+  registry.remove(schema_plugin);
+  delete schema_plugin;
+
   return 0;
 }
 
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
-  "schema_engine",
+  "SchemaEngine",
   "1.0",
   "Brian Aker",
-  N_("Schema engine"),
+  "This implements the default file based Schema engine.",
   PLUGIN_LICENSE_GPL,
-  init,
-  "signal_handler",
-  NULL
+  init,     /* Plugin Init */
+  finalize,     /* Plugin Deinit */
+  NULL,               /* system variables */
+  NULL                /* config options   */
 }
 DRIZZLE_DECLARE_PLUGIN_END;
