@@ -18,30 +18,39 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <plugin/session_dictionary/dictionary.h>
+#include "config.h"
+#include "plugin/session_dictionary/dictionary.h"
 
 using namespace drizzled;
 
-static int init(drizzled::module::Context &context)
+static ProcesslistTool *processlist;
+
+static int init(drizzled::plugin::Registry &registry)
 {
-  context.add(new ProcesslistTool);
-  context.add(new session_dictionary::Sessions);
-  context.add(new session_dictionary::Variables);
-  context.add(new session_dictionary::Savepoints);
+  processlist= new(std::nothrow)ProcesslistTool;
+  registry.add(processlist);
   
   return 0;
 }
 
+static int finalize(drizzled::plugin::Registry &registry)
+{
+  registry.remove(processlist);
+  delete processlist;
+
+  return 0;
+}
+
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
   "session_dictionary",
-  "1.1",
+  "1.0",
   "Brian Aker",
-  N_("Dictionaries of session variables and statements"),
+  "Dictionary for session information, aka proccesslist, etc.",
   PLUGIN_LICENSE_GPL,
   init,
+  finalize,
   NULL,
   NULL
 }
