@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2010 Sun Microsystems, Inc.
+ *  Copyright (C) 2010 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,10 +18,10 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_STATUS_DICTIONARY_VARIABLES_H
+#define PLUGIN_STATUS_DICTIONARY_VARIABLES_H
 
-#include <plugin/status_dictionary/dictionary.h>
-#include <drizzled/current_session.h>
+#include "plugin/status_dictionary/dictionary.h"
 
 class VariablesTool : public StateTool
 {
@@ -37,7 +37,8 @@ public:
 
   drizzled::drizzle_show_var *getVariables()
   {
-    return enumerate_sys_vars(current_session); // Final value unused
+    return enumerate_sys_vars(current_session, false); // Final value unused
   }
 };
 
+#endif /* PLUGIN_STATUS_DICTIONARY_VARIABLES_H */
