@@ -17,9 +17,10 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_TABLE_CACHE_DICTIONARY_TABLE_DEFINITION_CACHE_H
+#define PLUGIN_TABLE_CACHE_DICTIONARY_TABLE_DEFINITION_CACHE_H
 
-#include <drizzled/generator.h>
+#include "drizzled/table_share.h"
 
 namespace table_cache_dictionary {
 
@@ -30,12 +31,20 @@ public:
 
   class Generator : public drizzled::plugin::TableFunction::Generator 
   {
-    drizzled::generator::TableDefinitionCache table_definition_cache_generator;
+    bool is_primed;
+    drizzled::TableShare *share;
+    drizzled::TableDefinitionCache::iterator table_share_iterator;
+
+    void fill();
+
+    bool nextCore();
+    bool next();
 
   public:
     bool populate();
 
-    Generator(drizzled::Field**);
+    Generator(drizzled::Field **arg);
+    ~Generator();
   };
 
   Generator *generator(drizzled::Field **arg)
@@ -46,3 +55,4 @@ public:
 
 } /* namespace table_cache_dictionary */
 
+#endif /* PLUGIN_TABLE_CACHE_DICTIONARY_TABLE_DEFINITION_CACHE_H */
